! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

LOGICAL FUNCTION ALLOCBONDS (NDD)

!
! Memory allocation for bond properties
!

USE PARAMETERS

INTEGER, INTENT(IN) :: NDD

ALLOCBONDS = .true.

! For neighbors and environments

if (allocated(LA_COUNT)) deallocate(LA_COUNT)
allocate(LA_COUNT(NA,NSP,NS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Function: ALLOCBONDS"//CHAR(0), "Table: LA_COUNT"//CHAR(0))
  ALLOCBONDS = .false.
  goto 001
endif

if (allocated(SA_COUNT)) deallocate(SA_COUNT)
allocate(SA_COUNT(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Function: ALLOCBONDS"//CHAR(0), "Table: SA_COUNT"//CHAR(0))
  ALLOCBONDS = .false.
  goto 001
endif

if (allocated(MA_COUNT)) deallocate(MA_COUNT)
allocate(MA_COUNT(NSP,NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Function: ALLOCBONDS"//CHAR(0), "Table: MA_COUNT"//CHAR(0))
  ALLOCBONDS = .false.
  goto 001
endif

do i=1, NSP
  SA_COUNT(i)=0
  do j=1, NSP
    MA_COUNT(j,i)=0
  enddo
  do j=1, NA
  do k=1, NS
    LA_COUNT(j,i,k)=0
  enddo
  enddo
enddo

! Edge and Corner sharing !

if (allocated(EDGETA)) deallocate(EDGETA)
allocate(EDGETA(NSP,NSP), STAT=ERR)
if (ERR .ne. 0) then
call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Function: ALLOCBONDS"//CHAR(0), "Table: EDGETA"//CHAR(0))
  ALLOCBONDS = .false.
  goto 001
endif

if (allocated(CORTA)) deallocate(CORTA)
allocate(CORTA(NSP,NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Function: ALLOCBONDS"//CHAR(0), "Table: CORTA"//CHAR(0))
  ALLOCBONDS = .false.
  goto 001
endif

if (allocated(DEFTA)) deallocate(DEFTA)
allocate(DEFTA(NSP,NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Function: ALLOCBONDS"//CHAR(0), "Table: DEFTA"//CHAR(0))
  ALLOCBONDS = .false.
  goto 001
endif

if (allocated(TDA)) deallocate(TDA)
allocate(TDA(NSP,NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Function: ALLOCBONDS"//CHAR(0), "Table: TDA"//CHAR(0))
  ALLOCBONDS = .false.
  goto 001
endif
do m=1, NSP
do n=1, NSP
  EDGETA(n,m)=0
  CORTA(n,m)=0
  DEFTA(n,m)=0
  TDA(n,m)=0
enddo
enddo

if (allocated(CORNERA)) deallocate(CORNERA)
allocate(CORNERA(NSP,NSP,NS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Function: ALLOCBONDS"//CHAR(0), "Table: CORNERA"//CHAR(0))
  ALLOCBONDS = .false.
  goto 001
endif

if (allocated(EDGEA)) deallocate(EDGEA)
allocate(EDGEA(NSP,NSP,NS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Function: ALLOCBONDS"//CHAR(0), "Table: EDGEA"//CHAR(0))
  ALLOCBONDS = .false.
  goto 001
endif

if (allocated(DEFA)) deallocate(DEFA)
allocate(DEFA(NSP,NSP,NS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Function: ALLOCBONDS"//CHAR(0), "Table: DEFA"//CHAR(0))
  ALLOCBONDS = .false.
  goto 001
endif

if (allocated(TDSA)) deallocate(TDSA)
allocate(TDSA(NSP,NSP,NS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Function: ALLOCBONDS"//CHAR(0), "Table: TDSA"//CHAR(0))
  ALLOCBONDS = .false.
  goto 001
endif

do m=1, NSP
do n=1, NSP
do o=1, NS
  CORNERA(m,n,o)=0
  EDGEA(m,n,o)=0
  DEFA(m,n,o)=0
  TDSA(m,n,o)=0
enddo
enddo
enddo

if (allocated(NUM_GSA)) deallocate(NUM_GSA)
allocate(NUM_GSA(NSP,NA*10), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Function: bonding"//CHAR(0), "Table: NUM_GSA"//CHAR(0))
  ALLOCBONDS = .false.
  goto 001
endif

if (allocated(LISTE_GEOMSA)) deallocate(LISTE_GEOMSA)
allocate(LISTE_GEOMSA(NSP,NA*10), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Function: bonding"//CHAR(0), "Table: LISTE_GEOMSA"//CHAR(0))
  ALLOCBONDS = .false.
  goto 001
endif

if (allocated(TOT_GSA)) deallocate(TOT_GSA)
allocate(TOT_GSA(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Function: bonding"//CHAR(0), "Table: TOT_GSA"//CHAR(0))
 ALLOCBONDS = .false.
 goto 001
endif

if (allocated(TGSA)) deallocate(TGSA)
allocate(TGSA(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Function: bonding"//CHAR(0), "Table: TGSA"//CHAR(0))
  ALLOCBONDS = .false.
  goto 001
endif

if (allocated(STATBD)) deallocate(STATBD)
allocate(STATBD(NSP,NSP,0:NDD), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Function: ALLOCBONDS"//CHAR(0), "Table: STATBD"//CHAR(0))
  ALLOCBONDS = .false.
  goto 001
endif
do i=1, NSP
do j=1, NSP
do k=0, NDD
  STATBD(i,j,k)=0
enddo
enddo
enddo

ALLOCBONDS=.true.

001 continue

END FUNCTION

SUBROUTINE DEALLOCBONDS

!
! Memory allocation for bond properties
!

USE PARAMETERS

! For neighbors and environments

if (allocated(LA_COUNT)) deallocate(LA_COUNT)
if (allocated(SA_COUNT)) deallocate(SA_COUNT)
if (allocated(MA_COUNT)) deallocate(MA_COUNT)
if (allocated(EDGETA)) deallocate(EDGETA)
if (allocated(CORTA)) deallocate(CORTA)
if (allocated(DEFTA)) deallocate(DEFTA)
if (allocated(TDA)) deallocate(TDA)
if (allocated(CORNERA)) deallocate(CORNERA)
if (allocated(EDGEA)) deallocate(EDGEA)
if (allocated(DEFA)) deallocate(DEFA)
if (allocated(TDSA)) deallocate(TDSA)
if (allocated(ETYPEA)) deallocate(ETYPEA)
if (allocated(CTYPEA)) deallocate(CTYPEA)
if (allocated(DETYPEA)) deallocate(DETYPEA)
if (allocated(ETDA)) deallocate(ETDA)
if (allocated(NUM_GSA)) deallocate(NUM_GSA)
if (allocated(TOT_GSA)) deallocate(TOT_GSA)
if (allocated(TGSA)) deallocate(TGSA)

END SUBROUTINE
