/* This file is part of I.S.A.A.C.S.

   I.S.A.A.C.S. is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

    I.S.A.A.C.S. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>

#include "global.h"
#include "interface.h"
#include "callbacks.h"
#include "curve.h"
#include "bdcall.h"
#include "angcall.h"

int bond_angles_ (int *);
int bond_diedrals_ (int *);
int rundmtx_ (int *, int *);

G_MODULE_EXPORT void on_aspec_get ()
{
  int i, j;
  GtkEntry * EntryGtk;

  i=gtk_combo_box_get_active(GTK_COMBO_BOX(aspeca));
  j=gtk_combo_box_get_active(GTK_COMBO_BOX(aspecb));
  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "acut");
  update_entry_double (EntryGtk, & cutoffs[i][j]);
}

void initang ()
{
  int h, i, j, k, l;

  h = numgr+numgq+numsq+numsk+numbd;
  for ( i = 0 ; i < nspec ; i++ )
  {
    for ( j = 0 ; j < nspec ; j++ )
    {
      for ( k = 0 ; k < nspec ; k++ )
      {
        curves[h] -> title = g_strdup_printf("%s%s%s%s%s%s%s", "Angles [",
                          label[i], "-", label[j], "-", label[k], "]");
        h=h+1;
      }
    }
  }
  for ( i = 0 ; i < nspec ; i++ )
  {
    for ( j = 0 ; j < nspec ; j++ )
    {
      for ( k = 0 ; k < nspec ; k++ )
      {
        for ( l = 0 ; l < nspec ; l++ )
        {
          curves[h] -> title = g_strdup_printf("%s%s%s%s%s%s%s%s%s", "Dihedral [",
                            label[i], "-", label[j], "-", label[k], "-", label[l], "]");
          h=h+1;
        }
      }
    }
  }
  i = numgr+numgq+numsq+numsk+numbd;
  for ( j = i ; j < i+numan ; j++ )
  {
    addcurwidget (curves[j] -> title, & j, & i, tablegtkan);
    curves[j] -> xlabel = g_strdup_printf("%s", "Angle [°]");
    curves[j] -> cstart = i;
    curves[j] -> cend = i+numan;
  }
}

G_MODULE_EXPORT void on_acut_activate ()
{
  GtkEntry * EntryGtk;
  int i, j, k, l, m, n;
  const gchar *str;

  if (dmtx) dmtx=FALSE;
  i=gtk_combo_box_get_active(GTK_COMBO_BOX(aspeca));
  j=gtk_combo_box_get_active(GTK_COMBO_BOX(aspecb));
  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "acut");
  str=gtk_entry_get_text (EntryGtk);
  cutoffs[i][j] = atof(str);
  update_entry_double (EntryGtk, & cutoffs[i][j]);
  k=gtk_combo_box_get_active(GTK_COMBO_BOX(bspeca));
  l=gtk_combo_box_get_active(GTK_COMBO_BOX(bspecb));
  m=gtk_combo_box_get_active(GTK_COMBO_BOX(rspeca));
  n=gtk_combo_box_get_active(GTK_COMBO_BOX(rspecb));
  if (k==i && l==j)
  {
    EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "cut");
    update_entry_double (EntryGtk, & cutoffs[i][j]);
  }
  if (m==i && n==j)
  {
    EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "rcut");
    update_entry_double (EntryGtk, & cutoffs[i][j]);
  }
}

G_MODULE_EXPORT void on_atcut_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *str;

  if (dmtx) dmtx=FALSE;
  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "atcut");
  str=gtk_entry_get_text (EntryGtk);
  grtotcutoff = atof(str);
  update_entry_double (EntryGtk, & grtotcutoff);
  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "tcut");
  update_entry_double (EntryGtk, & grtotcutoff);
  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "rtcut");
  update_entry_double (EntryGtk, & grtotcutoff);
}

G_MODULE_EXPORT void on_deltaa_activate ()
{
  GtkWidget * info;
  GtkEntry * EntryGtk;
  const gchar * sta;
  double dela;
  int i, j;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "deltaa");
  sta=gtk_entry_get_text (EntryGtk);
  dela = atof(sta);
  i = dela;
  update_entry_int (EntryGtk, & i);
  if (i != num_delta_a)
  {
    num_delta_a = i;
    j= numgr+numgq+numsq+numsk+numbd;
    if (curves[j] -> data != NULL)
    {
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "angcheck");
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
      if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispang");
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
      if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
    }
    i= numgr+numgq+numsq+numsk+numbd+numan;
    hide_curves (&j, &i);
    for ( i = j ; i < j+numan ; i ++ )
    {
      free(curves[i] -> data);
      curves[i] -> data = NULL;
    }
  }
}

G_MODULE_EXPORT void on_Calcangles_released()
{
  GtkWidget * info;
  GtkWidget * pop;
  GtkWidget * win;
  int i, j, k, l, m;
  gboolean calca=TRUE;

  if (num_delta_a >= 2)
  {
    calca=testcutoffs();
    if (calca)
    {
      cutoffsend ();
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "angcheck");
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
      gtk_widget_set_sensitive (info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispang");
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
      gtk_widget_set_sensitive (info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "deltaa");
      gtk_widget_set_sensitive(info, 0);
      gtk_widget_set_sensitive(aspeca, 0);
      gtk_widget_set_sensitive(aspecb, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "acut");
      gtk_widget_set_sensitive(info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "atcut");
      gtk_widget_set_sensitive(info, 0);
      if (! angrun)
      {
        initang ();
        angrun = TRUE;
      }
      i = 0;
      setcalcs (& i);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Mainstatusbar");
      k=gtk_statusbar_push (GTK_STATUSBAR(info), 4, "Computing angle distributions");
      win = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "angles");
      if (! dmtx)
      {
        prepspbv (& i);
        gtk_window_set_opacity (GTK_WINDOW (win), opac);
        pop=show_popup ("Please wait computing the nearest neigbhors table", win);
        gtk_widget_show_all(pop);
        j = 0;
        i = rundmtx_ (& j, & j);
        gtk_widget_destroy(pop);
        gtk_window_set_opacity (GTK_WINDOW (win), 1.0);
      }
      else
      {
        i = 1;
      }

      if (i == 1)
      {
        dmtx = TRUE;
        gtk_window_set_opacity (GTK_WINDOW (win), opac);
        if (steps > 1)
        {
          pop=show_popup ("Please wait bond angles distribution calculation in progress", win);
          gtk_widget_show_all(pop);
        }
        l = bond_angles_ (& num_delta_a);
        if (steps > 1) gtk_widget_destroy(pop);
        if (l != 1)
        {
          show_error ("The bond angles distribution calculation has failed");
        }
        if (steps > 1)
        {
          pop=show_popup ("Please wait dihedral angles distribution calculation in progress", win);
          gtk_widget_show_all(pop);
        }
        m = bond_diedrals_ (& num_delta_a);
        if (steps > 1) gtk_widget_destroy(pop);
        gtk_window_set_opacity (GTK_WINDOW (win), 1.0);
        if (m != 1)
        {
          show_error ("The diedral angles distribution calculation has failed");
        }
        else
        {
          info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "angcheck");
          gtk_widget_set_sensitive (info, 1);
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 1);
          info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispang");
          gtk_widget_set_sensitive (info, 1);
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 1);
        }
      }
      else
      {
        show_error ("The nearest neighbors table calculation has failed");
        dmtx = FALSE;
      }
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Mainstatusbar");
      gtk_statusbar_remove (GTK_STATUSBAR(info), 4, k);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "deltaa");
      gtk_widget_set_sensitive(info, 1);
      gtk_widget_set_sensitive(aspeca, 1);
      gtk_widget_set_sensitive(aspecb, 1);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "acut");
      gtk_widget_set_sensitive(info, 1);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "atcut");
      gtk_widget_set_sensitive(info, 1);
      i = 1;
      setcalcs (& i);
    }
  }
  else
  {
    show_warning ("You must specify a number δθ >= 2");
  }
}

G_MODULE_EXPORT void on_angcheck_toggled (GtkToggleButton * Button)
{
  GtkWidget * info;
  gboolean status;

  status = gtk_toggle_button_get_active (Button);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "ANlist");
  if (status)
  {
    if (! GTK_WIDGET_VISIBLE(info))
    {
      gtk_widget_show_all(info);
    }
  }
  else
  {
    if (GTK_WIDGET_VISIBLE(info))
    {
      gtk_widget_hide(info);
    }
  }
}

