! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

INTEGER FUNCTION bond_angles(NDA)

USE PARAMETERS

INTEGER, INTENT(IN) :: NDA
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: ANGTAB
DOUBLE PRECISION :: pro
INTEGER, DIMENSION(:,:,:,:), ALLOCATABLE :: ANGLEA
INTEGER, DIMENSION(:,:,:), ALLOCATABLE :: SUM_ANGA

NUMBER_OF_DELTA_ANG=NDA

allocate(ANGLEA(NSP,NSP,NSP,NUMBER_OF_DELTA_ANG), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: bond_angles"//CHAR(0), "Table: ANGLEA"//CHAR(0))
  bond_angles=0
  goto 001
endif

do j=1, NUMBER_OF_DELTA_ANG
do k=1, NSP
do l=1, NSP
do m=1, NSP
  ANGLEA(m,l,k,j)=0
enddo
enddo
enddo
enddo

DELTA_ANG=180.0/dble(NUMBER_OF_DELTA_ANG)

do i=1, NS

  if (NS .gt. 1) then
    pro=(i-1)/(NS*1.0)
    call setprogress (pro)
  endif

  do j=1, NA

    if (CONTJ(j,i) .gt. 1) then

      do k=1, CONTJ(j,i)-1
      do l=k+1, CONTJ(j,i)
        m=VOISJ(k,j,i)
        n=VOISJ(l,j,i)
        call ANGIJK(m, j, n, i)
        ANG_I=AnINT (TETA/DELTA_ANG)
        if (ANG_I.le.0) ANG_I=1
        if (ANG_I.le.0) ANG_I=1
        if (ANG_I.gt.NUMBER_OF_DELTA_ANG) ANG_I=NUMBER_OF_DELTA_ANG
        ANGLEA(LOT(m),LOT(j),LOT(n),ANG_I)=ANGLEA(LOT(m),LOT(j),LOT(n),ANG_I)+1
        if (LOT(m) .ne. LOT(n)) ANGLEA(LOT(n),LOT(j),LOT(m),ANG_I)=ANGLEA(LOT(n),LOT(j),LOT(m),ANG_I)+1

      enddo
      enddo

    endif

  enddo
enddo

allocate (SUM_ANGA(NSP,NSP,NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: bond_angles"//CHAR(0), "Table: SUM_ANGA"//CHAR(0))
  bond_angles=0
  goto 001
endif

do i=1, NSP
  do j=1, NSP
    do k=1, NSP

      SUM_ANGA(i,j,k) = 0
      do l=1, NUMBER_OF_DELTA_ANG

        SUM_ANGA(i,j,k) = SUM_ANGA(i,j,k) + ANGLEA(i,j,k,l)

      enddo
    enddo
  enddo
enddo

if (NS .gt. 1) then
  pro=1.0d0
  call setprogress (pro)
endif

allocate(ANGTAB(NUMBER_OF_DELTA_ANG), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: bond_angles"//CHAR(0), "Table: ANGTAB"//CHAR(0))
  bond_angles=0
  goto 001
endif
do l=1, NUMBER_OF_DELTA_ANG
  ANGTAB(l)=0.0
enddo

m=GRNUM+SQNUM+SKNUM+GQNUM+BDNUM
do i=1, NSP
  do j=1, NSP
    do k=1, NSP
      if (SUM_ANGA(i,j,k) .ne. 0) then
        do l=1, NUMBER_OF_DELTA_ANG
          ANGTAB(l)=100.0*dble(ANGLEA(i,j,k,l))/dble(SUM_ANGA(i,j,k))
        enddo
        call save_curve (NUMBER_OF_DELTA_ANG, ANGTAB, m)
      else
        call save_curve (0, ANGTAB, m)
      endif
      m=m+1
    enddo
  enddo
enddo

bond_angles=1

if (NS .gt. 1) then
  pro=0.0d0
  call setprogress (pro)
endif

001 continue

if (allocated(ANGLEA)) deallocate(ANGLEA)
if (allocated(SUM_ANGA)) deallocate(SUM_ANGA)
if (allocated(ANGTAB)) deallocate(ANGTAB)

END FUNCTION bond_angles

INTEGER FUNCTION bond_diedrals(NDA)

USE PARAMETERS

INTEGER, INTENT(IN) :: NDA
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: ANGTAB
DOUBLE PRECISION :: pro
INTEGER, DIMENSION(:,:,:,:,:), ALLOCATABLE :: ANGLED
INTEGER, DIMENSION(:,:,:,:), ALLOCATABLE :: SUM_ANGD

NUMBER_OF_DELTA_ANG=NDA

allocate(ANGLED(NSP,NSP,NSP,NSP,NUMBER_OF_DELTA_ANG), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: bond_angles"//CHAR(0), "Table: ANGLED"//CHAR(0))
  bond_diedrals=0
  goto 001
endif

do j=1, NUMBER_OF_DELTA_ANG
do k=1, NSP
do l=1, NSP
do m=1, NSP
do n=1, NSP
  ANGLED(n,m,l,k,j)=0
enddo
enddo
enddo
enddo
enddo

DELTA_ANG=180.0/dble(NUMBER_OF_DELTA_ANG)

do i=1, NS
  if (NS .gt. 1) then
    pro=(i-1)/(NS*1.0)
    call setprogress (pro)
  endif
  do j=1, NA
    do k=1, CONTJ(j,i)
      m=VOISJ(k,j,i)
      if (CONTJ(m,i) .ge. 2) then
        do l=1, CONTJ(m,i)
          n=VOISJ(l,m,i)
          if (n .ne. j) then
            if (CONTJ(n,i) .ge. 2) then
              do o=1, CONTJ(n,i)
                p = VOISJ(o,n,i)
                if (p.ne.j .and. p.ne.m) then
                  call DIEDRE(j, m, n, p, i)
                  ANG_I=AnINT (TETA/DELTA_ANG)+1
                  if (ANG_I.le.0) ANG_I=1
                  if (ANG_I.gt.NUMBER_OF_DELTA_ANG) ANG_I=NUMBER_OF_DELTA_ANG
                  ANGLED(LOT(j),LOT(m),LOT(n),LOT(p),ANG_I)=ANGLED(LOT(j),LOT(m),LOT(n),LOT(p),ANG_I)+1
                  if (LOT(j).ne.LOT(m) .or. LOT(j).ne.LOT(n) .or. LOT(j).ne.LOT(p)) then
                    ANGLED(LOT(p),LOT(n),LOT(m),LOT(j),ANG_I)=ANGLED(LOT(p),LOT(n),LOT(m),LOT(j),ANG_I)+1
                  endif
                endif
              enddo
            endif
          endif
        enddo
      endif
    enddo
  enddo
enddo

allocate (SUM_ANGD(NSP,NSP,NSP,NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: bond_angles"//CHAR(0), "Table: SUM_ANGD"//CHAR(0))
  bond_diedrals=0
  goto 001
endif

do i=1, NSP
  do j=1, NSP
    do k=1, NSP
      do l=1, NSP
        SUM_ANGD(i,j,k,l) = 0
        do m=1, NUMBER_OF_DELTA_ANG
          SUM_ANGD(i,j,k,l) = SUM_ANGD(i,j,k,l) + ANGLED(i,j,k,l,m)
        enddo
      enddo
    enddo
  enddo
enddo

if (NS .gt. 1) then
  pro=1.0d0
  call setprogress (pro)
endif

allocate(ANGTAB(NUMBER_OF_DELTA_ANG), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: bond_angles"//CHAR(0), "Table: ANGTAB"//CHAR(0))
  bond_diedrals=0
  goto 001
endif
do l=1, NUMBER_OF_DELTA_ANG
  ANGTAB(l)=0.0
enddo

n=GRNUM+SQNUM+SKNUM+GQNUM+BDNUM+NSP*NSP*NSP
do i=1, NSP
  do j=1, NSP
    do k=1, NSP
      do l=1, NSP
        if (SUM_ANGD(i,j,k,l) .ne. 0) then
          o=1
          do m=1, NUMBER_OF_DELTA_ANG
            ANGTAB(m)=100.0*dble(ANGLED(i,j,k,l,m))/dble(SUM_ANGD(i,j,k,l))
          enddo
          call save_curve (NUMBER_OF_DELTA_ANG, ANGTAB, n)
        else
          call save_curve (0, ANGTAB, n)
        endif
        n=n+1
      enddo
    enddo
  enddo
enddo

bond_diedrals=1

if (NS .gt. 1) then
  pro=0.0d0
  call setprogress (pro)
endif

001 continue

if (allocated(ANGLED)) deallocate(ANGLED)
if (allocated(SUM_ANGD)) deallocate(SUM_ANGD)
if (allocated(ANGTAB)) deallocate(ANGTAB)

END FUNCTION bond_diedrals
