/* This file is part of I.S.A.A.C.S.

   I.S.A.A.C.S. is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

    I.S.A.A.C.S. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#ifdef G_OS_WIN32
#  include <windows.h>
#endif

#include "global.h"
#include "interface.h"
#include "callbacks.h"
#include "curve.h"
#include "bdcall.h"
#include "spcall.h"

void sendcuts_ (int *, int *, double *);
int rundmtx_ (int *, int *);
int bonding_ (int *, int *, int *, char *);
int convpow_ (int *, int *, int *);

void cutoffsend ()
{
  int i, j;
  for ( i=0; i < nspec; i++)
  {
    for (j=0; j < nspec; j++)
    {
       sendcuts_ (& i, & j, & cutoffs[i][j]);
    }
  }
  i = nspec;
  sendcuts_ (& i, & i, & grtotcutoff);
}

gboolean testcutoffs ()
{
  int i, j;
  gboolean val;
  char err1[117]="You need to specify all cutoff radii\n"
                 "For two chemical species i and j:\n"
                 "0.0 < Cutoff(i,j)) =< Dmax\n"
                 "With Dmax = ";
  char err2[50]=" Å\nmaximum inter-atomic distance in the system";
  char * vale;
  char err[202]="";

  val=TRUE;
  for ( i=0; i< nspec; i++)
  {
    for (j=0; j<nspec; j++)
    {
      if (cutoffs[i][j] == 0.0 || cutoffs[i][j] > dmax)
      {
        val=FALSE;
      }
    }
  }
  if (grtotcutoff == 0.0 || grtotcutoff > dmax) val=FALSE;
  if(! val)
  {
    strcpy(err, err1);
    vale=g_strdup_printf("%lf",dmax);
    strcat(err, vale);
    g_free(vale);
    strcat(err, err2);
    show_warning (err);
  }
  return(val);
}

void initbd ()
{
  int i, j, k;

  k = numgr+numgq+numsq+numsk;
  for ( i = 0 ; i < nspec ; i++ )
  {
    for ( j = 0 ; j < nspec ; j++ )
    {
      curves[k] -> title = g_strdup_printf("%s%s%s%s%s", "Dij [", label[i], "-", label[j], "]");
      k=k+1;
    }
  }
  i = numgr+numgq+numsq+numsk;
  for ( j = i ; j < i+numbd ; j++ )
  {
    addcurwidget (curves[j] -> title, & j, & i, tablegtkbd);
    curves[j] -> xlabel = g_strdup_printf("%s", "Dij [Å]");
    curves[j] -> cstart = i;
    curves[j] -> cend = i+numbd;
  }
}

void recup_dmin_ (double * min, double * ddij)
{
  bdmin = * min;
  delta_d = * ddij;
}

G_MODULE_EXPORT void on_deltad_activate ()
{
  GtkWidget * info;
  GtkEntry * EntryGtk;
  const gchar * std;
  double deld;
  int i, j;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "deltad");
  std=gtk_entry_get_text (EntryGtk);
  deld = atof(std);
  i = deld;
  update_entry_int (EntryGtk, & i);
  if (i != num_delta_d-1)
  {
    num_delta_d = i+1;
    if (curves[numgr+numgq+numsq+numsk] -> data != NULL)
    {
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "bdcheck");
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
      if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispbd");
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
      if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
    }
    j= numgr+numgq+numsq+numsk;
    i= numgr+numgq+numsq+numsk+numbd;
    hide_curves (&j, &i);
    for ( i = j ; i < j+numbd ; i ++ )
    {
      free (curves[i] -> data);
      curves[i] -> data = NULL;
    }
  }
}

G_MODULE_EXPORT void on_bspec_get ()
{
  int i, j;
  GtkEntry * info;

  i=gtk_combo_box_get_active(GTK_COMBO_BOX(bspeca));
  j=gtk_combo_box_get_active(GTK_COMBO_BOX(bspecb));
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "cut");
  update_entry_double (info, & cutoffs[i][j]);
}

G_MODULE_EXPORT void on_cut_activate ()
{
  GtkEntry * EntryGtk;
  int i, j, k, l, m, n;
  const gchar *str;

  if (dmtx) dmtx=FALSE;
  i=gtk_combo_box_get_active(GTK_COMBO_BOX(bspeca));
  j=gtk_combo_box_get_active(GTK_COMBO_BOX(bspecb));
  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "cut");
  str=gtk_entry_get_text (EntryGtk);
  cutoffs[i][j] = atof(str);
  update_entry_double (EntryGtk, & cutoffs[i][j]);
  k=gtk_combo_box_get_active(GTK_COMBO_BOX(aspeca));
  l=gtk_combo_box_get_active(GTK_COMBO_BOX(aspecb));
  m=gtk_combo_box_get_active(GTK_COMBO_BOX(rspeca));
  n=gtk_combo_box_get_active(GTK_COMBO_BOX(rspecb));
  if (k==i && l==j)
  {
    EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "acut");
    update_entry_double (EntryGtk, & cutoffs[i][j]);
  }
  if (m==i && n==j)
  {
    EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "rcut");
    update_entry_double (EntryGtk, & cutoffs[i][j]);
  }
}

G_MODULE_EXPORT void on_tcut_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *str;

  if (dmtx) dmtx=FALSE;
  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "tcut");
  str=gtk_entry_get_text (EntryGtk);
  grtotcutoff = atof(str);
  update_entry_double (EntryGtk, & grtotcutoff);
  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "atcut");
  update_entry_double (EntryGtk, & grtotcutoff);
  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "rtcut");
  update_entry_double (EntryGtk, & grtotcutoff);
}

void prepspbv (int * sp)
{
  GtkAction * action;
  GtkWidget * info;

  action = (GtkAction *) gtk_builder_get_object (BuilderGtk, "Spheric");
  gtk_action_set_sensitive(action, * sp);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Calcspheric");
  gtk_widget_set_sensitive(info, * sp);
  gtk_widget_set_sensitive(shspec, * sp);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "button-hsp");
  gtk_widget_set_sensitive(info, * sp);
  action = (GtkAction *) gtk_builder_get_object (BuilderGtk, "Valence");
  gtk_action_set_sensitive(action, * sp);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Calcvalence");
  gtk_widget_set_sensitive(info, * sp);
  gtk_widget_set_sensitive(bvspec, * sp);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "button-hbv");
  gtk_widget_set_sensitive(info, * sp);
  if (*sp == 1)
  {
    brun = TRUE;
    gtk_combo_box_set_active(GTK_COMBO_BOX(shspec), nspec-1);
    gtk_widget_show(shbox[nspec-1]);
    gtk_combo_box_set_active(GTK_COMBO_BOX(shbox[nspec-1]), 0);
    gtk_combo_box_set_active(GTK_COMBO_BOX(bvspec), nspec-1);
    gtk_widget_show(bvbox[nspec-1]);
    gtk_combo_box_set_active(GTK_COMBO_BOX(bvbox[nspec-1]), 0);
  }
  else
  {
    brun = FALSE;
  }
}

G_MODULE_EXPORT void on_Calcbonds_released ()
{
  int i, j, k, l, m, n, o;
  GtkWidget * info;
  GtkWidget * pop;
  GtkWidget * win;
  gboolean calcb=TRUE;

  if (num_delta_d >= 2)
  {
    calcb=testcutoffs();
    if (calcb)
    {
      cutoffsend ();
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "bdcheck");
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
      gtk_widget_set_sensitive (info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispbd");
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
      gtk_widget_set_sensitive (info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "deltad");
      gtk_widget_set_sensitive(info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "savengb");
      gtk_widget_set_sensitive(info, 0);
      gtk_widget_set_sensitive(bspeca, 0);
      gtk_widget_set_sensitive(bspecb, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "cut");
      gtk_widget_set_sensitive (info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "tcut");
      gtk_widget_set_sensitive (info, 0);
      if (! bdrun)
      {
        initbd ();
        bdrun = TRUE;
      }
      i = 0;
      setcalcs (& i);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Mainstatusbar");
      k=gtk_statusbar_push (GTK_STATUSBAR(info), 3, "Computing bond properties");
      win = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "bonds");
      if (! dmtx)
      {
        prepspbv (&i);
        gtk_window_set_opacity (GTK_WINDOW (win), opac);
        pop=show_popup ("Please wait computing the nearest neigbhors table", win);
        gtk_widget_show_all(pop);
        j = 0;
        i = rundmtx_ (& j, & j);
        gtk_widget_destroy(pop);
        gtk_window_set_opacity (GTK_WINDOW (win), 1.0);
      }
      else
      {
        i = 1;
      }

      if (i == 1)
      {
        dmtx = TRUE;
        gtk_window_set_opacity (GTK_WINDOW (win), opac);
        if (steps > 1)
        {
          pop=show_popup ("Please wait bond properties calculation in progress", win);
          gtk_widget_show_all(pop);
        }
        l = 0;
        m = 1;
        if (bondfile != NULL)
        {
          l = 1;
          m = StringLength(bondfile);
        }
        o = num_delta_d - 1;
        n = bonding_ (& m, & l, & o, bondfile);
        if (steps > 1) gtk_widget_destroy(pop);
        gtk_window_set_opacity (GTK_WINDOW (win), 1.0);
        prepspbv (& n);
        if (n != 1)
        {
          show_error ("The bond properties calculation has failed");
        }
        else
        {
          info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "bdcheck");
          gtk_widget_set_sensitive (info, 1);
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 1);
          info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispbd");
          gtk_widget_set_sensitive (info, 1);
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 1);
        }
      }
      else
      {
        show_error ("The nearest neighbors table calculation has failed");
        dmtx = FALSE;
      }
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Mainstatusbar");
      gtk_statusbar_remove (GTK_STATUSBAR(info), 3, k);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "deltad");
      gtk_widget_set_sensitive(info, 1);
      gtk_widget_set_sensitive(bspeca, 1);
      gtk_widget_set_sensitive(bspecb, 1);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "cut");
      gtk_widget_set_sensitive(info, 1);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "tcut");
      gtk_widget_set_sensitive(info, 1);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "savengb");
      gtk_widget_set_sensitive(info, 1);
      i = 1;
      setcalcs (& i);
    }
  }
  else
  {
    show_warning ("You must specify a number δr >= 2\n"
                  "to compute the bond distributions\n"
                  "for the first coordination sphere(s)\n");
  }
}

G_MODULE_EXPORT void on_bdcheck_toggled (GtkToggleButton * Button)
{
  GtkWidget * info;
  gboolean status;

  status = gtk_toggle_button_get_active (Button);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "BDlist");
  if (status)
  {
    if (! GTK_WIDGET_VISIBLE(info))
    {
      gtk_widget_show_all(info);
    }
  }
  else
  {
    if (GTK_WIDGET_VISIBLE(info))
    {
      gtk_widget_hide(info);
    }
  }
}

void coordout_ (int * sid, double * sac, double ssac[nspec])
{
  int j;
  GtkWidget * info;
  GtkTextBuffer * buffer = NULL;
  GtkTextIter bEnd;
  GtkTextIter bStart;
  char * ctot;
  char * tctot;
  char ta[23]="Coordination numbers: ";
  char tb[12]="(total):\t ";
  char tc[2]="(";
  char td[9]=")[rcut(";
  char te[2]=",";
  char tf[3]=")=";
  char tg[8]=" Å] = ";
  char th[9]="\t or \t";
  char tl[3]=" %";
  char tm[6]=" Å )";
  char inta[155]="Existence of a bond between two atoms i (α) and j (β) \n"
                 "if the two following conditions are verified: \n"
                 " 1) Dij < first minimum of the total RDF (";
  char intb[25]=" 2) Dij < rcut(α,β)\n";

  if (* sid == 0)
  {
    ctot = g_strdup_printf("%s%s%9.5lf%s%s%s%s%s",
           retl, inta, grtotcutoff, tm, retl, intb, retl, ta);
  }
  else
  {
    ctot = g_strdup_printf("%s", ta);
  }
  tctot = g_strdup_printf("%s%s%s%s%s%s%9.5lf%s",
          ctot, elt_name[* sid], retl, retl, label[* sid], tb, * sac, retl);
  g_free(ctot);
  for ( j=0 ; j < nspec ; j++ )
  {
    ctot = g_strdup_printf("%s%s%s%s%s%s%s%s%s%9.5lf%s%9.5lf%s",
           tctot, label[* sid], tc, label[j], td, label[* sid],
           te, label[j], tf, cutoffs[* sid][j], tg, ssac[j], th);
    g_free(tctot);
    if (* sac != 0.0)
    {
      tctot = g_strdup_printf("%s%7.3lf%s%s", ctot, ssac[j]*100.0/(* sac), tl, retl);
    }
    else
    {
      tctot = g_strdup_printf("%s%7.3lf%s%s", ctot, 0.0, tl, retl);
    }
    g_free(ctot);
  }
  ctot = g_strdup_printf("%s%s", tctot, retl);
  g_free(tctot);
  g_print("%s", ctot);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "TextInfo");
  buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(info));
  gtk_text_buffer_get_start_iter (buffer, &bStart);
  gtk_text_buffer_get_end_iter (buffer, &bEnd);
  gtk_text_buffer_insert (buffer, &bEnd, ctot, -1);
  gtk_text_view_set_buffer (GTK_TEXT_VIEW(info), buffer);
  g_free(ctot);
}

void envout_ (int * sid, int * totgsa, int * tgsa,
              double numgsa[* totgsa], int listegsa[* totgsa])
{
  int i, j, l, m;
  GtkWidget * info;
  GtkTextBuffer * buffer = NULL;
  GtkTextIter bStart;
  GtkTextIter bEnd;
  char * petot;
  char * tpetot;
  char tf[9]="\tNumber";
  char tg[8]="Percent";
  char th[9]="\t or \t";
  char ti[18]="Environments for ";
  char tj[8]=" atoms:";
  char tk[3]="\t";
  char tl[3]=" %";
  char tm[4]="   ";
  char tu[3]="  ";
  char tn[7]="N(tot)";
  char to[3]="N(";
  char tp[2]=")";
  int natpg[* totgsa];
  char ** listgsh;

/* Send data for spherical harmonics */

  listgsh = malloc((*totgsa)*sizeof*listgsh);

/* Write bond properties results */
  for ( i=0 ; i < * totgsa ; i++ )
  {
    natpg[i]=0;
    listgsh[i] = g_strdup_printf("%s", "");
    for ( j=0 ; j < nspec ; j++ )
    {
      l = listegsa[i];
      m = convpow_ ( & l, & nspec, & j);
      natpg[i]=natpg[i]+m;
      if (m > 1)
      {
        petot = g_strdup_printf("%s%s%d%s", listgsh[i], label[j], m, " ");
        g_free(listgsh[i]);
        listgsh[i] = g_strdup_printf("%s", petot);
        g_free(petot);
      }
      else if (m == 1)
      {
        petot = g_strdup_printf("%s%s%s", listgsh[i], label[j], " ");
        g_free(listgsh[i]);
        listgsh[i] = g_strdup_printf("%s", petot);
        g_free(petot);
      }
    }
  }

/* Send info for spherical harmonics */

  gtkcombosp (sid, totgsa, listgsh);

  petot = g_strdup_printf ("%s%s%s%s%s%s%s", retl, ti, label[* sid], tj, retl, retl, tn);
  for ( j=0 ; j < nspec ; j++ )
  {
    tpetot = g_strdup_printf ("%s%s%s%s%s", petot, tm, to, label[j], tp);
    g_free(petot);
    petot = g_strdup_printf ("%s", tpetot);
    g_free(tpetot);
  }
  tpetot = g_strdup_printf ("%s%s%s%s%s%s", petot, tk, tf, th, tg, retl);
  g_free(petot);
  for ( i=0 ; i < * totgsa ; i++ )
  {
    petot = g_strdup_printf("%s%3d%s", tpetot, natpg[i], tm);
    g_free(tpetot);
    for ( j=0 ; j < nspec ; j++ )
    {
      l = listegsa[i];
      m = convpow_ ( & l, & nspec, & j);
      tpetot = g_strdup_printf("%s%6d%s", petot, m, tu);
      g_free(petot);
      petot = g_strdup_printf("%s", tpetot);
      g_free(tpetot);
    }
    tpetot = g_strdup_printf("%s%16.5lf%s%7.3lf%s%s",
             petot, numgsa[i]/steps, th, 100.0*numgsa[i]/(* tgsa), tl, retl);
    g_free(petot);
  }
  g_print("%s", tpetot);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "TextInfo");
  buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(info));
  gtk_text_buffer_get_start_iter (buffer, &bStart);
  gtk_text_buffer_get_end_iter (buffer, &bEnd);
  gtk_text_buffer_insert (buffer, &bEnd, tpetot, -1);
  gtk_text_view_set_buffer (GTK_TEXT_VIEW(info), buffer);
  g_free(tpetot);
}

void tetraout_ (int * sid, double eda[nspec], double cda[nspec], double dda[nspec],
                double tepa[nspec], double tcpa[nspec], double tdda[nspec],
                double atd[nspec], double etd[nspec])
{
  int i;
  GtkWidget * info;
  GtkTextBuffer * buffer = NULL;
  GtkTextIter bStart;
  GtkTextIter bEnd;
  gboolean print;
  char * ctot;
  char * tctot;
  char th[9]="\t or \t";
  char ti[47]="Number and proportion of tetrahedra links for ";
  char te[47]="Number and proportion of tetrahedra units for ";
  char tj[8]=" atoms:";
  char tl[3]=" %";
  char tf[35]=" with the following bond defects: ";
  char tn[18]=" Edge-sharing:   ";
  char to[18]=" Corner-sharing: ";
  char tp[16]=")4 tetrahedra: ";
  char td[16]=" Links between ";
  char tq[2]="(";
  char tr[6]=" +/- ";
  char tc[30]=" Total number of tetrahedra: ";

  print=FALSE;
  for ( i=0 ; i < nspec ; i++)
  {
    if (eda[i] != 0.0 || cda[i] != 0.0) print=TRUE;
  }

  if (print)
  {
    ctot = g_strdup_printf("%s%s%s%s%s", retl, ti, label[* sid], tj, retl);
    for ( i=0 ; i < nspec ; i++ )
    {
      if (eda[i] != 0.0 || cda[i] != 0.0)
      {
        tctot = g_strdup_printf("%s%s%s%s%s%s%s",
                ctot, td, label[* sid], tq, label[i], tp, retl);
        g_free(ctot);
        if (eda[i] != 0.0)
        {
          ctot = g_strdup_printf("%s%s%11.5lf", tctot, tn, eda[i]/steps);
          g_free(tctot);
          if (steps > 1)
          {
            tctot = g_strdup_printf("%s%s%8.5lf", ctot, tr, tepa[i]);
            g_free(ctot);
            ctot = g_strdup_printf("%s", tctot);
            g_free(tctot);
          }
          tctot = g_strdup_printf("%s%s%7.3lf", ctot, th, 100*eda[i]/(eda[i]+cda[i]));
          g_free (ctot);
          if (steps > 1)
          {
            ctot = g_strdup_printf("%s%s%7.3lf", tctot, tr, 100*tepa[i]*steps/(eda[i]+cda[i]));
            g_free (tctot);
            tctot = g_strdup_printf("%s", ctot);
            g_free(ctot);
          }
          ctot = g_strdup_printf("%s%s%s", tctot, tl, retl);
          g_free(tctot);
          tctot = g_strdup_printf("%s", ctot);
          g_free(ctot);
        }
        if (cda[i] != 0.0)
        {
          ctot = g_strdup_printf("%s%s%11.5lf", tctot, to, cda[i]/steps);
          g_free(tctot);
          if (steps > 1)
          {
            tctot = g_strdup_printf("%s%s%8.5lf", ctot, tr, tcpa[i]);
            g_free(ctot);
            ctot = g_strdup_printf("%s", tctot);
            g_free(tctot);
          }
          tctot = g_strdup_printf("%s%s%7.3lf", ctot, th, 100*cda[i]/(eda[i]+cda[i]));
          g_free (ctot);
          if (steps > 1)
          {
            ctot = g_strdup_printf("%s%s%7.3lf", tctot, tr, 100*tcpa[i]*steps/(eda[i]+cda[i]));
            g_free (tctot);
            tctot = g_strdup_printf("%s", ctot);
            g_free(ctot);
          }
          ctot = g_strdup_printf("%s%s%s", tctot, tl, retl);
          g_free(tctot);
          tctot = g_strdup_printf("%s", ctot);
          g_free(ctot);
        }
        if (dda[i] != 0.0)
        {
          ctot = g_strdup_printf("%s%s%11.5lf", tctot, tf, dda[i]/steps);
          g_free(tctot);
          if (steps > 1)
          {
            tctot = g_strdup_printf("%s%s%8.5lf", ctot, tr, tdda[i]);
            g_free(ctot);
            ctot = g_strdup_printf("%s", tctot);
            g_free(tctot);
          }
          tctot = g_strdup_printf("%s%s%7.3lf", ctot, th, 100*dda[i]/(eda[i]+cda[i]));
          g_free (ctot);
          if (steps > 1)
          {
            ctot = g_strdup_printf("%s%s%7.3lf", tctot, tr, 100*tdda[i]*steps/(eda[i]+cda[i]));
            g_free (tctot);
            tctot = g_strdup_printf("%s", ctot);
            g_free(ctot);
          }
          ctot = g_strdup_printf("%s%s%s", tctot, tl, retl);
          g_free(tctot);
          tctot = g_strdup_printf("%s", ctot);
          g_free(ctot);
        }
        ctot = g_strdup_printf("%s", tctot);
        g_free(tctot);
      }
    }
    tctot = g_strdup_printf("%s%s%s%s%s%s", ctot, retl, te, label[* sid], tj, retl);
    g_free (ctot);
    for ( i=0 ; i < nspec ; i++ )
    {
      if (atd[i] != 0.0)
      {
        ctot = g_strdup_printf("%s%s%s%s%s%s%s%11.5lf",
               tctot, label[* sid], tq, label[i], tp, retl, tc, atd[i]/steps);
        g_free (tctot);
        if (steps > 1)
        {
            tctot = g_strdup_printf("%s%s%8.5lf", ctot, tr, etd[i]);
            g_free (ctot);
            ctot = g_strdup_printf("%s", tctot);
            g_free(tctot);
        }
        tctot = g_strdup_printf("%s%s", ctot, retl);
        g_free (ctot);
        if (eda[i] != 0.0 )
        {
          ctot = g_strdup_printf("%s%s%11.5lf", tctot, tn, 2*eda[i]/steps);
          g_free (tctot);
          if (steps > 1)
          {
            tctot = g_strdup_printf("%s%s%8.5lf", ctot, tr, 2*tepa[i]);
            g_free(ctot);
            ctot = g_strdup_printf("%s", tctot);
            g_free(tctot);
          }
          tctot = g_strdup_printf("%s%s%7.3lf", ctot, th, 100*2*eda[i]/atd[i]);
          g_free(ctot);
          if (steps > 1)
          {
            ctot = g_strdup_printf("%s%s%7.3lf", tctot, tr, 100*2*tepa[i]*steps/atd[i]);
            g_free(tctot);
            tctot = g_strdup_printf("%s", ctot);
            g_free(ctot);
          }
          ctot = g_strdup_printf("%s%s%s", tctot, tl, retl);
          g_free(tctot);
          tctot = g_strdup_printf("%s", ctot);
          g_free(ctot);
        }
        if (cda[i] != 0.0 )
        {
          ctot = g_strdup_printf("%s%s%11.5lf", tctot, to, (atd[i]-2*eda[i])/steps);
          g_free(tctot);
          if (steps > 1)
          {
            tctot = g_strdup_printf("%s%s%8.5lf", ctot, tr, 2*tepa[i]);
            g_free(ctot);
            ctot = g_strdup_printf("%s", tctot);
            g_free(tctot);
          }
          tctot = g_strdup_printf("%s%s%7.3lf", ctot, th, 100*(atd[i]-2*eda[i])/atd[i]);
          g_free(ctot);
          if (steps > 1)
          {
            ctot = g_strdup_printf("%s%s%7.3lf", tctot, tr, 100*2*tepa[i]*steps/atd[i]);
            g_free(tctot);
            tctot = g_strdup_printf("%s", ctot);
            g_free(ctot);
          }
          ctot = g_strdup_printf("%s%s%s", tctot, tl, retl);
          g_free(tctot);
          tctot = g_strdup_printf("%s", ctot);
          g_free(ctot);
        }
      }
    }
    g_print("%s", tctot);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "TextInfo");
    buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(info));
    gtk_text_buffer_get_start_iter (buffer, &bStart);
    gtk_text_buffer_get_end_iter (buffer, &bEnd);
    gtk_text_buffer_insert (buffer, &bEnd, tctot, -1);
    gtk_text_view_set_buffer (GTK_TEXT_VIEW(info), buffer);
    g_free(tctot);
  }
}

G_MODULE_EXPORT void on_savengb_toggled (GtkToggleButton * Button)
{
  GtkWidget * info;
  gboolean status;

  status = gtk_toggle_button_get_active (Button);
  if (status)
  {
    show_info ("Activate this option and the result of the nearest neighbors\n"
               "analysis will be saved during the search in a file that you may\n"
               "use afterwards.\n");
    info = gtk_file_chooser_dialog_new ("Save neighbors analysis in file",
       				      NULL,
                          GTK_FILE_CHOOSER_ACTION_SAVE,
     				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
     				      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
     				      NULL);
    gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (info), TRUE);
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(info), "");
    gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER(info), "neighbors.dat");

    if (gtk_dialog_run (GTK_DIALOG (info)) == GTK_RESPONSE_ACCEPT)
    {
      bondfile = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(info));
    }
    else
    {
       gtk_toggle_button_set_active (Button, FALSE);
    }
    gtk_widget_destroy (info);
  }
  else
  {
    bondfile = NULL;
  }
}
