! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

INTEGER (KIND=c_int) FUNCTION bonding (scf, sbf, NDB, sfil) BIND (C,NAME='bonding_')

USE PARAMETERS

INTEGER (KIND=c_int), INTENT(IN) :: scf, sbf, NDB
CHARACTER (KIND=c_char), DIMENSION(*), INTENT(IN) :: sfil
CHARACTER (LEN=scf) :: sfile

INTEGER :: TGESA
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: MAC, NGSA
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: EABL, CABL, DABL
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: ETABL, CTABL, DTABL
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: ECTABL, TDTABL
INTEGER, DIMENSION(:), ALLOCATABLE :: LGSA
DOUBLE PRECISION :: pro

INTERFACE
  LOGICAL FUNCTION ALLOCBONDS (NDD)
    INTEGER, INTENT(IN) :: NDD
  END FUNCTION
END INTERFACE

if (sbf .eq. 1 .and. scf .gt. 0) then
  do i=1, scf
    sfile(i:i) = sfil(i)
  enddo
endif

DELTA_DIJ = (MAXBD-MINBD) / NDB

if (.not. ALLOCBONDS(NDB)) then
  bonding=0
  goto 001
endif

if (sbf .eq. 1) open(unit=100, file=sfile, action="write", status='unknown')

! Décompte des nombres de coordination
! Evaluation of the coordination numbers

do k=1, NS

  if (NS .gt. 1) then
    pro=(k-1)/(NS*1.0)
    call setprogress (pro)
  endif
  do i=1, NA
    do j=1, CONTJ(i,k)
      LA_COUNT(i,LOT(VOISJ(j,i,k)),k)=LA_COUNT(i,LOT(VOISJ(j,i,k)),k)+1
      call CALCRIJ(i, VOISJ(j,i,k), k, k)
      if (MAXBD .eq. MINBD) then
        l=0
      else
        l=INT((Dij-MINBD)/DELTA_DIJ)
      endif
      STATBD(LOT(i),LOT(VOISJ(j,i,k)),l)=STATBD(LOT(i),LOT(VOISJ(j,i,k)),l) + 1
    enddo
  enddo

  if (sbf .eq. 1) call SAVE_NGB

  call ESCS(k)

  do i=1, NA
    do j=1, NSP
      MA_COUNT(LOT(i),j)=MA_COUNT(LOT(i),j)+LA_COUNT(i,j,k)
      SA_COUNT(LOT(i))=SA_COUNT(LOT(i))+LA_COUNT(i,j,k)
    enddo
  enddo

enddo

if (sbf .eq. 1) close(100)

do i=1, NSP
  do j=1, NA*10
    NUM_GSA(i,j)=0.0
    LISTE_GEOMSA(i,j)=0
  enddo
  TOT_GSA(i)=0
  TGSA(i)=0
enddo

do k=1, NS
  do i=1, NA
    TGESA=0
    do j=1, NSP
      TGESA=TGESA+LA_COUNT(i,j,k)*(100**(j-1))
    enddo
    call COMPARE(TGESA, LISTE_GEOMSA, TOT_GSA(LOT(i)), NUM_GSA, LOT(i))
  enddo
enddo

call EESCS

do i=1, NSP
  do j=1, TOT_GSA(i)
    TGSA(i)=TGSA(i)+NUM_GSA(i,j)
  enddo
enddo

do i=1, NSP
  SA_COUNT(i)=SA_COUNT(i)/NS/NBSPBS(i)
  do j=1, NSP
    MA_COUNT(i,j)=MA_COUNT(i,j)/NS/NBSPBS(i)
  enddo
enddo

if (NS .gt. 1) then
  pro=1.0d0
  call setprogress (pro)
endif

if (allocated(MAC)) deallocate(MAC)
allocate(MAC(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: bonding"//CHAR(0), "Table: MAC"//CHAR(0))
  bonding=0
  goto 001
endif

do i=1, NSP
  do j=1, NSP
    MAC(j)=MA_COUNT(i,j)
  enddo
  call coordout (i-1, SA_COUNT(i), MAC)
enddo

deallocate(MAC)

do i=1, NSP
  if (allocated(NGSA)) deallocate(NGSA)
  allocate(NGSA(TOT_GSA(i)), STAT=ERR)
  if (ERR .ne. 0) then
    call show_error ("Impossible to allocate memory"//CHAR(0), &
                     "Subroutine: bonding"//CHAR(0), "Table: NGSA"//CHAR(0))
    bonding=0
    goto 001
  endif
  do j=1, TOT_GSA(i)
    NGSA(j)=NUM_GSA(i,j)
  enddo
  if (allocated(LGSA)) deallocate(LGSA)
  allocate(LGSA(TOT_GSA(i)), STAT=ERR)
  if (ERR .ne. 0) then
    call show_error ("Impossible to allocate memory"//CHAR(0), &
                     "Subroutine: bonding"//CHAR(0), "Table: LGSA"//CHAR(0))
    bonding=0
    goto 001
  endif
  do j=1, TOT_GSA(i)
    LGSA(j)=LISTE_GEOMSA(i,j)
  enddo
  call envout (i-1, TOT_GSA(i), TGSA(i), NGSA, LGSA)
  deallocate(NGSA)
  deallocate(LGSA)
enddo

if (allocated(EABL)) deallocate(EABL)
allocate(EABL(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: bonding"//CHAR(0), "Table: EABL"//CHAR(0))
  bonding=0
  goto 001
endif
if (allocated(CABL)) deallocate(CABL)
allocate(CABL(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: bonding"//CHAR(0), "Table: CABL"//CHAR(0))
  bonding=0
  goto 001
endif
if (allocated(DABL)) deallocate(DABL)
allocate(DABL(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: bonding"//CHAR(0), "Table: DABL"//CHAR(0))
  bonding=0
  goto 001
endif
if (allocated(ETABL)) deallocate(ETABL)
allocate(ETABL(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: bonding"//CHAR(0), "Table: ETABL"//CHAR(0))
  bonding=0
  goto 001
endif
if (allocated(CTABL)) deallocate(CTABL)
allocate(CTABL(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: bonding"//CHAR(0), "Table: CTABL"//CHAR(0))
  bonding=0
  goto 001
endif
if (allocated(DTABL)) deallocate(DTABL)
allocate(DTABL(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: bonding"//CHAR(0), "Table: DTABL"//CHAR(0))
  bonding=0
  goto 001
endif
if (allocated(ECTABL)) deallocate(ECTABL)
allocate(ECTABL(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: bonding"//CHAR(0), "Table: ECTABL"//CHAR(0))
  bonding=0
  goto 001
endif
if (allocated(TDTABL)) deallocate(TDTABL)
allocate(TDTABL(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: bonding"//CHAR(0), "Table: TDTABL"//CHAR(0))
  bonding=0
  goto 001
endif
do i=1, NSP
  do j=1, NSP
    EABL(j)=EDGETA(i,j)
    CABL(j)=CORTA(i,j)
    DABL(j)=DEFTA(i,j)
    ETABL(j)=ETYPEA(i,j)
    CTABL(j)=CTYPEA(i,j)
    DTABL(j)=DETYPEA(i,j)
    TDTABL(j)=TDA(i,j)
    ECTABL(j)=ETDA(i,j)
  enddo
  call tetraout (i-1, EABL, CABL, DABL, ETABL, CTABL, DTABL, TDTABL, ECTABL)
enddo

if (allocated(CABL)) deallocate(CABL)
if (allocated(DABL)) deallocate(DABL)
if (allocated(ETABL)) deallocate(ETABL)
if (allocated(CTABL)) deallocate(CTABL)
if (allocated(DTABL)) deallocate(DTABL)
if (allocated(ECTABL)) deallocate(ECTABL)
if (allocated(TDTABL)) deallocate(TDTABL)

if (allocated(EABL)) deallocate(EABL)
allocate(EABL(0:NDB), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: bonding"//CHAR(0), "Table: EABL"//CHAR(0))
  bonding=0
  goto 001
endif
do k=0, NDB
  EABL(k) = 0.0
enddo

call recup_dmin (MINBD, DELTA_DIJ)
if (MAXBD .eq. MINBD) then
  call show_info (MAXBD);
else
  l=GRNUM+SQNUM+SKNUM+GQNUM
  do i=1, NSP
  do j=1, NSP
    TGESA=0
    do k=0, NDB
      TGESA = TGESA + STATBD(i,j,k)
    enddo
    if (TGESA .ne. 0) then
      do k=0, NDB
        EABL(k) = 100.0*STATBD(i,j,k)/TGESA
      enddo
      call save_curve (NDB+1, EABL, l)
    else
      call save_curve (0, EABL, l)
    endif
    l=l+1
  enddo
  enddo
endif

if (allocated(EABL)) deallocate(EABL)

if (NS .gt. 1) then
  pro=0.0d0
  call setprogress (pro)
endif
bonding=1

001 continue

call DEALLOCBONDS

CONTAINS

SUBROUTINE COMPARE(TEMP, LISTEB, LONGUEUR, NUM, SPEC)
!call COMPARE(TGESA, LISTE_GEOMSA, TOT_GSA(LOT(i)), NUM_GSA, LOT(i))
IMPLICIT NONE

INTEGER :: bu,bv
INTEGER, INTENT(IN) :: SPEC
INTEGER, INTENT(IN) :: TEMP
INTEGER, INTENT(INOUT) :: LONGUEUR
DOUBLE PRECISION, DIMENSION(NSP,NA*MAXN), INTENT(INOUT) :: NUM
INTEGER, DIMENSION(NSP,NA*MAXN), INTENT(INOUT) :: LISTEB

bv=0

do bu=1, LONGUEUR
  if (TEMP .eq. LISTEB(SPEC,bu)) then
    bv=1
    NUM(SPEC,bu)=NUM(SPEC,bu)+1.0
    exit
  endif
enddo

if (bv .eq. 0) then
  LONGUEUR=LONGUEUR+1
  LISTEB(SPEC,LONGUEUR)=TEMP
  NUM(SPEC,LONGUEUR)=1.0
endif

END SUBROUTINE

SUBROUTINE SAVE_NGB

write (100, '("Configuration N°",i6)') k
write (100, *)
do j=1, NA
  write (100, '(A2,i6)') TL(LOT(j)), j
  write (100, '(4x,"Nc[tot]= ",i2)') CONTJ(j,k)
  do i=1, NSP
    write (100, '(5x,"Nc[",A2,"]= ",i2)') TL(i), LA_COUNT(j,i,k)
    do m=1, CONTJ(j,k)
      if (LOT(VOISJ(m,j,k)) .eq. i) then
        call CALCRIJ(j, VOISJ(m,j,k), k, k)
        write (100, '(10x,A2,1x,i6,1x,"at",1x,f7.5,1x,"Å")') TL(i), VOISJ(m,j,k), Dij
      endif
    enddo
  enddo
enddo

END SUBROUTINE

END FUNCTION

SUBROUTINE sendcuts (cspa, cspb, cutab)

USE PARAMETERS

INTEGER, INTENT(IN) :: cspa, cspb
DOUBLE PRECISION, INTENT(IN) :: cutab

if (.not.allocated(Gr_CUT)) then
  allocate(Gr_CUT(NSP,NSP), STAT=ERR)
  if (ERR .ne. 0) then
    call show_error ("Impossible to allocate memory"//CHAR(0), &
                     "Subroutine: bonding"//CHAR(0), "Table: Gr_CUT"//CHAR(0))
  endif
endif

if (cspa .eq. NSP) then
  Gr_cutoff=cutab
else
  Gr_CUT(cspa+1,cspb+1) = cutab
endif

END SUBROUTINE
