/* This file is part of I.S.A.A.C.S.

   I.S.A.A.C.S. is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

    I.S.A.A.C.S. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#ifdef G_OS_WIN32
#  include <windows.h>
#endif

#include "global.h"
#include "interface.h"
#include "callbacks.h"
#include "curve.h"
#include "bdcall.h"
#include "bvcall.h"

int convpow_ (int *, int *, int *);
int bvalence_ (double *, double *, double *, int *, int *);

G_MODULE_EXPORT void on_bvspec_get (GtkComboBox * Box)
{
  int j;

  spbv = gtk_combo_box_get_active(Box);
  for ( j=0 ; j < nspec ; j++ )
  {
     gtk_widget_hide (bvbox[j]);
  }
  gtk_widget_show (bvbox[spbv]);
  idbv = gtk_combo_box_get_active(GTK_COMBO_BOX(bvbox[spbv]));
}

G_MODULE_EXPORT void on_bondvalence_box (GtkComboBox * Sbox)
{
  idbv = gtk_combo_box_get_active(Sbox);
}

G_MODULE_EXPORT void on_Do_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *str;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "Do");
  str = gtk_entry_get_text (EntryGtk);
  R0 = atof(str);
  update_entry_double (EntryGtk, & R0);
}

G_MODULE_EXPORT void on_b_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *str;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "b");
  str = gtk_entry_get_text (EntryGtk);
  B = atof(str);
  update_entry_double (EntryGtk, & B);
}

G_MODULE_EXPORT void on_N_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *str;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "N");
  str = gtk_entry_get_text (EntryGtk);
  N = atof(str);
  update_entry_double (EntryGtk, & N);
}

G_MODULE_EXPORT void on_Calcvalence_released ()
{
  int i, j, k;
  GtkWidget * info;
  GtkWidget * pop;
  GtkWidget * win;

  if (B == 0.0)
  {
     show_warning ("You must specify a value for the 'B' parameter\n");
  }
  else
  {
    if (R0 == 0.0)
    {
       show_warning ("You must specify a value for the 'R0' parameter\n");
    }
    else
    {
      if (N == 0.0)
      {
         show_warning ("You must specify a value for the 'N' parameter\n");
      }
      else
      {
        i = 0;
        setcalcs (& i);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Mainstatusbar");
        k=gtk_statusbar_push (GTK_STATUSBAR(info), 8, "Computing bond valence");
        win = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "valence");
        gtk_widget_set_sensitive (bvspec, 0);
        gtk_widget_set_sensitive (bvbox[spbv], 0);
        gtk_window_set_opacity (GTK_WINDOW (win), opac);
        bvrun=TRUE;
        if (steps > 1)
        {
          pop=show_popup ("Please wait bond valence calculation in progress", win);
          gtk_widget_show_all(pop);
        }
        win = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "valence");
        j = bvalence_ (& R0, & B, & N, & spbv , & idbv);
        if (steps > 1) gtk_widget_destroy(pop);
        gtk_window_set_opacity (GTK_WINDOW (win), 1.0);
        gtk_widget_set_sensitive (bvspec, j);
        gtk_widget_set_sensitive (bvbox[spbv], j);
        gtk_statusbar_remove (GTK_STATUSBAR(info), 8, k);
        i = 1;
        setcalcs (& i);
        if ( j == 0 )
        {
           show_error ("Bond valence calculation has failed\n");
        }
      }
    }
  }
}

void printvalence_ (int * sbv, double * bvdata, int * ids)
{
  GtkWidget * info;
  GtkTextBuffer * buffer = NULL;
  GtkTextIter bStart;
  GtkTextIter bEnd;
  char bv0[20]="\nBond valence \n\n";
  char bv1[14]="Average per [";
  char bv2[10]="] atom:\t";
  char bv3[19]="Exponential form\n";
  char bv4[18]="Fractional form\n";
  char * pbv;
  char * tpbv;

  if (*ids == 0 && *sbv == 0)
  {
    pbv = g_strdup_printf("%s%s%s", bv0, bv3, bv1);
  }
  else if (*sbv == 0)
  {
    pbv = g_strdup_printf("%s%s", bv4, bv1);
  }
  else
  {
    pbv = g_strdup_printf("%s", bv1);
  }
  tpbv = g_strdup_printf("%s%s%s%lf%s", pbv, label[* sbv], bv2, * bvdata, retl);
  g_free(pbv);
  g_print("%s", tpbv);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "TextInfo");
  buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(info));
  gtk_text_buffer_get_start_iter (buffer, &bStart);
  gtk_text_buffer_get_end_iter (buffer, &bEnd);
  gtk_text_buffer_insert (buffer, &bEnd, tpbv, -1);
  gtk_text_view_set_buffer (GTK_TEXT_VIEW(info), buffer);
  g_free(tpbv);
}

void printval_ (int * gidbv, double * bvdat)
{
  GtkWidget * info;
  GtkTextBuffer * buffer = NULL;
  GtkTextIter bStart;
  GtkTextIter bEnd;
  int i, j;
  char bv1[19]="Average per [";
  char bv2[17]="] environment:\t";
  char * psbv;
  char * tpsbv;

  psbv = g_strdup_printf("%s%s%s", bv1, label[spbv], "(");
  for ( j=0 ; j < nspec ; j++ )
  {
    i = convpow_ (gidbv, & nspec, & j);
    if (i > 1)
    {
      tpsbv = g_strdup_printf("%s%s%2d%s", psbv, label[j], i," ");
      g_free(psbv);
      psbv = g_strdup_printf("%s", tpsbv);
      g_free(tpsbv);
    }
    else if (i == 1)
    {
      tpsbv = g_strdup_printf("%s%s%s", psbv, label[j], " ");
      g_free(psbv);
      psbv = g_strdup_printf("%s", tpsbv);
      g_free(tpsbv);
    }
  }
  tpsbv = g_strdup_printf("%s%s%s%lf%s%s", psbv, ")", bv2, * bvdat, retl, retl);
  g_free(psbv);
  g_print("%s", tpsbv);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "TextInfo");
  buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(info));
  gtk_text_buffer_get_start_iter (buffer, &bStart);
  gtk_text_buffer_get_end_iter (buffer, &bEnd);
  gtk_text_buffer_insert (buffer, &bEnd, tpsbv, -1);
  gtk_text_view_set_buffer (GTK_TEXT_VIEW(info), buffer);
  g_free(tpsbv);
}
