/* This file is part of I.S.A.A.C.S.

   I.S.A.A.C.S. is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

    I.S.A.A.C.S. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <unistd.h>
#include <string.h>
#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#ifdef G_OS_WIN32
#  include <windows.h>
#endif

#include "global.h"
#include "interface.h"
#include "callbacks.h"
#include "curve.h"
#include "msdcall.h"
#include "xmlrw.h"

char ** las;
int write_xyz_ (char *, int *, int *, int *);
int read_xyz_ (char *, int *);
int read_c3d_ (char *, int *);
int read_pdb_ (char *, int *);
void getnspec_ (int *);
void getlabel_ (int *, char *);
int read_trj_ (char *, int *, int *, int *);
int read_vas_ (char *, int *, int *, int *);
double change_radius_ (int *, int *);
double new_mass_ (int *, double *);
double new_radius_ (int *, double *);
double new_nscatt_ (int *, double *);
double new_xscatt_ (int *, double *);
void isaacs_end_ ();
void isaacs_free_ ();
int prep_data_ ();
void chemistry_ ();
void lattice_ (double [3][3], double[3], double[3], int *, int *);
void prepos_ (int *, int *);
double fdmax_ ();
double fkmin_ ();
void initcwidgets ();

gboolean tovalidate = FALSE;

unsigned int StringLength (char *str)
{
  unsigned int  nb;
  char c, flg;

  for (nb = 0, flg = 0; (c = *str) != '\0'; ++str)
  {
    ++nb;
  }
  return (nb);
}

void free_data ()
{
  GtkEntry * info;
  GtkWidget * widg;
  int i, j;
 // if (coordfile != NULL) free(coordfile);
 // if (typefile != NULL) free(typefile);
  isaacs_free_ ();
  //newipf = TRUE;
  if (readata)
  {
#ifndef G_OS_WIN32
    for ( i = 0 ; i < nspec ; i ++ )
    {
      free(label[i]);
      free(elt_name[i]);
    }
#endif
    free(label);
    label=NULL;
    free(elt_name);
    elt_name=NULL;
    free(atid);
    atid=NULL;
    free(radid);
    radid=NULL;
    free(nscatt);
    nscatt=NULL;
    free(xscatt);
    xscatt=NULL;
    free(rad);
    rad=NULL;
    free(amass);
    amass=NULL;
    free(cutoffs);
    cutoffs=NULL;
    for ( i=0 ; i<3 ; i++ )
    {
      langles[i]=0.0;
      vmod[i]=0.0;
      for ( j=0 ; j<3 ; j++ )
      {
        cvect[i][j]=0.0;
      }
    }
    gtk_widget_destroy(specbox);
    tovalidate=FALSE;
  }

  widg = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "rdfgr");
  if (GTK_WIDGET_VISIBLE(widg)) gtk_widget_hide(widg);
  widg = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqfft");
  if (GTK_WIDGET_VISIBLE(widg)) gtk_widget_hide(widg);
  widg = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqkpoint");
  if (GTK_WIDGET_VISIBLE(widg)) gtk_widget_hide(widg);
  widg = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "gqfft");
  if (GTK_WIDGET_VISIBLE(widg)) gtk_widget_hide(widg);
  widg = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "bonds");
  if (GTK_WIDGET_VISIBLE(widg)) gtk_widget_hide(widg);
  widg = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "angles");
  if (GTK_WIDGET_VISIBLE(widg)) gtk_widget_hide(widg);
  widg = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "rings");
  if (GTK_WIDGET_VISIBLE(widg)) gtk_widget_hide(widg);
  widg = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "spheric");
  if (GTK_WIDGET_VISIBLE(widg)) gtk_widget_hide(widg);
  widg = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "valence");
  if (GTK_WIDGET_VISIBLE(widg)) gtk_widget_hide(widg);

  if (run)
  {
    num_delta_r = 0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "deltar");
    gtk_entry_set_text (info, (const gchar *)"");
    grfact = 0.0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "grfact");
    gtk_entry_set_text (info, (const gchar *)"");
    gtk_widget_set_sensitive(GTK_WIDGET(info), 0);
    num_delta_q = 0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "deltaq");
    gtk_entry_set_text (info, (const gchar *)"");
    qmax = 0.0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "qmax");
    gtk_entry_set_text (info, (const gchar *)"");
    sqfact = 0.0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "sqfact");
    gtk_entry_set_text (info, (const gchar *)"");
    gtk_widget_set_sensitive(GTK_WIDGET(info), 0);
    num_delta_k = 0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "deltak");
    gtk_entry_set_text (info, (const gchar *)"");
    kmax = 0.0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "kmax");
    gtk_entry_set_text (info, (const gchar *)"");
    skfact = 0.0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "skfact");
    gtk_entry_set_text (info, (const gchar *)"");
    gtk_widget_set_sensitive(GTK_WIDGET(info), 0);
    free (idsk);
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "qlim");
    qlim = 15.0;
    update_entry_double (info, & qlim);
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "plim");
    plim = 1.0;
    update_entry_double (info, & plim);
    num_delta_rq = 0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "deltarq");
    gtk_entry_set_text (info, (const gchar *)"");
    qmaxfft = 0.0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "qmaxfft");
    gtk_entry_set_text (info, (const gchar *)"");
    gqfact = 0.0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "gqfact");
    gtk_entry_set_text (info, (const gchar *)"");
    gtk_widget_set_sensitive(GTK_WIDGET(info), 0);
    num_delta_d = 0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "deltad");
    gtk_entry_set_text (info, (const gchar *)"");
    num_delta_a = 0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "deltaa");
    gtk_entry_set_text (info, (const gchar *)"");
    ringsize = 0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "tailled");
    gtk_entry_set_text (info, (const gchar *)"");
    maxring = 0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "numa");
    gtk_entry_set_text (info, (const gchar *)"");
    onlyabab = 0;
    widg = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "ababcheck");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widg), 0);
    gtk_widget_set_sensitive(widg, 0);
    homo = 0;
    widg = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "homocheck");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widg), 0);
    gtk_widget_set_sensitive(widg, 0);
    vlmax = 0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "lmax");
    gtk_entry_set_text (info, (const gchar *)"");
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "DeltaT");
    gtk_entry_set_text (info, (const gchar *)"");
    delta_t = 0.0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "Ndtbs");
    gtk_entry_set_text (info, (const gchar *)"");
    ndtbs = 0;
    steps = 1;
    R0 = 0.0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "Do");
    gtk_entry_set_text (info, (const gchar *)"");
    B = 0.0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "b");
    gtk_entry_set_text (info, (const gchar *)"");
    N = 0.0;
    info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "N");
    gtk_entry_set_text (info, (const gchar *)"");
    i = 0;
    hide_curves (&i, &numwid);
    free(curves);
    gtk_widget_destroy(GTK_WIDGET(tablegtkgr));
    gtk_widget_destroy(GTK_WIDGET(tablegtksq));
    gtk_widget_destroy(GTK_WIDGET(tablegtksk));
    gtk_widget_destroy(GTK_WIDGET(tablegtkgq));
    gtk_widget_destroy(GTK_WIDGET(tablegtkbd));
    gtk_widget_destroy(GTK_WIDGET(tablegtkan));
    gtk_widget_destroy(GTK_WIDGET(tablegtkri));
    gtk_widget_destroy(GTK_WIDGET(tablegtksh));
    bondfile = NULL;
    widg = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "savengb");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widg), FALSE);
    if (steps > 1)
    {
      gtk_widget_destroy(GTK_WIDGET(tablegtkms));
    }
  }
  if (trun)
  {
    gtk_widget_destroy(bspeca);
    bspeca=NULL;
    gtk_widget_destroy(bspecb);
    bspecb=NULL;
    gtk_widget_destroy(aspeca);
    aspeca=NULL;
    gtk_widget_destroy(aspecb);
    aspecb=NULL;
    gtk_widget_destroy(rspeca);
    rspeca=NULL;
    gtk_widget_destroy(rspecb);
    rspecb=NULL;
    gtk_widget_destroy(rispec);
    rispec=NULL;
    gtk_widget_destroy(shspec);
    shspec=NULL;
    gtk_widget_destroy(bvspec);
    bvspec=NULL;
    if (shbox != NULL)
    {
      for ( i=0 ; i< nspec ; i++)
      {
        if (shbox[i] !=NULL)
        {
          if (GTK_WIDGET_VISIBLE(shbox[i]))
          {
            gtk_widget_hide(shbox[i]);
          }
          gtk_widget_destroy(shbox[i]);
        }
      }
      free(shbox);
      shbox = NULL;
    }
    if (bvbox != NULL)
    {
       for ( i=0 ; i< nspec ; i++)
      {
        if (bvbox[i] !=NULL)
        {
          if (GTK_WIDGET_VISIBLE(bvbox[i]))
          {
            gtk_widget_hide(bvbox[i]);
          }
          gtk_widget_destroy(bvbox[i]);
        }
      }
      free(bvbox);
      bvbox = NULL;
    }
  }
}

void quit_gtk ()
{
  free_data ();
  isaacs_end_ ();
  gtk_main_quit ();
}

void show_Help ()
{
  GtkWidget * info;
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Help");
  gtk_widget_show (info);
}

void show_ProjectSettings ()
{
  GtkWidget * info;
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "ProjectSettings");
  gtk_widget_show (info);
}

G_MODULE_EXPORT void hide_ProjectSettings ()
{
  GtkWidget * info;
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "ProjectSettings");
  gtk_widget_hide (info);
}

G_MODULE_EXPORT void on_New_activate ()
{
  show_ProjectSettings();
  GtkWidget * info;
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "ProjectSettingsTabs");
  gtk_notebook_set_current_page (GTK_NOTEBOOK(info), 0);
}

G_MODULE_EXPORT void on_Open_activate ()
{
  int j;
  GtkWidget * info;
  GtkFileFilter *filter1, *filter2;

  info = gtk_file_chooser_dialog_new ("Open Project File",
     				      NULL,
     				      GTK_FILE_CHOOSER_ACTION_OPEN,
     				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
     				      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
     				      NULL);
  filter1 = gtk_file_filter_new();
  gtk_file_filter_set_name(GTK_FILE_FILTER(filter1), "IPF file (*.ipf)");
  gtk_file_filter_add_pattern(GTK_FILE_FILTER(filter1), "*.ipf");
  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(info), filter1);
  filter2 = gtk_file_filter_new();
  gtk_file_filter_set_name(GTK_FILE_FILTER(filter2), "All files (*)");
  gtk_file_filter_add_pattern(GTK_FILE_FILTER(filter2), "*");
  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(info), filter2);
  if (newipf)
  {
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(info), "");
  }
  else
  {
    gtk_file_chooser_set_filename (GTK_FILE_CHOOSER(info), ipffile);
  }
  if (gtk_dialog_run (GTK_DIALOG (info)) == GTK_RESPONSE_ACCEPT)
  {
    ipffile = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(info));
    newipf = FALSE;
    gtk_widget_destroy (info);
    j = open_xml(ipffile);
    if (j > 1)
    {
      show_error ("Impossible to use the Isaacs-XML project file\n");
    }
  }
  else
  {
    gtk_widget_destroy (info);
  }
}

G_MODULE_EXPORT void on_Save_activate ()
{
  int j;
  GtkWidget * info;
  GtkFileFilter *filter;

  info = gtk_file_chooser_dialog_new ("Save Project File",
     				      NULL,
     				      GTK_FILE_CHOOSER_ACTION_SAVE,
     				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
     				      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
     				      NULL);
  gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (info), TRUE);
  if (newipf)
  {
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(info), "");
    gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER(info), "isaacs-project.ipf");
  }
  else
  {
    gtk_file_chooser_set_filename (GTK_FILE_CHOOSER(info), ipffile);
  }
  filter = gtk_file_filter_new();
  gtk_file_filter_set_name(GTK_FILE_FILTER(filter), "IPF file (*.ipf)");
  gtk_file_filter_add_pattern(GTK_FILE_FILTER(filter), "*.ipf");
  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(info), filter);

  if (gtk_dialog_run (GTK_DIALOG (info)) == GTK_RESPONSE_ACCEPT)
  {
    ipffile = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(info));
    newipf = FALSE;
    j = write_xml(ipffile);
    if (j == 0)
    {
      show_error ("Impossible to write the IPF file\n");
    }
  }
  gtk_widget_destroy (info);
}

G_MODULE_EXPORT void on_Export_activate ()
{
  int j;
  int lgt;
  int tfr;
  int infrac;
  int cartorau;
  gboolean pfrac;
  gboolean exp;
  GtkWidget * info;
  GtkFileFilter *filter1, *filter2, *filter3, *filter4;
  GtkFileFilter *tmp;
  char errpbc[91]="To use fractional coordinates\nplease describe properly the lattice parameters";
  char * filename;

  if (prun)
  {
    info = gtk_file_chooser_dialog_new ("Export atomic coordinates",
       				      NULL,
                          GTK_FILE_CHOOSER_ACTION_SAVE,
     				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
     				      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
     				      NULL);
    gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (info), TRUE);
    filter1 = gtk_file_filter_new();
    filter2 = gtk_file_filter_new();
    filter3 = gtk_file_filter_new();
    filter4 = gtk_file_filter_new();
    gtk_file_filter_set_name(GTK_FILE_FILTER(filter1), "XYZ file (*.xyz) cartesian coordinates");
    gtk_file_filter_add_pattern(GTK_FILE_FILTER(filter1), "*.xyz");
    gtk_file_filter_set_name(GTK_FILE_FILTER(filter2), "XYZ file (*.xyz) atomic units");
    gtk_file_filter_add_pattern(GTK_FILE_FILTER(filter2), "*.xyz");
    gtk_file_filter_set_name(GTK_FILE_FILTER(filter3), "XYZ file (*.xyz) fractional (corner [0,0,0])");
    gtk_file_filter_add_pattern(GTK_FILE_FILTER(filter3), "*.xyz");
    gtk_file_filter_set_name(GTK_FILE_FILTER(filter4), "XYZ file (*.xyz) fractional (center [0,0,0])");
    gtk_file_filter_add_pattern(GTK_FILE_FILTER(filter4), "*.xyz");
    gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(info), filter1);
    gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(info), filter2);
    gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(info), filter3);
    gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(info), filter4);
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(info), "");
    gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER(info), "coordinates.xyz");

    if (gtk_dialog_run (GTK_DIALOG (info)) == GTK_RESPONSE_ACCEPT)
    {
      tmp = gtk_file_chooser_get_filter (GTK_FILE_CHOOSER(info));
      filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(info));
      lgt=StringLength(filename);
      if (tmp == filter3 || tmp == filter4)
      {
        infrac=1;
        pfrac = testpbc();
        if (! pfrac)
        {
          show_warning (errpbc);
          info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "ProjectSettingsTabs");
          gtk_notebook_set_current_page (GTK_NOTEBOOK(info), 2);
          exp=FALSE;
        }
        else
        {
          if (tmp == filter4)
          {
            tfr=1;
          }
          else
          {
            tfr=0;
          }
          lattice_ (cvect, vmod, langles, & cell, & tfr);
          exp=TRUE;
        }
      }
      else
      {
        infrac=0;
        exp=TRUE;
      }
      if (exp)
      {
        if (tmp == filter2)
        {
          cartorau=1;
        }
        j = write_xyz_ (filename, & lgt, & infrac, & cartorau);
        if (j == 0)
        {
          show_error ("Impossible to export the atomic coordinates\n");
        }
        else
        {
          if (tmp == filter1)
          {
            show_info ("Atom positions have been successfully exported\n"
                       "using cartesian coordinates.\n");
          }
          else if (tmp == filter2)
          {
            show_info ("Atom positions have been successfully exported\n"
                       "using atomic units.\n");
          }
          else if (tmp == filter2)
          {
            show_info ("Atom positions have been successfully exported\n"
                       "using fractional coordinates.\n"
                       "a corner of the simulation box being at (0,0,0)");
          }
          else if (tmp == filter4)
          {
            show_info ("Atom positions have been successfully exported\n"
                       "using fractional coordinates.\n"
                       "the center of the simulation box being at (0,0,0)");
          }
        }
      }
    }
    gtk_widget_destroy (info);
  }
  else
  {
     show_warning ("The project must have been validated\n"
                   "to export atomic coordinates\n");
  }
}

G_MODULE_EXPORT void on_Quit_activate ()
{
  quit_gtk ();
}

G_MODULE_EXPORT void on_FileType_changed (GtkComboBox *ComboBoxGtk)
{
  GtkTreeModel * p_model = NULL;
  GtkTreeIter iter;

  p_model = gtk_combo_box_get_model (ComboBoxGtk);
  if (gtk_combo_box_get_active_iter (ComboBoxGtk, &iter))
  {
    gtk_tree_model_get (p_model, &iter, 0, &typefile, -1);
  }
}

G_MODULE_EXPORT void on_CoordFileChooser_file_set (GtkFileChooser *CoordFileChooser)
{
  GtkWidget * info;
  int tfile;

  coordfile = gtk_file_chooser_get_filename (CoordFileChooser);
  g_print ("coordfile= %s\n", coordfile);
  if (coordfile != NULL)
  {
    if (typefile != NULL)
    {
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "FileType");
      tfile = gtk_combo_box_get_active (GTK_COMBO_BOX(info));
      open_file(& tfile);
    }
    else
    {
      /* Info Message Box */
      show_info("You need to specify a file format");
    }
  }
}

int to_read_trj_or_vas (int * len, int * ff)
{
  int i, j;
  int * nas;
  char * lbs;
  char * qas;

  natomes = iask("The total number of atoms: ");
  if (natomes < 1)
  {
    return (3);
  }
  nspec = iask("The number of chemical species: ");
  if (nspec < 1)
  {
    return (3);
  }
  getnspec_(& nspec);
  nas = calloc(nspec,sizeof*nas);
  las = calloc(nspec,sizeof*las);
  j = 0;
  for (i = 0; i < nspec; i++)
  {
    lbs = g_strdup_printf("The label of atomic spec. N° %d: ", i+1);
    las[i] = g_strdup_printf("%2s", cask(lbs));
    g_free(lbs);
    if (strcmp(las[i], " ") == 0)
    {
      g_free(las[i]);
      free(nas);
      free(las);
      return (3);
    }
    getlabel_ (& i, las[i]);
    qas = g_strdup_printf("The number of %s atoms: ",las[i]);
    nas[i] = iask(qas);
    free(qas);
    if (nas[i] == 0)
    {
      free(nas);
      free(las);
      return (3);
    }
    j=j+nas[i];
  }
  if (j != natomes) return (3);

  if (* ff == 3)
  {
    i= read_trj_ (coordfile, len, & natomes, nas);
  }
  else
  {
    i= read_vas_ (coordfile, len, & natomes, nas);
  }
  free(nas);
  free(las);
  return(i);
}

void open_file (int * ftype)
{
  int i;
  int res=0;
  int length;
  GtkWidget * info;

  free_data ();
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "CheckPBC");
  activate_PBC ();
  prun = FALSE;
  run = FALSE;
  trun = FALSE;
  grrun = FALSE;
  sqrun = FALSE;
  skrun = FALSE;
  gqrun = FALSE;
  brun = FALSE;
  bdrun = FALSE;
  angrun = FALSE;
  msdrun = FALSE;
  ringrun = FALSE;
  shrun = FALSE;
  bvrun = FALSE;
  dmtx = FALSE;
  changedpbc = FALSE;
  grtotcutoff = 0.0;
  i = 0;
  turnchecks(& i);
  turnactions(& i, & i);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "grmoother");
  gtk_widget_set_sensitive(info, 0);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqmoother");
  gtk_widget_set_sensitive(info, 0);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "skmoother");
  gtk_widget_set_sensitive(info, 0);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "gqmoother");
  gtk_widget_set_sensitive(info, 0);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "comboboxrings");
  gtk_combo_box_set_active(GTK_COMBO_BOX(info), 0);
  length = StringLength(coordfile);
  if (* ftype == 0)
  {
// 0 and ? to ensure compatibility with the first IPF files
    res=read_xyz_ (coordfile, & length);
  }
  else if (* ftype == 1)
  {
    res=read_pdb_ (coordfile, & length);
  }
  else if (* ftype == 2)
  {
// 2 and ? to ensure compatibility with the first IPF files
    res=read_c3d_ (coordfile, & length);
  }
  else if (* ftype == 3)
  {
    res=to_read_trj_or_vas (& length, ftype);
  }
  else if (* ftype == 4)
  {
    res=to_read_trj_or_vas (& length, ftype);
  }

  if (res == 0)
  {
#ifndef G_OS_WIN32
    g_print ("%s\n", coordfile);
    g_print ("%s\n", typefile);
#endif

    if (prep_data_ () == 1)
    {
      readata = TRUE;
      tovalidate = TRUE;
      runfrac = FALSE;
      initbonds ();
      if (cutoffs != NULL) free(cutoffs);
      cutoffs = malloc(nspec*sizeof *cutoffs);
      for ( i = 0 ; i < nspec ; i ++ )
      {
        /* allocation d'un tableau de tableau */
        cutoffs[i] = calloc(nspec, sizeof *cutoffs[i]);
      }
      show_info("Structure information successfully loaded");
      if (nspec > 1)
      {
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "ababcheck");
        gtk_widget_set_sensitive(info, 1);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "homocheck");
        gtk_widget_set_sensitive(info, 1);
      }
      else
      {
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "ababcheck");
        gtk_widget_set_sensitive(info, 0);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "homocheck");
        gtk_widget_set_sensitive(info, 0);
      }
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "comboboxrings");
      gtk_combo_box_set_active(GTK_COMBO_BOX(info), 0);
    }
    else
    {
      show_error("Error while parsing chemical information\n please check carefully the input file");
    }
  }
  if (res == 1)
  {
    /* Error Message Box - 1 */
    tovalidate = FALSE;
    show_error("Error loading file: file does not exist");
  }
  if (res == 2)
  {
    /* Error Message Box - 2 */
    tovalidate = FALSE;
    show_error ("Error loading file: format not supported");
  }
  if (res == 3)
  {
    /* Error Message Box - 3 */
    tovalidate = FALSE;
    show_error("Error at input: incorrect parameter");
  }
}

G_MODULE_EXPORT void on_ProjHelp_activate (GtkNotebook *notebook)
{
  show_Help();
  gtk_notebook_set_current_page (notebook, 0);
}

G_MODULE_EXPORT void on_RunHelp_activate (GtkNotebook *notebook)
{
  show_Help();
  gtk_notebook_set_current_page (notebook, 1);
}

G_MODULE_EXPORT void on_OpenConfigurationSettings_activate (GtkNotebook *notebook)
{
  show_ProjectSettings();
  gtk_notebook_set_current_page (notebook, 0);
}

G_MODULE_EXPORT void on_OpenChemicalSettings_activate (GtkNotebook *notebook)
{
  show_ProjectSettings();
  gtk_notebook_set_current_page (notebook, 1);
}

G_MODULE_EXPORT void on_OpenBoxSettings_activate (GtkNotebook *notebook)
{
  show_ProjectSettings();
  gtk_notebook_set_current_page (notebook, 2);
}

void initviewspec_ (int * nspc)
{
  nspec = * nspc;
  int i;
  if (label != NULL) free(label);
  label = NULL;
  label = malloc(nspec*sizeof*label);
  for ( i = 0 ; i < nspec ; i ++ )
  {
     /* allocation d'un tableau de tableau */
     label[i] = calloc(3, sizeof*label[i]);
  }
  if (elt_name != NULL) free(elt_name);
  elt_name = NULL;
  elt_name = malloc(nspec*sizeof*elt_name);
  for ( i = 0 ; i < nspec ; i ++ )
  {
     /* allocation d'un tableau de tableau */
     elt_name[i] = calloc(16, sizeof*elt_name[i]);
  }
  if (atid != NULL) free(atid);
  atid = NULL;
  atid = calloc(nspec, sizeof*atid);
  if (nscatt != NULL) free(nscatt);
  nscatt = NULL;
  nscatt = calloc(nspec, sizeof*nscatt);
  if (xscatt != NULL) free(xscatt);
  xscatt = NULL;
  xscatt = calloc(nspec, sizeof*xscatt);
  if (amass != NULL) free(amass);
  amass = NULL;
  amass = calloc(nspec, sizeof*amass);
  if (radid != NULL) free(radid);
  radid = NULL;
  radid = calloc(nspec, sizeof*radid);
  if (rad != NULL) free(rad);
  rad = NULL;
  rad = calloc(nspec, sizeof*rad);
}

void specdata_ (int * ind, int * atd, char * lbel, char * el_nme,
               double * scattlgth, double * xscattlgth, double * amss, double * rdus)
{
  int id = * ind;
  strcpy(label[id], lbel);
  strcpy(elt_name[id], el_nme);
  atid[id] = * atd;
  nscatt[id] = * scattlgth;
  xscatt[id] = * xscattlgth;
  amass[id] = * amss;
  rad[id] = * rdus;
  radid[id] = 0;
}

void display_spec_ ()
{
#ifndef G_OS_WIN32
  int i;
  for ( i = 0 ; i < nspec ; i ++ )
  {
    g_print ("%s\n", label[i]);
    g_print ("%s\n", elt_name[i]);
    g_print ("%i\n", atid[i]);
    g_print ("%f\n", amass[i]);
    g_print ("%f\n", rad[i]);
    g_print ("%f\n", nscatt[i]);
    g_print ("%f\n", xscatt[i]);
  }
#endif
}

G_MODULE_EXPORT void on_Species_get (GtkComboBox *ComboBoxGtk)
{
  int scb;
  GtkWidget * info;

  scb = gtk_combo_box_get_active (ComboBoxGtk);
  if (scb != -1)
  {
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Mass");
    if (!GTK_WIDGET_SENSITIVE(info)) gtk_widget_set_sensitive(info, 1);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Rad");
    if (!GTK_WIDGET_SENSITIVE(info)) gtk_widget_set_sensitive(info, 1);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Scatt");
    if (!GTK_WIDGET_SENSITIVE(info)) gtk_widget_set_sensitive(info, 1);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "radtype");
    if (!GTK_WIDGET_SENSITIVE(info)) gtk_widget_set_sensitive(info, 1);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "XScatt");
    if (!GTK_WIDGET_SENSITIVE(info)) gtk_widget_set_sensitive(info, 1);
    show_spec_info(& scb);
  }
}

void show_spec_info (int * spi)
{
  GtkWidget * info;

  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "ElemName");
  gtk_label_set_text (GTK_LABEL(info), (const gchar *)elt_name[* spi]);

  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Mass");
  update_entry_double (GTK_ENTRY(info), & amass[* spi]);

  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "radtype");
  gtk_combo_box_set_active (GTK_COMBO_BOX(info), radid[* spi]);

  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Rad");
  update_entry_double (GTK_ENTRY(info), & rad[* spi]);

  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Scatt");
  update_entry_double (GTK_ENTRY(info), & nscatt[* spi]);

  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "XScatt");
  update_entry_double (GTK_ENTRY(info), & xscatt[* spi]);
}

G_MODULE_EXPORT void on_radtype_changed (GtkComboBox *RadCombo)
{
  int rd;
  int sd;
  GtkWidget * info;

  rd = gtk_combo_box_get_active (RadCombo);
  sd = gtk_combo_box_get_active (GTK_COMBO_BOX(specbox));
  radid[sd] = rd;
  rad[sd] = change_radius_ (& sd, & rd);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Rad");
  update_entry_double (GTK_ENTRY(info), & rad[sd]);
  changedpbc=TRUE;
}

G_MODULE_EXPORT void on_Mass_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *str;
  int s;
  double val;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "Mass");
  str = gtk_entry_get_text (EntryGtk);
  val = atof(str);
  s = gtk_combo_box_get_active (GTK_COMBO_BOX(specbox));
  if (val != amass[s])
  {
     amass[s] = new_mass_ (& s, & val);
     changedpbc=TRUE;
  }
  update_entry_double (EntryGtk, & amass[s]);
}

G_MODULE_EXPORT void on_Rad_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *str;
  int s;
  double val;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "Rad");
  str = gtk_entry_get_text (EntryGtk);
  val = atof(str);
  s = gtk_combo_box_get_active (GTK_COMBO_BOX(specbox));
  if (val != rad[s])
  {
     rad[s] = new_radius_ (& s, & val);
     changedpbc=TRUE;
  }
  update_entry_double (EntryGtk, & rad[s]);
}

G_MODULE_EXPORT void on_Scatt_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *str;
  int s;
  double val;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "Scatt");
  str = gtk_entry_get_text (EntryGtk);
  val = atof(str);
  s = gtk_combo_box_get_active (GTK_COMBO_BOX(specbox));
  if (val != nscatt[s])
  {
     nscatt[s] = new_nscatt_ (& s, & val);
     changedpbc=TRUE;
  }
  update_entry_double (EntryGtk, & nscatt[s]);
}

G_MODULE_EXPORT void on_XScatt_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *str;
  int s;
  double val;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "XScatt");
  str = gtk_entry_get_text (EntryGtk);
  val = atof(str);
  s = gtk_combo_box_get_active (GTK_COMBO_BOX(specbox));
  if (val != xscatt[s])
  {
     xscatt[s] = new_xscatt_ (& s, & val);
     changedpbc=TRUE;
  }
  update_entry_double (EntryGtk, & xscatt[s]);
}

void activate_PBC ()
{
  GtkWidget * ax, * ay, * az;
  GtkWidget * bx, * by, * bz;
  GtkWidget * cx, * cy, * cz;
  GtkWidget * ma, * mb, * mc;
  GtkWidget * a, * b, * c;
  GtkWidget * info, * tov;

  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "CheckPBC");
  gtk_widget_set_sensitive(info, 1);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
  apbc=FALSE;
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Frac");
  gtk_widget_set_sensitive(info, 1);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
  fractional=FALSE;
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "combofrac");
  gtk_combo_box_set_active (GTK_COMBO_BOX(info), 0);
  gtk_widget_set_sensitive(info, 0);
  tfrac=0;
  ax = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "a.x");
  ay = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "a.y");
  az = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "a.z");
  bx = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "b.x");
  by = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "b.y");
  bz = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "b.z");
  cx = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "c.x");
  cy = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "c.y");
  cz = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "c.z");
  ma = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "moda");
  mb = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "modb");
  mc = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "modc");
  a = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "alpha");
  b = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "beta");
  c = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "gamma");
  tov = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "tovectors");

  gtk_widget_set_sensitive(ax, 1);
  update_entry_double (GTK_ENTRY(ax), & cvect[0][0]);
  gtk_widget_set_sensitive(ay, 1);
  update_entry_double (GTK_ENTRY(ay), & cvect[0][1]);
  gtk_widget_set_sensitive(az, 1);
  update_entry_double (GTK_ENTRY(az), & cvect[0][2]);
  gtk_widget_set_sensitive(bx, 1);
  update_entry_double (GTK_ENTRY(bx), & cvect[1][0]);
  gtk_widget_set_sensitive(by, 1);
  update_entry_double (GTK_ENTRY(by), & cvect[1][1]);
  gtk_widget_set_sensitive(bz, 1);
  update_entry_double (GTK_ENTRY(bz), & cvect[1][2]);
  gtk_widget_set_sensitive(cx, 1);
  update_entry_double (GTK_ENTRY(cx), & cvect[2][0]);
  gtk_widget_set_sensitive(cy, 1);
  update_entry_double (GTK_ENTRY(cy), & cvect[2][1]);
  gtk_widget_set_sensitive(cz, 1);
  update_entry_double (GTK_ENTRY(cz), & cvect[2][2]);
  gtk_widget_set_sensitive(ma, 1);
  update_entry_double (GTK_ENTRY(ma), & vmod[0]);
  gtk_widget_set_sensitive(mb, 1);
  update_entry_double (GTK_ENTRY(mb), & vmod[1]);
  gtk_widget_set_sensitive(mc, 1);
  update_entry_double (GTK_ENTRY(mc), & vmod[2]);
  gtk_widget_set_sensitive(a, 1);
  update_entry_double (GTK_ENTRY(a), & langles[0]);
  gtk_widget_set_sensitive(b, 1);
  update_entry_double (GTK_ENTRY(b), & langles[1]);
  gtk_widget_set_sensitive(c, 1);
  update_entry_double (GTK_ENTRY(c), & langles[2]);
  gtk_widget_set_sensitive(tov, 1);
}

G_MODULE_EXPORT void on_CheckPBC_toggled (GtkToggleButton *Button)
{
  apbc = gtk_toggle_button_get_active (Button);
  changedpbc=TRUE;
}

G_MODULE_EXPORT void on_Frac_toggled (GtkToggleButton *Button)
{
  GtkWidget * info;

  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "combofrac");
  gtk_combo_box_set_active (GTK_COMBO_BOX(info), 0);
  tfrac=0;
  fractional = gtk_toggle_button_get_active (Button);
  if (fractional)
  {
    gtk_widget_set_sensitive(info, 1);
    changedpbc=TRUE;
  }
  else
  {
    gtk_widget_set_sensitive(info, 0);
    changedpbc=TRUE;
  }
}

G_MODULE_EXPORT void on_combofrac_changed (GtkComboBox *ComboBoxGtk)
{
  tfrac = gtk_combo_box_get_active (ComboBoxGtk);
}

G_MODULE_EXPORT void on_ax_activate ()
{
  const gchar *str;
  double val;
  GtkEntry * EntryGtk;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "a.x");
  str = gtk_entry_get_text (EntryGtk);
  val = atof(str);
  if (val != cvect[0][0])
  {
     cvect[0][0]=val;
     changedpbc=TRUE;
  }
  update_entry_double (EntryGtk, & cvect[0][0]);
}

G_MODULE_EXPORT void on_ay_activate ()
{
  const gchar *str;
  double val;
  GtkEntry * EntryGtk;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "a.y");
  str = gtk_entry_get_text (EntryGtk);
  val = atof(str);
  if (val != cvect[0][1])
  {
     cvect[0][1]=val;
     changedpbc=TRUE;
  }
  update_entry_double (EntryGtk, & cvect[0][1]);
}

G_MODULE_EXPORT void on_az_activate ()
{
  const gchar *str;
  double val;
  GtkEntry * EntryGtk;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "a.z");
  str = gtk_entry_get_text (EntryGtk);
  val = atof(str);
  if (val != cvect[0][2])
  {
     cvect[0][2]=val;
     changedpbc=TRUE;
  }
  update_entry_double (EntryGtk, & cvect[0][2]);
}

G_MODULE_EXPORT void on_bx_activate ()
{
  const gchar *str;
  double val;
  GtkEntry * EntryGtk;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "b.x");
  str = gtk_entry_get_text (EntryGtk);
  val = atof(str);
  if (val != cvect[1][0])
  {
     cvect[1][0]=val;
     changedpbc=TRUE;
  }
  update_entry_double (EntryGtk, & cvect[1][0]);
}

G_MODULE_EXPORT void on_by_activate ()
{
  const gchar *str;
  double val;
  GtkEntry * EntryGtk;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "b.y");
  str = gtk_entry_get_text (EntryGtk);
  val = atof(str);
  if (val != cvect[1][1])
  {
     cvect[1][1]=val;
     changedpbc=TRUE;
  }
  update_entry_double (EntryGtk, & cvect[1][1]);
}

G_MODULE_EXPORT void on_bz_activate ()
{
  const gchar *str;
  double val;
  GtkEntry * EntryGtk;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "b.z");
  str = gtk_entry_get_text (EntryGtk);
  val = atof(str);
  if (val != cvect[1][2])
  {
     cvect[1][2]=val;
     changedpbc=TRUE;
  }
  update_entry_double (EntryGtk, & cvect[1][2]);
}

G_MODULE_EXPORT void on_cx_activate ()
{
  const gchar *str;
  double val;
  GtkEntry * EntryGtk;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "c.x");
  str = gtk_entry_get_text (EntryGtk);
  val = atof(str);
  if (val != cvect[2][0])
  {
     cvect[2][0]=val;
     changedpbc=TRUE;
  }
  update_entry_double (EntryGtk, & cvect[2][0]);
}

G_MODULE_EXPORT void on_cy_activate ()
{
  const gchar *str;
  double val;
  GtkEntry * EntryGtk;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "c.y");
  str = gtk_entry_get_text (EntryGtk);
  val = atof(str);
  if (val != cvect[2][1])
  {
     cvect[2][1]=val;
     changedpbc=TRUE;
  }
  update_entry_double (EntryGtk, & cvect[2][1]);
}

G_MODULE_EXPORT void on_cz_activate ()
{
  const gchar *str;
  double val;
  GtkEntry * EntryGtk;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "c.z");
  str = gtk_entry_get_text (EntryGtk);
  val = atof(str);
  if (val != cvect[2][2])
  {
     cvect[2][2]=val;
     changedpbc=TRUE;
  }
  update_entry_double (EntryGtk, & cvect[2][2]);
}

G_MODULE_EXPORT void on_moda_activate ()
{
  const gchar *str;
  double val;
  GtkEntry * EntryGtk;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "moda");
  str = gtk_entry_get_text (EntryGtk);
  val = atof(str);
  if (val != vmod[0])
  {
     vmod[0]=val;
     changedpbc=TRUE;
  }
  update_entry_double (EntryGtk, & vmod[0]);
}

G_MODULE_EXPORT void on_modb_activate ()
{
  const gchar *str;
  double val;
  GtkEntry * EntryGtk;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "modb");
  str = gtk_entry_get_text (EntryGtk);
  val = atof(str);
  if (val != vmod[1])
  {
     vmod[1]=val;
     changedpbc=TRUE;
  }
  update_entry_double (EntryGtk, & vmod[1]);
}

G_MODULE_EXPORT void on_modc_activate ()
{
  const gchar *str;
  double val;
  GtkEntry * EntryGtk;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "modc");
  str = gtk_entry_get_text (EntryGtk);
  val = atof(str);
  if (val != vmod[2])
  {
     vmod[2]=val;
     changedpbc=TRUE;
  }
  update_entry_double (EntryGtk, & vmod[2]);
}

G_MODULE_EXPORT void on_alpha_activate ()
{
  const gchar *str;
  double val;
  GtkEntry * EntryGtk;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "alpha");
  str = gtk_entry_get_text (EntryGtk);
  val = atof(str);
  if (val != langles[0])
  {
     langles[0]=val;
     changedpbc=TRUE;
  }
  update_entry_double (EntryGtk, & langles[0]);
}

G_MODULE_EXPORT void on_beta_activate ()
{
  const gchar *str;
  double val;
  GtkEntry * EntryGtk;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "beta");
  str = gtk_entry_get_text (EntryGtk);
  val = atof(str);
  if (val != langles[1])
  {
     langles[1]=val;
     changedpbc=TRUE;
  }
  update_entry_double (EntryGtk, & langles[1]);
}

G_MODULE_EXPORT void on_gamma_activate ()
{
  const gchar *str;
  double val;
  GtkEntry * EntryGtk;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "gamma");
  str = gtk_entry_get_text (EntryGtk);
  val = atof(str);
  if (val != langles[2])
  {
     langles[2]=val;
     changedpbc=TRUE;
  }
  update_entry_double (EntryGtk, & langles[2]);
}

gboolean testpbc ()
{
  gboolean val = TRUE;

  if (vmod[0] == 0.0 || vmod[1] == 0.0 || vmod[2] == 0.0)
  {
    val = FALSE;
  }
  else if (langles[0] == 0.0 || langles[1] == 0.0 || langles[2] == 0.0)
  {
    val = FALSE;
  }

  if (! val)
  {
    if (cvect[0][0] == 0.0 && cvect[0][1] == 0.0 && cvect[0][2] == 0.0)
    {
      return(FALSE);
    }
    else if (cvect[1][0] == 0.0 && cvect[1][1] == 0.0 && cvect[1][2] == 0.0)
    {
      return(FALSE);
    }
    else if (cvect[2][0] == 0.0 && cvect[2][1] == 0.0 && cvect[2][2] == 0.0)
    {
      return(FALSE);
    }
    else
    {
      cell = 1;
      return(TRUE);
    }
  }
  else
  {
    cell = 2;
    return(TRUE);
  }
}

void display_distances()
{
  GtkLabel * info;
  char * dtot;
  char legedmax[31]="Maximum distance in real space";
  char ldmax[18]="D<sub>max</sub>= ";
  char legeqmin[37]="Minimun distance in reciprocal space";
  char dunit[6]=" [Å]";
  char qunit[19]=" [Å<sup>-1</sup>]";
  char lqmin[18]="Q<sub>min</sub>= ";

  dtot = g_strdup_printf("%s%s%s%lf%s", legedmax, retl, ldmax, dmax, dunit);
  info = (GtkLabel *) gtk_builder_get_object (BuilderGtk, "dmax");
  gtk_label_set_text (info, (const gchar *)dtot);
  gtk_label_set_use_markup (info, TRUE);
  g_free(dtot);

  dtot = g_strdup_printf("%s%s%s%lf%s", legeqmin, retl, lqmin, qmin, qunit);
  info = (GtkLabel *) gtk_builder_get_object (BuilderGtk, "qmin");
  gtk_label_set_text (info, (const gchar *)dtot);
  gtk_label_set_use_markup (info, TRUE);
  g_free(dtot);

  dtot = g_strdup_printf("%s%s%s%lf%s", legeqmin, retl, lqmin, kmin, qunit);
  info = (GtkLabel *) gtk_builder_get_object (BuilderGtk, "kmin");
  gtk_label_set_text (info, (const gchar *)dtot);
  gtk_label_set_use_markup (info, TRUE);
  g_free(dtot);

  dtot = g_strdup_printf("%s%s%s%lf%s", legeqmin, retl, lqmin, kmin, qunit);
  info = (GtkLabel *) gtk_builder_get_object (BuilderGtk, "dqmin");
  gtk_label_set_text (info, (const gchar *)dtot);
  gtk_label_set_use_markup (info, TRUE);
  g_free(dtot);

  dtot = g_strdup_printf("%s%s%s%lf%s", legedmax, retl, ldmax, dmax, dunit);
  info = (GtkLabel *) gtk_builder_get_object (BuilderGtk, "drmax");
  gtk_label_set_text (info, (const gchar *)dtot);
  gtk_label_set_use_markup (info, TRUE);
  g_free(dtot);
}

G_MODULE_EXPORT void on_Apply_project_released ()
{
  GtkWidget * info;
  gboolean test = TRUE;
  int i;
  int disa = 0;
  int frct = 0;
  int pbcv = 0;

  char errpbc[104]="To apply PBC or use fractional coordinates\nplease describe properly the simulation box parameters";

  if (tovalidate)
  {
    test = testpbc ();
    if ((apbc || fractional) && ! test)
    {
      show_warning (errpbc);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "ProjectSettingsTabs");
      gtk_notebook_set_current_page (GTK_NOTEBOOK(info), 2);
    }
    else
    {
      if (apbc && test)
      {
        pbcv = 1;
      }
      else
      {
        pbcv = 0;
      }
      disa = 0;
      if (! apbc && ! test)
      {
        show_warning ("No simulation box parameters have been given\ng(r) and S(q) calculations are disabled.");
        cell = 0;
        disa = 1;
      }
      hide_ProjectSettings ();
      if ( changedpbc || ! prun )
      {
        wtfrac=tfrac;
        lattice_ (cvect, vmod, langles, & cell, & tfrac);
        if (fractional && !runfrac)
        {
          runfrac = TRUE;
          frct = 1;
        }
        else
        {
          frct = 0;
        }
        prepos_ (& pbcv, & frct);
        chemistry_ ();
        dmax = fdmax_ ();
        qmin = 2*pi/dmax;
        kmin = fkmin_ ();
        display_distances();
        i=0;
        turnchecks (& i);
        i=1;
        turnactions (& i, & disa);
        dmtx = FALSE;
        i = 0;
        if (run) hide_curves (&i, &numwid);
      }

      if (! run)
      {
        initcwidgets ();
      }

      if (fractional)
      {
        show_warning ("Fractional coordinates have been transformed into\n"
                      "cartesian coordinates using the simulation box parameters.\n"
                      "If you need to modify the simulation box paratemers you will\n"
                      "have to read again the file containing the 3D positions\n"
                      "of your atomic system in fractional coordinates.");
        fractional=FALSE;
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Frac");
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
      }
      prun=TRUE;
      changedpbc=FALSE;
      show_info ("The project has been validated");
    }
  }
  else
  {
    hide_ProjectSettings ();
    show_info ("No project to be validated to run calculation(s)");
  }
}

G_MODULE_EXPORT void on_rdf_activate ()
{
  GtkWidget * info;

  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "rdfgr");
  if (! GTK_WIDGET_VISIBLE(info))
  {
    gtk_widget_show_all(info);
  }
}

G_MODULE_EXPORT void on_sqfftrdf_activate ()
{
  GtkWidget * info;

  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqfft");
  if (! GTK_WIDGET_VISIBLE(info))
  {
    gtk_widget_show_all(info);
  }
}

G_MODULE_EXPORT void on_sqkpoints_activate ()
{
  GtkWidget * info;
  GtkEntry * inf;

  inf = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "qlim");
  qlim = 15.0;
  update_entry_double (inf, & qlim);
  inf = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "plim");
  plim = 1.0;
  update_entry_double (inf, & plim);
  show_warning ("Computing S(q) using the Debye equation\ncan be very CPU-time consuming");
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqkpoint");
  if (! GTK_WIDGET_VISIBLE(info))
  {
    gtk_widget_show_all(info);
  }
}

G_MODULE_EXPORT void on_grfft_activate ()
{
  GtkWidget * info;
  GtkLabel * lab;
  char * dtot;
  char legeqmax[37]="Maximum distance in reciprocal space";
  char qunit[19]=" [Å<sup>-1</sup>]";
  char lqmax[18]="Q<sub>max</sub>= ";

  dtot = g_strdup_printf("%s%s%s%lf%s", legeqmax, retl, lqmax, kmax, qunit);
  lab = (GtkLabel *) gtk_builder_get_object (BuilderGtk, "dqmax");
  gtk_label_set_text (lab, (const gchar *)dtot);
  gtk_label_set_use_markup (lab, TRUE);
  g_free(dtot);

  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "gqfft");
  if (! GTK_WIDGET_VISIBLE(info))
  {
    gtk_widget_show_all(info);
  }
}

G_MODULE_EXPORT void on_Bonds_activate ()
{
  GtkWidget * info;
  int i, j;

  i=gtk_combo_box_get_active(GTK_COMBO_BOX(bspeca));
  j=gtk_combo_box_get_active(GTK_COMBO_BOX(bspecb));
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "cut");
  update_entry_double (GTK_ENTRY(info), & cutoffs[i][j]);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "tcut");
  update_entry_double (GTK_ENTRY(info), & grtotcutoff);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "bonds");
  if (! GTK_WIDGET_VISIBLE(info))
  {
    gtk_widget_show_all(info);
  }
}

G_MODULE_EXPORT void on_Angles_activate ()
{
  GtkWidget * info;
  int i, j;

  i=gtk_combo_box_get_active(GTK_COMBO_BOX(aspeca));
  j=gtk_combo_box_get_active(GTK_COMBO_BOX(aspecb));
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "acut");
  update_entry_double (GTK_ENTRY(info), & cutoffs[i][j]);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "atcut");
  update_entry_double (GTK_ENTRY(info), & grtotcutoff);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "angles");
  if (! GTK_WIDGET_VISIBLE(info))
  {
    gtk_widget_show_all(info);
  }
}

G_MODULE_EXPORT void on_Rings_activate ()
{
  GtkWidget * info;
  int i, j;

  i=gtk_combo_box_get_active(GTK_COMBO_BOX(rspeca));
  j=gtk_combo_box_get_active(GTK_COMBO_BOX(rspecb));
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "rcut");
  update_entry_double (GTK_ENTRY(info), & cutoffs[i][j]);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "rtcut");
  update_entry_double (GTK_ENTRY(info), & grtotcutoff);
  show_warning ("Computing the ring statistics\ncan be very CPU-time consuming");
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "rings");
  if (! GTK_WIDGET_VISIBLE(info))
  {
    gtk_widget_show_all(info);
  }
}

G_MODULE_EXPORT void on_Spheric_activate ()
{
  GtkWidget * info;

  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "spheric");
  if (! GTK_WIDGET_VISIBLE(info))
  {
    gtk_widget_show_all(info);
  }
}

G_MODULE_EXPORT void on_Valence_activate ()
{
  GtkWidget * info;

  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "valence");
  if (! GTK_WIDGET_VISIBLE(info))
  {
    gtk_widget_show_all(info);
  }
}

G_MODULE_EXPORT void on_Msd_activate ()
{
  GtkWidget * info;

  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "msd");
  if (! GTK_WIDGET_VISIBLE(info))
  {
    gtk_widget_show_all(info);
  }
}

G_MODULE_EXPORT void on_GrHelp_released (GtkNotebook *notebook)
{
  GtkWidget * info;

  show_Help();
  gtk_notebook_set_current_page (notebook, 1);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "notebook-rc");
  gtk_notebook_set_current_page (GTK_NOTEBOOK(info), 0);
}

G_MODULE_EXPORT void on_SqHelp_released (GtkNotebook *notebook)
{
  GtkWidget * info;

  show_Help();
  gtk_notebook_set_current_page (notebook, 1);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "notebook-rc");
  gtk_notebook_set_current_page (GTK_NOTEBOOK(info), 1);
}

G_MODULE_EXPORT void on_SkHelp_released (GtkNotebook *notebook)
{
  GtkWidget * info;

  show_Help();
  gtk_notebook_set_current_page (notebook, 1);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "notebook-rc");
  gtk_notebook_set_current_page (GTK_NOTEBOOK(info), 2);
}

G_MODULE_EXPORT void on_SkoHelp_released (GtkNotebook *notebook)
{
  GtkWidget * info;

  show_Help();
  gtk_notebook_set_current_page (notebook, 1);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "notebook-rc");
  gtk_notebook_set_current_page (GTK_NOTEBOOK(info), 3);
}

G_MODULE_EXPORT void on_GqHelp_released (GtkNotebook *notebook)
{
  GtkWidget * info;

  show_Help();
  gtk_notebook_set_current_page (notebook, 1);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "notebook-rc");
  gtk_notebook_set_current_page (GTK_NOTEBOOK(info), 4);
}

G_MODULE_EXPORT void on_BdHelp_released (GtkNotebook *notebook)
{
  GtkWidget * info;

  show_Help();
  gtk_notebook_set_current_page (notebook, 1);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "notebook-rc");
  gtk_notebook_set_current_page (GTK_NOTEBOOK(info), 5);
}

G_MODULE_EXPORT void on_AnHelp_released (GtkNotebook *notebook)
{
  GtkWidget * info;

  show_Help();
  gtk_notebook_set_current_page (notebook, 1);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "notebook-rc");
  gtk_notebook_set_current_page (GTK_NOTEBOOK(info), 6);
}

G_MODULE_EXPORT void on_RiHelp_released (GtkNotebook *notebook)
{
  GtkWidget * info;

  show_Help();
  gtk_notebook_set_current_page (notebook, 1);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "notebook-rc");
  gtk_notebook_set_current_page (GTK_NOTEBOOK(info), 7);
}

G_MODULE_EXPORT void on_SpHelp_released (GtkNotebook *notebook)
{
  GtkWidget * info;

  show_Help();
  gtk_notebook_set_current_page (notebook, 1);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "notebook-rc");
  gtk_notebook_set_current_page (GTK_NOTEBOOK(info), 8);
}

G_MODULE_EXPORT void on_BvHelp_released (GtkNotebook *notebook)
{
  GtkWidget * info;

  show_Help();
  gtk_notebook_set_current_page (notebook, 1);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "notebook-rc");
  gtk_notebook_set_current_page (GTK_NOTEBOOK(info), 9);
}

G_MODULE_EXPORT void on_MsdHelp_released (GtkNotebook *notebook)
{
  GtkWidget * info;

  show_Help();
  gtk_notebook_set_current_page (notebook, 1);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "notebook-rc");
  gtk_notebook_set_current_page (GTK_NOTEBOOK(info), 10);
}

