! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

SUBROUTINE chemistry

!
! Density, number density, concentrations, empirical formula
!

USE PARAMETERS

IMPLICIT NONE

INTEGER :: REF1, REF2, REF3

LOGICAL :: UNCLEAR

if (allocated(VOL_DENSITY)) deallocate(VOL_DENSITY)
allocate(VOL_DENSITY(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: chemistry"//CHAR(0), "Table: VOL_DENSITY"//CHAR(0))
endif
if (allocated(Xi)) deallocate(Xi)
allocate(Xi(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: chemistry"//CHAR(0), "Table: Xi"//CHAR(0))
endif
if (allocated(INDEX_FORM)) deallocate(INDEX_FORM)
allocate(INDEX_FORM(NSP), STAT=ERR)
if (ERR .ne. 0) then
 call show_error ("Impossible to allocate memory"//CHAR(0), &
                  "Subroutine: chemistry"//CHAR(0), "Table: INDEX_FORM"//CHAR(0))
endif

!
! Evaluation of the volumic densities
! Calcul des densités volumiques
!

  REAL_DENSITY=0.0d0
  TOTAL_DENSITY=0.0d0
  do i=1, NSP
    VOL_DENSITY(i) = 0.0d0
  enddo
  if (VOLUME.ne.0.0) then
    do i=1, NSP
      VOL_DENSITY(i) = dble(NBSPBS(i))/VOLUME           ! Densité volumique
      REAL_DENSITY=REAL_DENSITY+NBSPBS(i)*MASS(i)
    enddo
    TOTAL_DENSITY = dble(NA)/VOLUME
    REAL_DENSITY=REAL_DENSITY/AVOGADRO
    REAL_DENSITY=REAL_DENSITY/VOLUME
    REAL_DENSITY=REAL_DENSITY*10
  endif

!
! Evalution of Xi number fraction of species i
!
!

  do k=1, NSP
    Xi(k) = dble(NBSPBS(k)) / dble(NA)
  enddo

!
! Deterlination of the empirical formula
! Détermination de la formule brute
!
allocate(REFP(NSP), STAT=ERR)
if (ERR .ne. 0) then
 call show_error ("Impossible to allocate memory"//CHAR(0), &
                  "Subroutine: chemistry"//CHAR(0), "Table: REFP"//CHAR(0))
endif

do i=1, NSP
  REFP(i)=NBSPBS(i)
enddo

  do i=1, NSP

    if (i .eq. 1)then

      REF1=NBSPBS(i)
      REF2=i

    else

      REF3=REF1
      REF1=min(NBSPBS(i),REF1)
      if (REF3 .ne. REF1) REF2=i

    endif

  enddo

  UNCLEAR=.true.
  l=1
  do while (UNCLEAR)

    UNCLEAR=.false.
    do i=1, NSP

      m=1
      do while (mod(m*REFP(i),REF1) .ne. 0)
        m=m+1
      enddo
      if (i .ne. REF2) then
        INDEX_FORM(i)=m*REFP(i)/REF1
        UNCLEAR=.true.
      else
        INDEX_FORM(i)=REFP(i)/NBSPBS(i)
      endif

    enddo

    if (UNCLEAR) then

      UNCLEAR=.false.
      do i=1, NSP
        do j=1, NSP
          if (j .ne. i) then
            y=dble(INDEX_FORM(i))/dble(INDEX_FORM(j))
            z=dble(NBSPBS(i))/dble(NBSPBS(j))
            if (y .ne. z) UNCLEAR=.true.
          endif
        enddo
      enddo
      if (UNCLEAR) then
        REFP=REFP/l
        l=l+1
        REFP=REFP*l
      endif

    endif

  enddo

deallocate(REFP)

if (VOLUME .eq. 0.0d0) then
  FREEVOL=0.0d0
  ECFREEV=0.0d0
else
  call FREEVOLUME
endif

call cheminfo (INDEX_FORM, VOLUME, REAL_DENSITY, FREEVOL, ECFREEV)

do i=1, NSP
  call display_cheminfo (i-1, NBSPBS(i), INDEX_FORM(i), Xi(i), VOL_DENSITY(i))
enddo

CONTAINS

SUBROUTINE FREEVOLUME

USE PARAMETERS

DOUBLE PRECISION :: frv, VAL, VL
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: FREEV

allocate(FREEV(NS), STAT=ERR)
if (ERR .ne. 0) then
 call show_error ("Impossible to allocate memory"//CHAR(0), &
                  "Subroutine: FREEVOLUME"//CHAR(0), "Table: FREEV"//CHAR(0))
endif

do k=1, NS

  frv=0.0d0
  do i=1, NA-1
  do j=i+1, NA

    call CALCRIJ(i, j, k, k)
    VAL=RVDW(LOT(i))+RVDW(LOT(j))
    if (Dij .lt. VAL) then
      if (Dij .ge. RVDW(LOT(i))) then
        VL=(VAL-Dij)/2.0d0
        frv=frv+(PI/3.0)*VL*VL*(3*RVDW(LOT(i))-VL)
      elseif (Dij .lt. RVDW(LOT(i))) then
        VL=RVDW(LOT(i))-Dij/2.0d0
        frv=frv+(PI/3.0)*VL*VL*(3*RVDW(LOT(i))-VL)
      endif
      if (Dij .ge. RVDW(LOT(j))) then
        VL=(VAL-Dij)/2.0d0
        frv=frv+(PI/3.0)*VL*VL*(3*RVDW(LOT(j))-VL)
      elseif (Dij .lt. RVDW(LOT(j))) then
        VL=RVDW(LOT(j))-Dij/2.0d0
        frv=frv+(PI/3.0)*VL*VL*(3*RVDW(LOT(j))-VL)
      endif
    endif

  enddo
  enddo

  FREEV(k)=0.0d0
  do i=1, NSP
!    write (*,*) i, TL(i), NBSPBS(i)
    FREEV(k)=FREEV(k)+NBSPBS(i)*(4.0/3.0)*PI*RVDW(i)*RVDW(i)*RVDW(i)
!    write (*,*) (4.0/3.0)*PI*RVDW(i)*RVDW(i)*RVDW(i), NBSPBS(i)*(4.0/3.0)*PI*RVDW(i)*RVDW(i)*RVDW(i)
  enddo
!  write (*,*) FREEV(k), frv
  if (frv .gt. FREEV(k)) then
    FREEV(k)=0.0d0
  else
    FREEV(k)=VOLUME-(FREEV(k)-frv)
  endif
enddo

if (NS .gt. 1) then
  call MOYENNE(FREEV, NS, FREEVOL)
  call ECT_TYPE(FREEVOL, FREEV, NS, ECFREEV)
else
  FREEVOL=FREEV(1)
  ECFREEV=0.0d0
endif

deallocate(FREEV)

END SUBROUTINE

END SUBROUTINE
