/* This file is part of I.S.A.A.C.S.

   I.S.A.A.C.S. is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

    I.S.A.A.C.S. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <cairo/cairo.h>
#include <cairo/cairo-pdf.h>
#include <cairo-svg.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "global.h"
#include "interface.h"
#include "callbacks.h"
#include "curve.h"

// Size of the GTK+ curve window / (Average height of the memu bar + size of the window)
// This is used of resize after test = 640 / 667
#define THESHIFT -27
// Grid lines width in pixels
#define GRIDSIZE 0.1
// Default position of the legend on the curve
#define LEGX 0.70
#define LEGY 0.20
// Default color of the data from 0 to 65535
#define RED 65535
#define GREEN 0
#define BLUE 0
// Default linewidth of the data line
#define DTHICK 1.0
// Number of dash types
#define NDASHS 11
// Number of glyph types
#define NGLYPHS 37
// The marges between the windows border and the axes
#define MARGX 70
#define MARGY 70

int forme;
int resol[2];
double XDRAW, YDRAW;
double DXDRAW, DYDRAW;
double xmin, xmax;
double ymin, ymax;
double DX, DY;
int xlog, ylog;
int dxlog, dylog;
char * sca;

// Marge sur X et Y dans les graphs
int xmarge = MARGX;
int ymarge = MARGY;
char * IMG_FILE = NULL;
GtkWidget * setcolorbox = NULL;
GtkWidget * datcolor = NULL;
GtkWidget * dataspect = NULL;
GtkWidget * linewidth = NULL;
GtkWidget * glyphsize = NULL;
GtkWidget * glyphfreq = NULL;
GtkWidget * dashbox = NULL;
GtkWidget * glyphbox = NULL;
GtkWidget * stylearea = NULL;
GtkWidget * pixarea = NULL;
GtkWidget * legcoord = NULL;
GtkWidget * xyl[2], * xyp[2];
GtkWidget * lxyc[2];
GtkWidget * vmin[2], * vmax[2];
GtkWidget * mjt[2], * nmi[2];
GtkWidget * xtitle = NULL;
GtkWidget * titlex = NULL;

cairo_t * cr;
cairo_surface_t *surface;

// Number of dash formats
int ndash = NDASHS;
int nglyph = NGLYPHS;

typedef struct {
   GtkWidget * a;
   GtkWidget * b;
} toimg;

typedef struct {
  gpointer a;
  gpointer b;
  GtkWidget * c;
} setdata;

typedef struct {
  int a;
  int b;
} axdata;

curve ** curves = NULL;

typedef struct {
  const double * a;
  int b;
} thedash;

static const double dashed0[] = {0.0, 0.0};
static int len0 = sizeof(dashed0) / sizeof(dashed0[0]);

static const double dashed1[] = {1.0, 0.0};
static int len1 = sizeof(dashed1) / sizeof(dashed1[0]);

static const double dashed2[] = {5.0, 2.5};
static int len2 = sizeof(dashed2) / sizeof(dashed2[0]);

static const double dashed3[] = {4.0, 6.0};
static int len3 = sizeof(dashed3) / sizeof(dashed3[0]);

static const double dashed4[] = {2.0, 5.0};
static int len4 = sizeof(dashed4) / sizeof(dashed4[0]);

static const double dashed5[] = {10.0, 5.0};
static int len5 = sizeof(dashed5) / sizeof(dashed5[0]);;

static const double dashed6[] = {10.0, 10.0};
static int len6 = sizeof(dashed6) / sizeof(dashed6[0]);;

static const double dashed7[] = {4.0, 1.0, 4.0};
static int len7 = sizeof(dashed7) / sizeof(dashed7[0]);

static const double dashed8[] = {10.0, 2.5, 10.0};
static int len8 = sizeof(dashed8) / sizeof(dashed8[0]);;

static const double dashed9[] = {10.0, 5.0, 2.0};
static int len9 = sizeof(dashed9) / sizeof(dashed9[0]);;

static const double pdashed[] = {1.0};
static int lenp = 1;

void save_to_file_ (int *, char *, int *, double *, int *, int *, int *, double *, double *, int *, int *);
void prep_file_ (int *, char *, int *, int *, int *, double *, int *);
void append_to_file_ (int *, double *, int *, double *, double *, int *, int *, int *, int *);

thedash * selectdash (int * iddash)
{
  thedash * dashtab;
  dashtab = calloc(1,sizeof*dashtab);

  if (* iddash == 0)
  {
     dashtab -> a = dashed0;
     dashtab -> b = len0;
  }
  if (* iddash == 1)
  {
     dashtab -> a = dashed1;
     dashtab -> b = len1;
  }
  if (* iddash == 2)
  {
     dashtab -> a = dashed2;
     dashtab -> b = len2;
  }
  if (* iddash == 3)
  {
     dashtab -> a = dashed3;
     dashtab -> b = len3;
  }
  if (* iddash == 4)
  {
     dashtab -> a = dashed4;
     dashtab -> b = len4;
  }
  if (* iddash == 5)
  {
     dashtab -> a = dashed5;
     dashtab -> b = len5;
  }
  if (* iddash == 6)
  {
     dashtab -> a = dashed6;
     dashtab -> b = len6;
  }
  if (* iddash == 7)
  {
     dashtab -> a = dashed7;
     dashtab -> b = len7;
  }
  if (* iddash == 8)
  {
     dashtab -> a = dashed8;
     dashtab -> b = len8;
  }
  if (* iddash == 9)
  {
     dashtab -> a = dashed9;
     dashtab -> b = len9;
  }
  if (* iddash == 10)
  {
     dashtab -> a = pdashed;
     dashtab -> b = lenp;
  }
  return (dashtab);
}

const gchar * default_xtitle (gpointer data)
{
  int d = GPOINTER_TO_INT(data);

  if (d < numgr)
  {
    return ("r [Å]");
  }
  else if (d < numsq+numgr)
  {
    return ("q [Å-1]");
  }
  else if (d < numsk+numsq+numgr)
  {
    return ("q [Å-1]");
  }
  else if (d < numsk+numsq+numgr+numgq)
  {
    return ("r [Å]");
  }
  else if (d < numsk+numsq+numgr+numgq+numbd)
  {
    return ("Dij [Å]");
  }
  else if (d < numsk+numsq+numgr+numgq+numbd+numan)
  {
    return ("Angle [°]");
  }
  else if (d < numsk+numsq+numgr+numgq+numbd+numan+numri)
  {
    return ("Size n of the ring [total number of nodes]");
  }
  else if (d < numsk+numsq+numgr+numgq+numbd+numan+numri+numsh)
  {
    return ("Ql");
  }
  else
  {
    return (untime[tunit-1]);
  }
}

double scale (double * axe)
{
  double xs;
  double axis;

  axis = fabs(* axe);

  if (axis < 0.0005)
  {
    xs=0.00005;
  }
  else if (axis < 0.0025)
  {
    xs=0.00025;
  }
  else if (axis < 0.005)
  {
    xs=0.0005;
  }
  else if (axis < 0.025)
  {
    xs=0.0025;
  }
  else if (axis < 0.05)
  {
    xs=0.005;
  }
  else if (axis < 0.25)
  {
    xs=0.025;
  }
  else if (axis < 0.5 )
  {
    xs=0.05;
  }
  else if (axis < 1.5 )
  {
    xs=0.1;
  }
  else if (axis < 2.5)
  {
    xs=0.25;
  }
  else if (axis < 5)
  {
    xs=0.5;
  }
  else if (axis < 15)
  {
    xs = 1.0;
  }
  else if (axis < 30)
  {
    xs = 2.0;
  }
  else if (axis < 50)
  {
    xs = 5.0;
  }
  else if (axis < 100)
  {
    xs = 10.0;
  }
  else if (axis < 250)
  {
    xs = 25.0;
  }
  else if (axis < 500)
  {
    xs = 50.0;
  }
  else if (axis < 1000)
  {
    xs = 100.0;
  }
  else if (axis < 5000)
  {
    xs = 500.0;
  }
  else if (axis < 10000)
  {
    xs = 1000.0;
  }
  else if (axis < 50000)
  {
    xs = 5000.0;
  }
  else if (axis < 100000)
  {
    xs = 10000.0;
  }
  else if (axis < 500000)
  {
    xs = 50000.0;
  }
  else if (axis < 1000000)
  {
    xs = 100000.0;
  }
  return (xs);
}

int findcurve (int * what, int * id)
{
  if (curves[* what] -> cstart == 0 || curves[* what] -> cstart == numgr + numsq + numsk)
  {
    if (curves[* what] -> cstart == 0 && * id  >= numgr)
    {
      return (* id + numsq + numsk);
    }
    else if (curves[* what] -> cstart == numgr+numsq+numsk && * id >= numgq)
    {
      return (* id - numgq);
    }
    else
    {
      return (curves[* what] -> cstart + * id);
    }
  }
  else if (curves[* what] -> cstart == numgr || curves[* what] -> cstart == numgr + numsq)
  {
    if (curves[* what] -> cstart == numgr && * id >= numsq)
    {
      return (numgr + * id);
    }
    else if (curves[* what] -> cstart == numgr+numsq && * id >= numsk)
    {
      return (numgr + * id - numsk);
    }
    else
    {
      return(curves[* what] -> cstart + * id);
    }
  }
  else
  {
    return (curves[* what] -> cstart + * id);
  }
}

void draw_poly (cairo_t * to, double * x, double * y, double * size, double * step, double * initp)
{
  double px, py;
  double alpha;
  double astep;
  double astart;

  astart = 2*pi / * initp;
  px = * x + * size * cos (astart);
  py = * y + * size * sin (astart);
  astep = 2*pi/ * step;
  cairo_move_to (to, px, py);
  for (alpha =0.0 ; alpha <= 2*pi ; alpha += astep)
  {
    px = * x + * size * cos (astart + alpha);
    py = * y + * size *sin (astart + alpha);
    cairo_line_to (to, px, py);
  }
}

void draw_glyph (cairo_t * in, int * theglyph, double * x, double * y, GdkColor * gcolor, double * size)
{
  double step;
  double start;
  double width;
  double cdash[3];
  double offset;
  int dcount;
  int idglyph;
  double starttab[] = {1.0, 4.0, 2.0, 1.33333};

  dcount = cairo_get_dash_count (in);
  cairo_get_dash (in, cdash, & offset);
  cairo_set_dash (in, dashed1, len1, 0.0);
  width = cairo_get_line_width (in);
  cairo_set_line_width (in, 1.0);
  cairo_set_source_rgba (in, gcolor -> red/65535.0, gcolor -> green/65535.0, gcolor -> blue/65535.0, 1.0);
  if (* theglyph == 1)
  {
    cairo_move_to (in, * x + * size/2.0, * y - * size/2.0);
    cairo_line_to (in, * x - * size/2.0, * y + * size/2.0);
    cairo_move_to (in, * x + * size/2.0, * y + * size/2.0);
    cairo_line_to (in, * x - * size/2.0, * y - * size/2.0);
    cairo_stroke (in);
  }
  if (* theglyph > 1 && * theglyph < 7)
  {
    cairo_move_to (in, * x, * y);
    cairo_rectangle (in, * x - * size/2.0, * y - * size/2.0, * size, * size);
    if (* theglyph == 2)
    {
      cairo_fill(in);
    }
    else if (* theglyph == 3)
    {
      cairo_stroke (in);
      cairo_set_source_rgba (in, gcolor -> red/65535.0, gcolor -> green/65535.0, gcolor -> blue/65535.0, 0.25);
      cairo_move_to (in, * x, * y);
      cairo_rectangle (in, * x - * size/2.0, * y - * size/2.0, * size, * size);
      cairo_fill (in);
    }
    else if (* theglyph == 4)
    {
      cairo_stroke (in);
      cairo_set_source_rgba (in, 1.0, 1.0, 1.0, 1.0);
      cairo_move_to (in, * x, * y);
      cairo_rectangle (in, * x - * size/2.0, * y - * size/2.0, * size, * size);
      cairo_fill (in);
      cairo_stroke (in);
      cairo_set_source_rgba (in, gcolor -> red/65535.0, gcolor -> green/65535.0, gcolor -> blue/65535.0, 0.25);
      cairo_move_to (in, * x, * y);
      cairo_rectangle (in, * x - * size/2.0, * y - * size/2.0, * size, * size);
      cairo_fill (in);
    }
    else if (* theglyph == 5)
    {
      cairo_stroke (in);
      cairo_set_source_rgba (in, 1.0, 1.0, 1.0, 1.0);
      cairo_move_to (in, * x, * y);
      cairo_rectangle (in, * x - * size/2.0, * y - * size/2.0, * size, * size);
      cairo_fill (in);
    }
    cairo_stroke (in);
  }
  else if (* theglyph >= 7 && * theglyph < 12)
  {
    step = 4.0;
    start = 1.0;
    draw_poly (in, x, y, size, & step, & start);
    if (* theglyph == 7)
    {
      cairo_fill(in);
    }
    else if (* theglyph == 8)
    {
      cairo_stroke (in);
      cairo_set_source_rgba (in, gcolor -> red/65535.0, gcolor -> green/65535.0, gcolor -> blue/65535.0, 0.25);
      draw_poly (in, x, y, size, & step, & start);
      cairo_fill (in);
    }
    else if (* theglyph == 9)
    {
      cairo_stroke (in);
      cairo_set_source_rgba (in, 1.0, 1.0, 1.0, 1.0);
      draw_poly (in, x, y, size, & step, & start);
      cairo_fill (in);
      cairo_stroke (in);
      cairo_set_source_rgba (in, gcolor -> red/65535.0, gcolor -> green/65535.0, gcolor -> blue/65535.0, 0.25);
      draw_poly (in, x, y, size, & step, & start);
      cairo_fill (in);
    }
    else if (* theglyph == 10)
    {
      cairo_stroke (in);
      cairo_set_source_rgba (in, 1.0, 1.0, 1.0, 1.0);
      draw_poly (in, x, y, size, & step, & start);
      cairo_fill (in);
    }
    cairo_stroke (in);
  }
  else if (* theglyph >= 12 && * theglyph < 17)
  {
    cairo_arc (in, * x, * y, * size, 0.0, 2*pi);
    if (* theglyph == 12)
    {
      cairo_fill(in);
    }
    else if (* theglyph == 13)
    {
      cairo_stroke (in);
      cairo_set_source_rgba (in, gcolor -> red/65535.0, gcolor -> green/65535.0, gcolor -> blue/65535.0, 0.25);
      cairo_arc (in, * x, * y, * size, 0.0, 2*pi);
      cairo_fill (in);
    }
    else if (* theglyph == 14)
    {
      cairo_stroke (in);
      cairo_set_source_rgba (in, 1.0, 1.0, 1.0, 1.0);
      cairo_arc (in, * x, * y, * size, 0.0, 2*pi);
      cairo_fill (in);
      cairo_stroke (in);
      cairo_set_source_rgba (in, gcolor -> red/65535.0, gcolor -> green/65535.0, gcolor -> blue/65535.0, 0.25);
      cairo_arc (in, * x, * y, * size, 0.0, 2*pi);
      cairo_fill (in);
    }
    else if (* theglyph == 15)
    {
      cairo_stroke (in);
      cairo_set_source_rgba (in, 1.0, 1.0, 1.0, 1.0);
      cairo_arc (in, * x, * y, * size, 0.0, 2*pi);
      cairo_fill (in);
    }
    cairo_stroke (in);
  }
  else if (* theglyph > 16)
  {
    step = 3.0;
    idglyph = (* theglyph - 17) / 5;
    start = starttab[idglyph];
    draw_poly (in, x, y, size, & step, & start);
    if (* theglyph == 18 + idglyph*5)
    {
      cairo_fill(in);
    }
    else if (* theglyph == 19 + idglyph*5)
    {
      cairo_stroke (in);
      cairo_set_source_rgba (in, gcolor -> red/65535.0, gcolor -> green/65535.0, gcolor -> blue/65535.0, 0.25);
      draw_poly (in, x, y, size, & step, & start);
      cairo_fill (in);
    }
    else if (* theglyph == 20 + idglyph*5)
    {
      cairo_stroke (in);
      cairo_set_source_rgba (in, 1.0, 1.0, 1.0, 1.0);
      draw_poly (in, x, y, size, & step, & start);
      cairo_fill (in);
      cairo_stroke (in);
      cairo_set_source_rgba (in, gcolor -> red/65535.0, gcolor -> green/65535.0, gcolor -> blue/65535.0, 0.25);
      draw_poly (in, x, y, size, & step, & start);
      cairo_fill (in);
    }
    else if (* theglyph == 21 + idglyph*5)
    {
      cairo_stroke (in);
      cairo_set_source_rgba (in, 1.0, 1.0, 1.0, 1.0);
      draw_poly (in, x, y, size, & step, & start);
      cairo_fill (in);
    }
    cairo_stroke (in);
  }
  cairo_set_dash (in, cdash, dcount, offset);
  cairo_set_source_rgba (in, gcolor -> red/65535.0, gcolor -> green/65535.0, gcolor -> blue/65535.0, 1.0);
  cairo_set_line_width (in, width);
}

void show_legend (int * who, double * lpos)
{
  double x, y, z;
  int j, k, l;
  thedash * dasht;

  x = lpos[0]*resol[0];
  y = lpos[1]*resol[1];
  if (curves[* who] -> dash > 0)
  {
    dasht = selectdash (& curves[* who] -> dash);
    cairo_set_dash(cr, dasht -> a, dasht -> b, 0.0);
    cairo_set_source_rgb (cr, curves[* who] -> datacolor.red/65535.0,
                            curves[* who] -> datacolor.green/65535.0,
                             curves[* who] -> datacolor.blue/65535.0);
    cairo_set_line_width (cr, curves[* who] -> thickness);
    cairo_move_to (cr, x-5, y);
    cairo_line_to (cr, x+20, y);
    cairo_stroke(cr);
    free(dasht);
  }
  z = x-5;
  draw_glyph (cr, & curves[* who] -> glyph, & z, & y, & curves[* who] -> datacolor, & curves[* who] -> gsize);
  z = x+20;
  draw_glyph (cr, & curves[* who] -> glyph, & z, & y, & curves[* who] -> datacolor, & curves[* who] -> gsize);
  cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
  cairo_move_to (cr, x+35.0, y+3.5);
  cairo_show_text (cr, curves[* who] -> title);
  cairo_stroke (cr);

  k = curves[* who] -> cend - curves[* who] -> cstart;
  if (curves[* who] -> cstart == 0 || curves[* who] -> cstart == numgr + numsq + numsk)
  {
    k = numgr+numgq;
  }
  else if (curves[* who] -> cstart == numgr || curves[* who] -> cstart == numgr + numsq)
  {
    k = numsq+numsk;
  }
  for ( j=0 ; j < k ; j++ )
  {
    if (curves[* who] -> extrasets[j] != 0)
    {
      y = y + 20;
      if (curves[* who] -> extradash[j] > 0)
      {
        dasht = selectdash (& curves[* who] -> extradash[j]);
        cairo_set_dash(cr, dasht -> a, dasht -> b, 0.0);
        cairo_set_source_rgb (cr, curves[* who] -> extracolor[j].red/65535.0,
                              curves[* who] -> extracolor[j].green/65535.0,
                              curves[* who] -> extracolor[j].blue/65535.0);
        cairo_set_line_width (cr, curves[* who] -> extrathick[j]);
        cairo_move_to (cr, x-5, y);
        cairo_line_to (cr, x+20, y);
        cairo_stroke(cr);
        free(dasht);
      }
      z = x-5;
      draw_glyph (cr, & curves[* who] -> extraglyph[j], & z, & y,
                      & curves[* who] -> extracolor[j], & curves[* who] -> extragsize[j]);
      z = x+20;
      draw_glyph (cr, & curves[* who] -> extraglyph[j], & z, & y,
                      & curves[* who] -> extracolor[j], & curves[* who] -> extragsize[j]);
      cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
      cairo_move_to (cr, x+35.0, y+3.5);
      l=findcurve(who, &j);
      cairo_show_text (cr, curves[l] -> title);
      cairo_stroke(cr);
    }
  }
}

void draw_curve (int * which,
                 int * points,
                 double * vmin,
                 GdkColor * withcolor,
                 int * xscale,
                 int * yscale,
                 int * asp,
                 int * vdash,
                 double * thick,
                 int * glyp,
                 double * gize,
                 int * freq,
                 int * sets,
                 int * id)
{
  int i, j, k, l, m;
  double x, y;
  double x1, x2, y1, y2;
  double slope;
  double ** plotdata;
  int * plotd;
  gboolean dglyp;
  gboolean plot;
  gboolean xgrid = FALSE;
  thedash *  dasht;
  int npts;

  npts = * points;
  if (* asp == 0 && * which >= numsq+numgr && * which < numsk+numsq+numgr)
  {
    j = 0;
    for ( i=0 ; i < * points; i++ )
    {
      if (idsk[i] == 1) j = j + 1;
    }
    npts = j;
    plotd = calloc(npts, sizeof* plotd);
    j = -1;
    for ( i=0 ; i < * points; i++ )
    {
      if (idsk[i] == 1)
      {
        j = j + 1;
        plotd[j] = i;
      }
    }
  }
  plotdata = malloc(npts*sizeof*plotdata);
  for ( i=0 ; i < npts ; i++ )
  {
    plotdata[i] = calloc(2, sizeof*plotdata[i]);
    if (* xscale == 0)
    {
      if (* which >= numsq+numgr && * which < numsk+numsq+numgr)
      {
        plotdata[i][0] = DX + plotd[i] * DXDRAW / (npts -1);
      }
      else
      {
        plotdata[i][0] = DX + i * DXDRAW / (npts -1);
      }
    }
    else
    {
      x = (i+1) * ndtbs * delta_t * pow(10, dxlog);
      x = log(x) / log(pow(10, xlog));
      plotdata[i][0] = DX + DXDRAW * x;
      //plotdata[i][0] = DX + DXDRAW * log(i + 1.0) / log(* points);
    }
    if (* yscale == 0)
    {
      if (* which >= numsq+numgr && * which < numsk+numsq+numgr)
      {
        plotdata[i][1] = DY - YDRAW * (curves[* which] -> data[plotd[i]] - * vmin) / ymax;
      }
      else
      {
        plotdata[i][1] = DY - YDRAW * (curves[* which] -> data[i] - * vmin) / ymax;
      }
    }
    else
    {
      y = curves[* which] -> data[i] * pow(10, dylog);
      y = log(y) / log(pow(10, ylog));
      plotdata[i][1] = DY - YDRAW * y;
      // plotdata[i][1] = DY - YDRAW * log (curves[* which] -> data[i] - * vmin) / log(ymax);
    }
    //g_print ("x= %f, y= %f\n", plotdata[i][0], plotdata[i][1]);
  }
  if (* vdash > 0)
  {
    dasht = selectdash (vdash);
    cairo_set_dash(cr, dasht -> a, dasht -> b, 0.0);
    cairo_set_line_width (cr, * thick);
    free(dasht);
  }
  else
  {
    cairo_set_line_width (cr, 0.0);
  }
  cairo_set_source_rgb (cr, withcolor -> red/65535.0,
                            withcolor -> green/65535.0,
                            withcolor -> blue/65535.0);

  if (* asp == 0)
  {
    k = 0;
    l = 1;
    if (* which >= numsk+numsq+numgr+numgq+numbd+numan && * which < numsk+numsq+numgr+numgq+numbd+numan+numri)
    {
      k = 2;
    }
    else if (* which >= numsk+numsq+numgr+numgq+numbd+numan+numri && * which < numsk+numsq+numgr+numgq+numbd+numan+numri+numsh)
    {
      k = 2;
      l = 2;
    }
    for ( i = k ; i < npts - l ; i = i + l )
    {
      plot = FALSE;
      dglyp = FALSE;
      if (plotdata[i][0] >= xmarge || plotdata[i+l][0] >= xmarge)
      {
        if (plotdata[i][1] <= YDRAW || plotdata[i+l][1] <= YDRAW)
        {
          plot=TRUE;
        }
      }
      if (plot)
      {
        if (plotdata[i][0] >= xmarge)
        {
          if (plotdata[i][1] <= YDRAW)
          {
            x1 = plotdata[i][0];
            y1 = plotdata[i][1];
            dglyp = TRUE;
          }
          else
          {
            y1 = YDRAW;
            slope = plotdata[i+l][1] - plotdata[i][1];
            slope = (plotdata[i+l][0] - plotdata[i][0]) / slope;
            x1 = plotdata[i][0] + slope * (y1 - plotdata[i][1]);
          }
        }
        else
        {
          if (plotdata[i][1] <= YDRAW)
          {
            x1 = xmarge;
            slope = plotdata[i+l][0] - plotdata[i][0];
            slope = (plotdata[i+l][1] - plotdata[i][1]) / slope;
            y1 = plotdata[i][1] + slope * (x1 - plotdata[i][0]);
          }
          else
          {
            y1 = YDRAW;
            slope = plotdata[i+l][1] - plotdata[i][1];
            slope = (plotdata[i+l][0] - plotdata[i][0]) / slope;
            x1 = plotdata[i][0] + slope * (y1 - plotdata[i][1]);
          }
        }
        if (plotdata[i+l][1] <= YDRAW)
        {
          x2 = plotdata[i+l][0];
          y2 = plotdata[i+l][1];
        }
        else
        {
          y2 = YDRAW;
          slope = plotdata[i+l][1] - plotdata[i][1];
          slope = (plotdata[i+l][0] - plotdata[i][0]) / slope;
          x2 = plotdata[i+l][0] + slope * (y2 - plotdata[i+l][1]);
        }
        cairo_move_to (cr, x1, y1);
        cairo_line_to (cr, x2, y2);
        cairo_stroke (cr);
        if (i % * freq == 0 && dglyp) draw_glyph (cr, glyp, & x1, & y1, withcolor, gize);
        if (i == * points - 2 && (i+l) % * freq == 0 && dglyp)
        {
          draw_glyph (cr, glyp, & x2, & y2, withcolor, gize);
        }
      }
    }
  }
  else if (* asp == 1)
  {
    k = 0;
    j = 0;
    l = 1;
    if (* which >= numsk+numsq+numgr+numgq+numbd+numan && * which < numsk+numsq+numgr+numgq+numbd+numan+numri)
    {
      k = 2;
      j = 0;
      l = 1;
    }
    else if (* which >= numsk+numsq+numgr+numgq+numbd+numan+numri && * which < numsk+numsq+numgr+numgq+numbd+numan+numri+numsh)
    {
      k = 2;
      j = 1;
      l = 2;
    }
    if (* id == 0)
    {
      m = 0;
    }
    else
    {
      m = * id + * sets / 2;
      if (m >= 0) m++;
    }
    for ( i = k ; i < npts ; i = i + l )
    {
      if (curves[* which] -> data[i] != 0.0)
      {
        if (plotdata[i][1] <= YDRAW)
        {
          if (plotdata[i-j][0] >= xmarge)
          {
            x1 = DX + (i - j - 0.25 + m * 0.1) * DXDRAW / (* points - 1);
            y1 = plotdata[i][1];
            x2 = 0.5 * DXDRAW / (* points - 1);
            y2 =  DY - plotdata[i][1];
            if (* vmin <= 0.0)
            {
              y2 = y2 + * vmin * YDRAW / ymax;
            }
            if (* sets > 0)
            {
              cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 1.0);
              cairo_rectangle (cr, x1, y1, x2, y2);
              cairo_fill(cr);
            }
            cairo_set_source_rgba (cr, withcolor -> red/65535.0,
                                       withcolor -> green/65535.0,
                                       withcolor -> blue/65535.0, 0.25);
            cairo_rectangle (cr, x1, y1, x2, y2);
            cairo_fill(cr);
            cairo_set_source_rgba (cr, withcolor -> red/65535.0,
                                       withcolor -> green/65535.0,
                                       withcolor -> blue/65535.0, 1.0);
            y2 = DY;
            if (* vmin <= 0.0)
            {
              y2 = y2 + * vmin * YDRAW / ymax;
            }
            cairo_move_to (cr, x1, y2);
            cairo_line_to (cr, x1, y1);
            cairo_move_to (cr, x1, y1);
            x2 = DX + (i - j + 0.25 + m * 0.1) * DXDRAW / (* points - 1);
            cairo_line_to(cr, x2, y1);
            cairo_move_to(cr, x2, y1);
            cairo_line_to(cr, x2, y2);
            cairo_stroke(cr);
          }
        }
      }
    }
    if (* sets > 0 && curves[* which] -> frame)
    {
      cairo_set_source_rgba (cr, curves[* which] -> framcolor.red/65535.0,
                                 curves[* which] -> framcolor.green/65535.0,
                                 curves[* which] -> framcolor.blue/65535.0, 1.0);
      cairo_set_line_width (cr, 2.0);
      cairo_set_line_width (cr, 2.0);
      cairo_set_dash (cr, dashed1, len1, 0.0);
      cairo_move_to (cr, xmarge, YDRAW);
      cairo_line_to (cr, resol[0], YDRAW);
      setup_xaxis_linear (which, & xgrid);
    }
  }
  cairo_stroke(cr);
  free (plotdata);
  if (* asp == 0 && * which >= numsq+numgr && * which < numsk+numsq+numgr)
  {
    free (plotd);
  }
}

void autoscale_xaxis (int * id)
{
  if (* id < numgr)
  {
    curves[* id] -> axmax[0] = dmax;
    curves[* id] -> axmin[0] = 0.0;
    curves[* id] -> cmin[0] = 0.0;
    curves[* id] -> cmax[0] = dmax;
  }
  else if (* id < numsq+numgr)
  {
    curves[* id] -> axmax[0] = qmax;
    curves[* id] -> axmin[0] = qmin;
    curves[* id] -> cmin[0] = qmin;
    curves[* id] -> cmax[0] = qmax;
  }
  else if (* id < numsk+numsq+numgr)
  {
    curves[* id] -> axmax[0] = kmax;
    curves[* id] -> axmin[0] = kmin;
    curves[* id] -> cmin[0] = kmin;
    curves[* id] -> cmax[0] = kmax;
  }
  else if (* id < numsk+numsq+numgr+numgq)
  {
    curves[* id] -> axmax[0] = dmax;
    curves[* id] -> axmin[0] = 0.0;
    curves[* id] -> cmin[0] = 0.0;
    curves[* id] -> cmax[0] = dmax;
  }
  else if (* id < numsk+numsq+numgr+numgq+numbd)
  {
    curves[* id] -> axmax[0] = bdmin + (num_delta_d-1)*delta_d;
    curves[* id] -> axmin[0] = bdmin;
    curves[* id] -> cmin[0] = bdmin;
    curves[* id] -> cmax[0] = bdmin + (num_delta_d-1)*delta_d;
  }
  else if (* id < numsk+numsq+numgr+numgq+numbd+numan)
  {
    curves[* id] -> axmax[0] = 180.0;
    curves[* id] -> axmin[0] = 0.0;
    curves[* id] -> cmin[0] = 0.0;
    curves[* id] -> cmax[0] = 180.0;
  }
  else if (* id < numsk+numsq+numgr+numgq+numbd+numan+numri)
  {
    curves[* id] -> axmax[0] = curves[* id] -> ndata + 1;
    curves[* id] -> axmin[0] = 2.0;
    curves[* id] -> cmin[0] = 1.0;
    curves[* id] -> cmax[0] = curves[* id] -> ndata;
  }
  else if (* id < numsk+numsq+numgr+numgq+numbd+numan+numri+numsh)
  {
    curves[* id] -> axmax[0] = curves[* id] -> ndata;
    curves[* id] -> axmin[0] = 1.0;
    curves[* id] -> cmin[0] = 1.0;
    curves[* id] -> cmax[0] = curves[* id] -> ndata;
  }
  else
  {
    curves[* id] -> axmax[0] = (steps-1)*delta_t*ndtbs;
    curves[* id] -> axmin[0] = delta_t;
    curves[* id] -> cmin[0] = delta_t;
    curves[* id] -> cmax[0] = (steps-1)*delta_t*ndtbs;
  }
}

void setup_xaxis_log (int * axa)
{
  int i, k, l;
  gboolean istrue;
  double v;

  istrue = TRUE;
  xlog = 1;
  if (xmin != 0.0)
  {
    k = (xmax+xmin)/xmin;
  }
  while (istrue)
  {
    k = k/10;
    if (k > 0)
    {
      xlog ++;
    }
    else
    {
      istrue = FALSE;
    }
  }
  istrue = TRUE;
  dxlog = 0;
  i = 100;
  while (istrue)
  {
    if (xmin >= pow(10, i))
    {
      istrue = FALSE;
      dxlog --;
    }
    else
    {
      dxlog ++;
      i=i-1;
    }
  }
  dxlog = dxlog - 100;
  for ( l = 0 ; l < xlog ; l = l + 1 )
  {
    cairo_move_to(cr, xmarge - 10 + l * XDRAW / xlog, YDRAW + 20);
    v = pow (10, l-dxlog);
    sca=g_strdup_printf("%.2f", v);
    cairo_show_text (cr, sca);
    g_free(sca);
    cairo_move_to(cr, xmarge + l * XDRAW / xlog, YDRAW);
    cairo_line_to(cr, xmarge + l * XDRAW / xlog, YDRAW - 10);
    if (curves[* axa] -> grid[0])
    {
      cairo_stroke(cr);
      cairo_set_line_width (cr, GRIDSIZE);
      cairo_set_dash (cr, pdashed, lenp, 0.0);
      cairo_move_to(cr, xmarge + l * XDRAW / xlog, YDRAW - 10);
      cairo_line_to(cr, xmarge + l * XDRAW / xlog, 0.0);
      cairo_stroke(cr);
      cairo_set_line_width (cr, 2.0);
      cairo_set_dash (cr, dashed1, len1, 0.0);
    }
    for ( k = 2 ; k < 10 ; k = k + 1 )
    {
      cairo_move_to(cr, xmarge + XDRAW * (l + log(k) / log(10.0)) / xlog, YDRAW);
      cairo_line_to(cr, xmarge + XDRAW * (l + log(k) / log(10.0)) / xlog, YDRAW - 5);
      if (curves[* axa] -> grid[0])
      {
        cairo_stroke(cr);
        cairo_set_line_width (cr, GRIDSIZE);
        cairo_set_dash (cr, pdashed, lenp, 0.0);
        cairo_move_to(cr, xmarge + XDRAW * (l + log(k) / log(10.0)) / xlog, YDRAW - 5);
        cairo_line_to(cr, xmarge + XDRAW * (l + log(k) / log(10.0)) / xlog, 0.0);
        cairo_stroke(cr);
        cairo_set_line_width (cr, 2.0);
        cairo_set_dash (cr, dashed1, len1, 0.0);
      }
    }
  }
  cairo_stroke (cr);
}

void setup_xaxis_linear (int * thexdata, gboolean * dogrid)
{
  int k;
  double u, v;
  double xmticks = curves[* thexdata] -> majt[0];
  int xnticks = curves[* thexdata] -> mint[0];

  k = xmin / xmticks;
  v = xmin - k * xmticks;
  if (k * xmticks < xmin) v = xmticks - v;
  v = fabs(v);
  DX = xmarge + XDRAW * v / xmax;
  for ( u = 0.0 ; u < xmax ; u = u + xmticks )
  {
    cairo_move_to(cr, DX - 10 + u * XDRAW / xmax, YDRAW + 20);
    sca=g_strdup_printf ("%.1f", xmin+v+u);
    cairo_show_text (cr, sca);
    g_free(sca);
    cairo_move_to(cr, DX + u * XDRAW / xmax, YDRAW);
    cairo_line_to(cr, DX + u * XDRAW / xmax, YDRAW - 10);
  }
  cairo_stroke (cr);
  k = xmin / xmticks;
  v = xmin - k * xmticks;
  if (k * xmticks < xmin) v = xmticks - v;
  v = fabs(v);
  for (u = v ; u > xmticks / xnticks ; u -= xmticks / xnticks);
  DX = xmarge + XDRAW * u / xmax;
  for ( u = 0.0 ; u < xmax ; u = u + xmticks / xnticks )
  {
    cairo_move_to(cr, DX + u * XDRAW / xmax, YDRAW);
    cairo_line_to(cr, DX + u * XDRAW / xmax, YDRAW - 5);
    if (* dogrid)
    {
      cairo_stroke(cr);
      cairo_set_dash (cr, pdashed, lenp, 0.0);
      cairo_set_line_width (cr, GRIDSIZE);
      cairo_move_to(cr, DX + u * XDRAW / xmax, YDRAW - 5);
      cairo_line_to(cr, DX + u * XDRAW / xmax, 0.0);
      cairo_stroke(cr);
      cairo_set_line_width (cr, 2.0);
      cairo_set_dash (cr, dashed1, len1, 0.0);
    }
  }
  cairo_stroke(cr);
}

void autoscale_yaxis (int * aya)
{
  int i, j, k, l, m;
  curves[* aya] -> axmax[1] = -100000.0;
  curves[* aya] -> axmin[1] = 100000.0;
  if (* aya >= numsk+numsq+numgr+numgq+numbd+numan+numri && * aya < numsk+numsq+numgr+numgq+numbd+numan+numri+numsh)
  {
    m=1;
  }
  else
  {
    m=0;
  }
  if (* aya >= numsq+numgr && * aya < numsk+numsq+numgr)
  {
    for ( i=m ; i < curves[* aya] -> ndata ; i++ )
    {
      if (idsk[i] == 1)
      {
        curves[* aya] -> axmax[1] = max(curves[* aya] -> axmax[1], curves[* aya] -> data[i]);
        curves[* aya] -> axmin[1] = min(curves[* aya] -> axmin[1], curves[* aya] -> data[i]);
      }
    }
  }
  else
  {
    for ( i=m ; i < curves[* aya] -> ndata ; i++ )
    {
      curves[* aya] -> axmax[1] = max(curves[* aya] -> axmax[1], curves[* aya] -> data[i]);
      curves[* aya] -> axmin[1] = min(curves[* aya] -> axmin[1], curves[* aya] -> data[i]);
    }
  }
  k = curves[* aya] -> cend - curves[* aya] -> cstart;
  if (curves[* aya] -> cstart == 0 || curves[* aya] -> cstart == numgr + numsq + numsk)
  {
    k = numgr+numgq;
  }
  else if (curves[* aya] -> cstart == numgr || curves[* aya] -> cstart == numgr + numsq)
  {
    k = numsq+numsk;
  }

  for ( j=0 ; j < k ; j++ )
  {
    if (curves[* aya] -> extrasets[j] != 0)
    {
      l = findcurve(aya, & j);
      if (l >= numsq+numgr && l < numsk+numsq+numgr)
      {
        for ( i=m ; i < curves[l] -> ndata ; i++ )
        {
          if (idsk[i] == 1)
          {
            curves[* aya] -> axmax[1] = max(curves[* aya] -> axmax[1], curves[l] -> data[i]);
            curves[* aya] -> axmin[1] = min(curves[* aya] -> axmin[1], curves[l] -> data[i]);
          }
        }
      }
      else
      {
        for ( i=m ; i < curves[l] -> ndata ; i++ )
        {
          curves[* aya] -> axmax[1] = max(curves[* aya] -> axmax[1], curves[l] -> data[i]);
          curves[* aya] -> axmin[1] = min(curves[* aya] -> axmin[1], curves[l] -> data[i]);
        }
      }
    }
  }
  curves[* aya] -> cmin[1] = curves[* aya] -> axmin[1];
  curves[* aya] -> cmax[1] = curves[* aya] -> axmax[1];
  curves[* aya] -> axmin[1] = curves[* aya] -> cmin[1] - fabs(curves[* aya] -> cmin[1]) / 10.0;
  curves[* aya] -> axmax[1] = curves[* aya] -> cmax[1] + fabs(curves[* aya] -> cmax[1]) / 10.0;
  if (* aya >= numsk+numsq+numgr+numgq && * aya < numsk+numsq+numgr+numgq+numbd+numan+numri+numsh)
  {
    curves[* aya] -> axmin[1] = 0.0;
  }
}

void setup_yaxis_linear (int * theydata)
{
  int k;
  double u, v;
  double ymticks = curves[* theydata] -> majt[1];
  int ynticks = curves[* theydata] -> mint[1];

  k = ymin / ymticks;
  v = ymin - k * ymticks;
  if (k * ymticks < ymin) v = ymticks - v;
  v = fabs(v);
  DY = YDRAW - YDRAW * v / ymax;
  for ( u = 0.0 ; u < ymax ; u = u + ymticks )
  {
    cairo_move_to(cr, 10, DY - u * YDRAW / ymax);
    sca=g_strdup_printf("%.2f", ymin+v+u);
    cairo_show_text (cr, sca);
    g_free(sca);
    cairo_move_to(cr, xmarge, DY - u * YDRAW / ymax);
    cairo_line_to(cr, xmarge + 10 , DY - u * YDRAW / ymax);
  }
  cairo_stroke (cr);
  k = ymin / ymticks;
  v = ymin - k * ymticks;
  if (k * ymticks < ymin) v = ymticks - v;
  v = fabs(v);
  for (u = v ; u > ymticks / ynticks ; u -= ymticks / ynticks);
  DY = YDRAW - YDRAW * u / ymax;
  for (  u = 0.0 ; u < ymax ; u = u + ymticks / ynticks )
  {
    cairo_move_to(cr, xmarge, DY - u * YDRAW / ymax);
    cairo_line_to(cr, xmarge + 5, DY - u * YDRAW / ymax);
    if (curves[* theydata] -> grid[1])
    {
      cairo_stroke(cr);
      cairo_set_line_width (cr, GRIDSIZE);
      cairo_set_dash (cr, pdashed, lenp, 0.0);
      cairo_move_to(cr, xmarge + 5, DY - u * YDRAW / ymax);
      cairo_line_to(cr, resol[0], DY - u * YDRAW / ymax);
      cairo_stroke(cr);
      cairo_set_line_width (cr, 2.0);
      cairo_set_dash (cr, dashed1, len1, 0.0);
    }
  }
  cairo_stroke (cr);
}

void setup_yaxis_log (int * aya)
{
  int i, k, l;
  gboolean istrue;
  double v;

  istrue = TRUE;
  ylog = 1;
  if (ymin != 0.0)
  {
    k = (ymax+ymin)/ymin;
  }
  while (istrue)
  {
    k = k/10;
    if (k > 0)
    {
      ylog ++;
    }
    else
    {
      ylog ++;
      istrue = FALSE;
    }
  }
  istrue = TRUE;
  dylog = 0;
  i = 100;
  while (istrue)
  {
    if (ymin >= pow(10, i))
    {
      istrue = FALSE;
    }
    else
    {
      dylog ++;
      i=i-1;
    }
  }
  dylog = dylog - 100;
  for ( l = 0 ; l < ylog ; l ++ )
  {
    cairo_move_to(cr, 10, YDRAW + 5 - YDRAW *l / ylog);
    v = pow (10, l-dylog);
    sca=g_strdup_printf("%.2f", v);
    cairo_show_text (cr, sca);
    g_free(sca);
    cairo_move_to(cr, xmarge, YDRAW - YDRAW * l / ylog);
    cairo_line_to(cr, xmarge+10, YDRAW - YDRAW * l / ylog);
    if (curves[* aya] -> grid[1])
    {
       cairo_stroke(cr);
       cairo_set_line_width (cr, GRIDSIZE);
       cairo_set_dash (cr, pdashed, lenp, 0.0);
       cairo_move_to(cr, xmarge + 10, YDRAW - YDRAW * l / ylog);
       cairo_line_to(cr, resol[0], YDRAW - YDRAW * l / ylog);
       cairo_stroke(cr);
       cairo_set_line_width (cr, 2.0);
       cairo_set_dash (cr, dashed1, len1, 0.0);
    }
    for ( k = 2 ; k < 10 ; k ++ )
    {
      cairo_move_to(cr, xmarge, YDRAW - YDRAW * (l + log(k) / log(10.0)) / ylog);
      cairo_line_to(cr, xmarge + 5, YDRAW - YDRAW *(l + log(k) / log(10.0)) / ylog);
      if (curves[* aya] -> grid[1])
      {
        cairo_stroke(cr);
        cairo_set_line_width (cr, GRIDSIZE);
        cairo_set_dash (cr, pdashed, lenp, 0.0);
        cairo_move_to(cr, xmarge + 5, YDRAW - YDRAW * (l + log(k) / log(10.0)) / ylog);
        cairo_line_to(cr, resol[0], YDRAW - YDRAW * (l + log(k) / log(10.0)) / ylog);
        cairo_stroke(cr);
        cairo_set_line_width (cr, 2.0);
        cairo_set_dash (cr, dashed1, len1, 0.0);
      }
    }
  }
  cairo_stroke (cr);
}

void draw_frame (int * axis)
{
  cairo_set_line_width (cr, 2.0);
  cairo_set_dash (cr, dashed1, len1, 0.0);
  cairo_move_to (cr, xmarge, 0.0);
  cairo_line_to (cr, xmarge, YDRAW);
  cairo_move_to (cr, xmarge, YDRAW);
  cairo_line_to (cr, resol[0], YDRAW);

// X axis

// Draw X axis ticks and labels
  if (curves[* axis] -> scale[0] == 0)
  {
    setup_xaxis_linear (axis, & curves[* axis] -> grid[0]);
  }
  else
  {
    setup_xaxis_log (axis);
  }

// Draw X axis label
  if (* axis >= numsk+numsq+numgr+numgq+numbd+numan && * axis < numsk+numsq+numgr+numgq+numbd+numan+numri)
  {
    cairo_move_to (cr, 10+resol[0]/4, resol[1]-15);
  }
  else
  {
    cairo_move_to (cr, 20+resol[0]/2, resol[1]-15);
  }
  cairo_show_text (cr, curves[* axis] -> xlabel);
  cairo_stroke(cr);

// Y axis

// Draw Y axis ticks and labels
  if (curves[* axis] -> scale[1] == 0)
  {
    setup_yaxis_linear (axis);
  }
  else
  {
    setup_yaxis_log (axis);
  }
}

void show_curve (GtkWidget * grwdget, gpointer curve)
{
  int what, where;
  int j, k, l, m;
  double alp;
  double xmx[2];

  what = GPOINTER_TO_INT(curve);
  where = curves[what] -> format;
  curves[what] -> format = 0;

  if (where == 0)
  {
    alp=1.0;
    cr = gdk_cairo_create (grwdget -> window);
    gtk_window_get_size (GTK_WINDOW(curves[what] -> window), &resol[0], &resol[1]);
    resol[1] = resol[1] + THESHIFT;
    for (j=0; j<2; j++)
    {
      if (xyp[j] != NULL) update_entry_int (GTK_ENTRY(xyp[j]), & resol[j]);
    }
  }
  else if (where == 1)
  {
    alp=alpha;
    surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, resol[0], resol[1]);
    cr = cairo_create(surface);
  }
  else if (where == 2)
  {
    alp=1.0;
    surface = cairo_pdf_surface_create(IMG_FILE, resol[0], resol[1]);
    cr = cairo_create(surface);
  }
  else if (where == 3)
  {
    alp=alpha;
    surface = cairo_svg_surface_create(IMG_FILE, resol[0], resol[1]);
    cr = cairo_create(surface);
  }
// Painting the background
  cairo_set_source_rgba (cr, curves[what] -> backcolor.red/65535.0,
                             curves[what] -> backcolor.green/65535.0,
                             curves[what] -> backcolor.blue/65535.0, alp);
  cairo_paint (cr);

// Axis color and line
  cairo_set_source_rgb (cr, curves[what] -> framcolor.red/65535.0,
                            curves[what] -> framcolor.green/65535.0,
                            curves[what] -> framcolor.blue/65535.0);
  cairo_set_line_width (cr, 2.0);
  cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_NORMAL,CAIRO_FONT_WEIGHT_NORMAL);

// Autoscale and autoticking for X axis
  if (curves[what] -> autoscale[0])
  {
    if (curves[what] -> cstart == numgr || curves[what] -> cstart == numgr + numsq)
    {
      xmx[0] = curves[what] -> axmin[0];
      xmx[1] = curves[what] -> axmax[0];
      k = numsq+numsk;
      for ( j=0 ; j < k ; j++ )
      {
        if (curves[what] -> extrasets[j] != 0)
        {
          l = findcurve (&what, &j);
          autoscale_xaxis (& l);
          xmx[0] = min(xmx[0],curves[l] -> axmin[0]);
          xmx[1] = max(xmx[1],curves[l] -> axmax[0]);
        }
      }
      autoscale_xaxis (& what);
      curves[what] -> axmin[0]=min(xmx[0], curves[what] -> axmin[0]);
      curves[what] -> axmax[0]=max(xmx[1], curves[what] -> axmax[0]);
    }
    else
    {
      autoscale_xaxis (& what);
    }
    curves[what] -> autoscale[0] = FALSE;
    curves[what] -> init[0] = TRUE;
    if (vmin[0] != NULL) update_entry_double (GTK_ENTRY(vmin[0]), & curves[what] -> axmin[0]);
    if (vmax[0] != NULL) update_entry_double (GTK_ENTRY(vmax[0]), & curves[what] -> axmax[0]);
  }
  xmin = curves[what] -> axmin[0];
  xmax = curves[what] -> axmax[0] - xmin;

// Check if autoscale on Y
  if (curves[what] -> autoscale[1])
  {
    autoscale_yaxis (& what);
    curves[what] -> autoscale[1] = FALSE;
    curves[what] -> init[1] = TRUE;
    if (vmin[1] != NULL) update_entry_double (GTK_ENTRY(vmin[1]), & curves[what] -> axmin[1]);
    if (vmax[1] != NULL) update_entry_double (GTK_ENTRY(vmax[1]), & curves[what] -> axmax[1]);
  }
  ymin = curves[what] -> axmin[1];
  ymax = curves[what] -> axmax[1] - ymin;

// The x size of the graph in pixels
  XDRAW = resol[0] - xmarge;
// The y size of the graph in pixels
  YDRAW = resol[1] - ymarge;

// Check if autoticking on X
  if (curves[what] -> init[0])
  {
    curves[what] -> majt[0] = scale (& xmax);
    curves[what] -> mint[0] = 2;
    curves[what] -> init[0] = FALSE;
    if (mjt[0] != NULL) update_entry_double (GTK_ENTRY(mjt[0]), & curves[what] -> majt[0]);
    j = curves[what] -> mint[0] - 1;
    if (nmi[0] != NULL) update_entry_int (GTK_ENTRY(nmi[0]), & j);
  }
// Check for the X axis font size
  if (abs(xmax) > xlim)
  {
    cairo_set_font_size (cr, 15.0*(resol[0]-2*xmarge)/resol[0]);
  }
  else
  {
    cairo_set_font_size (cr, 20.0*(resol[0]-2*xmarge)/resol[0]);
  }
// Check if autoticking on Y
  if (curves[what] -> init[1])
  {
    curves[what] -> majt[1] = scale (& ymax);
    curves[what] -> mint[1] = 2;
    curves[what] -> init[1] = FALSE;
    if (mjt[1] != NULL) update_entry_double (GTK_ENTRY(mjt[1]), & curves[what] -> majt[1]);
    j = curves[what] -> mint[1] - 1;
    if (nmi[1] != NULL) update_entry_int (GTK_ENTRY(nmi[1]), & j);
  }

// Check for the Y axis font size !!!!!!! abs (x) or abs(y)
  if (abs(curves[what] -> axmax[1]) > ylim)
  {
    cairo_set_font_size (cr, 15.0*(resol[1]-2.6*ymarge)/resol[1]);
  }
  else
  {
    cairo_set_font_size (cr, 20.0*(resol[1]-2.6*ymarge)/resol[1]);
  }

// Draw the axis bars - in option

  if (curves[what] -> ashow[1])
  {
    if (xmarge - XDRAW * xmin / xmax >= xmarge && xmarge - XDRAW * xmin / xmax <= resol[0])
    {
      cairo_set_line_width (cr, 1.0);
      cairo_set_dash (cr, pdashed, lenp, 0.0);
      cairo_move_to (cr, xmarge - XDRAW * xmin / xmax, 0.0);
      cairo_line_to (cr, xmarge - XDRAW * xmin / xmax, YDRAW);
      cairo_stroke (cr);
    }
  }
  if (curves[what] -> ashow[0])
  {
    if (YDRAW + YDRAW * ymin / ymax <= YDRAW && YDRAW + YDRAW * ymin / ymax >= ymarge)
    {
      cairo_set_line_width (cr, 1.0);
      cairo_set_dash (cr, pdashed, lenp, 0.0);
      cairo_move_to (cr, xmarge, YDRAW + YDRAW * ymin / ymax);
      cairo_line_to (cr, resol[0], YDRAW + YDRAW * ymin / ymax);
      cairo_stroke (cr);
    }
  }

  if (curves[what] -> frame) draw_frame (& what);

// Coordinate shift for the X axis
  if (curves[what] -> scale[0] == 0)
  {
    DXDRAW = XDRAW * (curves[what] -> cmax[0] - curves[what] -> cmin[0]) / xmax;
    DX = xmarge -  XDRAW * (xmin - curves[what] -> cmin[0]) / xmax;
  }
  else
  {
    DXDRAW = XDRAW * log(curves[what] -> cmax[0] - curves[what] -> cmin[0]) / log(xmax);
    DX = xmarge -  XDRAW * (log(xmin) - log(curves[what] -> cmin[0])) / log(xmax);
  }
  if (curves[what] -> scale[1] == 0)
  {
    DYDRAW = YDRAW * (curves[what] -> cmax[1] - curves[what] -> cmin[1]) / ymax;
  }
/*  else
  {
    DYDRAW = YDRAW * (log(curves[what] -> cmax[1]) - log(curves[what] -> cmin[1])) / log(ymax);
  } */
  DY = YDRAW;

// let's go for the plot
// first we start by the possible extra curves
  k = curves[what] -> cend - curves[what] -> cstart;
  if (curves[what] -> cstart == 0 || curves[what] -> cstart == numgr + numsq + numsk)
  {
    k = numgr+numgq;
  }
  else if (curves[what] -> cstart == numgr || curves[what] -> cstart == numgr + numsq)
  {
    k = numsq+numsk;
  }
  m = - curves[what] -> extra;
  for ( j=0 ; j < k ; j++ )
  {
    if (curves[what] -> extrasets[j] != 0)
    {
      l = findcurve (&what, &j);
      if (curves[what] -> cstart == numgr || curves[what] -> cstart == numgr+numsq)
      {
        DXDRAW = XDRAW * (curves[l] -> cmax[0] - curves[what] -> cmin[0]) / xmax;
        DX = xmarge -  XDRAW * (xmin - curves[l] -> cmin[0]) / xmax;
      }
      draw_curve (& l,
                  & curves[l] -> ndata,
                  & ymin,
                  & curves[what] -> extracolor[j],
                  & curves[what] -> scale[0],
                  & curves[what] -> scale[1],
                  & curves[what] -> extraspect[j],
                  & curves[what] -> extradash[j],
                  & curves[what] -> extrathick[j],
                  & curves[what] -> extraglyph[j],
                  & curves[what] -> extragsize[j],
                  & curves[what] -> extragfreq[j],
                  & curves[what] -> extra,
                  & m);
      m = m + 1;
    }
  }

// Coordinate shift for the X axis
  if (curves[what] -> scale[0] == 0)
  {
    DXDRAW = XDRAW * (curves[what] -> cmax[0] - curves[what] -> cmin[0]) / xmax;
    DX = xmarge -  XDRAW * (xmin - curves[what] -> cmin[0]) / xmax;
  }
  else
  {
    DXDRAW = XDRAW * log(curves[what] -> cmax[0] - curves[what] -> cmin[0]) / log(xmax);
    DX = xmarge -  XDRAW * (log(xmin) - log(curves[what] -> cmin[0])) / log(xmax);
  }

// and then the main curve so that it will be in front
// and centered if bar diagram has been chosen

  draw_curve (& what,
              & curves[what] -> ndata,
              & ymin,
              & curves[what] -> datacolor,
              & curves[what] -> scale[0],
              & curves[what] -> scale[1],
              & curves[what] -> aspect,
              & curves[what] -> dash,
              & curves[what] -> thickness,
              & curves[what] -> glyph,
              & curves[what] -> gsize,
              & curves[what] -> gfreq,
              & curves[what] -> extra,
              & m);

// and maybe for the legend
  if (curves[what] -> legend) show_legend (& what, curves[what] -> legpos);

// and we plot the complete curve
  if (where == 1)
  {
     cairo_surface_write_to_png(surface, IMG_FILE);
     cairo_surface_destroy(surface);
  }
  else if (where == 2)
  {
    cairo_show_page(cr);
    cairo_surface_destroy(surface);
  }
  else if (where == 3)
  {
    cairo_surface_destroy(surface);
  }
  cairo_destroy(cr);
}

G_MODULE_EXPORT void curve_hide (GtkWidget * thecurve, GdkEvent *event, gpointer data)
{
  int id = GPOINTER_TO_INT(data);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(curves[id] -> button), 0);
}

G_MODULE_EXPORT void hide_curve (GtkWidget * but, gpointer idc)
{
  gtk_widget_hide(curves[GPOINTER_TO_INT(idc)] -> window);
}

G_MODULE_EXPORT void to_show (GtkWidget * curve, GdkEventExpose *event, gpointer data)
{
  show_curve (curve, data);
}

GtkWidget * create_Curve (char * namecurve, int * idcurve)
{
  GtkWidget * Curve;
  GtkWidget * Cvbox;
  GtkWidget * menu;
  GtkWidget * menudata, * menucurve;
  GtkWidget * menud, * menuc;
  GtkWidget * savecurveas;
  GtkWidget * savecurve;
  GtkWidget * editcurve;
  GtkWidget * separatormenu;
  GtkWidget * closecurve;
  GtkAccelGroup * accel_group;
  int i, j;

  accel_group = gtk_accel_group_new ();
  Curve = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW(Curve), namecurve);
  gtk_window_set_default_size (GTK_WINDOW(Curve), 840, 677);
  gtk_window_set_resizable (GTK_WINDOW (Curve), TRUE);
  gtk_window_set_icon (GTK_WINDOW (Curve), THEMOL);
  Cvbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (Curve), Cvbox);
  menu = gtk_menu_bar_new ();
  gtk_box_pack_start (GTK_BOX (Cvbox), menu, FALSE, TRUE, 0);
  menudata = gtk_menu_item_new_with_mnemonic (("_Data"));
  gtk_container_add (GTK_CONTAINER (menu), menudata);
  menud = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menudata), menud);
  savecurveas = gtk_image_menu_item_new_from_stock ("gtk-save-as", accel_group);
  gtk_container_add (GTK_CONTAINER (menud), savecurveas);
  separatormenu = gtk_separator_menu_item_new ();
  gtk_container_add (GTK_CONTAINER (menud), separatormenu);
  gtk_widget_set_sensitive (separatormenu, FALSE);
  closecurve = gtk_image_menu_item_new_from_stock ("gtk-close", accel_group);
  gtk_container_add (GTK_CONTAINER (menud), closecurve);
  menucurve = gtk_menu_item_new_with_mnemonic (("_Curve"));
  gtk_container_add (GTK_CONTAINER (menu), menucurve);
  menuc = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menucurve), menuc);
  editcurve = gtk_image_menu_item_new_with_mnemonic (("E_dit curve"));
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM(editcurve), gtk_image_new_from_file(PACKAGE_EDIT));
  gtk_container_add (GTK_CONTAINER (menuc), editcurve);
  savecurve = gtk_image_menu_item_new_with_mnemonic (("_Export image"));
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM(savecurve), gtk_image_new_from_file(PACKAGE_PNG));
  gtk_container_add (GTK_CONTAINER (menuc), savecurve);

  curves[* idcurve] -> plot = gtk_drawing_area_new();
//  gtk_widget_set_size_request (curves[* idcurve] -> plot, 840, 640);
  gtk_box_pack_start (GTK_BOX (Cvbox), curves[* idcurve] -> plot, TRUE, TRUE, 0);

  curves[* idcurve] -> format = 0;
  curves[* idcurve] -> autoscale[0] = TRUE;
  curves[* idcurve] -> autoscale[1] = TRUE;
  curves[* idcurve] -> init[0] = TRUE;
  curves[* idcurve] -> init[1] = TRUE;
  curves[* idcurve] -> grid[0] = FALSE;
  curves[* idcurve] -> grid[1] = FALSE;
  curves[* idcurve] -> ashow[0] = FALSE;
  curves[* idcurve] -> ashow[1] = FALSE;
  curves[* idcurve] -> legend = FALSE;
  curves[* idcurve] -> frame = TRUE;
  curves[* idcurve] -> legpos[0] = LEGX;
  curves[* idcurve] -> legpos[1] = LEGY;
  curves[* idcurve] -> xdef = TRUE;
  curves[* idcurve] -> backcolor.red =  65535;
  curves[* idcurve] -> backcolor.green = 65535;
  curves[* idcurve] -> backcolor.blue = 65535;
  curves[* idcurve] -> datacolor.red = RED;
  curves[* idcurve] -> datacolor.green = GREEN;
  curves[* idcurve] -> datacolor.blue = BLUE;
  curves[* idcurve] -> framcolor.red = 0;
  curves[* idcurve] -> framcolor.green = 0;
  curves[* idcurve] -> framcolor.blue = 0;
  curves[* idcurve] -> thickness = DTHICK;
  curves[* idcurve] -> dash = 1;
  curves[* idcurve] -> gfreq = 1;
  curves[* idcurve] -> extra = 0;
  if (curves[* idcurve] -> cfile != NULL) free (curves[* idcurve] -> cfile);
  i = curves[* idcurve] -> cend - curves[* idcurve] -> cstart;
  if (curves[* idcurve] -> cstart == 0 || curves[* idcurve] -> cstart == numgr + numsq + numsk)
  {
    i = numgr+numgq;
  }
  else if (curves[* idcurve] -> cstart == numgr || curves[* idcurve] -> cstart == numgr + numsq)
  {
    i = numsq+numsk;
  }
  curves[* idcurve] -> extrasets = calloc(i, sizeof*curves[* idcurve] -> extrasets);
  curves[* idcurve] -> extracolor = calloc(i, sizeof*curves[* idcurve] -> extracolor);
  curves[* idcurve] -> extrathick = calloc(i, sizeof*curves[* idcurve] -> extrathick);
  curves[* idcurve] -> extradash = calloc(i, sizeof*curves[* idcurve] -> extradash);
  curves[* idcurve] -> extraglyph = calloc(i, sizeof*curves[* idcurve] -> extraglyph);
  curves[* idcurve] -> extragsize = calloc(i, sizeof*curves[* idcurve] -> extragsize);
  curves[* idcurve] -> extragfreq = calloc(i, sizeof*curves[* idcurve] -> extragfreq);
  curves[* idcurve] -> extraspect = calloc(i, sizeof*curves[* idcurve] -> extraspect);
  for ( j=0 ; j < i ; j++ )
  {
    curves[* idcurve] -> extracolor[j].red = 0;
    curves[* idcurve] -> extracolor[j].green = 0;
    curves[* idcurve] -> extracolor[j].blue = 0;
    curves[* idcurve] -> extrathick[j] = DTHICK;
    curves[* idcurve] -> extradash[j] = 1;
    curves[* idcurve] -> extraglyph[j] = 0;
    curves[* idcurve] -> extragsize[j] = 10;
    curves[* idcurve] -> extragfreq[j] = 1;
    curves[* idcurve] -> extraspect[j] = curves[* idcurve] -> aspect;
  }
  g_signal_connect ((gpointer)curves[* idcurve] -> plot, "expose_event", G_CALLBACK(to_show), GINT_TO_POINTER(* idcurve));
  g_signal_connect ((gpointer)Curve, "delete_event", G_CALLBACK(curve_hide), GINT_TO_POINTER(* idcurve));
  g_signal_connect ((gpointer)savecurveas, "activate", G_CALLBACK(write_curve), GINT_TO_POINTER(* idcurve));
  g_signal_connect ((gpointer)savecurve, "activate", G_CALLBACK(save_image), GINT_TO_POINTER(* idcurve));
  g_signal_connect ((gpointer)editcurve, "activate", G_CALLBACK(edit_image), GINT_TO_POINTER(* idcurve));
  g_signal_connect ((gpointer)closecurve, "activate", G_CALLBACK(hide_curve), GINT_TO_POINTER(* idcurve));
  gtk_window_add_accel_group (GTK_WINDOW (Curve), accel_group);

  return Curve;
}

G_MODULE_EXPORT void on_viewcurve_toggled (GtkToggleButton * Button, gpointer data)
{
  int j;
  gboolean status;

  status = gtk_toggle_button_get_active (Button);
  j = GPOINTER_TO_INT (data);
  if (status)
  {
    if (curves[j] -> window == NULL)
    {
      curves[j] -> window = create_Curve(curves[j] -> title, &j);
    }
    gtk_widget_show_all(curves[j] -> window);
  }
  else
  {
    if (curves[j] -> window != NULL)
    {
      gtk_widget_hide(curves[j] -> window);
    }
  }
}

void addcurwidget (char * wname, int * wid, int * pid, GtkTable * tablegtk)
{
  int id = * wid;
  int pd = * pid;

  curves[id] -> button = gtk_check_button_new_with_label (wname);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(curves[id] -> button), FALSE);
  gtk_widget_set_name (curves[id] -> button, wname);
  gtk_table_attach_defaults (tablegtk, curves[id] -> button, 0, 1, id-pd, id-pd+1);
  gtk_widget_set_sensitive(curves[id] -> button, 0);
  g_signal_connect (G_OBJECT(curves[id] -> button), "toggled", G_CALLBACK(on_viewcurve_toggled), GINT_TO_POINTER(id));
}

void curve_default_layout (int * cid)
{
  int i;

  if (* cid < numgr+numgq+numsk+numsq+numbd+numan)
  {
    curves[* cid] -> aspect = 0;
    curves[* cid] -> scale[0] = 0;
    curves[* cid] -> scale[1] = 0;
    curves[* cid] -> glyph = 0;
    curves[* cid] -> gsize = 10;
  }
  else if (* cid < numgr+numgq+numsk+numsq+numbd+numan+numri)
  {
    i = * cid - (numgr+numgq+numsk+numsq+numbd+numan);
    if ( i%4 == 0 || i%4 == 1 )
    {
      curves[* cid] -> aspect = 1;
      curves[* cid] -> scale[0] = 0;
      curves[* cid] -> scale[1] = 0;
      curves[* cid] -> glyph = 0;
      curves[* cid] -> gsize = 10;
    }
    else
    {
      curves[* cid] -> aspect = 0;
      curves[* cid] -> scale[0] = 0;
      curves[* cid] -> scale[1] = 0;
      curves[* cid] -> glyph = 13;
      curves[* cid] -> gsize = 5.0;
    }
  }
  else if (* cid < numgr+numgq+numsk+numsq+numbd+numan+numri+numsh)
  {
    curves[* cid] -> aspect = 1;
    curves[* cid] -> scale[0] = 0;
    curves[* cid] -> scale[1] = 0;
    curves[* cid] -> glyph = 0;
    curves[* cid] -> gsize = 10;
  }
  else if (* cid < numgr+numgq+numsk+numsq+numbd+numan+numri+numsh+nummsd-6)
  {
    curves[* cid] -> aspect = 0;
    curves[* cid] -> scale[0] = 1;
    curves[* cid] -> scale[1] = 1;
    curves[* cid] -> glyph = 0;
    curves[* cid] -> gsize = 10;
  }
  else
  {
    curves[* cid] -> aspect = 0;
    curves[* cid] -> scale[0] = 0;
    curves[* cid] -> scale[1] = 0;
    curves[* cid] -> glyph = 0;
    curves[* cid] -> gsize = 10;
  }
}

void save_curve_ (int * interv, double datacurve[* interv], int * id)
{
  int i;

  if (curves[* id] -> window != NULL)
  {
    gtk_widget_destroy(curves[* id] -> plot);
    curves[* id] -> plot = NULL;
    gtk_widget_destroy(curves[* id] -> window);
    curves[* id] -> window = NULL;
  }
  curve_default_layout (id);
  if (* interv != 0)
  {
    curves[* id] -> ndata = * interv;
    curves[* id] -> data = calloc(curves[* id] -> ndata, sizeof*curves[* id] -> data);
    if (*id >= numgr+numsq && *id < numgr+numsq+numsk)
    {
      if (* id == numgr+numsq)
      {
        for ( i=0 ; i < *interv ; i++ )
        {
          if (datacurve[i] != 0.0 || i==0)
          {
            idsk[i]=1;
            curves[* id] -> data[i]= datacurve[i];
          }
          else
          {
            idsk[i]=0;
            curves[* id] -> data[i]= -999999.0;
          }
        }
      }
      else
      {
        for ( i=0 ; i < *interv ; i++ )
        {
          curves[* id] -> data[i]= datacurve[i];
          if (idsk[i] == 0) curves[* id] -> data[i]= -999999.0;
        }
      }
    }
    else
    {
      for ( i=0 ; i < *interv ; i++ )
      {
        curves[* id] -> data[i]= datacurve[i];
      }
    }
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(curves[* id] -> button), FALSE);
    gtk_widget_set_sensitive(curves[* id] -> button, 1);
  }
  else
  {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(curves[* id] -> button), FALSE);
    gtk_widget_set_sensitive(curves[* id] -> button, 0);
  }
}

G_MODULE_EXPORT void write_curve (GtkWidget * butcurve, gpointer idata)
{
  int i, j, k, l, m, n, o;
  GtkWidget * info;
  GtkFileFilter *tmp, *filter1, *filter2;
  double delta, min;

  j = GPOINTER_TO_INT(idata);
  info = gtk_file_chooser_dialog_new ("Save data to disk",
     				      NULL,
     				      GTK_FILE_CHOOSER_ACTION_SAVE,
     				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
     				      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
     				      NULL);
  gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (info), TRUE);

  filter1 = gtk_file_filter_new();
  gtk_file_filter_set_name(GTK_FILE_FILTER(filter1), "Xmgrace file (*.agr)");
  gtk_file_filter_add_pattern(GTK_FILE_FILTER(filter1), "*.agr");
  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(info), filter1);
  filter2 = gtk_file_filter_new();
  gtk_file_filter_set_name(GTK_FILE_FILTER(filter2), "x/y ASCII file (*.dat)");
  gtk_file_filter_add_pattern(GTK_FILE_FILTER(filter2), "*.dat");
  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(info), filter2);
  if (curves[j] -> cfile != NULL)
  {
    gtk_file_chooser_set_filename (GTK_FILE_CHOOSER(info), curves[j] -> cfile);
  }
  else
  {
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(info), "");
    gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER(info), curves[j] -> title);
  }
  if (gtk_dialog_run (GTK_DIALOG (info)) == GTK_RESPONSE_ACCEPT)
  {
    curves[j] -> cfile = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(info));
    tmp = gtk_file_chooser_get_filter (GTK_FILE_CHOOSER(info));
    if (tmp == filter1)
    {
      l = 0;
    }
    else if (tmp == filter2)
    {
      l = 1;
    }
    k = StringLength(curves[j] -> cfile);
    if (j < numgr)
    {
      delta = delta_r;
      min = 0.0;
    }
    else if (j < numgr+numsq)
    {
      delta = (qmax-qmin)/num_delta_q;
      min = qmin;
    }
    else if (j < numgr+numsq+numsk)
    {
      delta = (kmax-kmin)/num_delta_k;
      min = kmin;
    }
    else if (j < numgr+numgq+numsq+numsk)
    {
      delta = delta_rq;
      min = 0.0;
    }
    else if (j < numgr+numgq+numsq+numsk+numbd)
    {
      delta = delta_d;
      min = bdmin;
    }
    else if (j < numgr+numgq+numsq+numsk+numbd+numan)
    {
      delta = 180.0/num_delta_a;
      min = 0.0;
    }
    else if (j < numgr+numgq+numsq+numsk+numbd+numan+numri)
    {
      delta = 1.0;
      min = 0.0;
    }
    else if (j < numgr+numgq + numsq + numsk + numbd + numan + numri + numsh)
    {
      delta = 1.0;
      min = 0.0;
    }
    else
    {
      delta = delta_t*ndtbs;
      min = tunit;
    }
    if (curves[j] -> extra == 0)
    {
      save_to_file_ (& k, curves[j] -> cfile,
                     & curves[j] -> ndata,
                     curves[j] -> data,
                     & curves[j] -> scale[0],
                     & curves[j] -> scale[1],
                     & curves[j] -> aspect,
                     & delta, & min, & j, & l);

    }
    else
    {
      o = curves[j] -> extra + 1;
      m = 0;
      prep_file_ (& k, curves[j] -> cfile, & l,
                  & curves[j] -> scale[0],
                  & curves[j] -> scale[1], & min, & j);
      append_to_file_ (& curves[j] -> ndata,
                       curves[j] -> data,
                       & curves[j] -> aspect,
                       & delta, & min, & j, & l, & m, & o);
      for (i=0 ; i < curves[j] -> cend - curves[j] -> cstart ; i++ )
      {
        if ( curves[j] -> extrasets[i] != 0 )
        {
          n = curves[j] -> cstart + i;
          m = m + 1;
          append_to_file_ (& curves[j] -> ndata,
                           curves[n] -> data,
                           & curves[j] -> extraspect[i],
                           & delta, & min, & n, & l, & m, & o);
        }
      }
    }
  }
  gtk_widget_destroy (info);
}

void hide_curves (int * wstart, int * wend)
{
  int i;

  for ( i = * wstart ; i < * wend ; i ++ )
  {
    if (curves[i] -> button != NULL)
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(curves[i] -> button), FALSE);
      gtk_widget_set_sensitive(curves[i] -> button, 0);
    }
    if (curves[i] -> window != NULL)
    {
      if (GTK_WIDGET_VISIBLE(curves[i] -> window)) gtk_widget_hide(curves[i] -> window);
    }
  }
}

G_MODULE_EXPORT void destroy_img_dialog (GtkWidget * button)
{
  gdk_window_destroy (button -> window);
}

G_MODULE_EXPORT void set_size (GtkEntry * val, gpointer data)
{
  const gchar *m;
  int shift[2];
  char * text[2];
  int i;
  axdata * adata = (axdata *)data;

  shift[0] = 0;
  shift[1] = THESHIFT;
  text[0] = "X size must be > 0";
  text[1] = "Y size must be > 0";
  m = gtk_entry_get_text (val);
  i = atof(m);
  if (i != 0)
  {
    resol[GPOINTER_TO_INT(data)] = i - shift[GPOINTER_TO_INT(data)];
  }
  else
  {
    show_warning (text[adata -> b]);
  }
  i = resol[GPOINTER_TO_INT(data)] + shift[GPOINTER_TO_INT(data)];
  update_entry_int (val, & i);
}

G_MODULE_EXPORT gboolean to_set_size (GtkWidget * val, GdkEventFocus * event, gpointer data)
{
  set_size (GTK_ENTRY(val), data);
  return (FALSE);
}

G_MODULE_EXPORT void set_background (GtkToggleButton * backb)
{
  gboolean stat;

  stat = gtk_toggle_button_get_active (backb);
  if (stat)
  {
    alpha = 0.0;
  }
  else
  {
    alpha = 1.0;
  }
}

G_MODULE_EXPORT void choose_format (GtkComboBox * box, gpointer cid)
{
  forme = gtk_combo_box_get_active (box);
  if (forme != -1)
  {
    toimg * imf  = (toimg *) cid;
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(imf -> a), 0);
    if (forme == 1)
    {
      gtk_widget_set_sensitive(imf -> a, 0);
      gtk_image_set_from_file (GTK_IMAGE(imf -> b), PACKAGE_PDF);
    }
    else
    {
      gtk_widget_set_sensitive(imf -> a, 1);
      if (forme == 0)
      {
        gtk_image_set_from_file (GTK_IMAGE(imf -> b), PACKAGE_PNG);
      }
      else
      {
        gtk_image_set_from_file (GTK_IMAGE(imf -> b), PACKAGE_SVG);
      }
    }
  }
}

G_MODULE_EXPORT void write_image (GtkWidget * but, gpointer curetow)
{
  GtkWidget * info;
  GtkFileFilter *filter;
  gchar * title;

  if (forme == -1)
  {
    show_warning ("To save an image please enter a file format");
  }
  else if (resol[0] == 0 || resol[1] == 0)
  {
    show_warning ("You need to specify the size of the image");
  }
  else
  {
    filter = gtk_file_filter_new();
    if (forme == 0)
    {
      gtk_file_filter_set_name(GTK_FILE_FILTER(filter), "PNG file (*.png)");
      gtk_file_filter_add_pattern(GTK_FILE_FILTER(filter), "*.png");
      title = "Save Portable Network Graphics";
    }
    else if (forme == 1)
    {
      gtk_file_filter_set_name(GTK_FILE_FILTER(filter), "PDF file (*.pdf)");
      gtk_file_filter_add_pattern(GTK_FILE_FILTER(filter), "*.pdf");
      title = "Save Portable Document File";
    }
    else
    {
      gtk_file_filter_set_name(GTK_FILE_FILTER(filter), "SVG file (*.svg)");
      gtk_file_filter_add_pattern(GTK_FILE_FILTER(filter), "*.svg");
      title = "Save Scalable Vector Graphics";
    }
    info = gtk_file_chooser_dialog_new (title,
                          NULL,
     				      GTK_FILE_CHOOSER_ACTION_SAVE,
     				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
     				      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
     				      NULL);
    gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (info), TRUE);
    gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(info), filter);

    if (gtk_dialog_run (GTK_DIALOG (info)) == GTK_RESPONSE_ACCEPT)
    {
      IMG_FILE = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(info));
      curves[GPOINTER_TO_INT(curetow)] -> format = forme + 1;
      show_curve (NULL, curetow);
      destroy_img_dialog (but);
    }
    gtk_widget_destroy(info);
  }
}

G_MODULE_EXPORT void save_image (GtkWidget * curve, gpointer cdata)
{
  GtkWidget *save_image;
  GtkWidget *dialog_savimg;
  GtkWidget *hbox2;
  GtkWidget *form;
  GtkWidget *comboformat;
  GtkWidget *hbox1;
  GtkWidget *size;
  GtkWidget *x;
  GtkWidget *times;
  GtkWidget *y;
  GtkWidget *pixels;
  GtkWidget *background;
  GtkWidget *dialog_action_img;
  GtkWidget *saving;
  GtkWidget *cancel;
  GtkWidget *log;
  int xlgt, ylgt;

  save_image = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (save_image), 5);
  gtk_window_set_type_hint (GTK_WINDOW (save_image), GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_window_set_resizable (GTK_WINDOW (save_image), FALSE);
  gtk_dialog_set_has_separator (GTK_DIALOG (save_image), FALSE);
  gtk_window_set_title (GTK_WINDOW(save_image), "Export image");

  dialog_savimg = GTK_DIALOG (save_image)->vbox;
  gtk_box_set_spacing (GTK_BOX(dialog_savimg), 15);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (dialog_savimg), hbox2, TRUE, TRUE, 0);

  form = gtk_label_new ("Format");
  gtk_box_pack_start (GTK_BOX (hbox2), form, TRUE, TRUE, 0);

  comboformat = gtk_combo_box_new_text ();
  gtk_combo_box_append_text (GTK_COMBO_BOX (comboformat), "PNG");
  gtk_combo_box_append_text (GTK_COMBO_BOX (comboformat), "PDF");
  gtk_combo_box_append_text (GTK_COMBO_BOX (comboformat), "SVG");
  gtk_box_pack_start (GTK_BOX (hbox2), comboformat, TRUE, TRUE, 0);
  gtk_widget_set_size_request (comboformat, -1, 30);

  log = gtk_image_new_from_file (PACKAGE_VOID);
  gtk_box_pack_start (GTK_BOX (hbox2), log, TRUE, TRUE, 0);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (dialog_savimg), hbox1, TRUE, TRUE, 0);

  size = gtk_label_new ("Size: ");
  gtk_box_pack_start (GTK_BOX (hbox1), size, TRUE, TRUE, 0);

  gtk_window_get_size (GTK_WINDOW(curves[GPOINTER_TO_INT(cdata)] -> window), &xlgt, &ylgt);
  x = gtk_entry_new ();
  gtk_entry_set_alignment (GTK_ENTRY(x), 1.0);
  gtk_box_pack_start (GTK_BOX (hbox1), x, TRUE, TRUE, 0);
  gtk_widget_set_size_request (x, 50, -1);
  update_entry_int (GTK_ENTRY(x), & xlgt);

  times = gtk_label_new ("x");
  gtk_box_pack_start (GTK_BOX (hbox1), times, TRUE, TRUE, 0);

  y = gtk_entry_new ();
  gtk_entry_set_alignment (GTK_ENTRY(y), 1.0);
  gtk_box_pack_start (GTK_BOX (hbox1), y, TRUE, TRUE, 0);
  gtk_widget_set_size_request (y, 50, -1);
  ylgt = ylgt + THESHIFT;
  update_entry_int (GTK_ENTRY(y), & ylgt);

  pixels = gtk_label_new ("pixels");
  gtk_box_pack_start (GTK_BOX (hbox1), pixels, TRUE, TRUE, 0);

  background = gtk_check_button_new_with_label ("Make background transparent");
  gtk_box_pack_start (GTK_BOX (dialog_savimg), background, TRUE, TRUE, 0);
  gtk_widget_set_sensitive (background, FALSE);

  dialog_action_img = GTK_DIALOG (save_image)->action_area;
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_img), GTK_BUTTONBOX_END);

  saving = gtk_button_new_from_stock ("gtk-save");
  gtk_dialog_add_action_widget (GTK_DIALOG (save_image), saving, 0);

  cancel = gtk_button_new_from_stock ("gtk-cancel");
  gtk_dialog_add_action_widget (GTK_DIALOG (save_image), cancel, 0);

  forme = -1;
  alpha = 1.0;

  toimg thedata;
  thedata.a = background;
  thedata.b = log;
  g_signal_connect ((gpointer)comboformat, "changed", G_CALLBACK(choose_format), &thedata);
  g_signal_connect ((gpointer)x, "focus-out-event", G_CALLBACK(to_set_size), GINT_TO_POINTER(0));
  g_signal_connect ((gpointer)y, "focus-out-event", G_CALLBACK(to_set_size), GINT_TO_POINTER(1));
  g_signal_connect ((gpointer)x, "activate", G_CALLBACK(set_size), GINT_TO_POINTER(0));
  g_signal_connect ((gpointer)y, "activate", G_CALLBACK(set_size), GINT_TO_POINTER(1));
  g_signal_connect ((gpointer)saving, "activate", G_CALLBACK(write_image), cdata);
  g_signal_connect ((gpointer)saving, "released", G_CALLBACK(write_image), cdata);
  g_signal_connect ((gpointer)cancel, "activate", G_CALLBACK(destroy_img_dialog), NULL);
  g_signal_connect ((gpointer)background, "toggled", G_CALLBACK(set_background), NULL);
  g_signal_connect_data ((gpointer)cancel, "activate", G_CALLBACK(gtk_widget_destroy),
                         GTK_OBJECT(save_image), NULL, G_CONNECT_AFTER | G_CONNECT_SWAPPED);
  g_signal_connect_data ((gpointer)cancel, "released", G_CALLBACK(gtk_widget_destroy),
                         GTK_OBJECT(save_image), NULL, G_CONNECT_AFTER | G_CONNECT_SWAPPED);
  g_signal_connect_data ((gpointer)save_image, "delete_event", G_CALLBACK(gtk_widget_destroy),
                         GTK_OBJECT(save_image), NULL, G_CONNECT_AFTER | G_CONNECT_SWAPPED);

  gtk_widget_show_all (save_image);
  gtk_dialog_run (GTK_DIALOG(save_image));
}

void update_curve (gpointer curve)
{
  gtk_widget_hide (curves[GPOINTER_TO_INT(curve)] -> plot);
  gtk_widget_show (curves[GPOINTER_TO_INT(curve)] -> plot);
}

G_MODULE_EXPORT void show_res (GtkEntry * maj, gpointer data)
{
  const gchar *m;
  double tmp;
  axdata * adata = (axdata *)data;

  m = gtk_entry_get_text (maj);
  tmp = atof(m);
  if (tmp != 0.0)
  {
    curves[adata -> a] -> majt[adata -> b] = tmp;
    update_curve (GINT_TO_POINTER(adata -> a));
  }
  else
  {
    show_warning ("Major tick must be > 0.0");
  }
  update_entry_double (maj, & curves[adata -> a] -> majt[adata -> b]);
}

G_MODULE_EXPORT gboolean to_show_res (GtkWidget * maj, GdkEventFocus * event, gpointer data)
{
   show_res (GTK_ENTRY(maj), data);
   return (FALSE);
}

G_MODULE_EXPORT void mod_res (GtkEntry * maj, gpointer data)
{
  const gchar *m;
  int size[2];
  int shift[2];
  char * text[2];
  int i;

  axdata * adata = (axdata *)data;

  shift[0] = 0;
  shift[1] = THESHIFT;
  size[0] = 0;
  size[1] = 0;
  text[0] = "X size must be > 0";
  text[1] = "Y size must be > 0";
  gtk_window_get_size(GTK_WINDOW(curves[adata -> a] -> window), &size[0], &size[1]);
  m = gtk_entry_get_text (maj);
  i = atof(m);
  if (i != 0)
  {
    size[adata -> b] = i - shift[adata -> b];
    gtk_window_resize(GTK_WINDOW(curves[adata -> a] -> window), size[0], size[1]);
  }
  else
  {
    show_warning (text[adata -> b]);
  }
  i = size[adata -> b] + shift[adata -> b];
  update_entry_int (maj, & i);
}

G_MODULE_EXPORT gboolean to_mod_res (GtkWidget * maj, GdkEventFocus * event, gpointer data)
{
  mod_res (GTK_ENTRY(maj), data);
  return (FALSE);
}

int lequel (int * id, gpointer data)
{
  int i, j, k, l, m;

  i = *id;
  j = curves[GPOINTER_TO_INT(data)] -> cend - curves[GPOINTER_TO_INT(data)] -> cstart;
  if (curves[GPOINTER_TO_INT(data)] -> cstart == 0 || curves[GPOINTER_TO_INT(data)] -> cstart == numgr + numsq + numsk)
  {
    j = numgr+numgq;
  }
  else if (curves[GPOINTER_TO_INT(data)] -> cstart == numgr || curves[GPOINTER_TO_INT(data)] -> cstart == numgr + numsq)
  {
    j = numsq+numsk;
  }
  l = 0;
  for ( k = 0 ; k < j ; k ++ )
  {
     if ( curves[GPOINTER_TO_INT(data)] -> extrasets[k] != 0 )
     {
        l = l + 1;
        if (l == i) m = k;
     }
  }
  return (m);
}

GdkPixmap * draw_pixmap (int * da, double * ti, GdkColor * dcol, GdkColor * bcol, int * tglyph, double * tgsize)
{
  GdkPixmap * pix;
  cairo_surface_t * cst;
  cairo_t * tcst;
  thedash * tdash;
  cairo_t * cr_pixmap;
  double x, y;

  pix = gdk_pixmap_new (NULL, 100, 30, 24);
  cst = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, 100, 30);
  tcst = cairo_create(cst);
  if (* da > 0)
  {
    tdash = selectdash (da);
    cairo_set_dash (tcst, tdash -> a, tdash -> b, 0);
    cairo_set_source_rgba (tcst, (* bcol).red/65535.0, (* bcol).green/65535.0, (* bcol).blue/65535.0, 1.0);
    cairo_paint (tcst);
    cairo_set_source_rgb (tcst, (* dcol).red/65535.0, (* dcol).green/65535.0, (* dcol).blue/65535.0);
    cairo_set_line_width (tcst, * ti);
    cairo_move_to (tcst, 0, 15);
    cairo_line_to (tcst, 100, 15);
    cairo_stroke (tcst);
    free(tdash);
  }
  else
  {
    cairo_set_source_rgba (tcst, (* bcol).red/65535.0, (* bcol).green/65535.0, (* bcol).blue/65535.0, 1.0);
    cairo_paint (tcst);
    cairo_stroke (tcst);
  }
  x = 25.0;
  y = 15.0;
  draw_glyph (tcst, tglyph, & x, & y, dcol, tgsize);
  x = 75.0;
  draw_glyph (tcst, tglyph, & x, & y, dcol, tgsize);
  cairo_destroy (tcst);
  cr_pixmap = gdk_cairo_create(pix);
  cairo_set_source_surface (cr_pixmap, cst, 0, 0);
  cairo_paint(cr_pixmap);
  cairo_destroy(cr_pixmap);
  cairo_surface_destroy(cst);
  return (pix);
}

G_MODULE_EXPORT void set_style (GtkWidget * box, gpointer data)
{
  int h, j;
  GdkPixmap * pix;

  h = gtk_combo_box_get_active (GTK_COMBO_BOX(setcolorbox));
  if (h > 0)
  {
    j = lequel (& h, data);
    pix = draw_pixmap (& curves[GPOINTER_TO_INT(data)] -> extradash[j],
                       & curves[GPOINTER_TO_INT(data)] -> extrathick[j],
                       & curves[GPOINTER_TO_INT(data)] -> extracolor[j],
                       & curves[GPOINTER_TO_INT(data)] -> backcolor,
                       & curves[GPOINTER_TO_INT(data)] -> extraglyph[j],
                       & curves[GPOINTER_TO_INT(data)] -> extragsize[j]);
  }
  else
  {
    pix = draw_pixmap (& curves[GPOINTER_TO_INT(data)] -> dash,
                       & curves[GPOINTER_TO_INT(data)] -> thickness,
                       & curves[GPOINTER_TO_INT(data)] -> datacolor,
                       & curves[GPOINTER_TO_INT(data)] -> backcolor,
                       & curves[GPOINTER_TO_INT(data)] -> glyph,
                       & curves[GPOINTER_TO_INT(data)] -> gsize);
  }
  gtk_widget_destroy (stylearea);
  stylearea =  gtk_image_new_from_pixmap  (pix, NULL);
  gtk_box_pack_start (GTK_BOX(pixarea), stylearea, TRUE, TRUE, 0);
  gtk_widget_show(stylearea);
  update_curve (data);
}

G_MODULE_EXPORT void set_glyph (GtkComboBox * gbox, gpointer data)
{
  int h, i, j;

  h = gtk_combo_box_get_active (GTK_COMBO_BOX(setcolorbox));
  i = gtk_combo_box_get_active (GTK_COMBO_BOX(gbox));
  if (h > 0)
  {
    j = lequel (& h, data);
    curves[GPOINTER_TO_INT(data)] -> extraglyph[j] = i;
  }
  else
  {
    curves[GPOINTER_TO_INT(data)] -> glyph = i;
  }
  if (i == 0)
  {
    gtk_widget_set_sensitive (glyphsize, 0);
    gtk_widget_set_sensitive (glyphfreq, 0);
  }
  else
  {
    gtk_widget_set_sensitive (glyphsize, 1);
    gtk_widget_set_sensitive (glyphfreq, 1);
  }
  set_style (NULL, data);
}

G_MODULE_EXPORT void set_line (GtkComboBox * lbox, gpointer data)
{
  int h, i, j;

  h = gtk_combo_box_get_active (GTK_COMBO_BOX(setcolorbox));
  i = gtk_combo_box_get_active (GTK_COMBO_BOX(lbox));
  if (h > 0)
  {
    j = lequel (& h, data);
    curves[GPOINTER_TO_INT(data)] -> extradash[j] = i;
  }
  else
  {
    curves[GPOINTER_TO_INT(data)] -> dash = i;
  }
  if (i == 0)
  {
    gtk_widget_set_sensitive (linewidth, 0);
  }
  else
  {
    gtk_widget_set_sensitive (linewidth, 1);
  }
  set_style (NULL, data);
}

G_MODULE_EXPORT void set_datacolor (GtkColorButton * colob, gpointer data)
{
  int i, j;

  i = gtk_combo_box_get_active (GTK_COMBO_BOX(setcolorbox));
  if (i > 0)
  {
    j = lequel (& i, data);
    gtk_color_button_get_color(colob, & curves[GPOINTER_TO_INT(data)] -> extracolor[j]);
  }
  else
  {
    gtk_color_button_get_color(colob, & curves[GPOINTER_TO_INT(data)] -> datacolor);
  }
  set_style (NULL, data);
}

G_MODULE_EXPORT void set_datathick (GtkEntry * thickd, gpointer data)
{
  int i, j;
  double k;
  const gchar * wid;;

  wid = gtk_entry_get_text (thickd);
  k = atof(wid);
  i = gtk_combo_box_get_active (GTK_COMBO_BOX(setcolorbox));
  j = lequel (& i, data);
  if (k > 0.0)
  {
    if (i > 0)
    {
      curves[GPOINTER_TO_INT(data)] -> extrathick[j] = k;
    }
    else
    {
      curves[GPOINTER_TO_INT(data)] -> thickness = k;
    }
    update_entry_double (thickd, & k);
    set_style (NULL, data);
  }
  else
  {
    show_warning ("Line width must be > 0.0");
    if (i > 0)
    {
      update_entry_double (thickd, & curves[GPOINTER_TO_INT(data)] -> extrathick[j]);
    }
    else
    {
      update_entry_double (thickd, & curves[GPOINTER_TO_INT(data)] -> thickness);
    }
  }
}

G_MODULE_EXPORT gboolean to_set_datathick (GtkWidget * bthick, GdkEventFocus * event, gpointer mdata)
{
  set_datathick (GTK_ENTRY(bthick), mdata);
  return (FALSE);
}

G_MODULE_EXPORT void set_glyphsize (GtkEntry * glsize, gpointer data)
{
  int i, j;
  double k;
  const gchar * wid;

  wid = gtk_entry_get_text (glsize);
  k = atof(wid);
  i = gtk_combo_box_get_active (GTK_COMBO_BOX(setcolorbox));
  j = lequel (& i, data);
  if (k > 0.0)
  {
    if (i > 0)
    {
      curves[GPOINTER_TO_INT(data)] -> extragsize[j] = k;
    }
    else
    {
      curves[GPOINTER_TO_INT(data)] -> gsize = k;
    }
    update_entry_double (glsize, & k);
    set_style (NULL, data);
  }
  else
  {
    show_warning ("Glyph size must be > 0.0");
    if (i > 0)
    {
      update_entry_double (glsize, & curves[GPOINTER_TO_INT(data)] -> extragsize[j]);
    }
    else
    {
      update_entry_double (glsize, & curves[GPOINTER_TO_INT(data)] -> gsize);
    }
  }
}

G_MODULE_EXPORT gboolean to_set_glyphsize (GtkWidget * glsize, GdkEventFocus * event, gpointer mdata)
{
  set_glyphsize (GTK_ENTRY(glsize), mdata);
  return(FALSE);
}

G_MODULE_EXPORT void set_glyphfreq (GtkEntry * glfreq, gpointer data)
{
  int i, j, k;
  const gchar * wid;

  wid = gtk_entry_get_text (glfreq);
  k = atof(wid);
  i = gtk_combo_box_get_active (GTK_COMBO_BOX(setcolorbox));
  j = lequel (& i, data);
  if (k > 0)
  {
    if (i > 0)
    {
      curves[GPOINTER_TO_INT(data)] -> extragfreq[j] = k;
    }
    else
    {
      curves[GPOINTER_TO_INT(data)] -> gfreq = k;
    }
    update_entry_int (glfreq, &  k);
    set_style (NULL, data);
  }
  else
  {
    show_warning ("Glyph frequency must be > 0");
    if (i > 0)
    {
      update_entry_int (glfreq, & curves[GPOINTER_TO_INT(data)] -> extragfreq[j]);
    }
    else
    {
      update_entry_int (glfreq, & curves[GPOINTER_TO_INT(data)] -> gfreq);
    }
  }
}

G_MODULE_EXPORT gboolean to_set_glyphfreq (GtkWidget * glfreq, GdkEventFocus * event, gpointer mdata)
{
  set_glyphfreq (GTK_ENTRY(glfreq), mdata);
  return (FALSE);
}

G_MODULE_EXPORT void set_backcolor (GtkColorButton * colob, gpointer data)
{
  gtk_color_button_get_color(colob, & curves[GPOINTER_TO_INT(data)] -> backcolor);
  set_style (NULL, data);
}

G_MODULE_EXPORT void set_framcolor (GtkColorButton * colob, gpointer data)
{
  gtk_color_button_get_color(colob, & curves[GPOINTER_TO_INT(data)] -> framcolor);
  update_curve (data);
}

G_MODULE_EXPORT void ediv_res (GtkSpinButton * res, gpointer data)
{
  axdata * adata = (axdata *) data;

  curves[adata -> a] -> mint[adata -> b] = gtk_spin_button_get_value_as_int(res) + 1;
  update_curve (GINT_TO_POINTER(adata -> a));
}

G_MODULE_EXPORT gboolean to_ediv_res (GtkWidget * res, GdkEventFocus * event, gpointer data)
{
  const gchar * m;
  axdata * adata = (axdata *)data;

  m = gtk_entry_get_text (GTK_ENTRY(res));
  curves[adata -> a] -> mint[adata -> b] = (int )atof(m) + 1;
  update_curve (GINT_TO_POINTER(adata -> a));
  return (FALSE);
}

G_MODULE_EXPORT void set_min (GtkEntry * res, gpointer data)
{
  const gchar * m;
  axdata * adata = (axdata *) data;

  m = gtk_entry_get_text (res);
  if (atof(m) < curves[adata -> a] -> axmax[adata -> b])
  {
    curves[adata -> a] -> axmin[adata -> b] = atof(m);
  }
  else
  {
      show_warning ("Axis min must be < to axis max");
  }
  update_entry_double (res, & curves[adata -> a] -> axmin[adata -> b]);
  update_curve (GINT_TO_POINTER(adata -> a));
}

G_MODULE_EXPORT gboolean to_set_min (GtkWidget * res, GdkEventFocus * event, gpointer data)
{
  set_min (GTK_ENTRY(res), data);
  return (FALSE);
}

G_MODULE_EXPORT void set_max (GtkEntry * res, gpointer data)
{
  const gchar * m;
  axdata * adata = (axdata *) data;

  m = gtk_entry_get_text (res);
  if (atof(m) > curves[adata -> a] -> axmin[adata -> b])
  {
    curves[adata -> a] -> axmax[adata -> b] = atof(m);
  }
  else
  {
    show_warning ("Axis max must be > to axis min");
  }
  update_entry_double (res, & curves[adata -> a] -> axmax[adata -> b]);
  update_curve (GINT_TO_POINTER(adata -> a));
}

G_MODULE_EXPORT gboolean to_set_max (GtkWidget * res, GdkEventFocus * event, gpointer data)
{
  set_max (GTK_ENTRY(res), data);
  return (FALSE);
}

G_MODULE_EXPORT void div_res (GtkSpinButton * res, gpointer data)
{
  axdata * adata = (axdata *)data;

  curves[adata -> a] -> mint[adata -> b] = gtk_spin_button_get_value_as_int(res) + 1;
  update_curve (GINT_TO_POINTER(adata -> a));
}

G_MODULE_EXPORT void to_xtitle (GtkToggleButton * but, gpointer data)
{
  if (gtk_toggle_button_get_active (but))
  {
    gtk_widget_set_sensitive (xtitle, 0);
    gtk_widget_set_sensitive (titlex, 0);
    gtk_entry_set_text (GTK_ENTRY(titlex), default_xtitle(data));
    update_curve (data);
  }
  else
  {
    gtk_widget_set_sensitive (xtitle, 1);
    gtk_widget_set_sensitive (titlex, 1);
  }
}

G_MODULE_EXPORT void set_xtitle (GtkEntry * xtit, gpointer mdata)
{
  curves[GPOINTER_TO_INT(mdata)] -> xlabel = (gchar *)gtk_entry_get_text (xtit);
  update_curve (mdata);
}

G_MODULE_EXPORT gboolean to_set_xtitle (GtkWidget * xtit, GdkEventFocus * event, gpointer mdata)
{
   set_xtitle (GTK_ENTRY(xtit), mdata);
   return (FALSE);
}

G_MODULE_EXPORT void choose_set (GtkComboBox * box, gpointer data)
{
  int i, j;

  i = gtk_combo_box_get_active (box);
  if (i > 0)
  {
    j = lequel (& i, data);
    gtk_color_button_set_color (GTK_COLOR_BUTTON(datcolor), & curves[GPOINTER_TO_INT(data)] -> extracolor[j]);
    gtk_combo_box_set_active (GTK_COMBO_BOX(dashbox), curves[GPOINTER_TO_INT(data)] -> extradash[j]);
    update_entry_double (GTK_ENTRY(linewidth),  & curves[GPOINTER_TO_INT(data)] -> extrathick[j]);
    gtk_combo_box_set_active (GTK_COMBO_BOX(glyphbox), curves[GPOINTER_TO_INT(data)] -> extraglyph[j]);
    update_entry_double (GTK_ENTRY(glyphsize), & curves[GPOINTER_TO_INT(data)] -> extragsize[j]);
    gtk_combo_box_set_active (GTK_COMBO_BOX(dataspect), curves[GPOINTER_TO_INT(data)] -> extraspect[j]);
    update_entry_int (GTK_ENTRY(glyphfreq), & curves[GPOINTER_TO_INT(data)] -> extragfreq[j]);
  }
  else
  {
    gtk_color_button_set_color (GTK_COLOR_BUTTON(datcolor), & curves[GPOINTER_TO_INT(data)] -> datacolor);
    gtk_combo_box_set_active (GTK_COMBO_BOX(dashbox), curves[GPOINTER_TO_INT(data)] -> dash);
    update_entry_double (GTK_ENTRY(linewidth), & curves[GPOINTER_TO_INT(data)] -> thickness);
    gtk_combo_box_set_active (GTK_COMBO_BOX(glyphbox), curves[GPOINTER_TO_INT(data)] -> glyph);
    update_entry_double (GTK_ENTRY(glyphsize), & curves[GPOINTER_TO_INT(data)] -> gsize);
    gtk_combo_box_set_active (GTK_COMBO_BOX(dataspect), curves[GPOINTER_TO_INT(data)] -> aspect);
    update_entry_int (GTK_ENTRY(glyphfreq), & curves[GPOINTER_TO_INT(data)] -> gfreq);
  }
  set_style (NULL, data);
}

void prepbox (gpointer data)
{
  int i, j, k, l;

  k = GPOINTER_TO_INT(data);
  gtk_combo_box_append_text (GTK_COMBO_BOX (setcolorbox), curves[k] -> title);
  j = curves[k] -> cend - curves[k] -> cstart;
  if (curves[k] -> cstart == 0 || curves[k] -> cstart == numgr + numsq + numsk)
  {
    j = numgr+numgq;
  }
  else if (curves[k] -> cstart == numgr || curves[k] -> cstart == numgr + numsq)
  {
    j = numsq+numsk;
  }

  for ( i=0 ; i < j ; i++ )
  {
    if ( curves[k] -> extrasets[i] != 0)
    {
      l = findcurve(&k ,&i);
      gtk_combo_box_append_text (GTK_COMBO_BOX (setcolorbox), curves[l] -> title);
    }
  }
  if (curves[k] -> extra > 0)
  {
    gtk_widget_set_sensitive (setcolorbox, 1);
  }
  else
  {
    gtk_widget_set_sensitive (setcolorbox, 0);
  }
  gtk_widget_set_size_request (setcolorbox, -1, 30);
  gtk_combo_box_set_active (GTK_COMBO_BOX (setcolorbox), 0);
  g_signal_connect ((gpointer)setcolorbox, "changed", G_CALLBACK(choose_set), data);
}

G_MODULE_EXPORT void set_set (GtkToggleButton *but, gpointer data)
{
  setdata * datagtk = (setdata *)data;

  gtk_widget_destroy (setcolorbox);
  setcolorbox = gtk_combo_box_new_text ();
  gtk_box_pack_start (GTK_BOX (datagtk -> c), setcolorbox, TRUE, TRUE, 0);
  gtk_widget_show(setcolorbox);
  if (gtk_toggle_button_get_active (but))
  {
    curves[GPOINTER_TO_INT(datagtk -> a)] -> extra += 1;
    curves[GPOINTER_TO_INT(datagtk -> a)] -> extrasets[GPOINTER_TO_INT(datagtk -> b)] = 1;
    curves[GPOINTER_TO_INT(datagtk -> a)] -> autoscale[0] = TRUE;
    curves[GPOINTER_TO_INT(datagtk -> a)] -> autoscale[1] = TRUE;
  }
  else
  {
    curves[GPOINTER_TO_INT(datagtk -> a)] -> extra -= 1;
    curves[GPOINTER_TO_INT(datagtk -> a)] -> extrasets[GPOINTER_TO_INT(datagtk -> b)] = 0;
    curves[GPOINTER_TO_INT(datagtk -> a)] -> autoscale[0] = TRUE;
    curves[GPOINTER_TO_INT(datagtk -> a)] -> autoscale[1] = TRUE;
  }
  prepbox (datagtk -> a);
  choose_set (GTK_COMBO_BOX(setcolorbox), datagtk -> a);
}

G_MODULE_EXPORT void set_grid (GtkToggleButton * grid, gpointer data)
{
  axdata * adata = (axdata *)data;

  curves[adata -> a] -> grid[adata -> b] = gtk_toggle_button_get_active (grid);
  update_curve (GINT_TO_POINTER(adata -> a));
}

G_MODULE_EXPORT void set_autoscale (GtkToggleButton * autosc, gpointer data)
{
  axdata * adata = (axdata *)data;

  curves[adata -> a] -> autoscale[adata -> b] = 1;
  update_curve (GINT_TO_POINTER(adata -> a));
  gtk_toggle_button_set_active (autosc, 0);
}

G_MODULE_EXPORT void set_axis (GtkToggleButton * axis, gpointer data)
{
  axdata * adata = (axdata *)data;

  curves[adata -> a] -> ashow[adata -> b] = gtk_toggle_button_get_active (axis);
  update_curve (GINT_TO_POINTER(adata -> a));
}

G_MODULE_EXPORT void data_legend (GtkToggleButton * leg, gpointer data)
{
  int i;

  curves[GPOINTER_TO_INT(data)] -> legend = gtk_toggle_button_get_active (leg);
  update_curve (data);
  for (i=0; i<2; i++)
  {
    gtk_widget_set_sensitive (lxyc[i], curves[GPOINTER_TO_INT(data)] -> legend);
    gtk_widget_set_sensitive (xyl[i], curves[GPOINTER_TO_INT(data)] -> legend);
  }
  gtk_widget_set_sensitive (legcoord, curves[GPOINTER_TO_INT(data)] -> legend);
}

G_MODULE_EXPORT void set_frame (GtkToggleButton * fram, gpointer data)
{
  curves[GPOINTER_TO_INT(data)] -> frame = gtk_toggle_button_get_active (fram);
  update_curve (data);
}

G_MODULE_EXPORT void leg_pos (GtkEntry * p, gpointer data)
{
  const gchar *c;
  axdata * adata = (axdata *)data;

  c = gtk_entry_get_text (p);
  curves[adata -> a] -> legpos[adata -> b] = atof(c);
  update_entry_double (p, & curves[adata -> a] -> legpos[adata -> b]);
  update_curve (GINT_TO_POINTER(adata -> a));
}

G_MODULE_EXPORT gboolean to_leg_pos (GtkWidget * p, GdkEventFocus * event, gpointer data)
{
  leg_pos (GTK_ENTRY(p), data);
  return (FALSE);
}

G_MODULE_EXPORT void set_aspect (GtkComboBox * box, gpointer data)
{
  int i, j;

  i = gtk_combo_box_get_active (GTK_COMBO_BOX(setcolorbox));
  if (i > 0)
  {
    j = lequel (& i, data);
    curves[GPOINTER_TO_INT(data)] -> extraspect[j] = gtk_combo_box_get_active (box);
  }
  else
  {
    curves[GPOINTER_TO_INT(data)] -> aspect = gtk_combo_box_get_active (box);
  }
  if (gtk_combo_box_get_active (box)  == 1) gtk_combo_box_set_active (GTK_COMBO_BOX(glyphbox), 0);
  update_curve (data);
}

G_MODULE_EXPORT void set_scale (GtkComboBox * sbox, gpointer data)
{
  axdata * adata = (axdata *)data;

  curves[adata -> a] -> scale[adata -> b] = gtk_combo_box_get_active(sbox);
  gtk_widget_set_sensitive (vmax[adata -> b], ! gtk_combo_box_get_active(sbox));
  gtk_widget_set_sensitive (vmin[adata -> b], ! gtk_combo_box_get_active(sbox));
  gtk_widget_set_sensitive (mjt[adata -> b], ! gtk_combo_box_get_active(sbox));
  gtk_widget_set_sensitive (nmi[adata -> b], ! gtk_combo_box_get_active(sbox));
  curves[adata -> a] -> autoscale[adata -> b] = TRUE;
  update_curve (GINT_TO_POINTER(adata -> a));
}

G_MODULE_EXPORT void edit_image (GtkWidget * curve, gpointer cdata)
{
  GtkWidget *edit_image;
  GtkWidget *ebox;
  GtkWidget *enoote;
  GtkWidget *cbox, *axebox, *dbox;
  GtkWidget *size;
  GtkWidget *hsep_a, *hsep_b;
  GtkWidget *hsep_c, *hsep_d;
  GtkWidget *x;
  GtkWidget *pixels;
  GtkWidget *backgroundcolor;
  GtkWidget *framecolor;
  GtkWidget *chbox_a, *chbox_b, *chbox_c;
  GtkWidget *chbox_d, *chbox_e, *chbox_f;
  GtkWidget *chbox_g, *chbox_h, *chbox_i;
  GtkWidget *chbox_j, *chbox_k, *chbox_l;
  GtkWidget *glyph;
  GtkWidget *stylebox;
  GtkWidget *datac, *dataf;
  GtkWidget *backc, *framc;
  GtkWidget *graph;
  GtkWidget *axis;
  GtkWidget *dialog_action_img;
  GtkWidget *close;
  GtkWidget *selectset;
  GtkWidget *setmess;
  GtkWidget *sets;
  GtkWidget *addset;
  GtkWidget *scrollsets;
  GtkWidget *vportsets;
  GtkWidget *fixedsets;
  GtkWidget **allsets;
  GtkObject *adj;
  GtkWidget *thickn;
  GtkWidget *style;
  GtkWidget *displegend, *displframe;
  GtkWidget *glips, *glipf;

// Axis data
  axdata dataxe[2];
  GtkWidget * axisbox[2];
  GtkWidget * axlabel[2];
  GtkWidget * amin[2], * amax[2];
  GtkWidget * ma[2], * mi[2];
  GtkObject * nmi_adj[2];
  GtkWidget * ahbox;
  GtkWidget * ahbox_a[2], * ahbox_b[2];
  GtkWidget * ahbox_c[2], * ahbox_d[2];
  GtkWidget * showaxe[2], * grid[2], * autos[2];
  GtkWidget *asc[2], * ascale[2];
  GtkWidget * dxtitle;

  int xlgt, ylgt;
  int id = GPOINTER_TO_INT(cdata);
  int i, j;

  const int naspects = 2;
  char * aspects[2];
  aspects[0]="x/y";
  aspects[1]="bar";

  edit_image = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (edit_image), 5);
  gtk_window_set_type_hint (GTK_WINDOW (edit_image), GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_window_set_resizable (GTK_WINDOW (edit_image), FALSE);

  gtk_dialog_set_has_separator (GTK_DIALOG (edit_image), FALSE);
  gtk_window_set_title (GTK_WINDOW(edit_image), "Edit curve");

  ebox = GTK_DIALOG (edit_image)-> vbox;
  gtk_box_set_spacing (GTK_BOX(ebox), 15);
  gtk_widget_set_size_request (ebox, 400, 620);
  enoote = gtk_notebook_new ();
  gtk_container_add (GTK_CONTAINER (ebox), enoote);

// The first tab of the notebook

  cbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (enoote), cbox);

  chbox_a = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (cbox), chbox_a, TRUE, TRUE, 0);

  size = gtk_label_new ("Size:");
  gtk_box_pack_start (GTK_BOX (chbox_a), size, TRUE, TRUE, 0);
  gtk_widget_set_size_request (size, 100, -1);
  gtk_misc_set_alignment (GTK_MISC (size), 0, 0.5);

  gtk_window_get_size (GTK_WINDOW(curves[id] -> window), &xlgt, &ylgt);
  xyp[0] = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (chbox_a), xyp[0], TRUE, TRUE, 0);
  gtk_widget_set_size_request (xyp[0], 100, -1);
  gtk_entry_set_alignment (GTK_ENTRY(xyp[0]), 1.0);
  update_entry_int (GTK_ENTRY(xyp[0]), & xlgt);

  x = gtk_label_new ("x");
  gtk_widget_set_size_request (x, 20, -1);
  gtk_box_pack_start (GTK_BOX (chbox_a), x, TRUE, TRUE, 0);

  xyp[1] = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (chbox_a), xyp[1], TRUE, TRUE, 0);
  gtk_widget_set_size_request (xyp[1], 100, -1);
  gtk_entry_set_alignment (GTK_ENTRY(xyp[1]), 1.0);
  ylgt = ylgt + THESHIFT;
  update_entry_int (GTK_ENTRY(xyp[1]), & ylgt);

  pixels = gtk_label_new ("pixels");
  gtk_widget_set_size_request (pixels, 100, -1);
  gtk_misc_set_alignment (GTK_MISC (pixels), 0.5, 0.5);
  gtk_box_pack_start (GTK_BOX (chbox_a), pixels, TRUE, TRUE, 0);

  hsep_a = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (cbox), hsep_a, TRUE, TRUE, 0);

  chbox_d = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (cbox), chbox_d, TRUE, TRUE, 0);
  backc = gtk_label_new ("Background color:");
  gtk_box_pack_start (GTK_BOX (chbox_d), backc, TRUE, TRUE, 0);
  gtk_widget_set_size_request (backc, 200, -1);
  gtk_misc_set_alignment (GTK_MISC (backc), 0, 0.5);
  backgroundcolor = gtk_color_button_new_with_color (& curves[id] -> backcolor);
  gtk_widget_set_size_request (backgroundcolor, 25, -1);
  gtk_box_pack_start (GTK_BOX (chbox_d), backgroundcolor, TRUE, TRUE, 0);

  displframe = gtk_check_button_new_with_label ("Show/hide frame");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(displframe), curves[id] -> frame);
  gtk_box_pack_start (GTK_BOX (cbox), displframe, TRUE, TRUE, 0);

  chbox_e = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (cbox), chbox_e, TRUE, TRUE, 0);
  framc = gtk_label_new ("Frame color:");
  gtk_box_pack_start (GTK_BOX (chbox_e), framc, TRUE, TRUE, 0);
  gtk_widget_set_size_request (framc, 200, -1);
  gtk_misc_set_alignment (GTK_MISC (framc), 0, 0.5);

  framecolor = gtk_color_button_new_with_color (& curves[id] -> framcolor);
  gtk_widget_set_size_request (framecolor, 25, -1);
  gtk_box_pack_start (GTK_BOX (chbox_e), framecolor, TRUE, TRUE, 0);
  displegend = gtk_check_button_new_with_label ("Show/hide legend");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(displegend), curves[id] -> legend);
  gtk_box_pack_start (GTK_BOX (cbox), displegend, TRUE, TRUE, 0);

  legcoord = gtk_label_new ("Position of the legend:");
  gtk_misc_set_alignment (GTK_MISC (legcoord), 0, 0.5);
  gtk_widget_set_sensitive (legcoord, curves[id] -> legend);
  gtk_box_pack_start (GTK_BOX (cbox), legcoord, TRUE, TRUE, 0);

  chbox_g = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (cbox), chbox_g, TRUE, TRUE, 0);
  xyl[0] = gtk_label_new ("x (0.0 -> 1.0): ");
  gtk_misc_set_alignment (GTK_MISC (xyl[0]), 1.0, 0.5);
  gtk_widget_set_size_request (xyl[0], 200, -1);
  gtk_widget_set_sensitive (xyl[0], curves[id] -> legend);
  gtk_box_pack_start (GTK_BOX (chbox_g), xyl[0], TRUE, TRUE, 0);
  lxyc[0] = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (chbox_g), lxyc[0], TRUE, TRUE, 0);
  gtk_widget_set_size_request (lxyc[0], 50, -1);
  gtk_entry_set_alignment (GTK_ENTRY(lxyc[0]), 1.0);
  update_entry_double (GTK_ENTRY(lxyc[0]), & curves[id] -> legpos[0]);
  gtk_widget_set_sensitive (lxyc[0], curves[id] -> legend);

  chbox_h = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (cbox), chbox_h, TRUE, TRUE, 0);
  xyl[1] = gtk_label_new ("y (0.0 -> 1.0): ");
  gtk_misc_set_alignment (GTK_MISC (xyl[1]), 1.0, 0.5);
  gtk_widget_set_size_request (xyl[1], 200, -1);
  gtk_widget_set_sensitive (xyl[1], curves[id] -> legend);
  gtk_box_pack_start (GTK_BOX (chbox_h), xyl[1], TRUE, TRUE, 0);
  lxyc[1] = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (chbox_h), lxyc[1], TRUE, TRUE, 0);
  gtk_widget_set_size_request (lxyc[1], 50, -1);
  gtk_entry_set_alignment (GTK_ENTRY(lxyc[1]), 1.0);
  update_entry_double (GTK_ENTRY(lxyc[1]), & curves[id] -> legpos[1]);
  gtk_widget_set_sensitive (lxyc[1], curves[id] -> legend);

  hsep_b = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (cbox), hsep_b, TRUE, TRUE, 0);

  chbox_b = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (cbox), chbox_b, TRUE, TRUE, 0);
  selectset = gtk_label_new ("<b>Select data set</b>");
  gtk_label_set_use_markup (GTK_LABEL(selectset), TRUE);
  gtk_box_pack_start (GTK_BOX (chbox_b), selectset, TRUE, TRUE, 0);
  setcolorbox = gtk_combo_box_new_text ();
  prepbox (cdata);
  gtk_box_pack_start (GTK_BOX (chbox_b), setcolorbox, TRUE, TRUE, 0);

  chbox_k = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (cbox), chbox_k, TRUE, TRUE, 0);
  dataf = gtk_label_new ("Plot type:");
  gtk_box_pack_start (GTK_BOX (chbox_k), dataf, TRUE, TRUE, 0);
  gtk_widget_set_size_request (dataf, 100, -1);
  gtk_misc_set_alignment (GTK_MISC (dataf), 0, 0.5);
  dataspect = gtk_combo_box_new_text ();
  for ( i=0 ; i < naspects ; i++ )
  {
    gtk_combo_box_append_text (GTK_COMBO_BOX(dataspect), aspects[i]);
  }
  gtk_widget_set_size_request (dataspect, 25, -1);
  gtk_combo_box_set_active (GTK_COMBO_BOX(dataspect), curves[id] -> aspect);
  gtk_box_pack_start (GTK_BOX (chbox_k), dataspect, TRUE, TRUE, 0);
  if ((id >= numgr && id < numgr+numgq+numsq+numsk) || id >= numwid-nummsd) gtk_widget_set_sensitive (dataspect, 0);

  pixarea = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (cbox), pixarea, TRUE, TRUE, 0);
  stylebox = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (pixarea), stylebox, TRUE, TRUE, 0);

// Data color

  chbox_c = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (stylebox), chbox_c, TRUE, TRUE, 0);
  datac = gtk_label_new ("Data color:");
  gtk_box_pack_start (GTK_BOX (chbox_c), datac, TRUE, TRUE, 0);
  gtk_widget_set_size_request (datac, 100, -1);
  gtk_misc_set_alignment (GTK_MISC (datac), 0, 0.5);
  datcolor = gtk_color_button_new_with_color (& curves[id] -> datacolor);
  gtk_widget_set_size_request (datcolor, 80, -1);
  gtk_box_pack_start (GTK_BOX (chbox_c), datcolor, TRUE, TRUE, 0);

// Line style
  chbox_i = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (stylebox), chbox_i, TRUE, TRUE, 0);
  style = gtk_label_new ("Line style:");
  gtk_widget_set_size_request (style, 100, -1);
  gtk_misc_set_alignment (GTK_MISC (style), 0, 0.5);
  gtk_box_pack_start (GTK_BOX (chbox_i), style, TRUE, TRUE, 0);
  dashbox = gtk_combo_box_new_text ();
  gtk_combo_box_append_text (GTK_COMBO_BOX(dashbox), "No line");
  for ( i=1 ; i < ndash ; i++)
  {
     gtk_combo_box_append_text (GTK_COMBO_BOX(dashbox), g_strdup_printf("%d", i));
  }
  gtk_widget_set_size_request (dashbox, 80, -1);
  gtk_box_pack_start (GTK_BOX (chbox_i), dashbox, TRUE, TRUE, 0);
  gtk_combo_box_set_active(GTK_COMBO_BOX(dashbox), curves[id] -> dash);

// Line linewidth

  chbox_f = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (stylebox), chbox_f, TRUE, TRUE, 0);
  thickn = gtk_label_new ("Line width:");
  gtk_widget_set_size_request (thickn, 100, -1);
  gtk_misc_set_alignment (GTK_MISC (thickn), 0, 0.5);
  gtk_box_pack_start (GTK_BOX (chbox_f), thickn, TRUE, TRUE, 0);
  linewidth = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (chbox_f), linewidth, TRUE, TRUE, 0);
  gtk_widget_set_size_request (linewidth, 80, -1);
  gtk_entry_set_alignment (GTK_ENTRY(linewidth), 1.0);
  update_entry_double (GTK_ENTRY(linewidth), & curves[id] -> thickness);

  if (curves[id] -> dash == 0)
  {
    gtk_widget_set_sensitive (linewidth, 0);
  }

// Glyph type

  chbox_j = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (stylebox), chbox_j, TRUE, TRUE, 0);
  glyph = gtk_label_new ("Glyph type:");
  gtk_widget_set_size_request (glyph, 100, -1);
  gtk_misc_set_alignment (GTK_MISC (glyph), 0, 0.5);
  gtk_box_pack_start (GTK_BOX (chbox_j), glyph, TRUE, TRUE, 0);
  glyphbox = gtk_combo_box_new_text ();
  gtk_combo_box_append_text (GTK_COMBO_BOX(glyphbox), "No glyph");
  for ( i=1 ; i < nglyph ; i++)
  {
     gtk_combo_box_append_text (GTK_COMBO_BOX(glyphbox), g_strdup_printf("%d", i));
  }
  gtk_widget_set_size_request (glyphbox, 80, -1);
  gtk_box_pack_start (GTK_BOX (chbox_j), glyphbox, TRUE, TRUE, 0);
  gtk_combo_box_set_active(GTK_COMBO_BOX(glyphbox), curves[id] -> glyph);

// Glyph size

  chbox_k = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (stylebox), chbox_k, TRUE, TRUE, 0);
  glips = gtk_label_new ("Glyph size:");
  gtk_widget_set_size_request (glips, 100, -1);
  gtk_misc_set_alignment (GTK_MISC (glips), 0, 0.5);
  gtk_box_pack_start (GTK_BOX (chbox_k), glips, TRUE, TRUE, 0);
  glyphsize = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (chbox_k), glyphsize, TRUE, TRUE, 0);
  gtk_widget_set_size_request (glyphsize, 80, -1);
  gtk_entry_set_alignment (GTK_ENTRY(glyphsize), 1.0);
  update_entry_double (GTK_ENTRY(glyphsize), & curves[id] -> gsize);

// Glyph frequency

  chbox_l = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (stylebox), chbox_l, TRUE, TRUE, 0);
  glipf = gtk_label_new ("Glyph freq.:");
  gtk_widget_set_size_request (glipf, 100, -1);
  gtk_misc_set_alignment (GTK_MISC (glipf), 0, 0.5);
  gtk_box_pack_start (GTK_BOX (chbox_l), glipf, TRUE, TRUE, 0);
  glyphfreq = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (chbox_l), glyphfreq, TRUE, TRUE, 0);
  gtk_widget_set_size_request (glyphfreq, 80, -1);
  gtk_entry_set_alignment (GTK_ENTRY(glyphfreq), 1.0);
  update_entry_int (GTK_ENTRY(glyphfreq), & curves[id] -> gfreq);

  if (curves[id] -> glyph == 0)
  {
    gtk_widget_set_sensitive (glyphsize, 0);
    gtk_widget_set_sensitive (glyphfreq, 0);
  }

// The corresponding sample pixmap

  GdkPixmap * pix = draw_pixmap (& curves[id] -> dash,
                                 & curves[id] -> thickness,
                                 & curves[id] -> datacolor,
                                 & curves[id] -> backcolor,
                                 & curves[id] -> glyph,
                                 & curves[id] -> gsize);
  stylearea =  gtk_image_new_from_pixmap  (pix, NULL);
  gtk_box_pack_start (GTK_BOX (pixarea), stylearea, TRUE, TRUE, 0);

// The complete tab

  graph = gtk_label_new ("Graph");
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (enoote), gtk_notebook_get_nth_page (GTK_NOTEBOOK (enoote), 0), graph);

// The second tab of the notebook

  axebox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (enoote), axebox);

  for ( i = 0 ; i < 2 ; i++ )
  {
    axisbox[i] = gtk_vbox_new (FALSE, 0);
    if (i == 0)
    {
      axlabel[i] = gtk_label_new ("<b>X axis</b>");
    }
    else
    {
      axlabel[i] = gtk_label_new ("<b>Y axis</b>");
    }
    gtk_box_pack_start (GTK_BOX (axisbox[i]), axlabel[i], TRUE, TRUE, 0);
    gtk_label_set_use_markup (GTK_LABEL (axlabel[i]), TRUE);

    ahbox_a[i] = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (axisbox[i]), ahbox_a[i], TRUE, TRUE, 0);

    asc[i] = gtk_label_new ("Scale type:");
    gtk_widget_set_size_request (asc[i], 30, -1);
    gtk_misc_set_alignment (GTK_MISC (asc[i]), 0.5, 0.5);
    gtk_box_pack_start (GTK_BOX (ahbox_a[i]), asc[i], TRUE, TRUE, 0);
    ascale[i] = gtk_combo_box_new_text ();
    gtk_combo_box_append_text (GTK_COMBO_BOX(ascale[i]), "Linear");
    gtk_combo_box_append_text (GTK_COMBO_BOX(ascale[i]), "Log");

    gtk_box_pack_start (GTK_BOX (ahbox_a[i]), ascale[i], TRUE, TRUE, 0);

    ahbox_b[i] = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (axisbox[i]), ahbox_b[i], TRUE, TRUE, 0);

    amin[i] = gtk_label_new ("Min:");
    gtk_box_pack_start (GTK_BOX (ahbox_b[i]), amin[i], TRUE, TRUE, 0);
    gtk_widget_set_size_request (amin[i], 30, -1);
    gtk_misc_set_alignment (GTK_MISC (amin[i]), 0.5, 0.5);
    vmin[i] = gtk_entry_new ();
    gtk_box_pack_start (GTK_BOX (ahbox_b[i]), vmin[i], TRUE, TRUE, 0);
    gtk_widget_set_size_request (vmin[i], 80, -1);
    gtk_entry_set_alignment (GTK_ENTRY(vmin[i]), 1.0);
    update_entry_double (GTK_ENTRY(vmin[i]), & curves[id] -> axmin[i]);

    amax[i] = gtk_label_new ("Max:");
    gtk_box_pack_start (GTK_BOX (ahbox_b[i]), amax[i], TRUE, TRUE, 0);
    gtk_widget_set_size_request (amax[i], 30, -1);
    gtk_misc_set_alignment (GTK_MISC (amax[i]), 0.50, 0.5);
    vmax[i] = gtk_entry_new ();
    gtk_box_pack_start (GTK_BOX (ahbox_b[i]), vmax[i], TRUE, TRUE, 0);
    gtk_widget_set_size_request (vmax[i], 80, -1);
    gtk_entry_set_alignment (GTK_ENTRY(vmax[i]), 1.0);
    update_entry_double (GTK_ENTRY(vmax[i]), & curves[id] -> axmax[i]);

    ahbox_c[i] = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (axisbox[i]), ahbox_c[i], TRUE, TRUE, 0);

    ma[i] = gtk_label_new ("Major ticks spacing:");
    gtk_box_pack_start (GTK_BOX (ahbox_c[i]), ma[i], TRUE, TRUE, 0);
    gtk_widget_set_size_request (ma[i], 200, -1);
    gtk_misc_set_alignment (GTK_MISC (ma[i]), 0, 0.5);
    mjt[i] = gtk_entry_new ();
    gtk_box_pack_start (GTK_BOX (ahbox_c[i]), mjt[i], TRUE, TRUE, 0);
    gtk_widget_set_size_request (mjt[i], 50, -1);
    gtk_entry_set_alignment (GTK_ENTRY(mjt[i]), 1.0);
    update_entry_double(GTK_ENTRY(mjt[i]), & curves[id] -> majt[i]);

    ahbox_d[i] = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (axisbox[i]), ahbox_d[i], TRUE, TRUE, 0);

    mi[i] = gtk_label_new ("Number of minor ticks:");
    gtk_box_pack_start (GTK_BOX (ahbox_d[i]), mi[i], TRUE, TRUE, 0);
    gtk_widget_set_size_request (mi[i], 200, -1);
    gtk_misc_set_alignment (GTK_MISC (mi[i]), 0, 0.5);

    nmi_adj[i] = gtk_adjustment_new (curves[id] -> mint[i]-1.0, 0.0, 100.0, 1.0, 1.0, 0.0);
    nmi[i] = gtk_spin_button_new (GTK_ADJUSTMENT (nmi_adj[i]), 1, 0);
    gtk_entry_set_alignment (GTK_ENTRY(nmi[i]), 1.0);
    gtk_box_pack_start (GTK_BOX (ahbox_d[i]), nmi[i], TRUE, TRUE, 0);

    if (id < numwid - nummsd)
    {
      gtk_combo_box_set_active (GTK_COMBO_BOX(ascale[i]), curves[id] -> scale[i]);
      gtk_widget_set_sensitive (ascale[i], 0);
    }
    else
    {
      gtk_combo_box_set_active (GTK_COMBO_BOX(ascale[i]), curves[id] -> scale[i]);
      gtk_widget_set_sensitive (ascale[i], 1);
      gtk_widget_set_sensitive (vmax[i], ! curves[id] -> scale[i]);
      gtk_widget_set_sensitive (vmin[i], ! curves[id] -> scale[i]);
      gtk_widget_set_sensitive (mjt[i], ! curves[id] -> scale[i]);
      gtk_widget_set_sensitive (nmi[i], ! curves[id] -> scale[i]);
    }

    if (i == 0)
    {
      showaxe[i] = gtk_check_button_new_with_label ("Show/hide X axis");
    }
    else
    {
      showaxe[i] = gtk_check_button_new_with_label ("Show/hide Y axis");
    }
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(showaxe[i]), curves[id] -> ashow[i]);
    gtk_box_pack_start (GTK_BOX (axisbox[i]), showaxe[i], TRUE, TRUE, 0);

    if (i == 0)
    {
      grid[i] = gtk_check_button_new_with_label ("Show/hide X grid");
    }
    else
    {
      grid[i] = gtk_check_button_new_with_label ("Show/hide Y grid");
    }
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(grid[i]), curves[id] -> grid[i]);
    gtk_box_pack_start (GTK_BOX (axisbox[i]), grid[i], TRUE, TRUE, 0);

    if (i == 0)
    {
      autos[i] = gtk_check_button_new_with_label ("Autoscale on X axis");
    }
    else
    {
      autos[i] = gtk_check_button_new_with_label ("Autoscale on Y axis");
    }
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(autos[i]), curves[id] -> autoscale[i]);
    gtk_box_pack_start (GTK_BOX (axisbox[i]), autos[i], TRUE, TRUE, 0);

    if (i == 0)
    {
      dxtitle = gtk_check_button_new_with_label ("Use default X axis title");
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(dxtitle), curves[id] -> xdef);
      gtk_box_pack_start (GTK_BOX (axisbox[i]), dxtitle, TRUE, TRUE, 0);

      ahbox = gtk_hbox_new (FALSE, 0);
      gtk_box_pack_start (GTK_BOX (axisbox[i]), ahbox, TRUE, TRUE, 0);

      xtitle = gtk_label_new ("X axis title:");
      gtk_widget_set_sensitive (xtitle, ! curves[id] -> xdef);
      gtk_box_pack_start (GTK_BOX (ahbox), xtitle, TRUE, TRUE, 0);

      titlex = gtk_entry_new ();
      gtk_entry_set_text (GTK_ENTRY(titlex), curves[id] -> xlabel);
      gtk_widget_set_sensitive (titlex, ! curves[id] -> xdef);
      gtk_box_pack_start (GTK_BOX (ahbox), titlex, TRUE, TRUE, 0);
      gtk_widget_set_size_request (titlex, 100, -1);
      gtk_entry_set_alignment (GTK_ENTRY(titlex), 1.0);

      hsep_d = gtk_hseparator_new ();
      gtk_box_pack_start (GTK_BOX (axisbox[i]), hsep_d, TRUE, TRUE, 0);
    }
    gtk_box_pack_start (GTK_BOX(axebox), axisbox[i], TRUE, TRUE, 0);
  }

  axis = gtk_label_new ("Axis");
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (enoote), gtk_notebook_get_nth_page (GTK_NOTEBOOK (enoote), 1), axis);

// The third tab of the notebook
  if (curves[id] -> cend - curves[id] -> cstart - 1 != 0)
  {
    dbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (enoote), dbox);

    setmess = gtk_label_new ("Select data sets to add to the active curve");
    gtk_widget_set_size_request (setmess, -1, 20);
    gtk_box_pack_start (GTK_BOX (dbox), setmess, TRUE, TRUE, 0);

    hsep_c = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (dbox), hsep_c, TRUE, TRUE, 0);

    adj = gtk_adjustment_new (0.0, 0.0, 100.0, 1.0, 10.0, 10.0);
    scrollsets = gtk_scrolled_window_new (NULL, GTK_ADJUSTMENT(adj));
    gtk_widget_set_size_request (scrollsets, 400, 350);
    gtk_box_pack_start (GTK_BOX (dbox), scrollsets, TRUE, TRUE, 0);
    vportsets = gtk_viewport_new (NULL, GTK_ADJUSTMENT(adj));
    gtk_container_add (GTK_CONTAINER(scrollsets), vportsets);
    fixedsets = gtk_fixed_new ();
    gtk_container_add (GTK_CONTAINER(vportsets), fixedsets);

    sets = gtk_vbutton_box_new ();
    gtk_container_add (GTK_CONTAINER(fixedsets), sets);

    if (curves[id] -> cstart == 0 || curves[id] -> cstart == numgr + numsq + numsk)
    {
      i = numgr+numgq;
      allsets = calloc (i, sizeof*allsets);
      setdata * theset = calloc(i, sizeof*theset);
      for ( i = curves[id] -> cstart ; i < curves[id] -> cend ; i ++ )
      {
        if (i != id)
        {
          j = i - curves[id] -> cstart;
          allsets[j] = gtk_check_button_new_with_label (curves[i] -> title);
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(allsets[j]), curves[id] -> extrasets[j]);
          gtk_widget_set_sensitive (allsets[j], GTK_WIDGET_SENSITIVE(curves[i] -> button));
          theset[j].a = cdata;
          theset[j].b = GINT_TO_POINTER(j);
          theset[j].c = chbox_b;
          g_signal_connect ((gpointer)allsets[j], "toggled", G_CALLBACK(set_set), &theset[j]);
          gtk_box_pack_start (GTK_BOX (sets), allsets[j], TRUE, TRUE, 0);
        }
      }
      if (curves[id] -> cstart == 0 && curves[numgr+numsq+numsk] -> ndata != 0)
      {
        gtk_box_pack_start (GTK_BOX (sets), gtk_hseparator_new (), TRUE, TRUE, 0);
        gtk_box_pack_start (GTK_BOX (sets), gtk_label_new("g(r) FFT[S(q) Debye]"), TRUE, TRUE, 0);
        gtk_box_pack_start (GTK_BOX (sets), gtk_hseparator_new (), TRUE, TRUE, 0);
        for ( i = 0 ; i < numgq ; i ++ )
        {
          j = i + numgr;
          allsets[j] = gtk_check_button_new_with_label (curves[i+numgr+numsq+numsk] -> title);
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(allsets[j]), curves[id] -> extrasets[j]);
          gtk_widget_set_sensitive (allsets[j], GTK_WIDGET_SENSITIVE(curves[i+numgr+numsq+numsk] -> button));
          theset[j].a = cdata;
          theset[j].b = GINT_TO_POINTER(j);
          theset[j].c = chbox_b;
          g_signal_connect ((gpointer)allsets[j], "toggled", G_CALLBACK(set_set), &theset[j]);
          gtk_box_pack_start (GTK_BOX (sets), allsets[j], TRUE, TRUE, 0);
        }
      }
      else if (curves[id] -> cstart == numgr+numsq+numsk && curves[0] -> ndata != 0)
      {
        gtk_box_pack_start (GTK_BOX (sets), gtk_hseparator_new (), TRUE, TRUE, 0);
        gtk_box_pack_start (GTK_BOX (sets), gtk_label_new("g(r) real space"), TRUE, TRUE, 0);
        gtk_box_pack_start (GTK_BOX (sets), gtk_hseparator_new (), TRUE, TRUE, 0);
        for ( i = 0 ; i < numgr ; i ++ )
        {
          j = i + numgq;
          allsets[j] = gtk_check_button_new_with_label (curves[i] -> title);
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(allsets[j]), curves[id] -> extrasets[j]);
          gtk_widget_set_sensitive (allsets[j], GTK_WIDGET_SENSITIVE(curves[i] -> button));
          theset[j].a = cdata;
          theset[j].b = GINT_TO_POINTER(j);
          theset[j].c = chbox_b;
          g_signal_connect ((gpointer)allsets[j], "toggled", G_CALLBACK(set_set), &theset[j]);
          gtk_box_pack_start (GTK_BOX (sets), allsets[j], TRUE, TRUE, 0);
        }
      }
    }
    else if (curves[id] -> cstart == numgr || curves[id] -> cstart == numgr + numsq)
    {
      i = numsq+numsk;
      allsets = calloc (i, sizeof*allsets);
      setdata * theset = calloc(i, sizeof*theset);
      for ( i = curves[id] -> cstart; i < curves[id] -> cend ; i ++ )
      {
        if (i != id)
        {
          j = i - curves[id] -> cstart;
          allsets[j] = gtk_check_button_new_with_label (curves[i] -> title);
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(allsets[j]), curves[id] -> extrasets[j]);
          gtk_widget_set_sensitive (allsets[j], GTK_WIDGET_SENSITIVE(curves[i] -> button));
          theset[j].a = cdata;
          theset[j].b = GINT_TO_POINTER(j);
          theset[j].c = chbox_b;
          g_signal_connect ((gpointer)allsets[j], "toggled", G_CALLBACK(set_set), &theset[j]);
          gtk_box_pack_start (GTK_BOX (sets), allsets[j], TRUE, TRUE, 0);
        }
      }
      if (curves[id] -> cstart == numgr && curves[numgr+numsq] -> ndata != 0)
      {
        gtk_box_pack_start (GTK_BOX (sets), gtk_hseparator_new (), TRUE, TRUE, 0);
        gtk_box_pack_start (GTK_BOX (sets), gtk_label_new("S(q) Debye"), TRUE, TRUE, 0);
        gtk_box_pack_start (GTK_BOX (sets), gtk_hseparator_new (), TRUE, TRUE, 0);
        for ( i = 0 ; i < numsk ; i ++ )
        {
          j = i + numsq;
          allsets[j] = gtk_check_button_new_with_label (curves[i+numgr+numsq] -> title);
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(allsets[j]), curves[id] -> extrasets[j]);
          gtk_widget_set_sensitive (allsets[j], GTK_WIDGET_SENSITIVE(curves[i+numgr+numsq] -> button));
          theset[j].a = cdata;
          theset[j].b = GINT_TO_POINTER(j);
          theset[j].c = chbox_b;
          g_signal_connect ((gpointer)allsets[j], "toggled", G_CALLBACK(set_set), &theset[j]);
          gtk_box_pack_start (GTK_BOX (sets), allsets[j], TRUE, TRUE, 0);
        }
      }
      else if (curves[id] -> cstart == numgr+numsq && curves[numgr] -> ndata != 0)
      {
        gtk_box_pack_start (GTK_BOX (sets), gtk_hseparator_new (), TRUE, TRUE, 0);
        gtk_box_pack_start (GTK_BOX (sets), gtk_label_new("S(q) FFT[g(r)]"), TRUE, TRUE, 0);
        gtk_box_pack_start (GTK_BOX (sets), gtk_hseparator_new (), TRUE, TRUE, 0);
        for ( i = 0 ; i < numsq ; i ++ )
        {
          j = i + numsk;
          allsets[j] = gtk_check_button_new_with_label (curves[i+numgr] -> title);
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(allsets[j]), curves[id] -> extrasets[j]);
          gtk_widget_set_sensitive (allsets[j], GTK_WIDGET_SENSITIVE(curves[i+numgr] -> button));
          theset[j].a = cdata;
          theset[j].b = GINT_TO_POINTER(j);
          theset[j].c = chbox_b;
          g_signal_connect ((gpointer)allsets[j], "toggled", G_CALLBACK(set_set), &theset[j]);
          gtk_box_pack_start (GTK_BOX (sets), allsets[j], TRUE, TRUE, 0);
        }
      }
    }
    else
    {
      i = curves[id] -> cend - curves[id] -> cstart;
      allsets = calloc (i, sizeof*allsets);
      setdata * theset = calloc(i, sizeof*theset);
      for ( i = curves[id] -> cstart; i < curves[id] -> cend ; i ++ )
      {
         if (i != id)
         {
           j = i - curves[id] -> cstart;
           allsets[j] = gtk_check_button_new_with_label (curves[i] -> title);
           gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(allsets[j]), curves[id] -> extrasets[j]);
           gtk_widget_set_sensitive (allsets[j], GTK_WIDGET_SENSITIVE(curves[i] -> button));
           theset[j].a = cdata;
           theset[j].b = GINT_TO_POINTER(j);
           theset[j].c = chbox_b;
           g_signal_connect ((gpointer)allsets[j], "toggled", G_CALLBACK(set_set), &theset[j]);
           gtk_box_pack_start (GTK_BOX (sets), allsets[j], TRUE, TRUE, 0);
         }
      }
    }
    addset = gtk_label_new ("Add data set");
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (enoote), gtk_notebook_get_nth_page (GTK_NOTEBOOK (enoote), 2), addset);
  }

// The button area

  dialog_action_img = GTK_DIALOG (edit_image)->action_area;
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_img), GTK_BUTTONBOX_END);

  close = gtk_button_new_from_stock ("gtk-close");
  gtk_dialog_add_action_widget (GTK_DIALOG (edit_image), close, 0);

  g_signal_connect ((gpointer)datcolor, "color-set", G_CALLBACK(set_datacolor), cdata);
  g_signal_connect ((gpointer)backgroundcolor, "color-set", G_CALLBACK(set_backcolor), cdata);
  g_signal_connect ((gpointer)framecolor, "color-set", G_CALLBACK(set_framcolor), cdata);
  g_signal_connect ((gpointer)linewidth, "focus-out-event", G_CALLBACK(to_set_datathick), cdata);
  g_signal_connect ((gpointer)linewidth, "activate", G_CALLBACK(set_datathick), cdata);
  g_signal_connect ((gpointer)glyphsize, "focus-out-event", G_CALLBACK(to_set_glyphsize), cdata);
  g_signal_connect ((gpointer)glyphsize, "activate", G_CALLBACK(set_glyphsize), cdata);
  g_signal_connect ((gpointer)glyphfreq, "focus-out-event", G_CALLBACK(to_set_glyphfreq), cdata);
  g_signal_connect ((gpointer)glyphfreq, "activate", G_CALLBACK(set_glyphfreq), cdata);
  g_signal_connect ((gpointer)dashbox, "changed", G_CALLBACK(set_line), cdata);
  g_signal_connect ((gpointer)glyphbox, "changed", G_CALLBACK(set_glyph), cdata);
  g_signal_connect ((gpointer)displegend, "toggled", G_CALLBACK(data_legend), cdata);
  g_signal_connect ((gpointer)dataspect, "changed", G_CALLBACK(set_aspect), cdata);
  g_signal_connect ((gpointer)displframe, "toggled", G_CALLBACK(set_frame), cdata);
  // Axis related signals
  for ( i=0 ; i < 2 ; i++ )
  {
   dataxe[i].a = id;
   dataxe[i].b = i;
   g_signal_connect ((gpointer)xyp[i], "activate", G_CALLBACK(mod_res), &dataxe[i]);
   g_signal_connect ((gpointer)xyp[i], "focus-out-event", G_CALLBACK(to_mod_res), &dataxe[i]);
   g_signal_connect ((gpointer)lxyc[i], "activate", G_CALLBACK(leg_pos), &dataxe[i]);
   g_signal_connect ((gpointer)lxyc[i], "focus-out-event", G_CALLBACK(to_leg_pos), &dataxe[i]);
   g_signal_connect ((gpointer)ascale[i], "changed", G_CALLBACK(set_scale),&dataxe[i]);
   g_signal_connect ((gpointer)vmin[i], "focus-out-event", G_CALLBACK(to_set_min), &dataxe[i]);
   g_signal_connect ((gpointer)vmin[i], "activate", G_CALLBACK(set_min), &dataxe[i]);
   g_signal_connect ((gpointer)vmax[i], "focus-out-event", G_CALLBACK(to_set_max), &dataxe[i]);
   g_signal_connect ((gpointer)vmax[i], "activate", G_CALLBACK(set_max), &dataxe[i]);
   g_signal_connect ((gpointer)mjt[i], "focus-out-event", G_CALLBACK(to_show_res), &dataxe[i]);
   g_signal_connect ((gpointer)mjt[i], "activate", G_CALLBACK(show_res), &dataxe[i]);
   g_signal_connect ((gpointer)nmi[i], "focus-out-event", G_CALLBACK(to_ediv_res), &dataxe[i]);
   g_signal_connect ((gpointer)nmi[i], "activate", G_CALLBACK(ediv_res), &dataxe[i]);
   g_signal_connect ((gpointer)nmi[i], "value-changed", G_CALLBACK(div_res), &dataxe[i]);
   g_signal_connect ((gpointer)showaxe[i], "toggled", G_CALLBACK(set_axis), &dataxe[i]);
   g_signal_connect ((gpointer)grid[i], "toggled", G_CALLBACK(set_grid), &dataxe[i]);
   g_signal_connect ((gpointer)autos[i], "toggled", G_CALLBACK(set_autoscale), &dataxe[i]);
  }
  g_signal_connect ((gpointer)dxtitle, "toggled", G_CALLBACK(to_xtitle), cdata);
  g_signal_connect ((gpointer)titlex, "focus-out-event", G_CALLBACK(to_set_xtitle), cdata);
  g_signal_connect ((gpointer)titlex, "activate", G_CALLBACK(set_xtitle), cdata);
  g_signal_connect_data ((gpointer)close, "released", G_CALLBACK(gtk_widget_destroy),
                         GTK_OBJECT(edit_image), NULL, G_CONNECT_AFTER | G_CONNECT_SWAPPED);
  g_signal_connect_data ((gpointer)close, "activate", G_CALLBACK(gtk_widget_destroy),
                         GTK_OBJECT(edit_image), NULL, G_CONNECT_AFTER | G_CONNECT_SWAPPED);
  g_signal_connect ((gpointer)edit_image, "delete_event", G_CALLBACK(gtk_widget_destroy), NULL);
  gtk_widget_show_all(edit_image);
  gtk_dialog_run (GTK_DIALOG(edit_image));

  for (i=0 ; i<2; i++)
  {
    xyp[i] = NULL;
    xyl[i] = NULL;
    lxyc[i] = NULL;
    vmax[i] = NULL;
    vmin[i] = NULL;
    mjt[i] = NULL;
    nmi[i] = NULL;
  }
}
