! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

INTEGER FUNCTION rundmtx (VABAB, VHOMO)

USE PARAMETERS

INTEGER, INTENT(IN) :: VABAB, VHOMO
LOGICAL :: DMTXOK=.false.

INTERFACE
  LOGICAL FUNCTION DISTMTX(NAN, LAN, INFP)
    INTEGER, INTENT(IN) :: NAN, INFP
    INTEGER, DIMENSION(NAN), INTENT(IN) :: LAN
  END FUNCTION
END INTERFACE

ABAB=.false.
if (VABAB .eq. 1) ABAB=.true.
TOMO=.false.
if (VHOMO .eq. 1) TOMO=.true.

DMTXOK=DISTMTX(NA, LOT, 0)
if (.not. DMTXOK) then
  if (allocated(VOISJ)) deallocate(VOISJ)
  if (allocated(CONTJ)) deallocate(CONTJ)
  rundmtx=0
  goto 001
endif

rundmtx=1

001 continue

END FUNCTION

LOGICAL FUNCTION DISTMTX(NAN, LAN, INFP)

!
! Compute the distance matrix - neighbors table
!

USE PARAMETERS

INTEGER, INTENT(IN) :: NAN, INFP
INTEGER, DIMENSION(NAN), INTENT(IN) :: LAN
DOUBLE PRECISION :: pro
LOGICAL :: CALCMAT=.false.

if (allocated(VOISJ)) deallocate(VOISJ)
allocate(VOISJ(MAXN,NAN,NS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: DMTX"//CHAR(0), "Table: VOISJ"//CHAR(0))
  DISTMTX=.false.
  goto 001
endif

if (allocated(CONTJ)) deallocate(CONTJ)
allocate(CONTJ(NAN,NS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: DMTX"//CHAR(0), "Table: CONTJ"//CHAR(0))
  DISTMTX=.false.
  goto 001
endif

do i=1, NS
do j=1, NAN
CONTJ(j,i)=0
do k=1, MAXN
VOISJ(k,j,i)=0
enddo
enddo
enddo

MAXBD=0.0d0
MINBD=10.0d0
do k=1, NS

  if (NS .gt. 1) then
    pro=(k-1)/(NS*1.0)
    call setprogress (pro)
  endif

  do i=1, NAN-1

    if (NS .eq. 1) then
      pro=(i-1)/(NAN*1.0)
      call setprogress (pro)
    endif

    do j=i+1, NAN

      if (ABAB.and.TOMO) then
        if (LAN(i).ne.LAN(j)) then
          CALCMAT=.true.
        else
          CALCMAT=.false.
        endif
      else
        CALCMAT=.true.
      endif

      if (CALCMAT) then
        if (INFP .eq. 0) then
          call CALCRIJ(i,j,k,k)
        else
! Box has been developped
! To apply new PBC
          call CALCRIJ(i,j,-2,k)
        endif
! / g(r) Partielles
        if (Dij .le. Gr_CUT(LAN(i),LAN(j))) then
          if (Dij .le. Gr_cutoff) then
            MAXBD=max(Dij,MAXBD)
            MINBD=min(Dij,MINBD)
            CONTJ(i,k)=CONTJ(i,k)+1
            if (CONTJ(i,k) .gt. MAXN) then
              call TOOMUCH(i, "DISTMTX")
              DISTMTX=.false.
              goto 001
            endif
            VOISJ(CONTJ(i,k),i,k)=j
            CONTJ(j,k)=CONTJ(j,k)+1
            if (CONTJ(j,k) .gt. MAXN) then
              call TOOMUCH(j, "DISTMTX")
              DISTMTX=.false.
              goto 001
            endif
            VOISJ(CONTJ(j,k),j,k)=i
          endif
        endif
      endif

    enddo
  enddo
enddo

if (NAN .eq. NA) NBX=1

DISTMTX=.true.

pro=1.0d0
call setprogress (pro)

001 continue

END FUNCTION

SUBROUTINE TOOMUCH(ATI, IDFCT)

INTEGER, INTENT(IN) :: ATI
CHARACTER (LEN=7), INTENT(IN) :: IDFCT
CHARACTER (LEN=15) :: IDATI

call CHARINT(IDATI, ATI)

call show_error ("Too much neighbors for atom "//IDATI(2:LEN_TRIM(IDATI))//" (>20)"//CHAR(0), &
                 "Please check the lattice parameters and/or the atomic coordinates"//CHAR(0), &
                 "Subroutine: "//IDFCT(1:LEN_TRIM(IDFCT))//CHAR(0))

END SUBROUTINE
