! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

INTEGER FUNCTION dvtbox(VABAB)

!
! Create a superlattice - primitive and strong rings analysis
! Size of the superlattice will be NBX*NBX*NBX with NBX >= 3
!

USE PARAMETERS

INTEGER, INTENT(IN) :: VABAB
INTEGER, DIMENSION(:), ALLOCATABLE :: TLOT

INTERFACE
  LOGICAL FUNCTION DISTMTX(NAN, LAN, INFP)
    INTEGER, INTENT(IN) :: NAN, INFP
    INTEGER, DIMENSION(NAN), INTENT(IN) :: LAN
  END FUNCTION
END INTERFACE

ABAB = .false.
if( VABAB .eq. 1) ABAB=.true.

! Set the size of the superlattice
NBX=3

NNA=NBX**3*NA

allocate(TLOT(NNA), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Function: dvtbox"//CHAR(0), "Table: TLOT"//CHAR(0))
  dvtbox=0
  goto 001
endif

allocate(NFULLPOS(NNA,3,NS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Function: dvtbox"//CHAR(0), "Table: NFULLPOS"//CHAR(0))
  dvtbox=0
  goto 001
endif

do i=1, NS

  n=0

  do j=0, NBX-1
    do k=0, NBX-1
      do l=0, NBX-1

        x=j*lvect(1,1)+k*lvect(2,1)+l*lvect(3,1)
        y=j*lvect(1,2)+k*lvect(2,2)+l*lvect(3,2)
        z=j*lvect(1,3)+k*lvect(2,3)+l*lvect(3,3)

        do m=1, NA
          n=n+1
          NFULLPOS(n,1,i)=FULLPOS(m,1,i)+x
          NFULLPOS(n,2,i)=FULLPOS(m,2,i)+y
          NFULLPOS(n,3,i)=FULLPOS(m,3,i)+z
          TLOT(n)=LOT(m)
        enddo

      enddo
    enddo
  enddo
enddo

NNP=(NBX**2 + NBX**2/2 - 1)*NA

! To give its new shape to the MD box
do i=1, 3
  modv(i)=modv(i)*NBX
  do j=1, 3
    lvect(i,j)=lvect(i,j)*NBX
  enddo
enddo

if (DISTMTX(NNA, TLOT, 1)) then
  dvtbox=1
else
  dvtbox=0
endif

001 continue

if (allocated(TLOT)) deallocate(TLOT)
if (allocated(NFULLPOS)) deallocate(NFULLPOS)

END FUNCTION
