! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

SUBROUTINE ESCS(ESCS_STEP)

!
! Edge vs. Corner sharing tetrahedra/bonds distributions
!

USE PARAMETERS

IMPLICIT NONE

INTEGER, INTENT(IN) :: ESCS_STEP
LOGICAL :: J_OK, L_OK

do j=1, NA-1
  do l=j+1, NA

    if (LOT(j) .eq. LOT(l))then
      t=LOT(j)

      do o=1, NSP

! Test if atoms j and l have 4 and only 4 neighbors of an other chemical species.

        J_OK=.false.
        if (LA_COUNT(j,o,ESCS_STEP).eq.4 .and. CONTJ(j,ESCS_STEP).eq.4) J_OK=.true.
        L_OK=.false.
        if (LA_COUNT(l,o,ESCS_STEP).eq.4 .and. CONTJ(l,ESCS_STEP).eq.4) L_OK=.true.

        if (J_OK .and. L_OK) then
          u=0
          do m=1, 4
            do n=1, 4
              if (VOISJ(m,j,ESCS_STEP) .eq. VOISJ(n,l,ESCS_STEP)) then
                u=u+1
              endif
            enddo
          enddo
          if (u.eq.1) then
            CORTA(t,o)=CORTA(t,o)+1
            if (NS .gt. 1) CORNERA(t,o,ESCS_STEP)=CORNERA(t,o,ESCS_STEP)+1
          elseif (u.eq.2) then
            EDGETA(t,o)=EDGETA(t,o)+1
            if (NS .gt. 1) EDGEA(t,o,ESCS_STEP)=EDGEA(t,o,ESCS_STEP)+1
          elseif (u.ge.3) then
            CORTA(t,o)=CORTA(t,o)+1
            EDGETA(t,o)=EDGETA(t,o)+1
            DEFTA(t,o)=DEFTA(t,o)+1
            if (NS .gt. 1) then
!            CORNERA(t,o,ESCS_STEP)=CORNERA(t,o,ESCS_STEP)+1
              EDGEA(t,o,ESCS_STEP)=EDGEA(t,o,ESCS_STEP)+1
              DEFA(t,o,ESCS_STEP)=DEFA(t,o,ESCS_STEP)+1
            endif
          endif
        endif

      enddo
    endif
  enddo
enddo

do j=1, NA
  do o=1, NSP
    if (CONTJ(j,ESCS_STEP).eq.4 .and. (LA_COUNT(j,o,ESCS_STEP) .eq. 4)) then
      if(NS .gt. 1) TDSA(LOT(j),o,ESCS_STEP)=TDSA(LOT(j),o,ESCS_STEP)+1
      TDA(LOT(j),o) = TDA(LOT(j),o)+1
    endif
  enddo
enddo

END SUBROUTINE ESCS

SUBROUTINE EESCS

USE PARAMETERS

IMPLICIT NONE

if (allocated(ETYPEA)) deallocate(ETYPEA)
allocate (ETYPEA(NSP,NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: EESCS"//CHAR(0), "Table: ETYPEA"//CHAR(0))
endif
if (allocated(CTYPEA)) deallocate(CTYPEA)
allocate (CTYPEA(NSP,NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: EESCS"//CHAR(0), "Table: CTYPEA"//CHAR(0))
endif
if (allocated(DETYPEA)) deallocate(DETYPEA)
allocate (DETYPEA(NSP,NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: EESCS"//CHAR(0), "Table: DETYPEA"//CHAR(0))
endif
if (allocated(ETDA)) deallocate(ETDA)
allocate (ETDA(NSP,NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: EESCS"//CHAR(0), "Table: EDTA"//CHAR(0))
endif

do i=1, NSP
do j=1,NSP
  ETYPEA(j,i)=0.0d0
  CTYPEA(j,i)=0.0d0
  DETYPEA(j,i)=0.0d0
  ETDA(j,i)=0.0d0
enddo
enddo

if (allocated(MTABL)) deallocate(MTABL)
allocate(MTABL(NS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: EESCS"//CHAR(0), "Table: MTABL"//CHAR(0))
endif

do j=1, NSP
  do o=1, NSP
    do i=1, NS
      MTABL(i)=EDGEA(j,o,i)
    enddo
    z=0.0d0
    call MOYENNE(MTABL, NS, z)
    call ECT_TYPE(z, MTABL, NS, ETYPEA(j,o))
    do i=1, NS
      MTABL(i)=CORNERA(j,o,i)
    enddo
    z=0.0d0
    call MOYENNE(MTABL, NS, z)
    call ECT_TYPE(z, MTABL, NS, CTYPEA(j,o))
    do i=1, NS
      MTABL(i)=DEFA(j,o,i)
    enddo
    z=0.0d0
    call MOYENNE(MTABL, NS, z)
    call ECT_TYPE(z, MTABL, NS, DETYPEA(j,o))
    do i=1, NS
      MTABL(i)=TDSA(j,o,i)
    enddo
    call MOYENNE(MTABL, NS, z)
    call ECT_TYPE(z, MTABL, NS, ETDA(j,o))
  enddo
enddo

deallocate(MTABL)

END SUBROUTINE
