/* This file is part of I.S.A.A.C.S.

   I.S.A.A.C.S. is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

    I.S.A.A.C.S. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>

#include "global.h"
#include "interface.h"
#include "callbacks.h"
#include "curve.h"
#include "grcall.h"
#include "gqcall.h"

void send_sq_ (int *, double *);
int g_of_r_fft_ (int *, double *, double *, double *, int *);
void smooth_and_save_ (double *, double *, int *, int *, int *);

void recup_sk_ (int * skid, int * end)
{
  int i;
  double * sdata;

  sdata=calloc(* end, sizeof*sdata);
  for (i=0 ; i< * end; i++)
  {
    sdata[i] = curves[* skid] -> data[i];
  }
  send_sq_ (end, sdata);
}

void hide_gq_curves ()
{
  GtkWidget * info;
  int i, j;

  if (curves[0] -> data != NULL)
  {
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "gqcheck");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
    if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispgq");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
    if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "gqfact");
    if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "gqmoother");
    if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
  }
  j= numgr+numsq+numsk;
  i= numgr+numgq+numsq+numsk;
  hide_curves (&j, &i);
  for ( i = j ; i < j+numgq ; i ++ )
  {
    free(curves[i] -> data);
    curves[i] -> data = NULL;
  }
}

G_MODULE_EXPORT void on_deltarq_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *str;
  double delr;
  int i;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "deltarq");
  str=gtk_entry_get_text (EntryGtk);
  delr = atof(str);
  i = delr;
  update_entry_int (EntryGtk, & i);
  if (i != num_delta_rq)
  {
    num_delta_rq = i;
    hide_gq_curves();
  }
}

G_MODULE_EXPORT void on_qmaxfft_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *str;
  double delr;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "qmaxfft");
  str = gtk_entry_get_text (EntryGtk);
  delr = atof(str);
  update_entry_double (EntryGtk, & delr);
  if (delr != qmaxfft)
  {
    qmaxfft = delr;
    hide_gq_curves ();
  }
}

G_MODULE_EXPORT void on_gqfact_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *stq;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "gqfact");
  stq=gtk_entry_get_text (EntryGtk);
  gqfact = atof(stq);
  update_entry_double (EntryGtk, & gqfact);
}
G_MODULE_EXPORT void on_Calcgq_released()
{
  GtkWidget * info;
  GtkWidget * win;
  GtkWidget * pop;

  int i, j;

  if (num_delta_rq < 2)
  {
    show_warning ("You must specify a number δr >= 2");
  }
  else if (qmaxfft > kmax || qmaxfft < kmin)
  {
    show_warning ("You must specify a Qmax to be used in the FFT\nwith Qmin < Qmax[FFT] <= Qmax");
  }
  else
  {
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "gqcheck");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
    gtk_widget_set_sensitive (info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispgq");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
    gtk_widget_set_sensitive (info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "deltarq");
    gtk_widget_set_sensitive(info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "qmaxfft");
    gtk_widget_set_sensitive(info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "gqfact");
    gtk_widget_set_sensitive(info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "gqmoother");
    gtk_widget_set_sensitive(info, 0);
    if (! gqrun)
    {
      i = numgr + numsq + numsk;
      initgr (& i, & numgq, tablegtkgq);
      gqrun = TRUE;
    }
    delta_rq = dmax / num_delta_rq;
    i = 0;
    setcalcs (& i);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Mainstatusbar");
    j=gtk_statusbar_push (GTK_STATUSBAR(info), 0, "Computing g(r)/G(r) from FFT[S(k)]");
    win = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "gqfft");
    pop=show_popup ("Please wait RDF's from FFT[S(k)] calculation in progress", win);
    gtk_widget_show_all(pop);
    gtk_window_set_opacity (GTK_WINDOW (win), opac);
    i = g_of_r_fft_ (& num_delta_rq, & delta_rq, & qmaxfft, & kmax, & num_delta_k);
    gtk_widget_destroy (pop);
    gtk_window_set_opacity (GTK_WINDOW (win), 1.0);
    gtk_statusbar_remove (GTK_STATUSBAR(info), 0, j);
    if (i != 1)
    {
      show_error ("The RDF's from FFT[S(k)] calculation has failed");
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "gqfact");
      gtk_widget_set_sensitive(info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "gqmoother");
      gtk_widget_set_sensitive(info, 0);
    }
    else
    {
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "gqcheck");
      gtk_widget_set_sensitive (info, 1);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 1);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispgq");
      gtk_widget_set_sensitive (info, 1);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 1);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "gqfact");
      gtk_widget_set_sensitive(info, 1);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "gqmoother");
      gtk_widget_set_sensitive(info, 1);

    }
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "deltarq");
    gtk_widget_set_sensitive(info, 1);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "qmaxfft");
    gtk_widget_set_sensitive(info, 1);
    i = 1;
    update_gr();
    setcalcs (& i);
  }
}

G_MODULE_EXPORT void on_gqmoother_released (GtkButton * button)
{
  int i, j, k, l;

  if (gqfact == 0.0 || gqfact > 1.0)
  {
    show_warning ("You must specify a smoothing factor\n0.0 < sfact < 1.0");
  }
  else
  {
    l = 3;
    i = numgr+numsq+numsk+1;
    smooth_and_save_ (curves[i-1] -> data, & gqfact, & i, & num_delta_rq, & l);
    i = i+2;
    smooth_and_save_ (curves[i-1] -> data, & gqfact, & i, & num_delta_rq, & l);
    i = i+2;
    smooth_and_save_ (curves[i-1] -> data, & gqfact, & i, & num_delta_rq, & l);
    i = i+2;
    smooth_and_save_ (curves[i-1] -> data, & gqfact, & i, & num_delta_rq, & l);
    k = i+2;
    for (i=0 ; i<nspec ; i++)
    {
      for (j=0 ; j<nspec ; j++)
      {
        smooth_and_save_ (curves[k-1] -> data, & gqfact, & k, & num_delta_rq, & l);
        k = k +2;
        smooth_and_save_ (curves[k-1] -> data, & gqfact, & k, & num_delta_rq, & l);
        k = k +3;
      }
    }
    if (nspec == 2)
    {
      for (j=0 ; j<3; j++)
      {
        smooth_and_save_ (curves[k-1] -> data, & gqfact, & k, & num_delta_rq, & l);
        k = k +2;
      }
    }
    update_gr();
  }
}


G_MODULE_EXPORT void on_gqcheck_toggled (GtkToggleButton * Button)
{
  GtkWidget * info;
  gboolean status;

  status = gtk_toggle_button_get_active (Button);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "GQlist");
  if (status)
  {
    if (! GTK_WIDGET_VISIBLE(info))
    {
      gtk_widget_show_all(info);
    }
  }
  else
  {
    if (GTK_WIDGET_VISIBLE(info))
    {
      gtk_widget_hide(info);
    }
  }
}
