! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

INTEGER FUNCTION g_of_r (NDR, DTR, FCR)

! Radial Pair Distribution function

!
!             -2                             V
!    g(r)= rho    < Sum Sum d(ri)d(rj-r) > = -- < Sum Sum d(r-rij) >
!                    i  j!i                  N²    i  j!i
!
!
!
!
!                1                    rho(b)                 rho(b)
!    g  (r)= ------------  Sum Sum  ----------  = Sum Sum ------------------
!     ab     4*PI*Delat(r)  a  b!a  d(ab)²*N(a)    a  b!a  Vshell(d(ab))*N(a)
!

USE PARAMETERS

INTEGER, INTENT(IN) :: NDR, FCR
DOUBLE PRECISION, INTENT(IN) :: DTR
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: GRTAB
DOUBLE PRECISION :: Hcap1, Hcap2, Vcap
DOUBLE PRECISION :: pro

NUMBER_OF_DELTA_R = NDR
DELTA = DTR

if (allocated(SHELL_VOL)) deallocate(SHELL_VOL)
allocate(SHELL_VOL(NUMBER_OF_DELTA_R+1), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: g_of_r"//CHAR(0), "Table: SHELL_VOL"//CHAR(0))
  g_of_r=0
  goto 001
endif
if (allocated(Gij)) deallocate(Gij)
allocate(Gij(NUMBER_OF_DELTA_R+1,NSP,NSP,NS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: g_of_r"//CHAR(0), "Table: Gij"//CHAR(0))
  g_of_r=0
  goto 001
endif
if (allocated(Dn)) deallocate(Dn)
allocate(Dn(NUMBER_OF_DELTA_R+1,NSP,NSP,NS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: g_of_r"//CHAR(0), "Table: Dn"//CHAR(0))
  g_of_r=0
  goto 001
endif
if (allocated(R_POINT)) deallocate(R_POINT)
allocate(R_POINT(NUMBER_OF_DELTA_R), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: g_of_r"//CHAR(0), "Table: R_POINT"//CHAR(0))
  g_of_r=0
  goto 001
endif

do i=1, NS
do j=1, NSP
do k=1, NSP
do l=1, NUMBER_OF_DELTA_R+1
  Gij(l,k,j,i)=0.0d0
  Dn(l,k,j,i)=0.0d0
enddo
enddo
enddo
enddo

if (GLASS) then
  BOX = modv(1)/2.0
else
  BOX = min(modv(1),modv(2))
  BOX = min(BOX,modv(3))
  BOX = BOX/2.0
endif

do i=1, NUMBER_OF_DELTA_R+1
  SHELL_VOL(i) = 0.0d0
  SHELL_VOL(i) = 4.0d0*PI*(((i-1)*DELTA+DELTA)**3 - ((i-1)*DELTA)**3)/3
! To take into account the atoms between a/2 and a srqt(3)/2
! We need the small volume they can be found in.
  if (GLASS .and. i*DELTA.gt.BOX) then
    Hcap1=i*DELTA-BOX
    if ((i-1)*DELTA .le. BOX) then
      Hcap2=0.0d0
    else
      Hcap2=(i-1)*DELTA-BOX
    endif
    Vcap=Hcap1**2*(3*i*DELTA - Hcap1) - Hcap2**2*(3*(i-1)*DELTA - Hcap2)
    Vcap=Vcap*PI*2
    SHELL_VOL(i)=SHELL_VOL(i)-Vcap
  endif
enddo

do i=1, NUMBER_OF_DELTA_R
  R_POINT(i) = 0.0d0
  R_POINT(i) = (i-0.5)*DELTA
enddo

do k=1, NS
  if (NS .gt. 1) then
    pro=(k-1)/(1.0*NS)
    call setprogress (pro)
  endif
  do i=1, NA-1
    if (NS .eq. 1) then
      pro=(i-1)/(1.0*NA)
      call setprogress (pro)
    endif
    do j=i+1, NA
      call CALCRIJ(i,j,k,k)
      if (Dij <= NUMBER_OF_DELTA_R*DELTA) then
        if (LOT(i) .eq. LOT(j)) then
          NUMBER_OF_I = NBSPBS(LOT(i)) -1
        else
          NUMBER_OF_I = NBSPBS(LOT(i))
        endif
        GR_INDEX=int(Dij/DELTA)+1
        NORM_FACT = 1.0d0/(SHELL_VOL(GR_INDEX)*dble(NUMBER_OF_I))
        Gij(GR_INDEX,LOT(i),LOT(j),k) = Gij(GR_INDEX,LOT(i),LOT(j),k) + NORM_FACT/VOL_DENSITY(LOT(j))
        Dn(GR_INDEX,LOT(i),LOT(j),k) = Dn(GR_INDEX,LOT(i),LOT(j),k) + 1.0d0
      endif
    enddo
  enddo
enddo

if (allocated(Gr_ij)) deallocate(Gr_ij)
allocate(Gr_ij(NUMBER_OF_DELTA_R,NSP,NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: g_of_r"//CHAR(0), "Table: Gr_ij"//CHAR(0))
  g_of_r=0
  goto 001
endif
if (allocated(Dn_ij)) deallocate(Dn_ij)
allocate(Dn_ij(NUMBER_OF_DELTA_R,NSP,NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: g_of_r"//CHAR(0), "Table: Dn_ij"//CHAR(0))
  g_of_r=0
  goto 001
endif

do i=1, NUMBER_OF_DELTA_R
  do j=1, NSP
  do k=1, NSP
    Gr_ij(i,j,k)=0.0d0
    Dn_ij(i,j,k)=0.0d0
  enddo
  enddo
enddo

do i=1, NUMBER_OF_DELTA_R
  do l=1, NS
    do j=1, NSP
      Dn(i,j,j,l) = 2.0d0*Dn(i,j,j,l)/dble(NBSPBS(j)-1)
    enddo
    do j=1, NSP-1
      do k=j+1, NSP
        Dn(i,j,k,l) = Dn(i,j,k,l)+Dn(i,k,j,l)
        Dn(i,k,j,l) = Dn(i,j,k,l)
        Dn(i,j,k,l) = Dn(i,j,k,l)/dble(NBSPBS(j))
        Dn(i,k,j,l) = Dn(i,k,j,l)/dble(NBSPBS(k))
      enddo
    enddo
  enddo
enddo

do i=2, NUMBER_OF_DELTA_R
  do l=1, NS
    do j=1, NSP
      do k=1, NSP
        Dn(i,j,k,l) = Dn(i-1,j,k,l) + Dn(i,j,k,l)
      enddo
    enddo
  enddo
enddo

do i=1, NSP
  do j=1, NSP
    do k=1, NUMBER_OF_DELTA_R
      do l=1, NS
          Dn_ij(k,i,j) = Dn_ij(k,i,j) + Dn(k,i,j,l)
          Gr_ij(k,i,j) = Gr_ij(k,i,j) + Gij(k,i,j,l)
      enddo
      Gr_ij(k,i,j) = Gr_ij(k,i,j)/dble(NS)
      Dn_ij(k,i,j) = Dn_ij(k,i,j)/dble(NS)
    enddo
  enddo
enddo

do k=1, NUMBER_OF_DELTA_R
  do l=1, NSP
    Gr_ij(k,l,l) = 2.0d0*Gr_ij(k,l,l)
  enddo
  do j=1, NSP-1
    do i=j+1, NSP
      Gr_ij(k,i,j) = Gr_ij(k,i,j) + Gr_ij(k,j,i)
      Gr_ij(k,j,i) = Gr_ij(k,i,j)
    enddo
  enddo
enddo

deallocate(Dn)
deallocate(Gij)

if (allocated(GRTAB)) deallocate(GRTAB)
allocate(GRTAB(NUMBER_OF_DELTA_R), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: g_of_r"//CHAR(0), "Table: GRTAB"//CHAR(0))
  g_of_r=0
  goto 001
endif
if (allocated(Ggr_ij)) deallocate(Ggr_ij)
allocate(Ggr_ij(NUMBER_OF_DELTA_R,NSP,NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: g_of_r"//CHAR(0), "Table: Ggr_ij"//CHAR(0))
  g_of_r=0
  goto 001
endif
do i=1, NUMBER_OF_DELTA_R
  do j=1, NSP
  do k=1, NSP
    Ggr_ij(i,j,k)=0.0d0
  enddo
  enddo
enddo

SUML=0.0d0
XSUML=0.0d0
do k=1, NSP
  SUML=SUML+NSCATTL(k)*Xi(k)
  XSUML=XSUML+XSCATTL(k)*Xi(k)
enddo

l = 8
do i=1, NSP
  do j=1, NSP
    do k=1, NUMBER_OF_DELTA_R
      GRTAB(k)=Gr_ij(k,i,j)
    enddo
    call save_curve (NUMBER_OF_DELTA_R, GRTAB, l)
    l = l+2
    do k=1, NUMBER_OF_DELTA_R
      Ggr_ij(k,i,j) = (Gr_ij(k,i,j)-1)*4*PI*(NA/VOLUME)*(k-0.5)*DELTA
      GRTAB(k) = Ggr_ij(k,i,j)
    enddo
    call save_curve (NUMBER_OF_DELTA_R, GRTAB, l)
    l = l+2
    do k=1, NUMBER_OF_DELTA_R
      GRTAB(k)=Dn_ij(k,i,j)
    enddo
    call save_curve (NUMBER_OF_DELTA_R, GRTAB, l)
    l = l+1
  enddo
enddo

if (NSP .eq. 2) then
  call GRBT(Gr_ij, NUMBER_OF_DELTA_R)
  do k=1, NUMBER_OF_DELTA_R
    GRTAB(k)=BTij(k,1)
  enddo
  call save_curve (NUMBER_OF_DELTA_R, GRTAB, l)
  l = l+2
  do k=1, NUMBER_OF_DELTA_R
    GRTAB(k)=BTij(k,2)
  enddo
  call save_curve (NUMBER_OF_DELTA_R, GRTAB, l)
  l = l+2
  do k=1, NUMBER_OF_DELTA_R
    GRTAB(k)=BTij(k,3)
  enddo
  call save_curve (NUMBER_OF_DELTA_R, GRTAB, l)
  if (allocated(BTij)) deallocate(BTij)
endif

if (allocated(GrTOT)) deallocate(GrTOT)
allocate(GrTOT(NUMBER_OF_DELTA_R), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: g_of_r"//CHAR(0), "Table: GrTOT"//CHAR(0))
  g_of_r=0
  goto 001
endif
if (allocated(GgrTOT)) deallocate(GrTOT)
allocate(GgrTOT(NUMBER_OF_DELTA_R), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: g_of_r"//CHAR(0), "Table: GgrTOT"//CHAR(0))
  g_of_r=0
  goto 001
endif
if (allocated(XGrTOT)) deallocate(XGrTOT)
allocate(XGrTOT(NUMBER_OF_DELTA_R), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: g_of_r"//CHAR(0), "Table: XGrTOT"//CHAR(0))
  g_of_r=0
  goto 001
endif
if (allocated(XGgrTOT)) deallocate(XGgrTOT)
allocate(XGgrTOT(NUMBER_OF_DELTA_R), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: g_of_r"//CHAR(0), "Table: XGgrTOT"//CHAR(0))
  g_of_r=0
  goto 001
endif

do i=1, NUMBER_OF_DELTA_R
  GrTOT(i)=0.0d0
  GgrTOT(i)=0.0d0
  XGrTOT(i)=0.0d0
  XGgrTOT(i)=0.0d0
enddo

SUML=0.0d0
XSUML=0.0d0
do k=1, NSP
  SUML=SUML+NSCATTL(k)*Xi(k)
  XSUML=XSUML+XSCATTL(k)*Xi(k)
enddo

! Weighting factors used in RMC calculations
!write (6, *)
!z=0.0d0
!y=0.0d0
!do k=1, NSP
!do l=1, NSP
!  z=z+NSCATTL(k)*Xi(k)*NSCATTL(l)*Xi(l)/(SUML**2)
!  y=y+XSCATTL(k)*Xi(k)*XSCATTL(l)*Xi(l)/(XSUML**2)
!  write (6, '("Neutrons: ",A2,"-",A2,f15.10)') TL(k), TL(l), NSCATTL(k)*Xi(k)*NSCATTL(l)*Xi(l)/(SUML**2)
!  write (6, '("X-rays: ",A2,"-",A2,f15.10)') TL(k), TL(l), XSCATTL(k)*Xi(k)*XSCATTL(l)*Xi(l)/(XSUML**2)
!enddo
!enddo
!write (6, *)
!write (6, '("Total weight neutron: ",f15.10)') z
!write (6, '("Total weight X-rays: ",f15.10)') y
!write (6, *)
!write (6, '("Neutrons for rmc++ : ")', advance='no')
!do k=1, NSP
!do l=k, NSP
!  if (k.eq.l) then
!    write (6, '(1x,f8.6)', advance='no') NSCATTL(k)*Xi(k)*NSCATTL(l)*Xi(l)/(SUML**2)
!  else
!    write (6, '(1x,f8.6)', advance='no') 2*NSCATTL(k)*Xi(k)*NSCATTL(l)*Xi(l)/(SUML**2)
!  endif
!enddo
!enddo
!write (6, *)
!write (6, '("Xrays for rmc++ : ")', advance='no')
!do k=1, NSP
!do l=k, NSP
!  if (k.eq.l) then
!    write (6, '(1x,f8.6)', advance='no') XSCATTL(k)*Xi(k)*XSCATTL(l)*Xi(l)/(XSUML**2)
!  else
!    write (6, '(1x,f8.6)', advance='no') 2*XSCATTL(k)*Xi(k)*XSCATTL(l)*Xi(l)/(XSUML**2)
!  endif
!enddo
!enddo
!write (6, *)
!write (6, *)

pro=1.0d0
call setprogress (pro)

! Attention bi(NSCATTL(i) dans le code) en fm et bi² en barn = fm*fm*1e-2 = 1e-24cm²

do i=1, NUMBER_OF_DELTA_R
  do j=1, NSP
    do k=1, NSP
      GrTOT(i) = GrTOT(i) + Xi(j)*Xi(k)*NSCATTL(j)*NSCATTL(k)*Gr_ij(i,j,k)
      GgrTOT(i) = GgrTOT(i) + Xi(j)*Xi(k)*NSCATTL(j)*NSCATTL(k)*Ggr_ij(i,j,k)
      XGrTOT(i) = XGrTOT(i) + Xi(j)*Xi(k)*XSCATTL(j)*XSCATTL(k)*Gr_ij(i,j,k)
      XGgrTOT(i) = XGgrTOT(i) + Xi(j)*Xi(k)*XSCATTL(j)*XSCATTL(k)*Ggr_ij(i,j,k)
    enddo
  enddo
  GrTOT(i) = GrTOT(i)/(SUML**2)
  GgrTOT(i) = GgrTOT(i)/(SUML**2)
  XGrTOT(i) = XGrTOT(i)/(XSUML**2)
  XGgrTOT(i) = XGgrTOT(i)/(XSUML**2)
enddo

do i=1, NUMBER_OF_DELTA_R
  GRTAB(i) = GrTOT(i)
enddo
call save_curve (NUMBER_OF_DELTA_R, GRTAB, 0)
do i=1, NUMBER_OF_DELTA_R
  GRTAB(i) = GgrTOT(i)
enddo
call save_curve (NUMBER_OF_DELTA_R, GRTAB, 2)
do i=1, NUMBER_OF_DELTA_R
  GRTAB(i) = XGrTOT(i)
enddo
call save_curve (NUMBER_OF_DELTA_R, GRTAB, 4)
do i=1, NUMBER_OF_DELTA_R
  GRTAB(i) = XGgrTOT(i)
enddo
call save_curve (NUMBER_OF_DELTA_R, GRTAB, 6)

if (FCR .eq. 1) call FITCUTOFFS

g_of_r=1

pro=0.0d0
call setprogress (pro)

001 continue

if (allocated(Dn)) deallocate (Dn)
if (allocated(Gij)) deallocate (Gij)
if (allocated(Dn_ij)) deallocate (Dn_ij)
if (allocated(GRTAB)) deallocate (GRTAB)
if (allocated(GgrTOT)) deallocate (GgrTOT)
if (allocated(XGgrTOT)) deallocate (XGgrTOT)
if (allocated(Ggr_ij)) deallocate (Ggr_ij)

CONTAINS

SUBROUTINE FITCUTOFFS

INTERFACE
  INTEGER FUNCTION CUTFIT (TABTOFIT, NPOINTS)

    INTEGER, INTENT(IN) :: NPOINTS
    DOUBLE PRECISION, DIMENSION(NPOINTS), INTENT(IN) :: TABTOFIT
  END FUNCTION
END INTERFACE

if (allocated(Gr_CUT)) deallocate(Gr_CUT)
allocate(Gr_CUT(NSP,NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: FITCUTOFFS"//CHAR(0), "Table: Gr_CUT"//CHAR(0))
endif
if (allocated(GFFT)) deallocate(GFFT)
allocate(GFFT(NUMBER_OF_DELTA_R), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: FITCUTOFFS"//CHAR(0), "Table: GFFT"//CHAR(0))
endif

do l=1, NSP
  do j=1, NSP
    do k=1, NUMBER_OF_DELTA_R
      GFFT(k)=Gr_ij(k,l,j)
    enddo
    Gr_CUT(l,j) = (CUTFIT(GFFT, NUMBER_OF_DELTA_R) - 0.5) * DELTA
  enddo
enddo
Gr_cutoff = (CUTFIT(GrTOT, NUMBER_OF_DELTA_R) - 0.5) * DELTA

call sendcutoffs(NSP, Gr_cutoff, Gr_CUT)

deallocate(GFFT)

END SUBROUTINE

END FUNCTION

INTEGER FUNCTION CUTFIT (TABTOFIT, NPOINTS)

USE PARAMETERS

INTEGER, INTENT(IN) :: NPOINTS
DOUBLE PRECISION, DIMENSION(NPOINTS), INTENT(IN) :: TABTOFIT

o = 1
p = 0
do while (p .eq. 0)
  r=INT(o+1+ANint(5.0*dble(NPOINTS)/dble(100*NS)))
  if (o.eq.NPOINTS .or. r.gt.NPOINTS) exit
  if (TABTOFIT(o) .le. TABTOFIT(o+1)) then
    o=o+1
  else if (TABTOFIT(o) .le. TABTOFIT(r)) then
    o=o+1
  else if (TABTOFIT(o) .le. 0.05) then
    o=o+1
  else
    p=1
  endif
enddo
p = 0
do while (p .eq. 0)
  r=INT(o+1+ANint(5.0*dble(NPOINTS)/dble(100*NS)))
  if (o.eq.NPOINTS .or. r.gt.NPOINTS) exit
  if (TABTOFIT(o) .le. TABTOFIT(o+1)) then
    o=o+1
  else if (TABTOFIT(o) .le. TABTOFIT(r)) then
    o=o+1
  else
    p=1
  endif
enddo

CUTFIT = o

END FUNCTION

DOUBLE PRECISION FUNCTION fdmax ()

USE PARAMETERS

if (.not. PBC) then
  fdmax=0.0d0
  do i=1, NS
    do j=1, NA-1
      do k=j+1, NA
        call CALCRIJ(j,k,i,i)
        fdmax = max(fdmax, Dij)
      enddo
    enddo
  enddo
else
  fdmax = min (modv(1), modv(2))
  fdmax = min (fdmax, modv(3))
  if (GLASS) then
    fdmax = sqrt(2.0)*fdmax/2.0
  else
    fdmax = fdmax/2.0
  endif
endif

END FUNCTION
