/* This file is part of I.S.A.A.C.S.

   I.S.A.A.C.S. is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

    I.S.A.A.C.S. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>

#include "global.h"
#include "interface.h"
#include "callbacks.h"
#include "curve.h"
#include "grcall.h"
#include "sqcall.h"

int fitc = 0;
int g_of_r_ (int *, double *, int *);
void smooth_and_save_ (double *, double *, int *, int *, int *);

void update_gr ()
{
  int i;

  if (grrun)
  {
    for (i=0; i < numgr; i++)
    {
      if (curves[i] -> window != NULL) update_curve (GINT_TO_POINTER(i));
    }
  }
  if (gqrun)
  {
    for (i= numgr+numsq+numsk; i < numgr+numsq+numsk+numgq; i++)
    {
      if (curves[i] -> window != NULL) update_curve (GINT_TO_POINTER(i));
    }
  }
}

void initgr (int * gstart, int * ngr, GtkTable * tablewidget)
{
  int i, j, k;

  k = * gstart;
  curves[k] -> title = g_strdup_printf("%s", "g(r) neutrons");
  k = k+1;
  curves[k] -> title = g_strdup_printf("%s", "g(r) neutrons - smoothed");
  k = k+1;
  curves[k] -> title = g_strdup_printf("%s", "G(r) neutrons");
  k = k+1;
  curves[k] -> title = g_strdup_printf("%s", "G(r) neutrons - smoothed");
  k = k+1;
  curves[k] -> title = g_strdup_printf("%s", "g(r) X-rays");
  k = k+1;
  curves[k] -> title = g_strdup_printf("%s", "g(r) X-rays - smoothed");
  k = k+1;
  curves[k] -> title = g_strdup_printf("%s", "G(r) X-rays");
  k = k+1;
  curves[k] -> title = g_strdup_printf("%s", "G(r) X-rays - smoothed");
  k = k+1;
  for ( i = 0 ; i < nspec ; i++ )
  {
    for ( j = 0 ; j < nspec ; j++ )
    {
      curves[k] -> title = g_strdup_printf("%s%s%s%s%s", "g(r)[", label[i], ",", label[j], "]");
      k=k+1;
      curves[k] -> title = g_strdup_printf("%s%s%s%s%s", "g(r)[", label[i], ",", label[j], "] - smoothed");
      k=k+1;
      curves[k] -> title = g_strdup_printf("%s%s%s%s%s", "G(r)[", label[i], ",", label[j], "]");
      k=k+1;
      curves[k] -> title = g_strdup_printf("%s%s%s%s%s", "G(r)[", label[i], ",", label[j], "] - smoothed");
      k=k+1;
      curves[k] -> title = g_strdup_printf("%s%s%s%s%s", "dn(r)[", label[i], ",", label[j], "]");
      k=k+1;
    }
  }
  if ( nspec == 2 )
  {
    curves[k] -> title = g_strdup_printf("%s", "BT(r)[NN]");
    k=k+1;
    curves[k] -> title = g_strdup_printf("%s", "BT(r)[NN] - smoothed");
    k=k+1;
    curves[k] -> title = g_strdup_printf("%s", "BT(r)[NC]");
    k=k+1;
    curves[k] -> title = g_strdup_printf("%s", "BT(r)[NC] - smoothed");
    k=k+1;
    curves[k] -> title = g_strdup_printf("%s", "BT(r)[CC]");
    k=k+1;
    curves[k] -> title = g_strdup_printf("%s", "BT(r)[CC] - smoothed");
  }
  j = * gstart;
  for ( k = * gstart ; k < * gstart + * ngr ; k++ )
  {
    addcurwidget (curves[k] -> title, & k, & j, tablewidget);
    curves[k] -> xlabel = g_strdup_printf("%s","r [Å]");
    curves[k] -> cstart = * gstart;
    curves[k] -> cend = j + * ngr;
  }
}

G_MODULE_EXPORT void on_deltar_activate ()
{
  GtkWidget * info;
  GtkEntry * EntryGtk;
  const gchar *str;
  double delr;
  int i, j;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "deltar");
  str=gtk_entry_get_text (EntryGtk);
  delr = atof(str);
  i = delr;
  update_entry_int (EntryGtk, & i);
  if (i != num_delta_r)
  {
    num_delta_r = i;
    if (curves[0] -> data != NULL)
    {
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "grcheck");
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
      if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispgr");
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
      if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "grfact");
      if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "grmoother");
      if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
    }
    j=0;
    i=numgr;
    hide_curves(&j, &i);
    for ( i = 0 ; i < numgr ; i ++ )
    {
      free(curves[i] -> data);
      curves[i] -> data = NULL;
    }
  }
}

G_MODULE_EXPORT void on_grfact_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *stq;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "grfact");
  stq=gtk_entry_get_text (EntryGtk);
  grfact = atof(stq);
  update_entry_double (EntryGtk, & grfact);
}

G_MODULE_EXPORT void on_Calcgr_released()
{
  GtkWidget * info;
  GtkWidget * win;
  GtkWidget * pop;
  gboolean value;
  int i, j;

  if (num_delta_r >= 2)
  {
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "grcheck");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
    gtk_widget_set_sensitive (info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "cutcheck");
    gtk_widget_set_sensitive (info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispgr");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
    gtk_widget_set_sensitive (info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "deltar");
    gtk_widget_set_sensitive(info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "grfact");
    gtk_widget_set_sensitive(info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "grmoother");
    gtk_widget_set_sensitive(info, 0);
    if (! grrun)
    {
      i = 0;
      initgr (& i, & numgr, tablegtkgr);
      grrun = TRUE;
    }
    delta_r = dmax / num_delta_r;
    i = 0;
    setcalcs (& i);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Mainstatusbar");
    j=gtk_statusbar_push (GTK_STATUSBAR(info), 0, "Computing g(r)/G(r)");
    win = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "rdfgr");
    pop=show_popup ("Please wait RDF's calculation in progress", win);
    gtk_widget_show_all(pop);
    gtk_window_set_opacity (GTK_WINDOW (win), opac);
    i = g_of_r_ (& num_delta_r, & delta_r, & fitc);
    gtk_widget_destroy (pop);
    gtk_window_set_opacity (GTK_WINDOW (win), 1.0);
    gtk_statusbar_remove (GTK_STATUSBAR(info), 0, j);
    if (i != 1)
    {
      show_error ("The RDF's calculation has failed");
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqfftrdf");
      value = gtk_action_get_sensitive (GTK_ACTION(info));
      if (value) gtk_action_set_sensitive (GTK_ACTION(info), 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "button-hsq");
      if (GTK_WIDGET_SENSITIVE(info)) gtk_widget_set_sensitive(info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqfft");
      if (GTK_WIDGET_VISIBLE(info)) gtk_widget_hide(info);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "grfact");
      gtk_widget_set_sensitive(info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "grmoother");
      gtk_widget_set_sensitive(info, 0);
    }
    else
    {
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "grcheck");
      gtk_widget_set_sensitive (info, 1);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 1);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispgr");
      gtk_widget_set_sensitive (info, 1);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 1);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "grfact");
      gtk_widget_set_sensitive(info, 1);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "grmoother");
      gtk_widget_set_sensitive(info, 1);

    }
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "cutcheck");
    gtk_widget_set_sensitive (info, 1);
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(info))) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "deltar");
    gtk_widget_set_sensitive(info, 1);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqfftrdf");
    value = gtk_action_get_sensitive (GTK_ACTION(info));
    if (! value) gtk_action_set_sensitive (GTK_ACTION(info), 1);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "button-hsq");
    if (!GTK_WIDGET_SENSITIVE(info)) gtk_widget_set_sensitive(info, 1);
    hide_sq_curves();
    i = 1;
    update_gr();
    setcalcs (& i);
  }
  else
  {
     show_warning ("You must specify a number δr >= 2");
  }
}

void sendcutoffs_ (int * nc, double * totc, double partc[* nc][* nc])
{
  int i, j;
  GtkEntry * info;

  grtotcutoff = * totc;
  for ( i=0; i< nspec; i++)
  {
    for (j=0; j<nspec; j++)
    {
      cutoffs[i][j]=partc[j][i];

    }
  }
  if (dmtx) dmtx=FALSE;
  i=gtk_combo_box_get_active(GTK_COMBO_BOX(bspeca));
  j=gtk_combo_box_get_active(GTK_COMBO_BOX(bspecb));
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "cut");
  update_entry_double (info, & cutoffs[i][j]);
  i=gtk_combo_box_get_active(GTK_COMBO_BOX(aspeca));
  j=gtk_combo_box_get_active(GTK_COMBO_BOX(aspecb));
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "acut");
   update_entry_double (info, & cutoffs[i][j]);
  i=gtk_combo_box_get_active(GTK_COMBO_BOX(rspeca));
  j=gtk_combo_box_get_active(GTK_COMBO_BOX(rspecb));
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "rcut");
  update_entry_double (info, & cutoffs[i][j]);
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "tcut");
  update_entry_double (info, & grtotcutoff);
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "atcut");
  update_entry_double (info, & grtotcutoff);
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "rtcut");
  update_entry_double (info, & grtotcutoff);
}

G_MODULE_EXPORT void on_grmoother_released (GtkButton * button)
{
  int i, j, k, l;

  if (grfact == 0.0 || grfact > 1.0)
  {
    show_warning ("You must specify a smoothing factor\n0.0 < sfact < 1.0");
  }
  else
  {
    l = 2;
    i = 1;
    smooth_and_save_ (curves[0] -> data, & grfact, & i, & num_delta_r, & l);
    i = 3;
    smooth_and_save_ (curves[2] -> data, & grfact, & i, & num_delta_r, & l);
    i = 5;
    smooth_and_save_ (curves[4] -> data, & grfact, & i, & num_delta_r, & l);
    i = 7;
    smooth_and_save_ (curves[6] -> data, & grfact, & i, & num_delta_r, & l);
    k = 9;
    for (i=0 ; i<nspec ; i++)
    {
      for (j=0 ; j<nspec ; j++)
      {
        smooth_and_save_ (curves[k-1] -> data, & grfact, & k, & num_delta_r, & l);
        k = k +2;
        smooth_and_save_ (curves[k-1] -> data, & grfact, & k, & num_delta_r, & l);
        k = k +3;
      }
    }
    if (nspec == 2)
    {
      for (j=0 ; j<3; j++)
      {
        smooth_and_save_ (curves[k-1] -> data, & grfact, & k, & num_delta_r, & l);
        k = k +2;
      }
    }
    update_gr();
  }
}

G_MODULE_EXPORT void on_grcheck_toggled (GtkToggleButton * Button)
{
  GtkWidget * info;
  gboolean status;

  status = gtk_toggle_button_get_active (Button);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "RDFlist");
  if (status)
  {
    if (! GTK_WIDGET_VISIBLE(info))
    {
      gtk_widget_show_all(info);
    }
  }
  else
  {
    if (GTK_WIDGET_VISIBLE(info))
    {
      gtk_widget_hide(info);
    }
  }
}

G_MODULE_EXPORT void on_cutcheck_toggled (GtkToggleButton * Button)
{
  gboolean status;

  status = gtk_toggle_button_get_active (Button);
  if (status)
  {
    fitc = 1;
  }
  else
  {
    fitc = 0;
  }
}
