! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

INTEGER FUNCTION g_of_r_fft (NDR, DTR, MMX, MXQ, NUMQ)

USE PARAMETERS

INTEGER :: NUM_DQ
INTEGER, INTENT(IN) :: NDR, NUMQ
DOUBLE PRECISION :: pro
DOUBLE PRECISION, INTENT(IN) :: DTR, MMX, MXQ

NUMBER_OF_DELTA_RQ = NDR
DELTA_RQ = DTR

if (allocated(GFFT)) deallocate(GFFT)
allocate(GFFT(NUMBER_OF_DELTA_RQ), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: g_of_r_fft"//CHAR(0), "Table: GFFT"//CHAR(0))
  g_of_r_fft=0
  goto 001
endif
if (allocated(DFFT)) deallocate(DFFT)
allocate(DFFT(NUMBER_OF_DELTA_RQ), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: g_of_r_fft"//CHAR(0), "Table: DFFT"//CHAR(0))
  g_of_r_fft=0
  goto 001
endif
if (allocated(F_POINT)) deallocate(F_POINT)
allocate(F_POINT(NUMBER_OF_DELTA_RQ), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: g_of_r_fft"//CHAR(0), "Table: F_POINT"//CHAR(0))
  g_of_r_fft=0
  goto 001
endif

i=0
do j=1, NUMQ
  z = (j-0.5)*MXQ/NUMQ
  if (z .le. MMX) i=i+1
enddo
NUM_DQ=i

if (allocated(R_PFFT)) deallocate(R_PFFT)
allocate(R_PFFT(NUMBER_OF_DELTA_RQ), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: g_of_r_fft"//CHAR(0), "Table: R_PFFT"//CHAR(0))
  g_of_r_fft=0
  goto 001
endif

do i=1, NUMBER_OF_DELTA_RQ
  R_PFFT(i)= 0.0d0
  R_PFFT(i)= 1.0 / (2.0*PI*PI*NA/VOLUME*(i-0.5)*DELTA_RQ)
  F_POINT(i) = 0.0d0
  F_POINT(i) = (i-0.5)*DELTA_RQ
enddo

if (allocated(S)) deallocate(S)
allocate(S(NUM_DQ), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: g_of_r_fft"//CHAR(0), "Table: S"//CHAR(0))
  g_of_r_fft=0
  goto 001
endif

j=GRNUM+SQNUM
l=GRNUM+SQNUM+SKNUM
call recup_sk (j, NUM_DQ)

pro=1.0/(3+NSP)
call setprogress (pro)

j=j+4
call recup_sk (j, NUM_DQ)

pro=2.0/(3+NSP)
call setprogress (pro)

if (NSP .eq. 2) then
  if (allocated(GQBT)) deallocate(GQBT)
  allocate(GQBT(NUMBER_OF_DELTA_RQ,NSP,NSP), STAT=ERR)
  if (ERR .ne. 0) then
    call show_error ("Impossible to allocate memory"//CHAR(0), &
                     "Subroutine: g_of_r_fft"//CHAR(0), "Table: GQBT"//CHAR(0))
    g_of_r_fft=0
    goto 001
  endif
endif

j=j+NSP*NSP*2+2
do n=1, NSP
do m=1, NSP
  j=j+2
  call recup_sk (j, NUM_DQ)
enddo
pro=(2.0+n)/(3+NSP)
call setprogress (pro)
enddo

if (NSP .eq. 2) then
  call GRBT (GQBT, NUMBER_OF_DELTA_RQ)
  do k=1, NUMBER_OF_DELTA_RQ
    GFFT(k)=BTij(k,1)
  enddo
  call save_curve (NUMBER_OF_DELTA_RQ, GFFT, l)
  l = l+2
  do k=1, NUMBER_OF_DELTA_RQ
    GFFT(k)=BTij(k,2)
  enddo
  call save_curve (NUMBER_OF_DELTA_RQ, GFFT, l)
  l = l+2
  do k=1, NUMBER_OF_DELTA_RQ
    GFFT(k)=BTij(k,3)
  enddo
  call save_curve (NUMBER_OF_DELTA_RQ, GFFT, l)
  if (allocated(BTij)) deallocate(BTij)
endif

pro=1.0d0
call setprogress (pro)

g_of_r_fft=1

pro=0.0d0
call setprogress (pro)

001 continue

deallocate(GFFT)
deallocate(DFFT)
deallocate(S)
deallocate(R_PFFT)
if (NSP .eq. 2) deallocate(GQBT)

END FUNCTION

SUBROUTINE send_sq (VAL, SDATA)

USE PARAMETERS

INTEGER, INTENT(IN) :: VAL
DOUBLE PRECISION, DIMENSION(VAL), INTENT(IN) :: SDATA

do i=1, VAL
  S(i) = SDATA(i)
enddo

call FFT_TO_GR (VAL, S, GFFT)
call save_curve (NUMBER_OF_DELTA_RQ, GFFT, l)
l=l+2
if (j.ge.GRNUM+SQNUM+6 .and. j.le.GRNUM+SQNUM+6+NSP*NSP*4) then
  if (nsp .eq. 2) then
    do i=1, NUMBER_OF_DELTA_RQ
      GQBT(i,n,m) = GFFT(i)
    enddo
  endif
  do i=1, NUMBER_OF_DELTA_RQ
    DFFT(i)= 4.0*PI*(((i-0.5)*DELTA_RQ)**2)*DELTA_RQ*GFFT(i)/VOLUME
    if (i .gt. 1) DFFT(i)= DFFT(i) + DFFT(i-1)
  enddo
  if (n .eq. m) then
    k = (NBSPBS(n)-1)
  else
    k = NBSPBS(m)
  endif
  do i=1, NUMBER_OF_DELTA_RQ
    DFFT(i)= DFFT(i)*k
  enddo
  do i=1, NUMBER_OF_DELTA_RQ
    GFFT(i)=(GFFT(i)-1.0)*4.0*PI*(NA/VOLUME)*(i-0.5)*DELTA_RQ
  enddo
  call save_curve (NUMBER_OF_DELTA_RQ, GFFT, l)
  l=l+2
  call save_curve (NUMBER_OF_DELTA_RQ, DFFT, l)
  l=l+1
else
  do i=1, NUMBER_OF_DELTA_RQ
    GFFT(i)=(GFFT(i)-1.0)*4.0*PI*(NA/VOLUME)*(i-0.5)*DELTA_RQ
  enddo
  call save_curve (NUMBER_OF_DELTA_RQ, GFFT, l)
  l=l+2
endif

CONTAINS

SUBROUTINE FFT_TO_GR (LTAB, TAB, RTAB)

USE PARAMETERS

INTEGER, INTENT(IN) :: LTAB
DOUBLE PRECISION, DIMENSION(LTAB), INTENT(IN) :: TAB
DOUBLE PRECISION, DIMENSION(NUMBER_OF_DELTA_RQ), INTENT(INOUT) :: RTAB

do i=1, NUMBER_OF_DELTA_RQ
  RTAB(i)= 0.0d0
  do k=1, LTAB-1
    if (TAB(k) .ne. -999999.0) then
      Phi = K_POINT(k)*(i-0.5)*DELTA_RQ
      do o=k+1, LTAB
        if (TAB(o) .ne. -999999.0) then
          RTAB(i) = RTAB(i) + K_POINT(k)*(TAB(k) - 1.0)*sin(Phi)*(K_POINT(o) - K_POINT(k))
          exit
        endif
      enddo
    endif
  enddo
  RTAB(i) = 1.0 + R_PFFT(i) * RTAB(i)
enddo

END SUBROUTINE

END SUBROUTINE
