! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

SUBROUTINE lattice(vectors, vmod, angles, lat, cfrac)

!
! Compute lattice angles from lattice vectors
! Lattice vector modules
! Lattice volume
! Reciprocal lattice parameters
!

USE PARAMETERS

IMPLICIT NONE

DOUBLE PRECISION, INTENT(IN), DIMENSION(3,3) :: vectors
DOUBLE PRECISION, INTENT(IN), DIMENSION(3) :: vmod, angles
INTEGER, INTENT(IN) :: lat, cfrac

! Transition from C/Gtk to Fortran !

GLASS=.false.

if (lat .gt. 0) then

  do i=1, 3
    do j=1, 3
      lvect(j,i) = vectors(i,j)
    enddo
  enddo

  modv(1) = vmod(1)
  modv(2) = vmod(2)
  modv(3) = vmod(3)
  ALPHA = angles(1)
  BETA = angles(2)
  GAMA = angles(3)

  if (lat .eq. 1) then

    modv(1)=sqrt(lvect(1,1)**2+lvect(1,2)**2+lvect(1,3)**2)
    modv(2)=sqrt(lvect(2,1)**2+lvect(2,2)**2+lvect(2,3)**2)
    modv(3)=sqrt(lvect(3,1)**2+lvect(3,2)**2+lvect(3,3)**2)

    ALPHA= (lvect(3,1)*lvect(2,1)+lvect(3,2)*lvect(2,2)+lvect(3,3)*lvect(2,3))/(modv(2)*modv(3))
    BETA = (lvect(1,1)*lvect(3,1)+lvect(1,2)*lvect(3,2)+lvect(1,3)*lvect(3,3))/(modv(1)*modv(3))
    GAMA = (lvect(1,1)*lvect(2,1)+lvect(1,2)*lvect(2,2)+lvect(1,3)*lvect(2,3))/(modv(1)*modv(2))

    if (ALPHA.eq.0.0d0 .and. BETA.eq.0.0d0 .and. GAMA.eq.0.0d0) then
      if (modv(1).eq.modv(2) .and. modv(2).eq.modv(3)) GLASS=.true.
    endif

    if (ALPHA.eq.0.0d0) then
      ALPHA = PI/2.0d0
      SALPHA = 1.0d0
      CALPHA = 0.0d0
    else
      ALPHA = acos(ALPHA)
      SALPHA = sin(ALPHA)
      CALPHA = cos(ALPHA)
    endif
    if (BETA.eq.0.0d0) then
      BETA = PI/2.0d0
      SBETA = 1.0d0
      CBETA = 0.0d0
    else
      BETA = acos(BETA)
      SBETA = sin(BETA)
      CBETA = cos(BETA)
    endif
    if (GAMA.eq.0.0d0) then
      GAMA = PI/2.0d0
      SGAMA = 1.0d0
      CGAMA = 0.0d0
    else
      GAMA = acos(GAMA)
      SGAMA = sin(GAMA)
      CGAMA = cos(GAMA)
    endif

    if (ALPHA.eq.0.0d0 .and. BETA.eq.0.0d0 .and. GAMA.eq.0.0d0) then
  ! If some problems display a GTK error message.
      call show_error ("Problem with the simulation box parameters"//CHAR(0), &
                       "Computed angles are equal to 0.0d0"//CHAR(0), "Subroutine: lattice"//CHAR(0))
    endif

  else

    if (ALPHA.eq.90.0 .and. BETA.eq.90.0 .and. GAMA.eq.90.0) then
      if (modv(1).eq.modv(2) .and. modv(2).eq.modv(3)) GLASS=.true.
    endif

    if (ALPHA.eq.90.0) then
      ALPHA = PI/2.0d0
      SALPHA = 1.0d0
      CALPHA = 0.0d0
    else
      ALPHA = ALPHA*PI/180.0d0
      SALPHA = sin(ALPHA)
      CALPHA = cos(ALPHA)
    endif
    if (BETA.eq.90.0) then
      BETA = PI/2.0d0
      SBETA = 1.0d0
      CBETA = 0.0d0
    else
      BETA = BETA*PI/180.0d0
      SBETA = sin(BETA)
      CBETA = cos(BETA)
    endif
    if (GAMA.eq.90.0) then
      GAMA = PI/2.0d0
      SGAMA = 1.0d0
      CGAMA = 0.0d0
    else
      GAMA = GAMA*PI/180.0d0
      SGAMA = sin(GAMA)
      CGAMA = cos(GAMA)
    endif

    lvect(1,1) = modv(1)
    lvect(1,2) = 0.0d0
    lvect(1,3) = 0.0d0
    lvect(2,1) = modv(2)*CGAMA
    lvect(2,2) = modv(2)*SGAMA
    lvect(2,3) = 0.d0
    lvect(3,1) = modv(3)*CBETA
    LTEMP = (CALPHA - CBETA*CGAMA)/SGAMA
    lvect(3,2) = modv(3)*LTEMP
    lvect(3,3) = modv(3)*sqrt(SBETA*SBETA - LTEMP*LTEMP)

  endif
!  write (*,*) lvect(1,1), lvect(1,2), lvect(1,3)
!  write (*,*) lvect(2,1), lvect(2,2), lvect(2,3)
!  write (*,*) lvect(3,1), lvect(3,2), lvect(3,3)
!  write (*,*) modv(1), modv(2), modv(3)
!  write (*,*) ALPHA*180/PI, BETA*180/PI, GAMA*180/PI
  VOLUME=(lvect(1,2)*lvect(2,3)-lvect(1,3)*lvect(2,2))*lvect(3,1)
  VOLUME=VOLUME+(lvect(1,3)*lvect(2,1)-lvect(1,1)*lvect(2,3))*lvect(3,2)
  VOLUME=VOLUME+(lvect(1,1)*lvect(2,2)-lvect(1,2)*lvect(2,1))*lvect(3,3)
  VOLUME=abs(VOLUME)
!  write (*,*) "VOLUME= ",VOLUME

  do i=1,3
    maxv(i)=max(lvect(1,i),lvect(2,i))
    maxv(i)=max(maxv(i),lvect(3,i))
  enddo

! Reciprocal lattice parameters !

  lrecp(1,1) = (lvect(2,2)*lvect(3,3)-lvect(2,3)*lvect(3,2)) / VOLUME
  lrecp(2,2) = (lvect(1,1)*lvect(3,3)-lvect(1,3)*lvect(3,1)) / VOLUME
  lrecp(3,3) = (lvect(1,1)*lvect(2,2)-lvect(1,2)*lvect(2,1)) / VOLUME
  lrecp(1,2) = (lvect(3,2)*lvect(1,3)-lvect(1,2)*lvect(3,3)) / VOLUME
  lrecp(1,3) = (lvect(1,2)*lvect(2,3)-lvect(2,2)*lvect(1,3)) / VOLUME
  lrecp(2,3) = (lvect(1,3)*lvect(2,1)-lvect(1,1)*lvect(2,3)) / VOLUME
  lrecp(2,1) = (lvect(2,3)*lvect(3,1)-lvect(2,1)*lvect(3,3)) / VOLUME
  lrecp(3,1) = (lvect(2,1)*lvect(3,2)-lvect(3,1)*lvect(2,2)) / VOLUME
  lrecp(3,2) = (lvect(3,1)*lvect(1,2)-lvect(1,1)*lvect(3,2)) / VOLUME

  lrecp = 2*PI*lrecp

! modules of the reciprocal lattice vectors

  modr(1)=sqrt(lrecp(1,1)**2+lrecp(1,2)**2+lrecp(1,3)**2)
  modr(2)=sqrt(lrecp(2,1)**2+lrecp(2,2)**2+lrecp(2,3)**2)
  modr(3)=sqrt(lrecp(3,1)**2+lrecp(3,2)**2+lrecp(3,3)**2)

  minr=min(modr(1),modr(2))
  minr=min(minr,modr(3))
! minr can be modified later if PBC are applied

! Creation of the matrix to convert fractional to cartesian coordinates

  z=sqrt(abs(1 - CALPHA*CALPHA-CBETA*CBETA-CGAMA*CGAMA + 2*CALPHA*CBETA*CGAMA))
  z=z/SGAMA
  fractocart(1,1)=modv(1)/(2.0**(cfrac))
  fractocart(1,2)=0.0d0
  fractocart(1,3)=0.0d0
  fractocart(2,1)=modv(2)*CGAMA/(2.0**(cfrac))
  fractocart(2,2)=modv(2)*SGAMA/(2.0**(cfrac))
  fractocart(2,3)=0.0d0
  fractocart(3,1)=modv(3)*CBETA/(2.0**(cfrac))
  fractocart(3,2)=modv(3)*((CALPHA-CBETA*CGAMA)/SGAMA)/(2.0**(cfrac))
  fractocart(3,3)=modv(3)*z/(2.0**(cfrac))

! Creation of the matrix to convert cartesian to fractional coordinates

  carttofrac(1,1)=1.0d0/fractocart(1,1)
  carttofrac(1,2)=0.0d0
  carttofrac(1,3)=0.0d0
  carttofrac(2,1)=-CGAMA/(SGAMA*(modv(1)/(2.0**(cfrac))))
  carttofrac(2,2)=1.0d0/fractocart(2,2)
  carttofrac(2,3)=0.0d0
  carttofrac(3,1)=((modv(2)/(2.0**(cfrac)))*(modv(3)/(2.0**(cfrac))))/VOLUME
  carttofrac(3,1)=carttofrac(3,1) * (CALPHA*CGAMA - CBETA)/SALPHA
  carttofrac(3,2)=((modv(1)/(2.0**(cfrac)))*(modv(3)/(2.0**(cfrac))))/VOLUME
  carttofrac(3,2)=carttofrac(3,2) * (CBETA*CGAMA - CALPHA)/SGAMA
  carttofrac(3,3)=1.0d0/fractocart(3,3)

else

  ALPHA=0.0
  BETA=0.0
  GAMA=0.0
  VOLUME=0.0d0
  do i=1, 3
    modv(i)=0.0d0
    modr(i)=0.0d0
    do j=1, 3
      lvect(i,j)=0.0d0
      lrecp(i,j)=0.0d0
      fractocart(i,j)=0.0d0
    enddo
  enddo

endif

END SUBROUTINE
