/* This file is part of I.S.A.A.C.S.

   I.S.A.A.C.S. is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

    I.S.A.A.C.S. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>. */

#include <glib.h>
#include <glib/gi18n.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <locale.h>

#if defined (HAVE_CONFIG_H)
#  include <config.h>
#endif

#include "version.h"
#include "global.h"
#include "callbacks.h"
#include "interface.h"
#include "xmlrw.h"
/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

#ifdef G_OS_WIN32
#  include <windows.h>
#endif

#ifdef OSX
#  include <gtkosxapplication.h>
#endif

void isaacs_init_ ();

void printhelp()
{
  char help[515] = "Usage: isaacs [OPTION]\n       isaacs [FILE]\n\n"
                "Interactive Structure Analysis of Amorphous and Crystalline Systems I.S.A.A.C.S.\n\n"
                "I.S.A.A.C.S. is a scientific program developed in Fortran90/C/Gtk+\n"
                " to analyze the results of computer simulations (MC, RMC, DM ...).\n\n"
                "options:\n"
                "  -v, --version             version informations\n"
                "  -h, --help                display this help message\n"
                "file:\n\n"
                "  I.S.A.A.C.S. project file see <http://www.phy.cmich.edu/people/petkov/3Dstru_anal.html> for details\n";
  char bug[20] = "\nReport a bug to <";
  char eh[4] = ">\n";

  printf("%s", help);
  printf("%s", bug);
  printf("%s", PACKAGE_BUGREPORT);
  printf("%s\n", eh);
}

void printversion()
{
  char scanid[80]="\nInteractive Structure Analysis of Amorphous and Crystalline I.S.A.A.C.S.\n";
  char bug[20] = "\nReport a bug to <";
  char eh[4] = ">\n";

  printf("%s", scanid);
  printf("\n%s version %s :\n", PACKAGE, VERSION);
  printf("FC Compiler: %s\n", FC);
  printf("FC Compiler flags: %s\n", FCFLAGS);
  printf("C Compiler: %s\n", CC);
  printf("C Compiler flags: %s\n", CFLAGS);
  printf("%s", bug);
  printf("%s", PACKAGE_BUGREPORT);
  printf("%s\n", eh);
}

gboolean
destroy_func (gpointer user_data)
{
  GtkWidget * splashi = (GtkWidget*) user_data;
  gtk_widget_hide (splashi);
  return TRUE;
}

GtkWidget*
create_splash_window ()
{
  GtkWidget *splash_window;
  GtkWidget *image;

  image = gtk_image_new_from_file (PACKAGE_LOGO);
  g_return_val_if_fail (image != NULL, NULL);

  splash_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_type_hint (GTK_WINDOW (splash_window), GDK_WINDOW_TYPE_HINT_SPLASHSCREEN);
  gtk_container_add (GTK_CONTAINER (splash_window), image);
  gtk_window_set_title(GTK_WINDOW(splash_window), "I.S.A.A.C.S.");

  return splash_window;
}

GtkWidget*
create_window (void)
{
  GtkWidget *window;
 /* Creation d'un nouveau GtkBuilder */
  BuilderGtk = gtk_builder_new ();

  if (BuilderGtk != NULL)
  {
    /* Chargement du XML dans p_builder */
    gtk_builder_add_from_file (BuilderGtk, PACKAGE_XML_FILE, & GtkErr);
    if (GtkErr == NULL)
    {
     /* 1.- Recuparation d'un pointeur sur la fenetre. */
      window = GTK_WIDGET(gtk_builder_get_object (BuilderGtk, "MainWindow"));
      THEMOL = gdk_pixbuf_new_from_file(PACKAGE_MOL, NULL);
      return window;
    }
    else
    {
      /* Affichage du message d'erreur de GTK+ */
      g_error ("%s", GtkErr -> message);
      g_error_free (GtkErr);
      quit_gtk ();
      return NULL;
    }
  }
  else
  {
    /* Affichage du message d'erreur de GTK+ */
    show_error("Impossible to create a GtkBuilder");
    quit_gtk ();
    return NULL;
  }
}

void setup_compute ()
{
  GtkAction * action;

  /* G(r) */
  action = (GtkAction *) gtk_builder_get_object (BuilderGtk, "rdf");
  gtk_action_set_sensitive(action, 0);
  /* S(q) FFT[g(r)] */
  action = (GtkAction *) gtk_builder_get_object (BuilderGtk, "sqfftrdf");
  gtk_action_set_sensitive(action, 0);
  /* S(q) k-points */
  action = (GtkAction *) gtk_builder_get_object (BuilderGtk, "sqkpoints");
  gtk_action_set_sensitive(action, 0);
  /* g(r) FFT[S(q) k-points] */
  action = (GtkAction *) gtk_builder_get_object (BuilderGtk, "grfft");
  gtk_action_set_sensitive(action, 0);
  /* Bonds */
  action = (GtkAction *) gtk_builder_get_object (BuilderGtk, "Bonds");
  gtk_action_set_sensitive(action, 0);
  /* Angles */
  action = (GtkAction *) gtk_builder_get_object (BuilderGtk, "Angles");
  gtk_action_set_sensitive(action, 0);
  /* Rings */
  action = (GtkAction *) gtk_builder_get_object (BuilderGtk, "Rings");
  gtk_action_set_sensitive(action, 0);
  /* Spherical harmonics */
  action = (GtkAction *) gtk_builder_get_object (BuilderGtk, "Spheric");
  gtk_action_set_sensitive(action, 0);
  /* Bond valence */
  action = (GtkAction *) gtk_builder_get_object (BuilderGtk, "Valence");
  gtk_action_set_sensitive(action, 0);
  /* MSD */
  action = (GtkAction *) gtk_builder_get_object (BuilderGtk, "Msd");
  gtk_action_set_sensitive(action, 0);
}

gboolean testipf (char * tipf)
{
  size_t larg;
  gboolean rtipf;
  larg=strlen(tipf);

  rtipf = FALSE;
  if (tipf[larg-3]=='i' || tipf[larg-3]=='I')
  {
    if (tipf[larg-2]=='p' || tipf[larg-2]=='P')
    {
      if (tipf[larg-1]=='f' || tipf[larg-1]=='F')
      {
        rtipf=TRUE;
      }
    }
  }
  return rtipf;
}

int main (int argc, char *argv[])
{
  GtkWidget * isaacs;
  GtkWidget * isplash;
  char * IPFILE;
  gboolean RUNC=FALSE;
  gboolean WITHXML=FALSE;
  int i;
  char nopts[43]="Not a valid option ... starting normally\n";

#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);
#endif

#ifdef G_OS_WIN32
  PACKAGE_PREFIX = g_win32_get_package_installation_directory (NULL, NULL);
  PACKAGE_DATA_DIR = g_build_filename (PACKAGE_PREFIX, "pixmaps", NULL);
  PACKAGE_LOCALE_DIR = g_build_filename (PACKAGE_PREFIX, "locale", NULL);
  PACKAGE_XML_FILE = g_build_filename (PACKAGE_PREFIX, "isaacs.gui", NULL);
  PACKAGE_LOGO = g_build_filename (PACKAGE_PREFIX, "pixmaps/logo.png", NULL);
  PACKAGE_PNG = g_build_filename (PACKAGE_PREFIX, "pixmaps/png.png", NULL);
  PACKAGE_PDF = g_build_filename (PACKAGE_PREFIX, "pixmaps/pdf.png", NULL);
  PACKAGE_SVG = g_build_filename (PACKAGE_PREFIX, "pixmaps/svg.png", NULL);
  PACKAGE_VOID = g_build_filename (PACKAGE_PREFIX, "pixmaps/void.png", NULL);
  PACKAGE_EDIT = g_build_filename (PACKAGE_PREFIX, "pixmaps/cedit.png", NULL);
  PACKAGE_MOL = g_build_filename (PACKAGE_PREFIX, "pixmaps/molecule.png", NULL);
  IMG_PIX = g_build_filename (PACKAGE_PREFIX, "pixmaps/img.png", NULL);
#endif

  switch (argc)
  {
    case 1:
     RUNC=TRUE;
     WITHXML=FALSE;
     break;

    case 2:
      if (argv[1][0] == '-')
      {
        if (argv[1][1] == 'h' && argv[1][2] == 0)
        {
          printhelp();
          RUNC=FALSE;
          WITHXML=FALSE;
        }
        else if (argv[1][1] == 'v' && argv[1][2] == 0)
        {
          printversion();
          RUNC=FALSE;
          WITHXML=FALSE;
        }
        else if (argv[1][1] == '-')
        {
          if (argv[1][2]=='h' && argv[1][3]=='e' && argv[1][4]=='l' && argv[1][5]=='p' && argv[1][6] == 0)
          {
            printhelp();
            RUNC=FALSE;
            WITHXML=FALSE;
          }
          else if (argv[1][2]=='v' && argv[1][3]=='e' && argv[1][4]=='r' && argv[1][5]=='s' &&
                   argv[1][6]=='i' && argv[1][7]=='o' && argv[1][8]=='n' && argv[1][9] == 0)
          {
            printversion();
            RUNC=FALSE;
            WITHXML=FALSE;
          }
          else
          {
            printf ("%s",nopts);
            RUNC=TRUE;
            WITHXML=testipf(argv[1]);
            if (WITHXML) IPFILE=argv[1];
          }
        }
        else
        {
          printf ("%s",nopts);
          RUNC=TRUE;
          WITHXML=testipf(argv[1]);
          if (WITHXML) IPFILE=argv[1];
        }
      }
      else
      {
        RUNC=TRUE;
        WITHXML=testipf(argv[1]);
        if (WITHXML) IPFILE=argv[1];
      }
      break;

    case 3:
      if (argv[1][0] == '-')
      {
        if (argv[1][1] == 'i' && argv[1][2] == 0)
        {
          RUNC=TRUE;
          WITHXML=TRUE;
          IPFILE=argv[2];
        }
        else if (argv[1][1] == '-')
        {
          if (argv[1][2]=='i' && argv[1][3]=='p' && argv[1][4]=='f')
          {
            RUNC=TRUE;
            WITHXML=TRUE;
            IPFILE=argv[2];
          }
          else
          {
            printf ("%s",nopts);
            RUNC=TRUE;
            WITHXML=FALSE;
          }
        }
      }
      else
      {
        RUNC=TRUE;
        WITHXML=FALSE;
      }
      break;

    default:
      RUNC=TRUE;
      WITHXML=FALSE;
      break;
  }

  if (RUNC)
  {

    isaacs_init_ ();

/*  Init threads
    g_thread_init (NULL);
    gdk_threads_init ();
    gdk_threads_enter (); */

//  setlocale(LC_ALL,"en_US");
    gtk_disable_setlocale();

  /* Cette fonction est appelée dans toutes les applications GTK.
     Les paramètres passés en ligne de commande sont analysés et
     retournés à l'application. */
    gtk_init (&argc, &argv);

#ifdef G_OS_WIN32
//  FreeConsole();
#endif
#ifdef OSX
   IsaacsOSX = g_object_new (GTKOSX_TYPE_APPLICATION, NULL);
#endif

  /* Création d'une nouvelle fenêtre. */
    isaacs = create_window ();
    setup_compute ();
    gtk_builder_connect_signals (BuilderGtk, NULL);
    gtk_widget_show (isaacs);
#ifdef OSX
  gtkosx_application_set_use_quartz_accelerators (IsaacsOSX, FALSE);
  gtkosx_application_ready (IsaacsOSX);
  GtkWidget * MenuBar = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "MainMenuBar");
  gtk_widget_hide (MenuBar);
  gtkosx_application_set_menu_bar (IsaacsOSX, GTK_MENU_SHELL (MenuBar));
  g_signal_connect (IsaacsOSX, "NSApplicationWillTerminate", G_CALLBACK(show_question), NULL);
#endif
    isplash = create_splash_window ();
    if (isplash == NULL)
    {
      g_print ("Impossible to load the splash screen");
    }
    else
    {
      gtk_widget_show_all (isplash);
      g_timeout_add_seconds (1, destroy_func, isplash);
    }
    if (WITHXML)
    {
      i = open_xml(IPFILE);
      if (i > 1)
      {
        show_error ("Impossible to use the Isaacs-XML project file\n");
      }
    }
    gtk_main ();
#ifdef OSX
    g_object_unref (IsaacsOSX);
#endif
//    gdk_threads_leave ();
  }
  return 0;
}
