/* This file is part of I.S.A.A.C.S.

   I.S.A.A.C.S. is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

    I.S.A.A.C.S. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>

#include "global.h"
#include "interface.h"
#include "callbacks.h"
#include "curve.h"
#include "msdcall.h"

int msd_ (double *, int *);

void initmsd()
{
  int i, j, k;

  j = numgr+numgq+numsq+numsk+numbd+numan+numri+numsh;
  for ( i = 0 ; i < nspec ; i++ )
  {
    curves[j] -> title = g_strdup_printf("%s%s%s", "MSD[", label[i], "]");
    j=j+1;
    curves[j] -> title = g_strdup_printf("%s%s%s", "MSD(nac)[", label[i], "]");
    j=j+1;
  }
  for ( i = 0 ; i < nspec ; i++ )
  {
    curves[j] -> title = g_strdup_printf("%s%s%s", "MSD(x)[", label[i], "]");
    j=j+1;
    curves[j] -> title = g_strdup_printf("%s%s%s", "MSD(y)[", label[i], "]");
    j=j+1;
    curves[j] -> title = g_strdup_printf("%s%s%s", "MSD(z)[", label[i], "]");
    j=j+1;
    curves[j] -> title = g_strdup_printf("%s%s%s", "MSD(xy)[", label[i], "]");
    j=j+1;
    curves[j] -> title = g_strdup_printf("%s%s%s", "MSD(xz)[", label[i], "]");
    j=j+1;
    curves[j] -> title = g_strdup_printf("%s%s%s", "MSD(yz)[", label[i], "]");
    j=j+1;
  }
  for ( i = 0 ; i < nspec ; i++ )
  {
    curves[j] -> title = g_strdup_printf("%s%s%s", "MSD(x/nac)[", label[i], "]");
    j=j+1;
    curves[j] -> title = g_strdup_printf("%s%s%s", "MSD(y/nac)[", label[i], "]");
    j=j+1;
    curves[j] -> title = g_strdup_printf("%s%s%s", "MSD(z/nac)[", label[i], "]");
    j=j+1;
    curves[j] -> title = g_strdup_printf("%s%s%s", "MSD(xy/nac)[", label[i], "]");
    j=j+1;
    curves[j] -> title = g_strdup_printf("%s%s%s", "MSD(xz/nac)[", label[i], "]");
    j=j+1;
    curves[j] -> title = g_strdup_printf("%s%s%s", "MSD(yz/nac)[", label[i], "]");
    j=j+1;
  }
  curves[j] -> title = g_strdup_printf("%s", "Correction[x]");
  j=j+1;
  curves[j] -> title = g_strdup_printf("%s", "Correction[y]");
  j=j+1;
  curves[j] -> title = g_strdup_printf("%s", "Correction[z]");
  j=j+1;
  curves[j] -> title = g_strdup_printf("%s", "Drift[x]");
  j=j+1;
  curves[j] -> title = g_strdup_printf("%s", "Drift[y]");
  j=j+1;
  curves[j] -> title = g_strdup_printf("%s", "Drift[z]");
  i = numgr+numgq+numsq+numsk+numbd+numan+numri+numsh;
  for ( k = i ; k < i+nummsd ; k++ )
  {
    addcurwidget (curves[k] -> title, & k, & i, tablegtkms);
    curves[k] -> xlabel = g_strdup_printf("%s", untime[tunit-1]);
    curves[k] -> cstart = i;
    curves[k] -> cend = i+nummsd;
  }
}

G_MODULE_EXPORT void on_DeltaT_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *str;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "DeltaT");
  str = gtk_entry_get_text (EntryGtk);
  delta_t = atof(str);
  update_entry_double (EntryGtk, & delta_t);
}

G_MODULE_EXPORT void on_Ndtbs_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *str;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "Ndtbs");
  str=gtk_entry_get_text (EntryGtk);
  ndtbs = atof(str);
  update_entry_int (EntryGtk, & ndtbs);
}

G_MODULE_EXPORT void on_TpsUnit_changed (GtkComboBox *TpsCombo)
{
  tunit = gtk_combo_box_get_active (TpsCombo)+1;
}

G_MODULE_EXPORT void on_msdcheck_toggled (GtkToggleButton * Button)
{
  GtkWidget * info;
  gboolean status;

  status = gtk_toggle_button_get_active (Button);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "MSDlist");
  if (status)
  {
    if (! GTK_WIDGET_VISIBLE(info))
    {
      gtk_widget_show_all(info);
    }
  }
  else
  {
    if (GTK_WIDGET_VISIBLE(info))
    {
      gtk_widget_hide(info);
    }
  }
}

gboolean teststep ()
{
  if(steps > 1)
  {
    if (delta_t == 0.0)
    {
      return (FALSE);
    }
    else if (tunit == 0)
    {
      return (FALSE);
    }
    else
    {
      return (TRUE);
    }
  }
  else
  {
    return (TRUE);
  }
}

G_MODULE_EXPORT void on_Calcmsd_released ()
{
  GtkWidget * info;
  GtkWidget * pop;
  GtkWidget * win;
  gboolean test = TRUE;
  int i, j;

  test = teststep ();
  if (! test)
  {
    show_warning ("You must specify the time step δt\n"
                  "used to integrate the Newton's equations\n"
                  "of motion during the dynamics\n");
  }
  else
  {
    if ( ndtbs < 1)
    {
      show_warning ("You must specify the number of steps\n"
                    "between each of the configurations read\n"
                    "in the structure file\n");
    }
    else
    {
      i = 0;
      setcalcs (& i);
      win = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "msd");
      gtk_window_set_opacity (GTK_WINDOW (win), opac);
      if (! msdrun)
      {
        initmsd ();
        msdrun=TRUE;
      }
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispmsd");
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
      gtk_widget_set_sensitive (info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "msdcheck");
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
      gtk_widget_set_sensitive (info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "DeltaT");
      gtk_widget_set_sensitive(info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "TpsUnit");
      gtk_widget_set_sensitive(info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Ndtbs");
      gtk_widget_set_sensitive(info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "MSDlist");
      gtk_widget_set_sensitive(info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Mainstatusbar");
      j=gtk_statusbar_push (GTK_STATUSBAR(info), 9, "Computing MSD");
      pop=show_popup ("Please wait MSD calculation in progress", win);
      gtk_widget_show_all(pop);
      i=msd_ (& delta_t, & ndtbs);
      gtk_widget_destroy(pop);
      gtk_window_set_opacity (GTK_WINDOW (win), 1.0);
      if (i == 1)
      {
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "msdcheck");
        gtk_widget_set_sensitive (info, 1);
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 1);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispmsd");
        gtk_widget_set_sensitive (info, 1);
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 1);
      }
      else
      {
        show_error ("The MSD calculation has failed");
      }
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Mainstatusbar");
      gtk_statusbar_remove (GTK_STATUSBAR(info), 9, j);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "DeltaT");
      gtk_widget_set_sensitive(info, 1);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "TpsUnit");
      gtk_widget_set_sensitive(info, 1);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Ndtbs");
      gtk_widget_set_sensitive(info, 1);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "MSDlist");
      gtk_widget_set_sensitive(info, 1);
      i = 1;
      setcalcs (& i);
    }
  }
}
