! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

!################################################################
!   This is the ISAACS code  main include file which contains:
!           -  Variables
!           -  Structures
!       used all along the code and classified as follow:
!
!           - LOGICAL type variables
!           - LOGICAL (:) type variables
!           - LOGICAL (:,:) type variables
!
!           - INTEGER type variables
!           - INTEGER(:)  type variables
!           - INTEGER(:,:) type variables
!           - INTEGER(:,:,:) type variables
!           - INTEGER(:,:,:,:) type variables
!           - INTEGER(:,:,:,:,:) type variables
!
!           - CHARACTER type variables
!           - CHARACTER(:) type variables
!           - CHARACTER(:,:) type variables
!           - CHARACTER(:,:,:) type variables
!
!           - DOUBLE PRECISION type variables
!
!           - DOUBLE PRECISION type variables
!           - DOUBLE PRECISION(:) type variables
!           - DOUBLE PRECISION(:,:) type variables
!           - DOUBLE PRECISION(:,:,:) type variables
!           - DOUBLE PRECISION(:,:,:,:) type variables
!
!           - STRUCTURES definitions
!
!           - MPI // Only variables
!
!    furthermore inside this first selection variables
!   have been sorted by routines when necessary/possible
!################################################################

MODULE PARAMETERS

USE, INTRINSIC :: iso_c_binding

IMPLICIT NONE

!#################################### LOGICAL VARIABLES ####################################!

!************************************ Outputting ******************************************!

! To be read in the file named 'options' !

LOGICAL :: CALC_RINGS=.false.    ! 1/0 Compute ring statistics
LOGICAL :: CALC_R0=.false.       ! 1/0 Find all existing rings in the box
LOGICAL :: CALC_R1=.false.       ! 1/0 Shortest Path analysis
LOGICAL :: CALC_R2=.false.       ! 1/0 King's analysis
LOGICAL :: CALC_R3=.false.       ! 1/0 SP + Homo
LOGICAL :: CALC_R4=.false.       ! 1/0 King's + Homo
LOGICAL :: CALC_PRINGS=.false.   ! 1/0 Compute primitive ring statistics
LOGICAL :: CALC_STRINGS=.false.  ! 1/0 Compute strongs ring statistics
LOGICAL :: RING_P1=.false.       ! 1/0 Compute first part of detailed ring properties
LOGICAL :: RING_P2=.false.       ! 1/0 Compute second part of detailed ring properties
LOGICAL :: RING_P3=.false.       ! 1/0 Compute third part of detailed ring properties
LOGICAL :: RING_P4=.false.       ! 1/0 Compute fourth part of detailed ring properties
LOGICAL :: RING_P5=.false.       ! 1/0 Compute fifth part of detailed ring properties
LOGICAL :: PBC=.false.           ! 1/0 Enable or disable periodic boundary conditions
LOGICAL :: FRAC=.false.          ! 1/0 Lattice relatives or DOUBLE PRECISIONs positions for atoms
LOGICAL :: ABAB=.false.          ! 1/0 Enable or disable only ABAB rings if more than 3 chemical species
LOGICAL :: TOMO=.false.          ! 1/0 Homopolar or no homopolar bonds in the rings

! utils.f90 !

LOGICAL :: GLASS=.false.         ! 1/0 if the structure is 'cubic' (90/90/90)

! *rings*.f90 !

LOGICAL :: FACTATRING=.false.    ! 1/0 Calculate atomic ring factor using the RN variable (thesis S. LE ROUX)
LOGICAL :: FACTATPNA=.false.     ! 1/0 Calculate atomic ring factor using the PN  variable (thesis S. LE ROUX)
LOGICAL :: FACTATRMAX=.false.    ! 1/0 Calculate atomic ring factor using the Pmax var (thesis S. LE ROUX)
LOGICAL :: FACTATRMIN=.false.    ! 1/0 Calculate atomic ring factor using the Pmin var (thesis S. LE ROUX)
LOGICAL :: RUNSEARCH             ! 1/0 Run ring statistics in the case of 3 or more species
LOGICAL :: ADDSPEC               ! 1/0 Add to chain in the ring case of 3 or more species
LOGICAL :: DOAMPAT=.false.       ! 1/0 if CALC_R1,R2,R3 or R4 to known if there is paths > TAILLD
LOGICAL :: SAUT, SSAUT
LOGICAL :: FOUND
LOGICAL :: PATHOUT
LOGICAL :: FIRR, PIRR
LOGICAL :: HOMO
LOGICAL :: DOSEARCH
LOGICAL :: TBR=.false.
LOGICAL :: ALC=.false.

!##########################################################################################!

!#################################### LOGICAL (:) VARIABLES ####################################!

! allocprims.f90 / primrings.F90 !

LOGICAL, DIMENSION(:), ALLOCATABLE :: CHK, CHK2

!##########################################################################################!

!################################### LOGICAL (:,:) VARIABLES ###################################!

! rings2.f90 / rings4.f90 !

LOGICAL, DIMENSION(:,:), ALLOCATABLE :: STMP

!##########################################################################################!


!###################################### INTEGER VARIABLES #####################################!

!***************************************** Misc *******************************************!

INTEGER :: h=0, i=0, j=0, k=0, l=0, m=0, n=0, o=0, p=0, r=0, t=0, u=0, v=0
INTEGER :: NOA, NOB, NOC, NOD, NOE
INTEGER :: COUNTER, RATE_COUNT, MAX_COUNT
INTEGER :: NSUB, NTSUB, TSUB, ERR
INTEGER :: NL, NC, NSI, NSJ
INTEGER :: NUMBER_OF_I
INTEGER :: DTV, PROBE
INTEGER :: NBONDS, ANBONDS
INTEGER :: NDX, NBX, IDR
INTEGER :: GR_INDEX
INTEGER :: SUM_INDEX
INTEGER :: ANG_I
INTEGER :: FINISH, INDEXMSF
INTEGER :: Id1, Id2, Id, GtBsize
INTEGER :: SPIRR
INTEGER :: L_TOT, LA_TOT
INTEGER :: NUMA, SC
INTEGER :: PATH, NNP, NNA
INTEGER :: LOA, LOB, LOC
INTEGER :: MAXAT, MINAT
INTEGER :: MAXST, MINST
INTEGER :: GRNUM, SQNUM, SKNUM, GQNUM, BDNUM, ANNUM, RINUM, SHNUM, MSNUM

!*************************************** Defined ******************************************!

INTEGER :: NA, NUMBER_OF_A              ! Number of atom in the box
INTEGER :: NSP, NSP_BY_STEP             ! Number of species
INTEGER :: NS, NOS, NSTP                ! Number of DM steps = Number of configurations
INTEGER :: NUMBER_OF_DELTA_R            ! Number of Delta(r) for the evaluation of g(r)
INTEGER :: NUMBER_OF_DELTA_Q            ! Number of Delta(q) for the evaluation of S(k)
INTEGER :: NUMBER_OF_Q_POINTS, NQP      ! Number of q-points for the evaluation of S(k)
INTEGER :: NUMBER_OF_DELTA_RQ           ! Number of Delta(r) g(r)=FFT[S(k)]
INTEGER :: NUMBER_OF_DELTA_D            ! Number of Delta(r) for the bond distribution
INTEGER :: NUMBER_OF_DELTA_ANG          ! Number of Delta(teta) for the angle distribution
INTEGER :: NSBG                         ! Number of step between each geomerty
INTEGER :: TAILLE_REF, TAILLD           ! Depth for rings hunt
INTEGER :: TAILLE, TAILLH, TAILLT       ! Depth for rings hunt
INTEGER :: TLT, NTLT
INTEGER :: NUMBER_OF_QMOD               ! Number of Qvect modulus
INTEGER :: NUMBER_OF_QVECT              ! Number of Qvectors
INTEGER :: LTLT                         ! Ring's hunt species (0=all, 1=highest coord, 2=lowest coord)
INTEGER :: t_p                          ! Node number in MPI / 0 otherwise

INTEGER :: MAXN=20                      ! The maximun number of neighbors an atom can have

!##########################################################################################!


!#################################### INTEGER(:) VARIABLES #####################################!

!************************************* defined ********************************************!

INTEGER, DIMENSION(:), ALLOCATABLE :: LOT
INTEGER, DIMENSION(:), ALLOCATABLE :: NBSPBS               ! Number of atom of species=index_1
INTEGER, DIMENSION(:), ALLOCATABLE :: INDEX_FORM, REFP     ! Formula index

! prepdata.F90 !

INTEGER, DIMENSION(:), ALLOCATABLE :: ATOMID

! bonds.F90 !

INTEGER, DIMENSION(:), ALLOCATABLE :: TOT_GSA
INTEGER, DIMENSION(:), ALLOCATABLE :: TGSA
INTEGER, DIMENSION(:), ALLOCATABLE :: NTETRA

! pathdx.F90 !

INTEGER, DIMENSION(:), ALLOCATABLE :: PNAT, NVAL

! alloc*rings.F90 - rings.F90 !

INTEGER, DIMENSION(:), ALLOCATABLE :: RPAT, CPAT
INTEGER, DIMENSION(:), ALLOCATABLE :: RES_LIST
INTEGER, DIMENSION(:), ALLOCATABLE :: APNA, SPNA
INTEGER, DIMENSION(:), ALLOCATABLE :: VALID

! sk.f90 !

INTEGER, DIMENSION(:), ALLOCATABLE :: degeneracy

! primrings.F90 !

INTEGER, DIMENSION(2) :: PRIMSTAT
INTEGER, DIMENSION(:), ALLOCATABLE :: QUEUE, RINGSTAT, STRRNG
INTEGER, DIMENSION(:), ALLOCATABLE :: NPRING, MATDIST, TOTEST, TOTEST2

! spherical.f90 - valence.f90 !

INTEGER, DIMENSION(:), ALLOCATABLE :: COOSPH, NEIGH
INTEGER, DIMENSION(:), ALLOCATABLE :: NSPSH

!#################################### INTEGER (:,:) VARIABLES ###################################!

! sk.f90 - utils.f90 !

INTEGER, DIMENSION(:,:), ALLOCATABLE :: NFLOT, NFBSP

! alloc*rings.F90 - rings.F90 !

INTEGER, DIMENSION(:,:), ALLOCATABLE, TARGET :: INDR, INDSR
INTEGER, DIMENSION(:,:), ALLOCATABLE, TARGET :: NRING
INTEGER, DIMENSION(:,:), ALLOCATABLE :: VPAT
INTEGER, DIMENSION(:,:), ALLOCATABLE :: INDRAT, NIRRAT, NIRR
INTEGER, DIMENSION(:,:), ALLOCATABLE :: ATRMSX, ATRMSN
INTEGER, DIMENSION(:,:), ALLOCATABLE :: AMPAT, ATRMAX, ATRMIN
INTEGER, DIMENSION(:,:), ALLOCATABLE :: MAXPNA, MINPNA, MAXSNA, MINSNA

! primrings.F90 !

INTEGER, DIMENSION(:,:), ALLOCATABLE :: PRING, SRING, SNIRR
INTEGER, DIMENSION(:,:), ALLOCATABLE :: QUERNG, VREF, TABNUR2
INTEGER, DIMENSION(:,:), ALLOCATABLE :: PRINGORD, PRINGSAV
INTEGER, DIMENSION(:,:), ALLOCATABLE, TARGET :: SNRING
INTEGER, DIMENSION(:,:), POINTER :: TABNUR, TABID
INTEGER, DIMENSION(:,:), ALLOCATABLE :: MATDIST2

! rfact.f90 !

INTEGER, DIMENSION(:,:), ALLOCATABLE :: NBATRING

! bonds.F90 !

INTEGER, DIMENSION(:,:), ALLOCATABLE :: TOT_GEOMSA
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: NUM_GSA
INTEGER, DIMENSION(:,:), ALLOCATABLE :: TETRA
INTEGER, DIMENSION(:,:), ALLOCATABLE :: LISTE_GEOMSA

! dmtx.f90 !

INTEGER, DIMENSION(:,:), ALLOCATABLE :: CONTJ

! rfact.f90 - cdqsr.f90 !

INTEGER, DIMENSION(:,:), ALLOCATABLE :: CONTX

! escs.F90 !

INTEGER, DIMENSION(:,:), ALLOCATABLE :: CORT
INTEGER, DIMENSION(:,:), ALLOCATABLE :: EDGET
INTEGER, DIMENSION(:,:), ALLOCATABLE :: DEFT
INTEGER, DIMENSION(:,:), ALLOCATABLE :: CORTA
INTEGER, DIMENSION(:,:), ALLOCATABLE :: EDGETA
INTEGER, DIMENSION(:,:), ALLOCATABLE :: DEFTA
INTEGER, DIMENSION(:,:), ALLOCATABLE :: TDA

!##########################################################################################!


!#################################### INTEGER VARIABLES #################################!

! bonds.F90 !

INTEGER, DIMENSION(:,:,:), ALLOCATABLE :: LA_COUNT

! escs.F90 !

INTEGER, DIMENSION(:,:,:), ALLOCATABLE :: CORNER
INTEGER, DIMENSION(:,:,:), ALLOCATABLE :: EDGE
INTEGER, DIMENSION(:,:,:), ALLOCATABLE :: DEF
INTEGER, DIMENSION(:,:,:), ALLOCATABLE :: CORNERA
INTEGER, DIMENSION(:,:,:), ALLOCATABLE :: EDGEA
INTEGER, DIMENSION(:,:,:), ALLOCATABLE :: DEFA
INTEGER, DIMENSION(:,:,:), ALLOCATABLE :: TDSA

! dmtx.f90 !

INTEGER, DIMENSION(:,:,:), ALLOCATABLE :: VOISJ

! bonds.f90 !

INTEGER, DIMENSION(:,:,:), ALLOCATABLE :: STATBD

! rfact.f90 - cdqsr.f90 !

INTEGER, DIMENSION(:,:,:), ALLOCATABLE :: VOISX

! allocrings.F90 - rings.F90 !

INTEGER, DIMENSION(:,:,:), ALLOCATABLE :: INDRING, INDSRING, LIRR
INTEGER, DIMENSION(:,:,:), ALLOCATABLE :: ATRING, ATPNA, MATPNA
INTEGER, DIMENSION(:,:,:), ALLOCATABLE :: ATRING2, STPNA
INTEGER, DIMENSION(:,:,:), ALLOCATABLE :: PNA, SNA
INTEGER, DIMENSION(:,:,:), ALLOCATABLE, TARGET :: RINGSAVED

! primrings.F90 !

INTEGER, DIMENSION(:,:,:), ALLOCATABLE, TARGET :: SRINGSAVED
INTEGER, DIMENSION(:,:,:), POINTER :: TABSAVR
INTEGER, DIMENSION(:,:,:), ALLOCATABLE :: TABSAVR2

!##########################################################################################!


!################################## INTEGER(:,:,:,:) VARIABLES #################################!

! angles.f90 !

! allocrings.F90 - rings.F90 !

INTEGER, DIMENSION(:,:,:,:), ALLOCATABLE, TARGET :: RINGORD

! allocprims.f90 - primrings.F90 !

INTEGER, DIMENSION(:,:,:,:), ALLOCATABLE, TARGET :: SRINGORD
INTEGER, DIMENSION(:,:,:,:), POINTER :: TABORDR
INTEGER, DIMENSION(:,:,:,:), ALLOCATABLE :: TABORDR2

!##########################################################################################!



!###################################### CHARACTER VARIABLES ####################################!

! Misc !

CHARACTER (LEN=15) :: sdate, stime, szone                ! Starting date and time
CHARACTER (LEN=15) :: edate, etime, ezone                ! Ending date and time

! misc !

CHARACTER (LEN=2) :: CHOOSE
CHARACTER (LEN=2) :: A1, A2
CHARACTER (LEN=2) :: ATOT
CHARACTER (LEN=20) :: NOM_TMP
CHARACTER (LEN=15) :: NTMP, NTMP2, NTMP3, Nstep, Nomx, Nomy
CHARACTER (LEN=15) :: Nom, Nom1, Nom2, Nom3, Nom4
CHARACTER (LEN=40) :: NomSys, NomS
CHARACTER (LEN=16) :: SPIND
CHARACTER (LEN=15) :: NDGRij_Dnij
CHARACTER (LEN=3) :: writegr
CHARACTER (LEN=16) :: DISTGR

!##########################################################################################!


!##################################### CHAR VARIABLES ###################################!

! Misc !

CHARACTER (LEN=2), DIMENSION(:), ALLOCATABLE :: TL, FTAB          ! Label of species=index_1
CHARACTER (LEN=2), DIMENSION(:), ALLOCATABLE :: TAB_OF_TYPE       ! List of atom by name(species)=indx_1, by DM step=indx_2
CHARACTER (LEN=2), DIMENSION(:), ALLOCATABLE :: LABEL             ! Label of species=index in input

! prepdata.F90 !
CHARACTER (LEN=15), DIMENSION(:), ALLOCATABLE :: ELEMID


!##########################################################################################!

!#################################### CHARACTER (:,:) VARIABLES ##################################!

CHARACTER (LEN=12), DIMENSION(:,:), ALLOCATABLE :: LISTE_GEOMS

!##########################################################################################!

!################################### CHARACTER (:,:,:) VARIABLES #################################!

! CHARACTER (LEN=12), DIMENSION(:,:,:), ALLOCATABLE :: LGEO
CHARACTER (LEN=12), DIMENSION(:,:,:), ALLOCATABLE :: LGEO

!##########################################################################################!



!###################################### DOUBLE PRECISION VARIABLES ####################################!

DOUBLE PRECISION :: ALPHA, BETA, GAMA                            ! Lattice Angles
DOUBLE PRECISION :: CALPHA, SALPHA, CBETA, SBETA, CGAMA, SGAMA   ! Cosinus and Sinus
DOUBLE PRECISION :: LTEMP

!##########################################################################################!



!###################################### double VARIABLES ##################################!

! Misc !

DOUBLE PRECISION :: w=0.0d0, x=0.0d0, y=0.0d0, z=0.0d0
DOUBLE PRECISION, PARAMETER :: PI=acos(-1.0)
DOUBLE PRECISION, PARAMETER :: AVOGADRO=6.02214179d0
DOUBLE PRECISION, PARAMETER :: ANGTOBOHR=0.52917721
DOUBLE PRECISION :: p1, p2, p3
DOUBLE PRECISION :: TIME_TOT, tstat
DOUBLE PRECISION :: BOX, BOXS2
DOUBLE PRECISION :: BASE
DOUBLE PRECISION :: TETA
DOUBLE PRECISION :: DC
DOUBLE PRECISION :: DELTA_ANG
DOUBLE PRECISION :: DELTA_T                             ! Pas de temps du calcul de DM, pour output
DOUBLE PRECISION :: DELTA                               ! Discretisation for the computation of g(r)
DOUBLE PRECISION :: DELTA_Q                             ! Discretisation for the computation of S(k)
DOUBLE PRECISION :: DELTA_RQ                            ! Discretisation for the computation of g(r)
DOUBLE PRECISION :: TOTAL_DENSITY, REAL_DENSITY
DOUBLE PRECISION :: SUM_OF_LENGTH, XSUM_OF_LENGTH
DOUBLE PRECISION :: SUML, XSUML
DOUBLE PRECISION :: minr                                ! Minimum module of reciprocal lattice vectors
DOUBLE PRECISION :: FREEVOL, ECFREEV
DOUBLE PRECISION :: MAXBD, MINBD
DOUBLE PRECISION :: DELTA_DIJ

! gr.F90 !

DOUBLE PRECISION :: Dij, Dil, Din, Vij
DOUBLE PRECISION :: VOLUME
DOUBLE PRECISION :: SHELL_VOLUME
DOUBLE PRECISION :: Gr_cutoff

! sq.f90 !

DOUBLE PRECISION :: Phi, Sinus_phi

! sk.f90 !

DOUBLE PRECISION :: NORM_FACT
DOUBLE PRECISION :: NORM_TOT
DOUBLE PRECISION :: Rmax, Sinus_Fact_Rmax, Fact_Rmax
DOUBLE PRECISION :: SIGMA_LISS
DOUBLE PRECISION :: qvmod, qvmax, qvmin
DOUBLE PRECISION :: QMAX, QMIN

!##########################################################################################!


!#################################### double VARIABLES ##################################!

! Misc !

DOUBLE PRECISION, DIMENSION(3) :: fact
DOUBLE PRECISION, DIMENSION(3) :: Rij, Ril, Rim, Dab, VAR                       ! Position vector
DOUBLE PRECISION, DIMENSION(3) :: R2ij, R2Cor, RCm, RCm2
DOUBLE PRECISION, DIMENSION(3) :: modv, maxv                                    ! Lattice parameters
DOUBLE PRECISION, DIMENSION(3) :: modr                                          ! Reciprocal lattice parameters
DOUBLE PRECISION, DIMENSION(27) :: pDij
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: S_LENGTH
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: VOL_DENSITY                      ! Volumic densities for each type of atom=index_1
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: MASS, M_SS
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: RVDW, R_DW
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: Dcte                             ! Diffusion Cte
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: DIST_IJ, DIST_JI
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: MTABL

! sq.f90 !

DOUBLE PRECISION, DIMENSION(2) :: Gd, Gn, Go, C_Box
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: NSCATTL          ! Neutron Scattering Length of species=index
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: XSCATTL          ! X-ray Scattering Length of species=index
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: S, XS            ! Total structure factor \\ Go=f(G(r))
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: Xi               ! Number fraction of species=index
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: Gr_T
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: Q_POINT, K_POINT

! bonds.F90 !

DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: SA_COUNT

! gr.f90 !

DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: R_POINT
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: GrTOT, GgrTOT    ! Total RDF
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: XGrTOT, XGgrTOT  ! Total RDF
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: SHELL_VOL        ! Shell volume as a function of d=(index-0.5)DELTA

! grfft.f90 !

DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: GFFT, DFFT, R_PFFT, F_POINT

! sk.f90 - utils.f90 !

DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: FNBSPBS
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: modq
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: qvectx, qvecty, qvectz
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: cij, sik

! rfact.f90 !

DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: TOTRING
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: FCTRTOT

! resrings.f90 !

DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: MOYPUR, MOYRED
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: MOYPAT, MOYRAT
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: ECTYPE, ECTYP, ECTYPAT, ECTAT
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: IRRED, RED
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: REDAT, IRRAT
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: TOTPSTEP
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: RNAMAX, RNAMIN
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: EVMAX, EVMIN

!##########################################################################################!


!################################## double VARIABLES ##################################!

! Misc !

DOUBLE PRECISION, DIMENSION(3,3) :: lvect                                      ! Lattice vectors
DOUBLE PRECISION, DIMENSION(3,3) :: fractocart, carttofrac                     ! Conversion matrix
DOUBLE PRECISION, DIMENSION(3,3) :: lrecp                                      ! Reciprocal lattice vectors
DOUBLE PRECISION, DIMENSION(27,3) :: pRij, pvect

DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: EPNA
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: POA, POB                      ! Atm=indx_1, coord=indx_2(1=x, 2=y, 3=z)

! gr.F90 !

DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: Gr_CUT                        ! g(r) cutoff for first neighbors.


! fzbt.f90 !

DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: BTij                         ! Bathia-Thornton Partial g(r)/s(q)

! bonds.F90 !

DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: MA_COUNT

! escs.F90 !

DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: ETYPE
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: CTYPE
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: DETYPE
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: ETYPEA
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: CTYPEA
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: DETYPEA
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: ETDA

! msd.f90 !

DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: DRIFT, COR
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: D2i, D2iNAC

! rfact.f90 !

!DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: FCTRTOT
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: CMRING
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: MATRING

! sk.f90 !

DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: qvect

!##########################################################################################!


!################################# DOUBLE PRECISION (:,:,:) VARIABLES #################################!

! Misc !

DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: FULLPOS
DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: FULLVEL
DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: NFULLPOS, NFPOS
DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: ECART_TYPE

! gr.f90 !

DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: Gr_ij        ! Average of the RDF on all the MD run
DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: Ggr_ij       ! Average of the RDF on all the MD run
DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: Dn_ij        ! Average of the Dn on all the MD run

! grfft.f90 !

DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: GQBT

! sq.f90 !

DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: Sij, Spij            ! Partial structure factor \\ Goij
DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: Ss_ij                ! Partial structure factor \\ (sin(piqr/Rmax)/(piqr/Rmax))

! fzbt.f90 !

DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: FZSij

! msd.f90 !

DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: D2ij, D2ijNAC, D2dir, D2dirNAC
DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: CORNAC

! rfact.f90 !

DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: DRATRING, VATRING, RATRING

! skr.f90 !

DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: BTRSij, FULLBTQ

!##########################################################################################!


!################################ DOUBLE PRECISION (:,:,:,:) VARIABLES ################################!

! gr.F90 !

DOUBLE PRECISION, DIMENSION(:,:,:,:), ALLOCATABLE :: Gij ! RDF distance=index_1, atom_a=index_2, atom_b=index_3, step=index_4
DOUBLE PRECISION, DIMENSION(:,:,:,:), ALLOCATABLE :: Dn  ! Dn distance=index_1, atom_a=index_2, atom_b=index_3, step=index_4

!##########################################################################################!


!################################ Ring Structure Definition ###############################!
                                                                   !
TYPE RING                                                          !
  INTEGER :: INDICE                                                !
  INTEGER :: ELEMENT                                               !
  INTEGER :: NEIGHBOR                                              !      Ring structure definition
  TYPE (RING), POINTER :: PAST                                     ! used in the ring hunt subroutine
  TYPE (RING), POINTER :: NEXT                                     !
END TYPE RING                                                      !
                                                                   !
TYPE (RING), POINTER :: FIRST                                      !
TYPE (RING), POINTER :: ANNEAUX                                    !
TYPE (RING), POINTER :: LINK_A, LINK_B                             !
TYPE (RING), POINTER :: TMP, TMP_SAVE                              !
                                                                   !
!##########################################################################################!


END MODULE PARAMETERS

! ########################################  EOF ###########################################!

