! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

#if defined (HAVE_CONFIG_H)
#  include <config.h>
#endif

INTEGER FUNCTION prep_data ()

!
! Data initialization
!

USE PARAMETERS
USE MENDELEIEV

IMPLICIT NONE

if (allocated(LOT)) deallocate(LOT)
allocate(LOT(NA), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: prep_data"//CHAR(0), "Table: LOT"//CHAR(0))
  prep_data = 0
  goto 001
endif

do NOA=1, NA
  LOT(NOA)=0        ! Initialisation du tableau
enddo

NOC=1
do NOA=1, NA
  if (NOA .eq. 1) then
    LOT(NOA)=NOC
  else if (LOT(NOA) .eq. 0) then
    do NOB=1, NOA
      if (TAB_OF_TYPE(NOA) .eq. TAB_OF_TYPE(NOB)) then
        LOT(NOA)= LOT(NOB)
      endif
    enddo
    if (LOT(NOA) .eq. 0) then
      LOT(NOA)=NOC+1
      NOC=NOC+1
    endif
  endif
enddo
NSP=NOC

if (allocated(NBSPBS)) deallocate(NBSPBS)
allocate(NBSPBS(NSP+1), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: prep_data"//CHAR(0), "Table: NBSPBS"//CHAR(0))
  prep_data = 0
  goto 001
endif
if (allocated(TL)) deallocate(TL)
allocate(TL(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: prep_data"//CHAR(0), "Table: TL"//CHAR(0))
  prep_data = 0
  goto 001
endif
if (allocated(ATOMID)) deallocate(ATOMID)
allocate(ATOMID(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: prep_data"//CHAR(0), "Table: ATOMID"//CHAR(0))
  prep_data = 0
  goto 001
endif
if (allocated(ELEMID)) deallocate(ELEMID)
allocate(ELEMID(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: prep_data"//CHAR(0), "Table: ELEMID"//CHAR(0))
  prep_data = 0
  goto 001
endif
if (allocated(NSCATTL)) deallocate(NSCATTL)
allocate(NSCATTL(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: prep_data"//CHAR(0), "Table: NSCATTL"//CHAR(0))
  prep_data = 0
  goto 001
endif
if (allocated(XSCATTL)) deallocate(XSCATTL)
allocate(XSCATTL(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: prep_data"//CHAR(0), "Table: XSCATTL"//CHAR(0))
  prep_data = 0
  goto 001
endif
if (allocated(MASS)) deallocate(MASS)
allocate(MASS(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: prep_data"//CHAR(0), "Table: MASS"//CHAR(0))
  prep_data = 0
  goto 001
endif
if (allocated(RVDW)) deallocate(RVDW)
allocate(RVDW(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: prep_data"//CHAR(0), "Table: RVDW"//CHAR(0))
  prep_data = 0
  goto 001
endif

do NOA=1, NSP
  NOC=0
  do NOB=1, NA
    if (LOT(NOB) .eq. NOA) then
      NOC=NOC+1
    endif
  enddo
  NBSPBS(NOA)=NOC
enddo

NBSPBS(NSP+1)=NA

!
! Link between index and label
! Correspondance entre indice de type et label
!

do NOB=1, NSP
  do NOA=1, NA
    if (LOT(NOA) .eq. NOB) then
      TL(NOB)=TAB_OF_TYPE(NOA)
      exit
    endif
  enddo
enddo

do NOA=1, NSP
  ATOMID(NOA) = FINDID(TL(NOA))
  if (ATOMID(NOA) .ne. 0) then
    ELEMID(NOA) = ELEMENT(ATOMID(NOA))
    NSCATTL(NOA) = COHEB(ATOMID(NOA))
    XSCATTL(NOA) = ATOMID(NOA)
    if (NSCATTL(NOA).eq.0.00 .and. XSCATTL(NOA).eq.0.00) then
      call show_warning ("Element "//TL(NOA)//" does not have X-rays scattering length "//CHAR(0), &
                         "If this is a bug please report it to"//CHAR(0), PACKAGE_BUGREPORT//CHAR(0))
    elseif (NSCATTL(NOA) .eq. 0.00) then
      call show_warning ("Element "//TL(NOA)//" does not have neutron scattering length "//CHAR(0), &
                         "If this is a bug please report it to"//CHAR(0), PACKAGE_BUGREPORT//CHAR(0))
    elseif (XSCATTL(NOA).eq.0.00) then
      call show_warning ("Element "//TL(NOA)//" does not have X-rays scattering length "//CHAR(0), &
                         "If this is a bug please report it to"//CHAR(0), PACKAGE_BUGREPORT//CHAR(0))
    endif
    MASS(NOA) = AMASS(ATOMID(NOA))
! Covalent radius are the defaut values
    RVDW(NOA) = ARCOV(ATOMID(NOA))
! To use ionic radius uncomment the following line
!    RVDW(NOA) = ARION(ATOMID(NOA))
! To use Wander Waals radius uncomment the following line
!    RVDW(NOA) = ARVDW(ATOMID(NOA))
  else
    prep_data = 0
    goto 001
  endif
enddo

! Now we are calling the GTK+ routines
call initview (NA, NSP, NS)
call initviewspec (NSP)
do i=1, NSP
! In C all string must be terminated by a CHAR(0)
  call specdata (i-1, ATOMID(i), TL(i)//CHAR(0), ELEMID(i)//CHAR(0), &
                 NSCATTL(i), XSCATTL(i), MASS(i), RVDW(i))
enddo
call initcombospec
call display_spec

GRNUM=8+5*NSP*NSP
SQNUM=8+4*NSP*NSP
SKNUM=8+4*NSP*NSP
GQNUM=GRNUM
BDNUM=NSP*NSP
ANNUM=NSP*NSP*NSP+NSP*NSP*NSP*NSP
if (NSP .eq. 2) then
  GRNUM=GRNUM+6
  SQNUM=SQNUM+8
  SKNUM=SKNUM+8
  GQNUM=GQNUM+6
endif
RINUM=20*(NSP+1)
SHNUM=NSP+1
MSNUM=0
if (NS .gt. 1) MSNUM=14*NSP+6

prep_data = 1

001 continue

CONTAINS

FUNCTION FINDID(NAMEAT)

USE MENDELEIEV

INTEGER :: ELEMT
INTEGER :: FINDID
CHARACTER (LEN=2), INTENT(IN) :: NAMEAT

FINDID=0
do ELEMT=1, MAXE

 if (ATSYM(ELEMT) .eq. NAMEAT) then
   FINDID=ELEMT
   exit
 endif

enddo

if (FINDID .eq. 0) then
  call show_error ("Atom "//NAMEAT//" does not exist "//CHAR(0), &
                   "If this is a bug please report it to"//CHAR(0), PACKAGE_BUGREPORT//CHAR(0))
endif

END FUNCTION

END FUNCTION

DOUBLE PRECISION FUNCTION change_radius (SW, RD)

USE MENDELEIEV
USE PARAMETERS

INTEGER, INTENT(IN) :: SW, RD

if (RD .eq. 0) RVDW(SW+1) = ARCOV(ATOMID(SW+1))
! To use ionic radius uncomment the following line
if (RD .eq. 1) RVDW(SW+1) = ARION(ATOMID(SW+1))
! To use Wander Waals radius uncomment the following line
if (RD .eq. 2) RVDW(SW+1) = ARVDW(ATOMID(SW+1))
! To use Wander Waals radius uncomment the following line
if (RD .eq. 3) RVDW(SW+1) = ARCRY(ATOMID(SW+1))

change_radius = RVDW(SW+1)

END FUNCTION

DOUBLE PRECISION FUNCTION new_radius (SW, NRW)

USE MENDELEIEV
USE PARAMETERS

INTEGER, INTENT(IN) :: SW
DOUBLE PRECISION, INTENT(IN) :: NRW

RVDW(SW+1) = NRW
new_radius = RVDW(SW+1)

END FUNCTION

DOUBLE PRECISION FUNCTION new_mass (SM, MAS)

USE PARAMETERS

INTEGER, INTENT(IN) :: SM
DOUBLE PRECISION, INTENT(IN) :: MAS

MASS(SM+1) = MAS
new_mass = MASS(SM+1)

END FUNCTION

DOUBLE PRECISION FUNCTION new_nscatt (SL, NSL)

USE PARAMETERS

INTEGER, INTENT(IN) :: SL
DOUBLE PRECISION, INTENT(IN) :: NSL

NSCATTL(SL+1) = NSL
new_nscatt = NSCATTL(SL+1)

END FUNCTION

DOUBLE PRECISION FUNCTION new_xscatt (XL, XSL)

USE PARAMETERS

INTEGER, INTENT(IN) :: XL
DOUBLE PRECISION, INTENT(IN) :: XSL

XSCATTL(XL+1) = XSL
new_xscatt = XSCATTL(XL+1)

END FUNCTION

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!                Reconstruction des trajectoires réelles
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SUBROUTINE prepos (PINFO, FINFO)

USE PARAMETERS

INTEGER, INTENT(IN) :: PINFO, FINFO

PBC=.false.
if (PINFO .eq. 1) then
  PBC=.true.
endif
FRAC=.false.
if (FINFO .eq. 1) FRAC=.true.

do NOC=1, NS
  do NOA=1, NA
    if (FRAC) then
      x=FULLPOS(NOA,1,NOC)*fractocart(1,1)+FULLPOS(NOA,2,NOC)*fractocart(2,1)+FULLPOS(NOA,3,NOC)*fractocart(3,1)
      y=FULLPOS(NOA,1,NOC)*fractocart(1,2)+FULLPOS(NOA,2,NOC)*fractocart(2,2)+FULLPOS(NOA,3,NOC)*fractocart(3,2)
      z=FULLPOS(NOA,1,NOC)*fractocart(1,3)+FULLPOS(NOA,2,NOC)*fractocart(2,3)+FULLPOS(NOA,3,NOC)*fractocart(3,3)
      FULLPOS(NOA,1,NOC)=x
      FULLPOS(NOA,2,NOC)=y
      FULLPOS(NOA,3,NOC)=z
    endif
  enddo
enddo

END SUBROUTINE
