! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

INTEGER FUNCTION PRIM_DIJKSTRA()

!
! Primitive and strong ring statistics
!

USE PARAMETERS

DOUBLE PRECISION :: pro

INTERFACE
  INTEGER FUNCTION RECRINGS(VES)
    INTEGER, INTENT(IN) :: VES
  END FUNCTION
END INTERFACE

PRIM_DIJKSTRA=0

if (.not.PBC) then
  NBX=1
  NNA=NA
  NNP=0
endif

! Dynamic allocation of pointers and tables used in the "RINGS" subroutine

do i=1, NS

  if (NS .gt. 1) then
    pro=(i-1)/(1.0*NS)
    call setprogress (pro)
  endif

  do j=1, TAILLD
  do k=1, NUMA
  do n=1, TAILLD
    RINGSAVED(j,k,n)=0
  enddo
  enddo
  enddo

  call SETUP_CPAT_VPAT (i, NNA)

  if (CALC_STRINGS) then

    do j=1, TAILLD
    do k=1, NUMA
    do n=1, TAILLD
      SRINGSAVED(j,k,n)=0
    enddo
    enddo
    enddo

  endif

  o=0

  do j=NNP+1, NNP+NA ! atoms-loop

    if (TLT .eq. NSP+1 .or. LOT(j-NNP) .eq. TLT) then

      if (NS .eq. 1 .and. TAILLD.ge.20) then
        pro=(j-NNP)/(1.0*NA)
        call setprogress (pro)
      endif

      do k=1, TAILLD
        APNA(k)=0
      enddo
      MINAT=TAILLD
      MAXAT=1
      SAUT=.true.
      if (CALC_STRINGS) then
        do k=1, TAILLD
          SPNA(k)=0
        enddo
        MINST=TAILLD
        MAXST=1
        SSAUT=.true.
      endif

      o=o+1
      call DIJKSTRA(j)
      if (TBR .or. ALC) goto 001
      do k=1, TAILLE_REF ! ring-sizes-loop

        do l=1, 2
          do n=1, NUMA
            INDR(l,n)=0
          enddo
          if (CALC_STRINGS) then
            do n=1, NUMA
              INDSR(l,n)=0
            enddo
          endif
        enddo

        PATH=0
        do l=1, NNA
          if (MATDIST(l) .eq. k) then
            call SPATH_REC(l,k,k)
          endif
        enddo

        if (mod(PATH,2) .eq. 0) then
          p=PATH/2
          p=p*(PATH-1)
        else
          p=(PATH-1)/2
          p=p*PATH
        endif

        allocate(QUERNG(p,2), STAT=ERR)
        if (ERR .ne. 0) then
          call show_error ("Impossible to allocate memory"//CHAR(0), &
                           "Function: PRIM_DIJKSTRA"//CHAR(0), "Table: QUERNG"//CHAR(0))
          ALC=.true.
          goto 001
        endif

        l=0
        do m=1, PATH-1
          do n=m+1, PATH
            l=l+1
            QUERNG(l,1)=m
            QUERNG(l,2)=n
!           Paths which share atoms ... so with overlap ... are deleted
            PATHOUT=.false.
            do v=1, k-1
              do p=1, k-1
                if (PRINGORD(m,v) .eq. PRINGORD(n,p)) then
                  PATHOUT=.true.
                  exit
                endif
              enddo
              if (PATHOUT) exit
            enddo
            if (PATHOUT) then
              QUERNG(l,1)=0
              QUERNG(l,2)=0
              l=l-1
            endif
          enddo
        enddo

        PATH=l
        PRIMSTAT(1)=0
        PRIMSTAT(2)=0
        if(CALC_STRINGS) then
          STRRNG(k)=0
          STRRNG(k+1)=0
        endif

        call PRRING(k,i)
        if (TBR .or. ALC) goto 001

        n=2*k
        if (APNA(n).eq.0 .and. PRIMSTAT(1).ne.0) then
          APNA(n)=APNA(n)+1
          MINAT=min(MINAT,n)
          MAXAT=max(MAXAT,n)
          if (FACTATPNA) then
            ATPNA(n,o,i)=1
          endif
          SAUT=.false.
        endif
        if (n .lt. TAILLD) then
          if (APNA(n+1).eq.0 .and. PRIMSTAT(2).ne.0) then
            APNA(n+1)=APNA(n+1)+1
            MINAT=min(MINAT,n+1)
            MAXAT=max(MAXAT,n+1)
            if (FACTATPNA) then
              ATPNA(n+1,o,i)=1
            endif
            SAUT=.false.
          endif
        endif

        PRING(n,o)=PRIMSTAT(1)
        NIRR(n,i)=NIRR(n,i)+PRIMSTAT(1)
        do m=1, NRING(n,i)
          INDRING(n,m,i)=INDRING(n,m,i)+INDR(1,m)
        enddo
        if (n .lt. TAILLD) then
          PRING(n+1,o)=PRIMSTAT(2)
          NIRR(n+1,i)=NIRR(n+1,i)+PRIMSTAT(2)
          do m=1, NRING(n+1,i)
            INDRING(n+1,m,i)=INDRING(n+1,m,i)+INDR(2,m)
          enddo
        endif

        if (CALC_STRINGS) then ! Calc_string if

          if (SPNA(n).eq.0 .and. STRRNG(1).ne.0) then
            SPNA(n)=SPNA(n)+1
            MINST=min(MINST,n)
            MAXST=max(MAXST,n)
            if (FACTATPNA) STPNA(n,o,i)=1
            SSAUT=.false.
          endif
          if (n .lt. TAILLD) then
            if (SPNA(n+1).eq.0 .and. STRRNG(2).ne.0) then
              SPNA(n+1)=SPNA(n+1)+1
              MINST=min(MINST,n+1)
              MAXST=max(MAXST,n+1)
              if (FACTATPNA) STPNA(n+1,o,i)=1
              SSAUT=.false.
            endif
          endif

          SRING(n,o)=STRRNG(1)
          SNIRR(n,i)=SNIRR(n,i)+STRRNG(1)
          do m=1, SNRING(n,i)
            INDSRING(n,m,i)=INDSRING(n,m,i)+INDSR(1,m)
          enddo
          if (n .lt. TAILLD) then
            SRING(n+1,o)=STRRNG(2)
            SNIRR(n+1,i)=SNIRR(n+1,i)+STRRNG(2)
            do m=1, SNRING(n+1,i)
              INDSRING(n+1,m,i)=INDSRING(n+1,m,i)+INDSR(2,m)
            enddo
          endif

        endif ! End Calc_string if

        if (allocated(QUERNG)) deallocate(QUERNG)

      enddo ! end ring-sizes-loop

      if (.not. SAUT) then
        do k=3, TAILLD
          do m=3, TAILLD
            if (APNA(k).eq.1 .and. APNA(m).eq.1) then
              PNA(k,m,i)=PNA(k,m,i)+1
            endif
          enddo
        enddo
        MAXPNA(MAXAT,i)=MAXPNA(MAXAT,i)+1
        MINPNA(MINAT,i)=MINPNA(MINAT,i)+1
      endif
      if (CALC_STRINGS) then
        if (.not. SSAUT) then
          do k=3, TAILLD
            do m=3, TAILLD
              if (SPNA(k).eq.1 .and. SPNA(m).eq.1) then
                SNA(k,m,i)=SNA(k,m,i)+1
              endif
            enddo
          enddo
          MAXSNA(MAXST,i)=MAXSNA(MAXST,i)+1
          MINSNA(MINST,i)=MINSNA(MINST,i)+1
        endif
      endif

    endif

  enddo ! end atoms-loop

  do k=3, TAILLD
    if (NRING(k,i) .ne. 0) then

!      do j=1, NRING(k,i)
!!  The algorithm implies that you may have a problem for
!!  the biggest size of ring therefore lets put this size out of the proof checking.
!!  Furthemore we check resutls only if all atoms are used to initiate the search.
!          if (mod(INDRING(k,j,i), k).ne.0 .and. k.lt.TAILLD .and. LTLT.eq.0) then
!            write (6, 003) i, k, j, INDRING(k,j,i)
!            write (6, 007)
!            do l=1, k
!              write (6, '(i4,2x)', advance='no') RINGORD(k,j,l,i)
!            enddo
!            write (6, *)
!            write (6, 004)
!            if (ABAB) then
!              write (6, 008)
!            else
!              write (6, 005)
!            endif
!            write (6, *)
!          endif
!      enddo
    endif
  enddo

  if (CALC_STRINGS) then
    do k=3, TAILLD
      if (SNRING(k,i) .ne. 0) then

!        do j=1, SNRING(k,i)

!  The algorithm implies that you may have a problem for
!  the biggest size of ring therefore lets put this size out of the proof checking.
!  Furthemore we check resutls only if all atoms are used to initiate the search.
!          if (mod(INDSRING(k,j,i), k).ne.0 .and. k.lt.TAILLD .and. LTLT.eq.0) then
!            write (6, 006) i, k, j, INDSRING(k,j,i)
!            write (6, 007)
!            do l=1, k
!              write (6, '(i4,2x)', advance='no') SRINGORD(k,j,l,i)
!            enddo
!            write (6, *)
!            write (6, 004)
!            if (ABAB) then
!              write (6, 008)
!            else
!              write (6, 005)
!            endif
!            write (6, *)
!          endif

!        enddo
      endif
    enddo
  endif
enddo

if (allocated(APNA)) deallocate(APNA)
if (CALC_STRINGS) then
 if (allocated(SPNA)) deallocate(SPNA)
endif

! To give back is original shape to the MD box
do i=1, 3
  modv(i)=modv(i)/NBX
  do j=1, 3
    lvect(i,j)=lvect(i,j)/NBX
  enddo
enddo

001 continue

if (allocated(TOTEST)) deallocate(TOTEST)
if (allocated(TOTEST2)) deallocate(TOTEST2)
if (allocated(QUEUE)) deallocate(QUEUE)
if (allocated(CHK)) deallocate(CHK)
if (allocated(PRING)) deallocate(PRING)
if (allocated(NPRING)) deallocate(NPRING)
if (allocated(PRINGORD)) deallocate(PRINGORD)
if (allocated(PRINGSAV)) deallocate(PRINGSAV)
if (allocated(MATDIST)) deallocate(MATDIST)
if (allocated(RINGSAVED)) deallocate(RINGSAVED)

if (CALC_STRINGS) then

 if (allocated(CHK2)) deallocate(CHK2)
 if (allocated(SRINGSAVED)) deallocate(SRINGSAVED)
 if (allocated(STRRNG)) deallocate(STRRNG)
 if (allocated(SRING)) deallocate(SRING)
 if (allocated(INDSR)) deallocate(INDSR)
 if (allocated(INDSRING)) deallocate(INDSRING)

endif

PRIM_DIJKSTRA=RECRINGS(5)

if (NS.gt.1 .or. (NS.eq.1 .and. TAILLD.ge.20)) then
  pro=0.0d0
  call setprogress (pro)
endif

if (CALC_STRINGS) then

  if (FACTATRING) then
    if (allocated(ATRING)) deallocate(ATRING)
    allocate(ATRING(TAILLD,NTLT,NS), STAT=ERR)
    if (ERR .ne. 0) then
      call show_error ("Impossible to allocate memory"//CHAR(0), &
                       "Function: PRIM_DIJKSTRA"//CHAR(0), "Table: MAXPNA"//CHAR(0))
      ALC=.true.
      goto 002
    endif
    do i=1, TAILLD
      do j=1, NTLT
        do k=1, NS
          ATRING(i,j,k)=ATRING2(i,j,k)
        enddo
      enddo
    enddo
    if (allocated(ATRING2)) deallocate(ATRING2)
  endif

  if (FACTATPNA) then
    if (allocated(ATPNA)) deallocate(ATPNA)
    allocate(ATPNA(TAILLD,NTLT,NS), STAT=ERR)
    if (ERR .ne. 0) then
      call show_error ("Impossible to allocate memory"//CHAR(0), &
                       "Function: PRIM_DIJKSTRA"//CHAR(0), "Table: MAXPNA"//CHAR(0))
      ALC=.true.
      goto 002
    endif
    do i=1, TAILLD
      do j=1, NTLT
        do k=1, NS
          ATPNA(i,j,k)=STPNA(i,j,k)
        enddo
      enddo
    enddo
    if (allocated(ATPNA)) deallocate(ATPNA)
  endif

  if (allocated(MAXPNA)) deallocate(MAXPNA)
  allocate(MAXPNA(TAILLD,NS), STAT=ERR)
  if (ERR .ne. 0) then
    call show_error ("Impossible to allocate memory"//CHAR(0), &
                     "Function: PRIM_DIJKSTRA"//CHAR(0), "Table: MAXPNA"//CHAR(0))
    ALC=.true.
    goto 002
  endif
  if (allocated(MINPNA)) deallocate(MINPNA)
  allocate(MINPNA(TAILLD,NS), STAT=ERR)
  if (ERR .ne. 0) then
    call show_error ("Impossible to allocate memory"//CHAR(0), &
                     "Function: PRIM_DIJKSTRA"//CHAR(0), "Table: MINPNA"//CHAR(0))
    ALC=.true.
    goto 002
  endif
  do i=3, TAILLD
    do j=1, NS
      MAXPNA(i,j)=MAXSNA(i,j)
      MINPNA(i,j)=MINSNA(i,j)
    enddo
  enddo
  if (allocated(MAXSNA)) deallocate(MAXSNA)
  if (allocated(MINSNA)) deallocate(MINSNA)
  if (allocated(PNA)) deallocate(PNA)
  allocate(PNA(TAILLD,TAILLD,NS), STAT=ERR)
  if (ERR .ne. 0) then
    call show_error ("Impossible to allocate memory"//CHAR(0), &
                     "Function: PRIM_DIJKSTRA"//CHAR(0), "Table: PNA"//CHAR(0))
    ALC=.true.
    goto 002
  endif
  do i=1, TAILLD
    do j=1, TAILLD
      do k=1, NS
        PNA(i,j,k)=SNA(i,j,k)
      enddo
    enddo
  enddo
  if (allocated(SNA)) deallocate(SNA)

  do i=3, TAILLD
  do j=1, NS
    NRING(i,j)=SNRING(i,j)
    NIRR(i,j)=SNIRR(i,j)
  enddo
  enddo
  if (allocated(SNRING)) deallocate(SNRING)
  if (allocated(SNIRR)) deallocate(SNIRR)

  if (allocated(RINGORD)) deallocate(RINGORD)
  allocate(RINGORD(TAILLD,NUMA/NBX,TAILLD,NS), STAT=ERR)
  if (ERR .ne. 0) then
    call show_error ("Impossible to allocate memory"//CHAR(0), &
                     "Function: PRIM_DIJKSTRA"//CHAR(0), "Table: RINGORD"//CHAR(0))
    ALC=.true.
    goto 002
  endif
  do i=1, NS
  do j=3, TAILLD
  do k=1, NRING(j,i)
  do l=1, TAILLD
    RINGORD(j,k,l,i)=SRINGORD(j,k,l,i)
  enddo
  enddo
  enddo
  enddo
  if (allocated(SRINGORD)) deallocate(SRINGORD)

  002 continue

  PRIM_DIJKSTRA=RECRINGS(6)

endif

END FUNCTION

SUBROUTINE DIJKSTRA(NODE)

USE PARAMETERS

INTEGER, INTENT(IN) :: NODE
INTEGER :: QBEGIN, QEND, QID, AT1, AT2, DAT1

do QID=1,NNA
  QUEUE(QID)=0
  MATDIST(QID)=NNA+2
enddo

MATDIST(NODE)=0

QUEUE(1)=NODE
QBEGIN=0
QEND=1

! Trouver les plus cours chemin reliant un atome aux autres
! Find the shortest paths between atoms

do while (QBEGIN < QEND)

  QBEGIN=QBEGIN+1
  AT1=QUEUE(QBEGIN)
  DAT1=MATDIST(AT1)+1

  do QID=1, CPAT(AT1)

    AT2=VPAT(AT1,QID)
    if (MATDIST(AT2) .gt. DAT1) then

      MATDIST(AT2) = DAT1
      if (DAT1 < NNA) then

        QEND=QEND+1
        QUEUE(QEND)= AT2

      endif

    endif

  enddo

enddo

END SUBROUTINE

RECURSIVE SUBROUTINE SPATH_REC(NODE, LENGTH, LNGTH)

USE PARAMETERS

INTEGER, INTENT(IN) :: NODE, LENGTH, LNGTH
INTEGER :: DISTNN, IDV, VDI, IDT

NPRING(LENGTH) = NODE

do IDV=1, CPAT(NODE)

! Boucle sur tous les voisins du noeud "NODE"
! Loop on all the neighbors of node "NODE"

  VDI=VPAT(NODE,IDV)                  ! Selection du voisin - neighbor selection
  DISTNN=MATDIST(VDI)                 ! Distance to starting atom start = VDI in the node distance table

  if (DISTNN .eq. 0) then

    PATH=PATH+1
    do IDT=1, LNGTH
      PRINGORD(PATH,IDT)=NPRING(IDT)
    enddo

  elseif (DISTNN .eq. LENGTH-1) then

    call SPATH_REC(VDI, DISTNN, LNGTH)

  endif

enddo

END SUBROUTINE

SUBROUTINE PRRING(LGTH, STEPR)

USE PARAMETERS

INTEGER, INTENT(IN) :: STEPR, LGTH
INTEGER :: RM, RN, PR, PTH1, PTH2, IRS, IRX
INTEGER :: ATB, ATA, ATC, ATD, ATE, MAXD, MIND

LOGICAL:: GOAL, TOSAVE

GOAL=.false.

do PR=1, PATH

! Premier chemin i->j - First path i->j
  PTH1= QUERNG(PR,1)
! Second chemin i->k - second path i->k
  PTH2= QUERNG(PR,2)

  PROBE=0
  ATA= PRINGORD(PTH1,LGTH)
  ATB= PRINGORD(PTH2,LGTH)
  do RM=1, CPAT(ATA)
    ATE=VPAT(ATA,RM)
    if (ATE .eq. ATB) PROBE=1
  enddo

  if (ATA .eq. ATB .or. PROBE .eq. 1) then

    do RM=1, LGTH-1
    do RN=RM, LGTH-1+PROBE

      ATC= PRINGORD(PTH1,RM)
      ATD= PRINGORD(PTH2,RN)
      MAXD=MATDIST(ATC)+MATDIST(ATD)
      MIND=2*LGTH+PROBE-MAXD
      do IRX=1, NNA
        CHK(IRX)=.false.
      enddo

      call PAIR_SEARCH(ATC, ATD, 1, MAXD, MIND, GOAL, STEPR)
      if (GOAL) then
        GOAL= .false.
        goto 001
      endif

    enddo
    enddo

    do RN=1, LGTH-1
    do RM=RN, LGTH-1+PROBE

      ATC= PRINGORD(PTH1,RM)
      ATD= PRINGORD(PTH2,RN)
      MAXD=MATDIST(ATC)+MATDIST(ATD)
      MIND=2*LGTH+PROBE-MAXD
      do IRX=1, NNA
        CHK(IRX)=.false.
      enddo

      call PAIR_SEARCH(ATC, ATD, 1, MAXD, MIND, GOAL, STEPR)
      if (GOAL) then
        GOAL= .false.
        goto 001
      endif

    enddo
    enddo

    if (PROBE .eq. 0 .or. (PROBE .eq. 1 .and. LGTH.lt.TAILLE_REF)) then

      do IRX=1, TAILLD
        TOTEST(IRX)=0
        TOTEST2(IRX)=0
      enddo
      do IRX=1, LGTH
        TOTEST(IRX)=PRINGORD(PTH1,IRX)
      enddo
      if (PROBE .eq. 1) TOTEST(LGTH+1)=PRINGORD(PTH2,LGTH)
      IRS=LGTH+PROBE
      do IRX=LGTH-1, 1, -1
        IRS=IRS+1
        TOTEST(IRS)=PRINGORD(PTH2,IRX)
      enddo
      TOTEST(IRS+1)=j

!      To find atom id in the primitive unit cell
      call INDORD(LGTH,PROBE)
      TOSAVE=.true.
      if (ABAB) then
        if (PROBE .eq. 0) then
          call TESTABAB(TOSAVE,LGTH)
        else
          TOSAVE=.false.
        endif
      endif
      if (TOMO) then
        call TESTHOMO(TOSAVE,LGTH)
      endif
      if (TOSAVE) then
        call SAVE_RING2(LGTH, 1, PROBE, STEPR)
        if (TBR .or. ALC) goto 002
        PRIMSTAT(1+PROBE) = PRIMSTAT(1+PROBE)+1
        if (CALC_STRINGS) then
          call STRRINGS(LGTH, PROBE, STEPR)
          if (TBR .or. ALC) goto 002
        endif
      endif

    endif

  endif

001 continue

enddo

002 continue

END

RECURSIVE SUBROUTINE PAIR_SEARCH(AT1, AT2, LG, MAXM, MINM, GOAL, PSTEP)

USE PARAMETERS

INTEGER, INTENT(IN) :: AT1, AT2, PSTEP, LG, MAXM, MINM
INTEGER :: PSC, AT3

LOGICAL, INTENT(INOUT) :: GOAL

CHK(AT1)=.true.

if (AT1 .eq. AT2) then

  GOAL=.true.

else

  do PSC=1, CPAT(AT1)

    AT3=VPAT(AT1,PSC)

    if (.not.CHK(AT3)) then

      if (AT3.eq.AT2) then

        GOAL=.true.
        goto 001

      elseif (LG.lt.MAXM-1 .and. LG.lt.MINM-1) then

        call PAIR_SEARCH(AT3, AT2, LG+1, MAXM, MINM, GOAL, PSTEP)

        if (GOAL) then
           goto 001
        endif

      endif

    endif

  enddo

endif

001 continue

CHK(AT1)=.false.

END SUBROUTINE

SUBROUTINE STRRINGS(RLGTH, RPROBE, RSTEPR)

USE PARAMETERS

INTEGER, INTENT(IN) :: RLGTH, RPROBE, RSTEPR
INTEGER :: RM, RN, STLGT

LOGICAL :: DVSTR
LOGICAL :: SGOAL=.false.
LOGICAL :: FGOAL=.false.

STLGT=2*RLGTH+RPROBE

do RM=1, NNA
  CHK2(RM)=.false.
enddo

do RM=1, STLGT-1

  do RN=RM+1, STLGT

    DVSTR=.false.
    call SEARCH_RINGS(DVSTR, TOTEST, RM, RN, STLGT, STLGT, SGOAL, FGOAL, RSTEPR)
    if (TBR .or. ALC) goto 002
    if (FGOAL) goto 001
    if (SGOAL) goto 002

  enddo

enddo

001 continue

call SAVE_RING2(RLGTH, 2, RPROBE, RSTEPR)
if (TBR .or. ALC) goto 002
STRRNG(1+PROBE)=STRRNG(1+PROBE) +1

002 continue

END SUBROUTINE

RECURSIVE SUBROUTINE SEARCH_RINGS(DLOW, TOTER, IDX, IDY, DLGTR, LGTR, SRGOAL, FRGOAL, SRSTEP)

USE PARAMETERS

INTEGER, INTENT(IN) :: DLGTR, LGTR, IDX, IDY, SRSTEP
INTEGER :: DXY, DYX, DTEST, DMIN
INTEGER :: RG, RF, RH, RJ, RL, RO, RP, RQ, RR
INTEGER :: LGNEW
INTEGER :: NBPATH,  IDZ
INTEGER, DIMENSION(DLGTR), INTENT(IN) :: TOTER
INTEGER, DIMENSION(:), ALLOCATABLE :: TOTER2, STRST

INTEGER, DIMENSION(:,:), ALLOCATABLE :: TOTPATH

LOGICAL, INTENT(INOUT) :: DLOW, SRGOAL, FRGOAL
LOGICAL, DIMENSION(:), ALLOCATABLE :: CUTRING

!   At the begin TOTER is the ring to test, within the process
!   it changes to become the biggest ring piece of the one tested

do RG=1, DLGTR
  CHK2(TOTER(RG))=.true.
enddo

DXY=abs(IDY-IDX)
DYX=DLGTR-DXY
DMIN=min(DXY,DYX)
DTEST=LGTR-1-DMIN

!   First we find find a ring starting from an atom i and enclosing
!   all tested atoms, then this ring will be tested and so on
!   if the size of the enclosing ring became smaller than
!   DLGTR then the initial ring is not strong

if (DTEST.ge.2) then

  do RJ=2, DTEST

    if (allocated(TOTPATH)) deallocate(TOTPATH)
    allocate(TOTPATH(RJ*RJ,RJ), STAT=ERR)
    if (ERR .ne. 0) then
      call show_error ("Impossible to allocate memory"//CHAR(0), &
                       "Subroutine: SEARCH_RINGS"//CHAR(0), "Table: TOTPATH"//CHAR(0))
      ALC=.true.
      goto 001
    endif
    if (allocated(CUTRING)) deallocate(CUTRING)
    allocate(CUTRING(RJ*RJ), STAT=ERR)
    if (ERR .ne. 0) then
      call show_error ("Impossible to allocate memory"//CHAR(0), &
                       "Subroutine: SEARCH_RINGS"//CHAR(0), "Table: CUTRING"//CHAR(0))
      ALC=.true.
      goto 001
    endif
    if (allocated(STRST)) deallocate(STRST)
    allocate(STRST(RJ*RJ), STAT=ERR)
    if (ERR .ne. 0) then
      call show_error ("Impossible to allocate memory"//CHAR(0), &
                       "Subroutine: SEARCH_RINGS"//CHAR(0), "Table: STRST"//CHAR(0))
      ALC=.true.
      goto 001
    endif

    do RG=1, RJ*RJ
      CUTRING(RG)=.false.
      STRST=0
      do RF=1, RJ
        TOTPATH(RG,RF)=0
      enddo
    enddo
    NBPATH=0

    do RG=1, NNA
      CHK(RG)=.false.
    enddo

    call CREAT_LIST(TOTER(IDX), 1)
    ANNEAUX => FIRST

! On recherche tous les chemins de taille RJ entre les atomes TOTER(IDX) et TOTER(IDY)
! On stock ces NBPATH chemins dans le tableau TOTPATH

! We are loking for all path of size RJ between atoms TOTER(IDX) and TOTER(IDY)
! Then these NBPATH are saved in the tab TOTPATH

    call PSEARCH(DLOW,TOTPATH,CUTRING,STRST,NBPATH,TOTER,TOTER(IDX),TOTER(IDX),TOTER(IDY),IDX,IDY,RJ,DLGTR,LGTR,SRSTEP)
    deallocate(FIRST)

    if (NBPATH .ne. 0) then

      do RH=1, NBPATH

        IDZ=0
        if (CUTRING(RH)) then
          do RG=1, DLGTR
            if (TOTER(RG) .eq. STRST(RH)) then
              IDZ=RG
              exit
            endif
          enddo
        endif

        if(IDZ.eq.0)then

          if (.not.CUTRING(RH)) then
            call CREAT_LIST(TOTER(IDX), 1)
            ANNEAUX => FIRST
            do RG=1, RJ
              call DO_LIST(TOTPATH(RH,RG), 1)
            enddo
            if (DXY .le. DYX) then
              do RG=IDY+1, DLGTR
                call DO_LIST(TOTER(RG), 1)
              enddo
              do RG=1, IDX
                call DO_LIST(TOTER(RG), 1)
              enddo
            else
              do RG=IDY-1, IDX, -1
                call DO_LIST(TOTER(RG), 1)
              enddo
            endif
          else
            call CREAT_LIST(TOTER(1), 1)
            ANNEAUX => FIRST
            do RG=2, DLGTR
              call DO_LIST(TOTER(RG), 1)
            enddo
            CHK2(TOTPATH(RH,1))=.true.
          endif

        else

          call CREAT_LIST(TOTPATH(RH,1), 1)
          ANNEAUX => FIRST
          call SHORTCUTRING(IDX, IDY, IDZ, TOTER, DLGTR)

        endif

!    A new path as been found, so a new rings as been built according
!    to the newlist of atoms.
!    The size of this new ring has to be tested

        if (ANNEAUX%INDICE+1 .lt. LGTR) then

          SRGOAL=.true.
          call CLEAN_LIST(ANNEAUX%INDICE)
           if (allocated(TOTPATH)) deallocate(TOTPATH)
            if (allocated(CUTRING)) deallocate(CUTRING)
            if (allocated(STRST)) deallocate(STRST)
          goto 001

        else

! If all the atoms are CHK2 and ring is bigger than the initial ring
! the initial ring is strong .. we have to check this
          RF=0
          do RG=1, NNA
            if (CHK2(RG)) RF=RF+1
          enddo
          if (RF .eq. NNA) then

            FRGOAL=.true.
            call CLEAN_LIST(ANNEAUX%INDICE)
            if (allocated(TOTPATH)) deallocate(TOTPATH)
            if (allocated(CUTRING)) deallocate(CUTRING)
            if (allocated(STRST)) deallocate(STRST)
            goto 001

          else

!       Else the new ring has to be test
            LGNEW=ANNEAUX%INDICE+1
            if (allocated(TOTER2)) deallocate(TOTER2)
            allocate(TOTER2(LGNEW), STAT=ERR)
            if (ERR .ne. 0) then
              call show_error ("Impossible to allocate memory"//CHAR(0), &
                               "Subroutine: SEARCH_RINGS"//CHAR(0), "Table: TOTER2"//CHAR(0))
              ALC=.true.
              goto 001
            endif
            SRGOAL=.false.
            LINK_A => FIRST
            TOTER2(1)=LINK_A%ELEMENT
            do RG=2, LGNEW
              LINK_A => LINK_A%NEXT
              TOTER2(RG)=LINK_A%ELEMENT
            enddo

            call CLEAN_LIST(ANNEAUX%INDICE)
            DLOW=.false.

            if (IDZ.eq.0) then
              if (DMIN .eq. 1) then
                RO=2
                RP=RJ
                DLOW=.true.
              else
                RO=1
                RP=LGNEW-1
              endif
            else
              RO=1
              RP=LGNEW-1
            endif

            do RG=RO, RP

              if (IDZ.eq.0) then
                if (DMIN .eq. 1) then
                  DLOW=.true.
                  RQ=RJ+2
                  RR=LGNEW
                else
                  RQ=RG+1
                  RR=LGNEW
                endif
              else
                 RQ=RG+1
                 RR=LGNEW
              endif

              do RF=RQ, RR

                call SEARCH_RINGS(DLOW, TOTER2, RG, RF, LGNEW, LGTR, SRGOAL, FRGOAL, SRSTEP)
                if (TBR .or. ALC) goto 001
                if (SRGOAL .or. FRGOAL) then

                  if (allocated(TOTER2)) deallocate(TOTER2)
                  if (allocated(TOTPATH)) deallocate(TOTPATH)
                  if (allocated(CUTRING)) deallocate(CUTRING)
                  if (allocated(STRST)) deallocate(STRST)
                  goto 001

                endif
                do RL=1, DLGTR
                  CHK2(TOTER(RL))=.true.
                enddo
                if (IDZ.eq.0 .and. CUTRING(RH)) CHK2(TOTPATH(RH,1))=.true.

              enddo
            enddo

            if (allocated(TOTER2)) deallocate(TOTER2)

          endif

        endif

        if (IDZ.eq.0 .and. CUTRING(RH)) CHK2(TOTPATH(RH,1))=.false.

      enddo

    else

      SRGOAL=.false.

    endif

    if (allocated(TOTPATH)) deallocate(TOTPATH)
    if (allocated(CUTRING)) deallocate(CUTRING)
    if (allocated(STRST)) deallocate(STRST)

  enddo

endif

001 continue

do RG=1, DLGTR
  CHK2(TOTER(RG))=.false.
enddo

END SUBROUTINE

RECURSIVE SUBROUTINE PSEARCH(DLPATH,TPATH,CUTPATH,STPATH,NPATH,ACRING,AT0,AT1,AT2,IDT1,IDT2,DMAX,DMED,DARING,CSTEP)

USE PARAMETERS

INTEGER, INTENT(IN) :: AT0, AT1, AT2, IDT1, IDT2
INTEGER, INTENT(IN) :: DMAX, DMED, DARING, CSTEP
INTEGER :: AT3, AT4, AT5, AT6, AT7
INTEGER :: DUV, DUW, DVW

INTEGER, INTENT(INOUT) :: NPATH

INTEGER, DIMENSION(DMED) :: ACRING
INTEGER, DIMENSION(DMAX*DMAX), INTENT(INOUT) :: STPATH
INTEGER, DIMENSION(DMAX*DMAX,DMAX), INTENT(INOUT) :: TPATH

LOGICAL, DIMENSION(DMAX*DMAX), INTENT(INOUT) :: CUTPATH

LOGICAL :: VAL1, VAL2
LOGICAL :: TOUCH
LOGICAL, INTENT(INOUT) :: DLPATH

CHK(AT1)=.true.

if (AT1 .eq. AT2 .and. ANNEAUX%INDICE .eq. DMAX) then

  NPATH=NPATH+1
  TMP => ANNEAUX
  TPATH(NPATH,DMAX)=TMP%ELEMENT
  do AT3=DMAX-1, 1, -1
    TMP=>TMP%PAST
    TPATH(NPATH,AT3)=TMP%ELEMENT
  enddo

else

  do AT4=1, CPAT(AT1)

    AT3=VPAT(AT1,AT4)

    if (.not.CHK(AT3)) then

      call DO_LIST(AT3, 1)
      if (AT3 .eq. AT2 .and. ANNEAUX%INDICE .eq. DMAX) then

        NPATH=NPATH+1
        TMP => ANNEAUX
        TPATH(NPATH,DMAX)=TMP%ELEMENT
        do AT5=DMAX-1, 1, -1
          TMP => TMP%PAST
          TPATH(NPATH,AT5)=TMP%ELEMENT
        enddo

      elseif (ANNEAUX%INDICE.lt.DMAX .and. .not.CHK2(AT3)) then

        TOUCH=.false.
        AT7=0
        VAL1=.false.
        VAL2=.false.
        do AT5=1, CPAT(AT3)
          AT6=VPAT(AT3,AT5)
          if (AT6 .eq. AT0) VAL1=.true.
          if (AT6 .eq. AT2) VAL2=.true.
          if (CHK2(AT6)) then
            AT7=AT7+1
            if (AT6.ne.AT0 .and. AT6.ne.AT2) STPATH(NPATH+1)=AT6
          endif
        enddo
        AT6=0
        do AT5=1, DMED
          if (ACRING(AT5) .eq. STPATH(NPATH+1)) then
            AT6=AT5
            exit
          endif
        enddo
        if (AT7 .eq. 0) then
          TOUCH=.false.
        elseif (AT7.eq.1 .and. VAL1 .and. .not.VAL2) then
          TOUCH=.false.
        elseif (AT7.eq.1 .and. VAL2 .and. .not.VAL1) then
          TOUCH=.false.
        elseif (AT7.eq.1 .and. .not.VAL1 .and. .not.VAL2) then
          TOUCH=.true.
        elseif (AT7.eq.2 .and. VAL1 .and. VAL2 .and. DMAX.eq.2) then
          TOUCH=.false.
        elseif (AT7.gt.2 .and. VAL1 .and. VAL2 .and. DMAX.eq.2) then
          if (DLPATH) then
            TOUCH=.true.
          else
            if (AT6 .ne. 0) then
              if (AT6 .lt. IDT1) then
                DUV=IDT1-AT6
                DVW=IDT2-IDT1
                DUW=DMED-(IDT2-AT6)
              elseif(AT6 .lt. IDT2) then
                DUV=AT6-IDT1
                DVW=DMED-(AT2-AT1)
                DUW=IDT2-AT6
              elseif(AT6 .gt. IDT2) then
                DUV=DMED-(AT6-IDT1)
                DVW=IDT2-IDT1
                DUW=AT6-IDT2
              endif
              if (DUV.lt.DARING-1 .and. DUW.lt.DARING-1) then
                CUTPATH(NPATH+1)=.true.
                TOUCH=.false.
              elseif (DVW.lt.DARING-1 .and. DUW.lt.DARING-1) then
                CUTPATH(NPATH+1)=.true.
                TOUCH=.false.
              elseif (DUV.lt.DARING-1 .and. DVW.lt.DARING-1) then
                CUTPATH(NPATH+1)=.true.
                TOUCH=.false.
              else
                TOUCH=.true.
              endif
            else
               CUTPATH(NPATH+1)=.true.
               TOUCH=.false.
            endif
          endif
        else
          TOUCH=.true.
        endif
        if (.not.TOUCH) call PSEARCH(DLPATH,TPATH,CUTPATH,STPATH,NPATH,ACRING,AT0,AT3,AT2,IDT1,IDT2,DMAX,DMED,DARING,CSTEP)

      endif

      ANNEAUX => ANNEAUX%PAST
      deallocate(ANNEAUX%NEXT)

    endif

  enddo

endif

CHK(AT1)=.false.

END SUBROUTINE

SUBROUTINE SHORTCUTRING(IDA, IDB, IDC, TABAB, DLT)

USE PARAMETERS

INTEGER :: IDD
INTEGER :: LXY, LXZ, LYZ
INTEGER, INTENT(IN) :: IDA, IDB, IDC, DLT

INTEGER, DIMENSION(DLT), INTENT(IN) :: TABAB

if (IDA .gt. IDC) then
! IDB > IDA > IDC
  LXZ=IDA-IDC
  LXY=IDB-IDA
  LYZ=DLT-(IDB-IDC)
  if (LXY .gt. LYZ) then
    if (LXY .gt. LXZ) then
      do IDD=IDA, IDB
        call DO_LIST(TABAB(IDD), 1)
      enddo
    else
      do IDD=IDC, IDA
        call DO_LIST(TABAB(IDD), 1)
      enddo
    endif
  else
    if (LXZ .gt. LYZ) then
      do IDD=IDC, IDA
        call DO_LIST(TABAB(IDD), 1)
      enddo
    else
       do IDD=IDB, DLT
        call DO_LIST(TABAB(IDD), 1)
      enddo
      do IDD=1, IDC
        call DO_LIST(TABAB(IDD), 1)
      enddo
    endif
  endif
else
! IDB > IDA .and. IDC .gt. IDA
  if (IDC .gt. IDB) then
! IDB > IDA .and. IDC .gt. IDA .and. IDC .gt. IDB
! =>  IDC > IDB > IDA
    LXY=IDB-IDA
    LYZ=IDC-IDB
    LXZ=DLT-(IDC-IDA)
    if (LXY .gt. LYZ) then
      if (LXY .gt. LXZ) then
        do IDD=IDA, IDB
          call DO_LIST(TABAB(IDD), 1)
        enddo
      else
        do IDD=IDC, DLT
          call DO_LIST(TABAB(IDD), 1)
        enddo
        do IDD=1, IDA
          call DO_LIST(TABAB(IDD), 1)
        enddo
      endif
    else
      if (LYZ .ge. LXZ) then
        do IDD=IDB, IDC
          call DO_LIST(TABAB(IDD), 1)
        enddo
      else
        do IDD=IDA, IDB
          call DO_LIST(TABAB(IDD), 1)
        enddo
      endif
    endif
  else
! IDB > IDA .and. IDC .gt. IDA .and. IDB .gt. IDC
! =>  IDB > IDC > IDA
    LXY=DLT-(IDB-IDA)
    LYZ=IDB-IDC
    LXZ=IDC-IDA
    if (LXY .gt. LYZ) then
      if (LXY .gt. LXZ) then
        do IDD=IDB, DLT
          call DO_LIST(TABAB(IDD), 1)
        enddo
        do IDD=1, IDA
          call DO_LIST(TABAB(IDD), 1)
        enddo
      else
        do IDD=IDA, IDC
          call DO_LIST(TABAB(IDD), 1)
        enddo
      endif
    else
      if (LYZ .gt. LXZ) then
        do IDD=IDC, IDB
          call DO_LIST(TABAB(IDD), 1)
        enddo
      else
        do IDD=IDA, IDC
          call DO_LIST(TABAB(IDD), 1)
        enddo
      endif
    endif
  endif
endif

END SUBROUTINE

SUBROUTINE SAVE_RING2(TLES, TRNG, IOD, S_SAVE)

USE PARAMETERS

IMPLICIT NONE

INTEGER, INTENT(IN) :: TLES, S_SAVE, TRNG, IOD
INTEGER :: idx, idy, NEWRING, IDD

INTEGER, DIMENSION(:), ALLOCATABLE :: CERCLE

IDD=TLES*2+IOD

if (allocated(CERCLE)) deallocate(CERCLE)
allocate(CERCLE(IDD), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: SAVE_RING2"//CHAR(0), "Table: CERCLE"//CHAR(0))
  ALC=.true.
  goto 001
endif

! A ring has been found, we need to check if it has already been found or not

do idx=1, IDD

! Creation of the tab for the numerical sorting of the list
  CERCLE(idx)=TOTEST2(idx)

enddo

! Sorting
call TRI(CERCLE, IDD)
!call SORT3(CERCLE, IDD)

NEWRING=0

if (TRNG .eq. 1) then

  TABNUR => NRING
  TABSAVR => RINGSAVED
  TABORDR => RINGORD
  TABID => INDR

else

  TABNUR => SNRING
  TABSAVR => SRINGSAVED
  TABORDR => SRINGORD
  TABID => INDSR

endif


if (TABNUR(IDD,S_SAVE) .ne. 0) then

  do idx=1, TABNUR(IDD,S_SAVE)
! do-loop on existing rings to check if the new on has already been found

    NEWRING=0
    do idy=1, IDD
      if (CERCLE(idy) .eq. TABSAVR(IDD,idx,idy)) then
        NEWRING=0
      else
        NEWRING=1
        exit
      endif
    enddo

    if (NEWRING .eq. 0) then
      ! Already been found n-times, increment of the counter
      TABID(1+IOD,idx)=TABID(1+IOD,idx)+1
      exit
    endif

  enddo

else

  NEWRING=1

endif

if (NEWRING .eq. 1) then

! A new ring has been found
! We save the data
  TABNUR(IDD,S_SAVE)=TABNUR(IDD,S_SAVE)+1
  if (TABNUR(IDD,S_SAVE) .gt. NUMA) then
    TBR=.true.
    goto 001
  endif
  TABID(1+IOD,TABNUR(IDD,S_SAVE))=TABID(1+IOD,TABNUR(IDD,S_SAVE))+1
  do idx=1, IDD

    TABSAVR(IDD,TABNUR(IDD,S_SAVE),idx)=CERCLE(idx)
    TABORDR(IDD,TABNUR(IDD,S_SAVE),idx,S_SAVE)=TOTEST2(idx)

  enddo

endif

001 continue

if (allocated(CERCLE)) deallocate(CERCLE)

END SUBROUTINE

SUBROUTINE TESTABAB(VALTEST, LGTEST)

USE PARAMETERS

LOGICAL, INTENT(INOUT) :: VALTEST
INTEGER, INTENT(IN) :: LGTEST
INTEGER :: LTA, LTB, LTC, LTD

LTA=LOT(TOTEST2(1))
LTB=LOT(TOTEST2(2))

VALTEST=.true.
if (LTA .ne. LTB) then
  do LTD=3, 2*LGTEST
    LTC=LOT(TOTEST2(LTD))
    if (mod(LTD,2).eq.0 .and. LTC.ne.LTB) then
      VALTEST=.false.
      exit
    elseif (mod(LTD,2).ne.0 .and. LTC.ne.LTA) then
      VALTEST=.false.
      exit
    endif
  enddo
else
  VALTEST=.false.
endif

END SUBROUTINE

SUBROUTINE TESTHOMO(VALTEST, LGTEST)

USE PARAMETERS

LOGICAL, INTENT(INOUT) :: VALTEST
INTEGER, INTENT(IN) :: LGTEST
INTEGER :: LTA, LTB

VALTEST=.true.
do LTA=1, 2*LGTEST
  LTB=LTA+1
  if (LTA .eq. 2*LGTEST) LTB=1
  if (LOT(TOTEST2(LTA)) .eq. LOT(TOTEST2(LTB))) then
    VALTEST=.false.
    exit
  endif
enddo

END SUBROUTINE

!********************************************************************
!
! Trouver l'index d'une particule dans une superboite
! Find the index of an atom in the supercell
!

SUBROUTINE INDORD(IDX,PR)

USE PARAMETERS

IMPLICIT NONE

INTEGER, INTENT(IN) :: IDX, PR
INTEGER :: IDY, IDZ

do IDY=1, 2*IDX+PR

IDZ=mod(TOTEST(IDY),NA)

if (IDZ .eq. 0) IDZ=NA

TOTEST2(IDY)=IDZ

enddo

END SUBROUTINE
