! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

INTEGER FUNCTION RINGS4()

USE PARAMETERS

DOUBLE PRECISION :: pro

INTERFACE
  INTEGER FUNCTION RECRINGS(VES)
    INTEGER, INTENT(IN) :: VES
  END FUNCTION
END INTERFACE

RINGS4=0
! Dynamic allocation of pointers and tables used in the "RINGS" subroutine
if (allocated(STMP)) deallocate(STMP)
allocate(STMP(TAILLD,NUMA), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Function: RINGS4"//CHAR(0), "Table: STMP"//CHAR(0))
  goto 001
endif

do i=1, NS

  if (NS .gt. 1) then
    pro=(i-1)/(1.0*NS)
    call setprogress (pro)
  endif

  do j=1, TAILLD
  do k=1, NUMA
  do n=1, TAILLD
    RINGSAVED(j,k,n)=0
  enddo
  enddo
  enddo

  call SETUP_CPAT_VPAT (i, NA)

  o=0
  do j=1, NA

! All Atoms do-loop
    if (NS .eq. 1 .and. TAILLD.ge.20) then
      pro=(j-1)/(1.0*NA)
      call setprogress (pro)
    endif

    if (TLT.eq. NSP+1 .or. LOT(j) .eq. TLT) then

     o=o+1
      do k=1, NA
        RPAT(k) = 0
      enddo
      do k=1, TAILLD
        APNA(k)=0
      enddo
      MINAT=TAILLD
      MAXAT=1
      SAUT=.true.

      do l=1, CPAT(j)

        if (ABAB) then
          LOB=LOT(j)
          LOA=LOT(VPAT(j,l))
        endif
        FOUND=.false.

        do k=1, CPAT(j)
            RPAT(VPAT(j,k)) = 0
        enddo
        RPAT(j) = 2

        call CREAT_LIST(j, 1)
        ANNEAUX => FIRST
        call DO_LIST(VPAT(j,l), CPAT(VPAT(j,l)))
        TAILLE=TAILLD

        do k=1, TAILLD
          RES_LIST(k)=0
          do n=1, NUMA
            INDR(k,n)=0
          enddo
        enddo

        call INSIDE_LIST_2(i, VPAT(j,l))
        if (TBR .or. ALC) goto 001

        call CLEAN_LIST(ANNEAUX%INDICE)

        if (APNA(TAILLE) .eq. 0 .and. TAILLE .lt. TAILLD) then
          APNA(TAILLE)=1
          MINAT=min(MINAT,TAILLE)
          MAXAT=max(MAXAT,TAILLE)
          if (FACTATPNA) ATPNA(TAILLE,o,i)=1
          SAUT=.false.
        elseif (APNA(TAILLE) .eq. 0) then
          do k=1, NRING(TAILLE,i)
            if (INDR(TAILLE,k) .ne. 0) then
              APNA(TAILLE)=1
              MINAT=min(MINAT,TAILLE)
              MAXAT=max(MAXAT,TAILLE)
              if (FACTATPNA) ATPNA(TAILLE,o,i)=1
              SAUT=.false.
              exit
            endif
          enddo
        endif

        do k=1, NRING(TAILLE,i)
          if (FACTATRING) ATRING(TAILLE,o,i)=ATRING(TAILLE,o,i)+INDR(TAILLE,k)
          INDRING(TAILLE,k,i)=INDRING(TAILLE,k,i)+INDR(TAILLE,k)
        enddo

        if (TAILLE.eq.TAILLD .and. .not.FOUND) then
           if (CONTJ(VPAT(j,l),i) .ge. 2 .and. CPAT(j).ge.2) AMPAT(o,i)=AMPAT(o,i)+1
        endif

      enddo

      if (.not. SAUT) then
        do k=3, TAILLD
          do m=3, TAILLD
            if (APNA(k).eq.1 .and. APNA(m).eq.1) then
              PNA(k,m,i)=PNA(k,m,i)+1
            endif
          enddo
        enddo
        MAXPNA(MAXAT,i)=MAXPNA(MAXAT,i)+1
        MINPNA(MINAT,i)=MINPNA(MINAT,i)+1
      endif
    endif

  enddo

  do l=3, TAILLD

    if (NRING(l,i) .ne. 0) then

      do k=1, NRING(l,i)

        INDRAT(l,i)=INDRAT(l,i)+INDRING(l,k,i)
        FIRR=.false.

        if (mod(INDRING(l,k,i), l) .eq. 0) then
          FIRR=.true.
        elseif (NSP.eq.2 .and. LTLT.ne.0) then
          if (mod(l,2) .eq. 0) then
            if(INDRING(l,k,i) .eq. l/2) then
              FIRR=.true.
            endif
          else
            if(INDRING(l,k,i) .gt. l/2) then
              FIRR=.true.
            endif
          endif
        endif

        if (FIRR) then

          NIRR(l,i)=NIRR(l,i)+1
          NIRRAT(l,i)=NIRRAT(l,i)+INDRING(l,k,i)
          LIRR(l,NIRR(l,i),i)=k

        endif
      enddo
    endif
  enddo
enddo

001 continue

RINGS4=RECRINGS(4)

if (NS.gt.1 .or. (NS.eq.1 .and. TAILLD.ge.20)) then
  pro=1.0d0
  call setprogress (pro)
endif

END FUNCTION RINGS4
