/* This file is part of I.S.A.A.C.S.

   I.S.A.A.C.S. is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

    I.S.A.A.C.S. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>

#ifdef G_OS_WIN32
#  include <windows.h>
#endif

#include "global.h"
#include "interface.h"
#include "callbacks.h"
#include "curve.h"
#include "bdcall.h"
#include "ringscall.h"

int rundmtx_ (int *, int *);
int dvtbox_ (int *, int *);
int initrings_ (int *, int *, int *, int *, int *);

void initrng ()
{
  int i, j, k, l;
  GtkWidget * info;
  char * cp[4] = {"Rc(n)[", "Pn(n)[", "Pmax(n)[", "Pmin(n)["};

  l = numgr+numgq+numsq+numsk+numbd+numan;
  for ( i = 0 ; i < 5 ; i++ )
  {
    for ( j = 0 ; j < 4 ; j++ )
    {
        curves[l] -> title = g_strdup_printf("%s%s%s%s", trc[i], " - ", cp[j], "All]");
        l=l+1;
    }
    for ( j = 0 ; j < nspec ; j++ )
    {
      for ( k = 0 ; k < 4 ; k++ )
      {
        curves[l] -> title = g_strdup_printf("%s%s%s%s%s", trc[i], " - ", cp[k], label[j], "]");
        l=l+1;
      }
    }
  }
  i = numgr+numgq+numsq+numsk+numbd+numan;
  for ( j = i ; j < i+numri ; j++ )
  {
    addcurwidget (curves[j] -> title, & j, & i, tablegtkri);
    curves[j] -> xlabel = g_strdup_printf("%s", "Size n of the ring [total number of nodes]");
    curves[j] -> cstart = i;
    curves[j] -> cend = i+numri;
  }

  if (curves[i] -> data != NULL)
  {
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "ringcheck");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
    if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
  }
  for ( j = i ; j < i+numri ; j ++ )
  {
    free(curves[j] -> data);
    curves[j] -> data = NULL;
  }
}

G_MODULE_EXPORT void on_comboboxrings_changed (GtkComboBox * Ringbox)
{
  search = gtk_combo_box_get_active(Ringbox);
}

G_MODULE_EXPORT void on_rispec_get (GtkComboBox * Ringbox)
{
  initnode = gtk_combo_box_get_active(Ringbox);
}

G_MODULE_EXPORT void on_rspec_get ()
{
  int i, j;
  GtkEntry * info;

  i=gtk_combo_box_get_active(GTK_COMBO_BOX(rspeca));
  j=gtk_combo_box_get_active(GTK_COMBO_BOX(rspecb));
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "rcut");
  update_entry_double (info, & cutoffs[i][j]);
}

G_MODULE_EXPORT void on_rcut_activate ()
{
  GtkEntry * EntryGtk;
  int i, j, k, l, m, n;
  const gchar *str;

  if (dmtx) dmtx=FALSE;
  i=gtk_combo_box_get_active(GTK_COMBO_BOX(rspeca));
  j=gtk_combo_box_get_active(GTK_COMBO_BOX(rspecb));
  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "rcut");
  str=gtk_entry_get_text (EntryGtk);
  cutoffs[i][j] = atof(str);
  update_entry_double (EntryGtk, & cutoffs[i][j]);
  k=gtk_combo_box_get_active(GTK_COMBO_BOX(bspeca));
  l=gtk_combo_box_get_active(GTK_COMBO_BOX(bspecb));
  m=gtk_combo_box_get_active(GTK_COMBO_BOX(aspeca));
  n=gtk_combo_box_get_active(GTK_COMBO_BOX(aspecb));
  if (k==i && l==j)
  {
    EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "cut");
    update_entry_double (EntryGtk, & cutoffs[i][j]);
  }
  if (m==i && n==j)
  {
    EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "acut");
    update_entry_double (EntryGtk, & cutoffs[i][j]);
  }
}

G_MODULE_EXPORT void on_rtcut_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *str;

  if (dmtx) dmtx=FALSE;
  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "rtcut");
  str=gtk_entry_get_text (EntryGtk);
  grtotcutoff = atof(str);
  update_entry_double (EntryGtk, & grtotcutoff);
  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "tcut");
  update_entry_double (EntryGtk, & grtotcutoff);
  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "atcut");
  update_entry_double (EntryGtk, & grtotcutoff);
}

G_MODULE_EXPORT void on_tailled_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *str;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "tailled");
  str = gtk_entry_get_text (EntryGtk);
  ringsize = atof(str);
  update_entry_int (EntryGtk, & ringsize);
}

G_MODULE_EXPORT void on_numa_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *str;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "numa");
  str = gtk_entry_get_text (EntryGtk);
  maxring = atof(str);
  update_entry_int (EntryGtk, & maxring);
}

G_MODULE_EXPORT void on_ababcheck_toggled (GtkToggleButton * Button)
{
  gboolean status;

  status = gtk_toggle_button_get_active (Button);
  if (status)
  {
    if (onlyabab == 0 && dmtx) dmtx=FALSE;
    onlyabab = 1;
  }
  else
  {
    if (onlyabab == 1 && dmtx) dmtx=FALSE;
    onlyabab = 0;
  }
}

G_MODULE_EXPORT void on_homocheck_toggled (GtkToggleButton * Button)
{
  gboolean status;

  status = gtk_toggle_button_get_active (Button);
  if (status)
  {
    if (homo == 0 && dmtx) dmtx=FALSE;
    homo = 1;
  }
  else
  {
    if (homo == 1 && dmtx) dmtx=FALSE;
    homo = 0;
  }
}

G_MODULE_EXPORT void on_Calcrings_released()
{
  int i, j, k;
  GtkWidget * info;
  GtkWidget * pop;
  GtkWidget * win;
  gboolean calcr=TRUE;

  calcr=testcutoffs();
  if (calcr)
  {
    cutoffsend ();
    if (ringsize > 1)
    {
      if (maxring > 99)
      {
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "ringcheck");
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
        gtk_widget_set_sensitive (info, 0);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "disprings");
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
        gtk_widget_set_sensitive (info, 0);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "tailled");
        gtk_widget_set_sensitive (info, 0);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "comboboxrings");
        gtk_widget_set_sensitive (info, 0);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "numa");
        gtk_widget_set_sensitive(info, 0);
        gtk_widget_set_sensitive(rispec, 0);
        gtk_widget_set_sensitive(rspeca, 0);
        gtk_widget_set_sensitive(rspecb, 0);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "rcut");
        gtk_widget_set_sensitive(info, 0);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "rtcut");
        gtk_widget_set_sensitive(info, 0);
        if (nspec > 1)
        {
          info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "ababcheck");
          gtk_widget_set_sensitive(info, 0);
          info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "homocheck");
          gtk_widget_set_sensitive(info, 0);
        }
        if (! ringrun)
        {
          initrng ();
          ringrun = TRUE;
        }
        i = 0;
        setcalcs (& i);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Mainstatusbar");
        k=gtk_statusbar_push (GTK_STATUSBAR(info), 6, "Ring statistics");
        win = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "rings");
        if (search > 2) dmtx = FALSE;
        if (! dmtx)
        {
          prepspbv (& i);
          gtk_window_set_opacity (GTK_WINDOW (win), opac);
          pop=show_popup ("Please wait computing the nearest neigbhors table", win);
          gtk_widget_show_all(pop);
          if (search > 2 && apbc)
          {
            i = dvtbox_ (& onlyabab, & homo);
          }
          else
          {
            i = rundmtx_ (& onlyabab, & homo);
          }
          gtk_widget_destroy(pop);
          gtk_window_set_opacity (GTK_WINDOW (win), 1.0);
        }
        else
        {
          i = 1;
        }

        if (i == 1)
        {
          if (search < 3) dmtx=TRUE;
          gtk_window_set_opacity (GTK_WINDOW (win), opac);
          if (steps > 1 || (steps == 1 && ringsize >= 10))
          {
            pop=show_popup ("Please wait ring statistics in progress", win);
            gtk_widget_show_all(pop);
          }
          j = initrings_ (& search, & ringsize, & initnode, & maxring, & homo);
          if (steps > 1 || (steps == 1 && ringsize >= 10)) gtk_widget_destroy(pop);
          gtk_window_set_opacity (GTK_WINDOW (win), 1.0);
          gtk_statusbar_remove (GTK_STATUSBAR(info), 6, k);
          if (j == 0)
          {
            show_error ("The ring statistics calculation has failed");
          }
          else if (j == 2)
          {
            show_error ("The ring statistics have failed, during the search\n"
                        "the number of ring per size 'n' and per node appears\n"
                        "to be bigger than the value used to allocate memory\n"
                        "initially to store results.");
          }
        }
        else
        {
          show_error ("The nearest neighbors table calculation has failed");
          dmtx = FALSE;
        }
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "ringcheck");
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 1);
        gtk_widget_set_sensitive (info, 1);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "disprings");
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 1);
        gtk_widget_set_sensitive (info, 1);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "tailled");
        gtk_widget_set_sensitive (info, 1);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "comboboxrings");
        gtk_widget_set_sensitive (info, 1);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "numa");
        gtk_widget_set_sensitive(info, 1);
        gtk_widget_set_sensitive(rispec, 1);
        gtk_widget_set_sensitive(rspeca, 1);
        gtk_widget_set_sensitive(rspecb, 1);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "rcut");
        gtk_widget_set_sensitive(info, 1);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "rtcut");
        gtk_widget_set_sensitive(info, 1);
        if (nspec > 1)
        {
          info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "ababcheck");
          gtk_widget_set_sensitive(info, 1);
          info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "homocheck");
          gtk_widget_set_sensitive(info, 1);
        }
        i = 1;
        setcalcs (& i);
      }
      else
      {
        show_warning ("You must specify a number of ring per size 'n' and per node\n"
                      "this value is used when allocating the memory to store results\n"
                      "and depends on both the maximum ring size used in the search\n"
                      "and the system studied.\n"
                      "We recommand a value at least equal to 100.\n"
                      "If it appears that the value given is not big enought\n"
                      "(more than 100 different rings of size 'n' for a single node)\n"
                      "then the search will failed but the program will propose\n"
                      "you to initiate a new search using a higher value");
      }
    }
    else
    {
      show_warning ("You must specify a maximum ring size > 1 for the search");
    }
  }
}

G_MODULE_EXPORT void on_ringcheck_toggled (GtkToggleButton * Button)
{
  GtkWidget * info;
  gboolean status;

  status = gtk_toggle_button_get_active (Button);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "RINGSlist");
  if (status)
  {
    if (! GTK_WIDGET_VISIBLE(info))
    {
      gtk_widget_show_all(info);
    }
  }
  else
  {
    if (GTK_WIDGET_VISIBLE(info))
    {
      gtk_widget_hide(info);
    }
  }
}

void printrings_ (int * idnodes,  int * nodes, double nc[ringsize*2], double pna[ringsize*2],
                 double rmax[ringsize*2], double rmin[ringsize*2],
                 double ectrc[ringsize*2], double ectpna[ringsize*2],
                 double ectmax[ringsize*2], double ectmin[ringsize*2],
                 double * rpstep, double * ectrpst, double * nampat, double * ectampat)
{
  int i, j;
  char * value;
  char * nelt;
  char * vala, * valb;
  char ra[10]=" n    Rc[";
  char rb[13]="](n)  Pn(n)[";
  char rm[14]="]   Pmax(n)[";
  char rn[13]="]  Pmin(n)[";
  char rc[28]="](n)      +/-      Pn(n)[";
  char ro[28]="]     +/-        Pmax(n)[";
  char rp[28]="]     +/-       Pmin(n)[";
  char rq[8]="    +/-";
  char rs[27]="](n)     +/-     Pn(n)[";
  char rt[28]="]    +/-       Pmin(n)[";
  char ru[15]="]    Pmax(n)[";
  char rw[15]="]   Pmin(n)[";
  char rv[14]="](n)   Pn(n)[";
  char re[20]=" ring statistics:\n";
  char rf[5]="    ";
  char rg[2]=" ";
  char rh[35]=" Average number of rings with n > ";
  char ri[33]=" nodes which potentially exist: ";
  char rj[6]=" +/- ";
  char rk[45]=" Average number of rings per configuration: ";
  char rd[42]=" * only ABAB rings have been considered\n";
  char rl[47]=" * homopolar bonds can not shorten the rings\n";
  GtkWidget * info;
  GtkTextBuffer * buffer = NULL;
  GtkTextIter bStart;
  GtkTextIter bEnd;

  j = * idnodes;
  j = j-1;

  if (j < nspec)
  {
    nelt=label[j];
  }
  else
  {
    nelt="All";
  }

  value=g_strdup_printf ("%s%s%s", retl, trc[search], re);
  if (onlyabab == 1)
  {
    vala=g_strdup_printf ("%s", value);
    g_free(value);
    value=g_strdup_printf("%s%s", vala, rd);
    g_free(vala);
  }
  if (homo ==1)
  {
    vala=g_strdup_printf ("%s", value);
    g_free(value);
    value=g_strdup_printf("%s%s", vala, rl);
    g_free(vala);
  }
  vala=g_strdup_printf ("%s", value);
  g_free(value);
  value=g_strdup_printf("%s%s%s%lf", vala, retl, rk, * rpstep);
  g_free(vala);
  if (steps > 1)
  {
    vala=g_strdup_printf ("%s", value);
    g_free(value);
    value=g_strdup_printf ("%s%s%lf", vala, rj,* ectrpst);
    g_free(vala);
  }
  vala=g_strdup_printf ("%s%s", value, retl);
  g_free(value);
  value=g_strdup_printf ("%s", vala);
  g_free(vala);
  if (search  == 1 || search == 2)
  {
    vala=g_strdup_printf ("%s%d%s%lf", rh, 2*ringsize, ri, * nampat);
    if (steps > 1)
    {
      valb=g_strdup_printf ("%s%s%lf", vala, rj, * ectampat);
      g_free(vala);
      vala=g_strdup_printf ("%s", valb);
      g_free(valb);
    }
    valb=g_strdup_printf ("%s%s%s", value, vala, retl);
    g_free(vala);
    g_free(value);
    value=g_strdup_printf ("%s", valb);
    g_free(valb);
  }
  if (steps > 1)
  {
    if (j == nspec)
    {
      vala=g_strdup_printf ("%s%s%s%s%s%s%s%s%s%s%s%s", value, ra, nelt, rs, nelt, ro, nelt, rt, nelt, "]", rq, retl);
    }
    else
    {
      vala=g_strdup_printf ("%s%s%s%s%s%s%s%s%s%s%s%s", value, ra, nelt, rc, nelt, ro, nelt, rp, nelt, "]", rq, retl);
    }
    g_free(value);
    value=g_strdup_printf ("%s", vala);
    g_free(vala);
  }
  else
  {
    if (j == nspec)
    {
      vala=g_strdup_printf ("%s%s%s%s%s%s%s%s%s%s%s", value, ra, nelt, rb, nelt, rm, nelt, rn, nelt, "]", retl);
    }
    else
    {
      vala=g_strdup_printf ("%s%s%s%s%s%s%s%s%s%s%s", value, ra, nelt, rv, nelt, ru, nelt, rw, nelt, "]", retl);
    }
    g_free(value);
    value=g_strdup_printf ("%s", vala);
    g_free(vala);
  }
  for ( i=2 ; i < 2*ringsize ; i++ )
  {
    if (pna[i] != 0.0)
    {
      if (i < 9)
      {
        vala=g_strdup_printf ("%s%s", value, rg);
        g_free(value);
        value=g_strdup_printf ("%s", vala);
        g_free(vala);
      }
      vala=g_strdup_printf("%s%d%s%lf%s", value, i+1, rf, nc[i]/(* nodes), rf);
      g_free(value);
      value=g_strdup_printf("%s",vala);
      g_free(vala);
      if (steps > 1)
      {
        vala=g_strdup_printf("%s%lf%s", value, ectrc[i]/(* nodes), rf);
        g_free(value);
        value=g_strdup_printf("%s", vala);
        g_free(vala);
      }
      vala=g_strdup_printf("%s%lf%s", value, pna[i]/(* nodes), rf);
      g_free(value);
      value=g_strdup_printf("%s", vala);
      if (steps > 1)
      {
        vala=g_strdup_printf("%s%lf%s", value, ectpna[i]/(* nodes), rf);
        g_free(value);
        value=g_strdup_printf("%s", vala);
        g_free(vala);
      }
      vala=g_strdup_printf("%s%s%lf%s", value, "  ", rmax[i], rf);
      g_free(value);
      value=g_strdup_printf("%s", vala);

      if (steps > 1)
      {
        vala=g_strdup_printf("%s%lf%s", value, ectmax[i], rf);
        g_free(value);
        value=g_strdup_printf("%s", vala);
        g_free(vala);
      }
      vala=g_strdup_printf("%s%s%lf", value, "  ", rmin[i]);
      g_free(value);
      value=g_strdup_printf("%s", vala);
      if (steps > 1)
      {
        vala=g_strdup_printf("%s%s%lf", value, rf, ectmin[i]);
        g_free(value);
        value=g_strdup_printf("%s", vala);
        g_free(vala);
      }
      vala=g_strdup_printf("%s%s", value, retl);
      g_free(value);
      value=g_strdup_printf("%s", vala);
    }
  }
  g_print("%s", value);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "TextInfo");
  buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(info));
  gtk_text_buffer_get_start_iter (buffer, &bStart);
  gtk_text_buffer_get_end_iter (buffer, &bEnd);
  gtk_text_buffer_insert (buffer, &bEnd, value, -1);
  gtk_text_view_set_buffer (GTK_TEXT_VIEW(info), buffer);
  g_free(value);
}
