! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

INTEGER FUNCTION s_of_k ()

! Total structure factor
! Partial structure factors from k-points

USE PARAMETERS

IMPLICIT NONE

DOUBLE PRECISION :: factor, xfactor
DOUBLE PRECISION :: pro

if(allocated(Sij)) deallocate(Sij)
allocate(Sij(NUMBER_OF_Q_POINTS,NSP,NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: s_of_k"//CHAR(0), "Table: Sij"//CHAR(0))
  s_of_k = 0
  goto 001
endif
do i=1, NSP
do j=1, NSP
do k=1, NUMBER_OF_Q_POINTS
  Sij(k,j,i)=0.0d0
enddo
enddo
enddo

if(allocated(cij)) deallocate(cij)
allocate(cij(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: s_of_k"//CHAR(0), "Table: cij"//CHAR(0))
  s_of_k = 0
  goto 001
endif
if(allocated(sik)) deallocate(sik)
allocate(sik(NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: s_of_k"//CHAR(0), "Table: sik"//CHAR(0))
  s_of_k = 0
  goto 001
endif

do k=1, NS
  if (NS .gt. 1) then
    pro=(k-1)/(1.0*NS)
    call setprogress (pro)
  endif
  call FOURIER_TRANS
enddo

deallocate(cij)
deallocate(sik)
deallocate(qvectx)
deallocate(qvecty)
deallocate(qvectz)
deallocate(modq)

if(allocated(S)) deallocate(S)
allocate(S(NUMBER_OF_Q_POINTS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: s_of_k"//CHAR(0), "Table: S"//CHAR(0))
  s_of_k = 0
  goto 001
endif
if(allocated(XS)) deallocate(XS)
allocate(XS(NUMBER_OF_Q_POINTS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: s_of_k"//CHAR(0), "Table: XS"//CHAR(0))
  s_of_k = 0
  goto 001
endif

factor=0.0d0
xfactor=0.0d0
do i=1, NSP
  factor=factor + NBSPBS(i)*NSCATTL(i)**2
  xfactor=xfactor + NBSPBS(i)*XSCATTL(i)**2
enddo

factor=factor*NS
xfactor=xfactor*NS

do k=1, NUMBER_OF_Q_POINTS
  S(k)=0.0d0
  XS(k)=0.0d0
enddo

do i=1,  NUMBER_OF_Q_POINTS
  do k=1, NSP
  do j=1, NSP
    S(i)=S(i)+Sij(i,k,j)*NSCATTL(k)*NSCATTL(j)
    XS(i)=XS(i)+Sij(i,k,j)*XSCATTL(k)*XSCATTL(j)
  enddo
  enddo
  S(i)=S(i)/(factor*degeneracy(i))
  XS(i)=XS(i)/(xfactor*degeneracy(i))
  do j=1, NSP
  do k=1, NSP
    Sij(i,j,k)=Sij(i,j,k)/(degeneracy(i)*sqrt(dble(NBSPBS(k)*NBSPBS(j)))*NS)
  enddo
  enddo
enddo

deallocate(degeneracy)

pro=1.0d0
call setprogress (pro)

s_of_k = SK_SAVE ()

pro=0.0d0
call setprogress (pro)

001 continue

if (allocated(qvectx)) deallocate(qvectx)
if (allocated(qvecty)) deallocate(qvecty)
if (allocated(qvectz)) deallocate(qvectz)
if (allocated(modq)) deallocate(modq)
if (allocated(degeneracy)) deallocate(degeneracy)
if (allocated(cij)) deallocate(cij)
if (allocated(sik)) deallocate(sik)
if (allocated(Sij)) deallocate(Sij)
if (allocated(S)) deallocate(S)
if (allocated(XS)) deallocate(XS)

CONTAINS

!************************************************************
!
! this subroutine computes the sine and cosine sums from the
! configuration for all q-vectors needed
!
SUBROUTINE FOURIER_TRANS

  USE PARAMETERS

  IMPLICIT NONE

  INTEGER :: PFT, IFT
  DOUBLE PRECISION :: qx, qy, qz, qtr, sini, cosi

! now compute the various fourier transforms
!
  PFT=NUMBER_OF_QVECT/10
  IFT=1
  do j=1, NUMBER_OF_QVECT

    if (NS.eq.1 .and. j.eq.IFT*PFT) then
      pro=(IFT-1)*PFT/(1.0*NUMBER_OF_QVECT)
      call setprogress (pro)
      IFT=IFT+1
    endif

    l=AnINT((modq(j)-qvmin)/DELTA_Q)+1

    if (l .le. NUMBER_OF_Q_POINTS) then

      qx=qvectx(j)
      qy=qvecty(j)
      qz=qvectz(j)

      do i=1, NSP
        cij(i)=0.d0
        sik(i)=0.d0
      enddo

      do i=1, NA
        qtr=qx*FULLPOS(i,1,k)+qy*FULLPOS(i,2,k)+qz*FULLPOS(i,3,k)
        sini=sin(qtr)
        cosi=cos(qtr)
        sik(LOT(i))=sik(LOT(i))+sini
        cij(LOT(i))=cij(LOT(i))+cosi
      enddo

      do i=1, NSP
      do m=1, NSP
        Sij(l,i,m)=Sij(l,i,m) + cij(i)*cij(m) + sik(i)*sik(m)
      enddo
      enddo

   endif

  enddo

END SUBROUTINE

INTEGER FUNCTION SK_SAVE()

USE PARAMETERS

DOUBLE PRECISION, DIMENSION (:), ALLOCATABLE :: SQTAB

if (allocated(SQTAB)) deallocate(SQTAB)
allocate(SQTAB(NUMBER_OF_Q_POINTS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: SMOOTH_SAVE"//CHAR(0), "Table: SQTAB"//CHAR(0))
  SK_SAVE = 0
  goto 001
endif

l=GRNUM+SQNUM
do k=1, NUMBER_OF_Q_POINTS
  SQTAB(k)= S(k)
enddo
call save_curve (NUMBER_OF_Q_POINTS, SQTAB, l)

l=l+2
do k=1, NUMBER_OF_Q_POINTS
  SQTAB(k)= (S(k)-1.0)*K_POINT(k)
  if (S(k) .eq. 0.0) SQTAB(k)=0.0
enddo
call save_curve (NUMBER_OF_Q_POINTS, SQTAB, l)

l=l+2
do k=1, NUMBER_OF_Q_POINTS
  SQTAB(k)= XS(k)
enddo
call save_curve (NUMBER_OF_Q_POINTS, SQTAB, l)

l=l+2
do k=1, NUMBER_OF_Q_POINTS
  SQTAB(k)= (XS(k)-1.0)*K_POINT(k)
  if (XS(k) .eq. 0.0) SQTAB(k)=0.0
enddo
call save_curve (NUMBER_OF_Q_POINTS, SQTAB, l)

l=l+2
do i=1, NSP
  do j=1, NSP
    do k=1, NUMBER_OF_Q_POINTS
      SQTAB(k)=Sij(k,i,j)
      if (S(k) .eq. 0.0) SQTAB(k)=0.0
    enddo
    call save_curve (NUMBER_OF_Q_POINTS, SQTAB, l)
    l=l+2
  enddo
enddo

!  To compute FZ and BT partials
call FZBT

do i=1, NSP
do j=1, NSP
  do k=1, NUMBER_OF_Q_POINTS
    SQTAB(k)= FZSij(k,i,j)
    if (S(k) .eq. 0.0) SQTAB(k)=0.0
  enddo
  call save_curve (NUMBER_OF_Q_POINTS, SQTAB, l)
  l=l+2
enddo
enddo
if (NSP .eq. 2) then
  do i=1, 4
    do j=1, NUMBER_OF_Q_POINTS
      SQTAB(j)= BTij(j,i)
      if (S(j) .eq. 0.0) SQTAB(j)=0.0
    enddo
    call save_curve (NUMBER_OF_Q_POINTS, SQTAB, l)
    l=l+2
  enddo
endif

SK_SAVE=1

001 continue

if (allocated(FZSij)) deallocate(FZSij)
if (NSP.eq.2 .and. allocated(BTij)) deallocate(BTij)
if (allocated(SQTAB)) deallocate(SQTAB)

END FUNCTION

END FUNCTION s_of_k

SUBROUTINE smooth_and_save(CTS, SFC, IDC, NQPTS, IDSQ)

USE PARAMETERS

DOUBLE PRECISION, DIMENSION(NQPTS), INTENT(IN) :: CTS
DOUBLE PRECISION, INTENT(IN) :: SFC
INTEGER, INTENT(IN) :: IDC, IDSQ, NQPTS
DOUBLE PRECISION, DIMENSION (:), ALLOCATABLE :: SQTAB
DOUBLE PRECISION, DIMENSION (:), ALLOCATABLE :: DQTAB

SIGMA_LISS = SFC

if (allocated(DQTAB)) deallocate(DQTAB)
allocate(DQTAB(NQPTS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: smooth_and_save"//CHAR(0), "Table: DQTAB"//CHAR(0))
endif
if (allocated(SQTAB)) deallocate(SQTAB)
allocate(SQTAB(NQPTS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: smooth_and_save"//CHAR(0), "Table: SQTAB"//CHAR(0))
endif

do k=1, NQPTS
  DQTAB(k)=CTS(k)
enddo
if (IDSQ .eq. 0) then
  call SMOOTH (DQTAB, Q_POINT, NQPTS, SIGMA_LISS)
elseif (IDSQ .eq. 1) then
  call SMOOTHSK (DQTAB, K_POINT, NQPTS, SIGMA_LISS)
elseif (IDSQ .eq. 2) then
  call SMOOTH (DQTAB, R_POINT, NQPTS, SIGMA_LISS)
else
  call SMOOTH (DQTAB, F_POINT, NQPTS, SIGMA_LISS)
endif
do k=1, NQPTS
  SQTAB(k)=DQTAB(k)
enddo

call save_curve (NQPTS, SQTAB, IDC)

if (allocated(SQTAB)) deallocate(SQTAB)
if (allocated(DQTAB)) deallocate(DQTAB)

END SUBROUTINE

DOUBLE PRECISION FUNCTION fkmin ()

USE PARAMETERS

fkmin = minr

END FUNCTION
