/* This file is part of I.S.A.A.C.S.

   I.S.A.A.C.S. is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

    I.S.A.A.C.S. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>

#include "global.h"
#include "interface.h"
#include "callbacks.h"
#include "curve.h"
#include "sqcall.h"
#include "skcall.h"
#include "gqcall.h"

int cqvf_ (double *, int *, double *, double *);
int s_of_k_ ();
void smooth_and_save_ (double *, double *, int *, int *, int *);

G_MODULE_EXPORT void on_deltak_activate ()
{
  GtkWidget * info;
  GtkEntry * EntryGtk;
  const gchar *stk;
  int i, j;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "deltak");
  stk=gtk_entry_get_text (EntryGtk);
  i = atof(stk);
  update_entry_int (EntryGtk, & i);
  if (i != num_delta_k)
  {
    num_delta_k = i;
    if (curves[numgr+numsq] -> data != NULL)
    {
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "skcheck");
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
      if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispsk");
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
      if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "skfact");
      if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "skmoother");
      if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
    }
    j= numgr+numsq;
    i= numgr+numsq+numsk;
    hide_curves (&j, &i);
    for ( i = j ; i < j+numsk ; i ++ )
    {
      free(curves[i] -> data);
      curves[i] -> data = NULL;
    }
  }
}

G_MODULE_EXPORT void on_kmax_activate ()
{
  GtkWidget * info;
  GtkEntry * EntryGtk;
  const gchar *stk;
  double delk;
  int i, j;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "kmax");
  stk=gtk_entry_get_text (EntryGtk);
  delk = atof(stk);
  update_entry_double (EntryGtk, & delk);
  if (delk != kmax)
  {
    kmax = delk;
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "skcheck");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
    if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispsk");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
    if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "skfact");
    if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "skmoother");
    if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
    j = numgr+numsq;
    i = numgr+numsq+numsk;
    hide_curves (&j, &i);
  }
}

G_MODULE_EXPORT void on_skfact_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *stk;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "skfact");
  stk = gtk_entry_get_text (EntryGtk);
  skfact = atof(stk);
  update_entry_double (EntryGtk, & skfact);
}

G_MODULE_EXPORT void on_Calcsk_released()
{
  int i, j, k;
  GtkWidget * info;
  GtkWidget * pop;
  GtkWidget * win;
  gboolean value;

  if (kmax <= qmin)
  {
    show_warning ("You must specify a Qmax\nand Qmax must be > Qmin");
  }
  else if (num_delta_k < 2)
  {
    show_warning ("You must specify a number of δq >= 2");
  }
  else
  {
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "skcheck");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
    gtk_widget_set_sensitive (info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispsk");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
    gtk_widget_set_sensitive (info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "deltak");
    gtk_widget_set_sensitive(info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "kmax");
    gtk_widget_set_sensitive(info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "skfact");
    gtk_widget_set_sensitive(info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "skmoother");
    gtk_widget_set_sensitive(info, 0);
    if (! skrun)
    {
      i = numgr + numsq;
      initsq (& i, tablegtksk);
      idsk = calloc(num_delta_k, sizeof*idsk);
      skrun = TRUE;
    }
    i = 0;
    setcalcs (& i);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Mainstatusbar");
    j=gtk_statusbar_push (GTK_STATUSBAR(info), 2, "Computing S(q) from Debye equation");
    win = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqkpoint");
    gtk_window_set_opacity (GTK_WINDOW (win), opac);
    pop=show_popup ("Please wait k points selection in progress", win);
    gtk_widget_show_all(pop);
    i = cqvf_ (& kmax, & num_delta_k, & qlim, & plim);
    gtk_widget_destroy(pop);
    if (i == 1)
    {
      pop=show_popup ("Please wait S(q) calculation in progress", win);
      gtk_widget_show_all(pop);
      k = s_of_k_ ();
      gtk_widget_destroy(pop);
      gtk_window_set_opacity (GTK_WINDOW (win), 1.0);
      gtk_statusbar_remove (GTK_STATUSBAR(info), 2, j);
      if (k != 1)
      {
        show_error ("The S(q) calculation has failed");
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "skmoother");
        gtk_widget_set_sensitive(info, 0);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "skfact");
        gtk_widget_set_sensitive(info, 0);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "grfft");
        value = gtk_action_get_sensitive (GTK_ACTION(info));
        if (value) gtk_action_set_sensitive (GTK_ACTION(info), 0);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "button-hgq");
        if (GTK_WIDGET_SENSITIVE(info)) gtk_widget_set_sensitive(info, 0);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "gqfft");
        if (GTK_WIDGET_VISIBLE(info)) gtk_widget_hide(info);
      }
      else
      {
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "skcheck");
        gtk_widget_set_sensitive (info, 1);
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 1);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispsk");
        gtk_widget_set_sensitive (info, 1);
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 1);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "skmoother");
        gtk_widget_set_sensitive(info, 1);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "skfact");
        gtk_widget_set_sensitive(info, 1);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "grfft");
        value = gtk_action_get_sensitive (GTK_ACTION(info));
        if (! value) gtk_action_set_sensitive (GTK_ACTION(info), 1);
        info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "button-hgq");
      }
      hide_gq_curves ();
    }
    else
    {
      show_error ("Problem during the selection of the k-points used to sample\nthe recipocal lattice");
      gtk_statusbar_remove (GTK_STATUSBAR(info), 2, j);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "smoother");
      gtk_widget_set_sensitive(info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "skfact");
      gtk_widget_set_sensitive(info, 0);
    }
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "deltak");
    gtk_widget_set_sensitive(info, 1);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "kmax");
    gtk_widget_set_sensitive(info, 1);
    i = 1;
    update_sq();
    setcalcs (& i);
  }
}

G_MODULE_EXPORT void on_skmoother_released (GtkButton * button)
{
  int i, j, k, l;

  if (skfact == 0.0 || skfact > 1.0)
  {
    show_warning ("You must specify a smoothing factor\n0.0 < sfact < 1.0");
  }
  else
  {
    l = 1;
    i = numgr+numsq+1;
    smooth_and_save_ (curves[i-1] -> data, & skfact, & i, & num_delta_k, & l);
    i = i+2;
    smooth_and_save_ (curves[i-1] -> data, & skfact, & i, & num_delta_k, & l);
    i = i+2;
    smooth_and_save_ (curves[i-1] -> data, & skfact, & i, & num_delta_k, & l);
    i = i+2;
    smooth_and_save_ (curves[i-1] -> data, & skfact, & i, & num_delta_k, & l);
    k = i+2;
    for (i=0 ; i<nspec ; i++)
    {
      for (j=0 ; j<nspec ; j++)
      {
        smooth_and_save_ (curves[k-1] -> data, & skfact, & k, & num_delta_k, & l);
        k = k +2;
      }
    }
    for (i=0 ; i<nspec ; i++)
    {
      for (j=0 ; j<nspec ; j++)
      {
        smooth_and_save_ (curves[k-1] -> data, & skfact, & k, & num_delta_k, & l);
        k = k +2;
      }
    }
    if (nspec == 2)
    {
      for (j=0 ; j<4; j++)
      {
        smooth_and_save_ (curves[k-1] -> data, & skfact, & k, & num_delta_k, & l);
        k = k +2;
      }
    }
    update_sq();
  }
}

G_MODULE_EXPORT void on_skcheck_toggled (GtkToggleButton * Button)
{
  GtkWidget * info;
  gboolean status;

  status = gtk_toggle_button_get_active (Button);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "SKlist");
  if (status)
  {
    if (! GTK_WIDGET_VISIBLE(info))
    {
      gtk_widget_show_all(info);
    }
  }
  else
  {
    if (GTK_WIDGET_VISIBLE(info))
    {
      gtk_widget_hide(info);
    }
  }
}

G_MODULE_EXPORT void on_plim_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *stk;
  double delk;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "plim");
  stk=gtk_entry_get_text (EntryGtk);
  delk = atof(stk);
  if (delk != plim)
  {
    if (delk < 0.0 || delk > 1.0)
    {
      show_warning ("You must specify a probability between 0.0 and 1.0");
    }
    else
    {
      plim = delk;
    }
  }
  update_entry_double (EntryGtk, & plim);
}

G_MODULE_EXPORT void on_qlim_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *stk;
  double delk;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "qlim");
  stk=gtk_entry_get_text (EntryGtk);
  delk = atof(stk);
  if (delk != qlim)
  {
    if (delk <= kmin)
    {
      show_warning ("Qlim must be > Qmin");
    }
    else
    {
      qlim = delk;
    }
  }
  update_entry_double (EntryGtk, & qlim);
}
