/* This file is part of I.S.A.A.C.S.

   I.S.A.A.C.S. is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

    I.S.A.A.C.S. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#ifdef G_OS_WIN32
#  include <windows.h>
#endif

#include "global.h"
#include "interface.h"
#include "callbacks.h"
#include "curve.h"
#include "bdcall.h"
#include "spcall.h"
#include "bvcall.h"

char sphname[50]="";

int convpow_ (int *, int *, int *);
int sphericals_ (int *, int *, int *);

void gtkcombosp (int * sp, int * ngsp, char ** lsp)
{
  int i;
  GtkFixed * boxfixed;

  if (shbox[*sp] != NULL)
  {
    if (GTK_WIDGET_VISIBLE(shbox[*sp])) gtk_widget_hide(shbox[*sp]);
    gtk_widget_destroy(shbox[*sp]);
  }
  if (bvbox[*sp] != NULL)
  {
    if (GTK_WIDGET_VISIBLE(bvbox[*sp])) gtk_widget_hide(bvbox[*sp]);
    gtk_widget_destroy(bvbox[*sp]);
  }
  shbox[*sp] = gtk_combo_box_new_text();
  bvbox[*sp] = gtk_combo_box_new_text();
  for ( i=0 ; i < * ngsp ; i++ )
  {
    gtk_combo_box_append_text (GTK_COMBO_BOX(shbox[*sp]), lsp[i]);
    gtk_combo_box_append_text (GTK_COMBO_BOX(bvbox[*sp]), lsp[i]);
  }
  gtk_combo_box_set_active(GTK_COMBO_BOX(shbox[*sp]), 0);
  gtk_combo_box_set_active(GTK_COMBO_BOX(bvbox[*sp]), 0);

  boxfixed = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixed-s2");
  gtk_fixed_put(boxfixed, shbox[*sp], -1, -1);
  gtk_widget_set_sensitive(shbox[*sp], 1);
  gtk_combo_box_set_active(GTK_COMBO_BOX(shbox[*sp]), 0);
  g_signal_connect(G_OBJECT(shbox[*sp]), "changed", G_CALLBACK(on_spherical_box), NULL);
  gtk_widget_hide(shbox[*sp]);

  boxfixed = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixed-b2");
  gtk_fixed_put(boxfixed, bvbox[*sp], -1, -1);
  gtk_widget_set_sensitive(bvbox[*sp], 1);
  gtk_combo_box_set_active(GTK_COMBO_BOX(bvbox[*sp]), 0);
  g_signal_connect(G_OBJECT(bvbox[*sp]), "changed", G_CALLBACK(on_bondvalence_box), NULL);
  gtk_widget_hide(bvbox[*sp]);
}

void initsh ()
{
  int i, j;

  i = numgr+numgq+numsq+numsk+numbd+numan+numri;
  for ( j = 0 ; j < nspec ; j++ )
  {
      curves[j+i] -> title = g_strdup_printf("%s%s%s", "Q(l) [", label[j], "] (l=0 -> lmax)");
  }
  for ( j = i ; j < i+numsh-1 ; j++ )
  {
    addcurwidget (curves[j] -> title, & j, & i, tablegtksh);
    curves[j] -> xlabel = g_strdup_printf("%s", "Ql");
    curves[j] -> cstart = i;
    curves[j] -> cend = i+numsh;
  }
}

void cadwidget_ (int * sspsh, int * sidsh)
{
  int i, j, m;
  char * val;
  char cname[50];

  i=numgr+numgq+numsq+numsk+numbd+numan+numri+numsh-1;
  strcpy(cname, "Q(l) ");
  strcat(cname, label[*sspsh]);
  strcat(cname, "(");

  strcpy(sphname, label[*sspsh]);
  strcat(sphname, "(");

  for ( j=0 ; j < nspec ; j++ )
  {
    m = convpow_ ( sidsh, & nspec, & j);
    val=g_strdup_printf( "%2d", m);
    if (m > 1)
    {
      strcat(sphname, label[j]);
      strcat(sphname, val);
      strcat(sphname, " ");
      strcat(cname, label[j]);
      strcat(cname, val);
      strcat(cname, " ");
    }
    else if (m == 1)
    {
      strcat(sphname,label[j]);
      strcat(sphname, " ");
      strcat(cname,label[j]);
      strcat(cname, " ");
    }
  }
  strcat(sphname, ")");
  strcat(cname, ")");
  curves[i] -> title = g_strdup_printf("%s", cname);
  if (curves[i] -> button != NULL)
  {
    gtk_button_set_label(GTK_BUTTON(curves[i] -> button), curves[i] -> title);
    gtk_widget_set_name (curves[i] -> button, curves[i] -> title);
    if (curves[i] -> window != NULL) gtk_window_set_title (GTK_WINDOW(curves[i] -> window), curves[i] -> title);
  }
  else
  {
    j=numgr+numgq+numsq+numsk+numbd+numan+numri;
    addcurwidget (curves[i] -> title, & i, & j, tablegtksh);
    curves[i] -> xlabel = g_strdup_printf("%s", "Ql");
    curves[i] -> cstart = j;
    curves[i] -> cend = j+numsh;
  }
}

G_MODULE_EXPORT void on_shspec_get (GtkComboBox * Box)
{
  int j;

  spsh = gtk_combo_box_get_active(Box);
  for ( j=0 ; j < nspec ; j++ )
  {
     gtk_widget_hide (shbox[j]);
  }
  gtk_widget_show (shbox[spsh]);
  idsh = gtk_combo_box_get_active(GTK_COMBO_BOX(shbox[spsh]));
}

G_MODULE_EXPORT void on_spherical_box (GtkComboBox * Sbox)
{
  idsh = gtk_combo_box_get_active(Sbox);
}

G_MODULE_EXPORT void on_lmax_activate ()
{
  GtkWidget * info;
  GtkEntry * EntryGtk;
  const gchar * sts;
  int dels;
  int i, j;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "lmax");
  sts=gtk_entry_get_text (EntryGtk);
  dels = atof(sts);
  i = dels + 1;
  update_entry_int (EntryGtk, & dels);
  if (i != vlmax)
  {
    vlmax = i;
    j= numgr+numgq+numsq+numsk+numbd+numan+numri;
    if (curves[j] -> data != NULL)
    {
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "spcheck");
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
      if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispsp");
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
      if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
    }
    i= numgr+numgq+numsq+numsk+numbd+numan+numri+numsh;
    hide_curves (&j, &i);
    for ( i = j ; i < j+numsh ; i ++ )
    {
      free(curves[j] -> data);
      curves[j] -> data = NULL;
    }
  }
}

G_MODULE_EXPORT void on_Calcspheric_released ()
{
  int i, k, m, n;
  GtkWidget * info;
  GtkWidget * pop;
  GtkWidget * win;

  if (vlmax >= 2 && vlmax < 40 && vlmax & 1)
  {
    i = 0;
    setcalcs (& i);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "spcheck");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
    gtk_widget_set_sensitive (info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispsp");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
    gtk_widget_set_sensitive (info, 0);
    gtk_widget_set_sensitive (shspec, 0);
    gtk_widget_set_sensitive (shbox[spsh], 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Mainstatusbar");
    k=gtk_statusbar_push (GTK_STATUSBAR(info), 7, "Computing spherical harmonics");
    win = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "spheric");
    gtk_window_set_opacity (GTK_WINDOW (win), opac);
    if (! shrun)
    {
      initsh ();
      shrun=TRUE;
    }
    if (steps > 1)
    {
      pop=show_popup ("Please wait spherical harmonics calculation in progress", win);
      gtk_widget_show_all(pop);
    }
    win = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "spheric");
    n = vlmax -1;
    m=sphericals_ (& n, & spsh , & idsh);
    if (steps > 1) gtk_widget_destroy(pop);
    gtk_window_set_opacity (GTK_WINDOW (win), 1.0);
    gtk_statusbar_remove (GTK_STATUSBAR(info), 7, k);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "spcheck");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), m);
    gtk_widget_set_sensitive (info, m);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispsp");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), m);
    gtk_widget_set_sensitive (info, m);
    gtk_widget_set_sensitive (shspec, m);
    gtk_widget_set_sensitive (shbox[spsh], m);
    i = 1;
    setcalcs (& i);
  }
  else
  {
     show_warning ("You must specify an 'even' l value: 2 =< lmax < 40");
  }
}

G_MODULE_EXPORT void on_spcheck_toggled (GtkToggleButton * Button)
{
  GtkWidget * info;
  gboolean status;

  status = gtk_toggle_button_get_active (Button);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "SHlist");
  if (status)
  {
    if (! GTK_WIDGET_VISIBLE(info))
    {
      gtk_widget_show_all(info);
    }
  }
  else
  {
    if (GTK_WIDGET_VISIBLE(info))
    {
      gtk_widget_hide(info);
    }
  }
}

void prinsphericals_ (double average[vlmax])
{
  GtkWidget * info;
  GtkTextBuffer * buffer = NULL;
  GtkTextIter bStart;
  GtkTextIter bEnd;
  int i;
  char sph1[31]="\nSpherical harmonics - Q(l)\n";
  char sph2[24]="Average Q(l) per bond\n";
  char sph3[10]="l\t   Q(l)\n";
  char sph4[38]="\nAverage Q(l) per chemical species\n";
  char tab[3]="\t";
  char * psph;
  char * lpsph;

  lpsph=g_strdup_printf("%s%s%s%s", sph1, retl, sph2, sph3);
  for ( i=0 ; i < vlmax ; i++ )
  {
    psph=g_strdup_printf("%s%d%s%lf%s", lpsph, i, tab, average[i], retl);
    g_free(lpsph);
    lpsph=g_strdup_printf("%s", psph);
    g_free(psph);
  }
  psph=g_strdup_printf("%s%s", lpsph, sph4);
  g_free(lpsph);
  g_print("%s", psph);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "TextInfo");
  buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(info));
  gtk_text_buffer_get_start_iter (buffer, &bStart);
  gtk_text_buffer_get_end_iter (buffer, &bEnd);
  gtk_text_buffer_insert (buffer, &bEnd, psph, -1);
  gtk_text_view_set_buffer (GTK_TEXT_VIEW(info), buffer);
  g_free(psph);
}

void spprintspherics_ (int * shp, double spdata[vlmax])
{
  GtkWidget * info;
  GtkTextBuffer * buffer = NULL;
  GtkTextIter bStart;
  GtkTextIter bEnd;
  int i;
  char ssph0[31]="\nSpherical harmonics - Q(l)\n";
  char ssph1[21]="\nAverage Q(l) per [";
  char ssph2[10]="] atom:\n";
  char ssph3[10]="l\tQ(l)\n";
  char tab[3]="\t";
  char * psphs;
  char * tpsphs;

  if (*shp == 0)
  {
    psphs=g_strdup_printf("%s%s", ssph0, ssph1);
  }
  else
  {
    psphs=g_strdup_printf("%s", ssph1);
  }
  tpsphs=g_strdup_printf("%s%s%s%s", psphs, label[* shp], ssph2, ssph3);
  g_free(psphs);
  for ( i=0 ; i < vlmax ; i++ )
  {
    psphs=g_strdup_printf("%s%d%s%lf%s", tpsphs, i, tab, spdata[i], retl);
    g_free(tpsphs);
    tpsphs=g_strdup_printf("%s", psphs);
    g_free(psphs);
  }
  g_print("%s", tpsphs);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "TextInfo");
  buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(info));
  gtk_text_buffer_get_start_iter (buffer, &bStart);
  gtk_text_buffer_get_end_iter (buffer, &bEnd);
  gtk_text_buffer_insert (buffer, &bEnd, tpsphs, -1);
  gtk_text_view_set_buffer (GTK_TEXT_VIEW(info), buffer);
  g_free(tpsphs);
}

void spprintsph_ (double spdat[vlmax])
{
  GtkWidget * info;
  GtkTextBuffer * buffer = NULL;
  GtkTextIter bStart;
  GtkTextIter bEnd;
  int i;
  char ssph1[21]="\nAverage Q(l) per [";
  char ssph2[17]="] environment:\n";
  char ssph3[10]="l\tQ(l)\n";
  char tab[3]="\t";
  char * psphs;
  char * tpsphs;

  psphs = g_strdup_printf("%s%s%s%s", ssph1, sphname, ssph2, ssph3);
  for ( i=0 ; i < vlmax ; i++ )
  {
    tpsphs=g_strdup_printf("%s%d%s%lf%s", psphs, i, tab, spdat[i], retl);
    g_free(psphs);
    psphs=g_strdup_printf("%s", tpsphs);
    g_free(tpsphs);
  }
  g_print("%s", psphs);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "TextInfo");
  buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(info));
  gtk_text_buffer_get_start_iter (buffer, &bStart);
  gtk_text_buffer_get_end_iter (buffer, &bEnd);
  gtk_text_buffer_insert (buffer, &bEnd, psphs, -1);
  gtk_text_view_set_buffer (GTK_TEXT_VIEW(info), buffer);
  g_free(psphs);
}
