! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

! Bond order parameter from spherical harmonics

LOGICAL FUNCTION PREPNEIGH(SP, GEO)

  USE PARAMETERS

  INTEGER, INTENT(IN) :: SP, GEO
  LOGICAL :: NEIRUN
  INTERFACE
    INTEGER FUNCTION convpow (VALC, VALS, VALI)
      INTEGER, INTENT(IN) :: VALC, VALS, VALI
    END FUNCTION
  END INTERFACE

  if (allocated(COOSPH)) deallocate(COOSPH)
  allocate(COOSPH(NSP), STAT=ERR)
  if (ERR .ne. 0) then
    call show_error ("Impossible to allocate memory"//CHAR(0), &
                     "Function: PREPNEIGH"//CHAR(0), "Table: COOSPH"//CHAR(0))
    PREPNEIGH=.false.
    goto 001
  endif
  do j=0, NSP-1
    COOSPH(j+1) = convpow (LISTE_GEOMSA(SP+1,GEO+1), NSP, j)
  enddo

  if (allocated(NEIGH)) deallocate(NEIGH)
  allocate(NEIGH(NSP), STAT=ERR)
  if (ERR .ne. 0) then
    call show_error ("Impossible to allocate memory"//CHAR(0), &
                     "Function: PREPNEIGH"//CHAR(0), "Table: NEIGH"//CHAR(0))
    PREPNEIGH=.false.
    goto 001
  endif
  if (allocated(NSPSH)) deallocate(NSPSH)
  allocate(NSPSH(NSP), STAT=ERR)
  if (ERR .ne. 0) then
    call show_error ("Impossible to allocate memory"//CHAR(0), &
                     "Function: PREPNEIGH"//CHAR(0), "Table: NSPSH"//CHAR(0))
    PREPNEIGH=.false.
    goto 001
  endif

  do i=1, NSP
    NSPSH(i)=0
  enddo

!  NBONDS=0
  ANBONDS=0
  do i=1, NS
    do j=1, NA
      NEIRUN=.true.
      do k=1, NSP
        NEIGH(k)=0
      enddo
      do k=1, CONTJ(j,i)
!        NBONDS=NBONDS+1
        NSPSH(LOT(j))=NSPSH(LOT(j))+1
        NEIGH(LOT(VOISJ(k,j,i)))=NEIGH(LOT(VOISJ(k,j,i)))+1
      enddo
      do k=1, NSP
        if (NEIGH(k) .ne. COOSPH(k)) then
          NEIRUN=.false.
          exit
        endif
      enddo
      if (NEIRUN) then
        do k=1, CONTJ(j,i)
          ANBONDS=ANBONDS+1
        enddo
      endif
    enddo
  enddo

  PREPNEIGH=.true.

  001 continue

END FUNCTION

INTEGER FUNCTION sphericals (MAXL, SPSH, GEOSH)

  USE PARAMETERS

  INTEGER, INTENT(IN) :: MAXL, SPSH, GEOSH

  LOGICAL :: SPHRUN
  DOUBLE PRECISION :: XC, YC, ZC
  DOUBLE PRECISION :: SR, ST, SP
  DOUBLE PRECISION, DIMENSION(0:MAXL,-MAXL:MAXL) :: HSP
! DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: THSP
  DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: ATHSP
  DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: SPTSHP
  DOUBLE PRECISION, DIMENSION(0:MAXL) :: SPHA
  DOUBLE PRECISION, DIMENSION(0:MAXL) :: SPHTAB
  DOUBLE PRECISION :: pro
  INTERFACE
    LOGICAL FUNCTION PREPNEIGH(SPSH, GEOSH)
      INTEGER, INTENT(IN) :: SPSH, GEOSH
    END FUNCTION
  END INTERFACE
  INTERFACE
    DOUBLE PRECISION FUNCTION PLEGENDRE (l, m, x)
      INTEGER, INTENT(IN) :: l, m
      DOUBLE PRECISION, INTENT(IN) :: x
    END FUNCTION
  END INTERFACE

  if (.not.PREPNEIGH(SPSH, GEOSH)) then
    sphericals=0
    goto 001
  endif

!  if (allocated(THSP)) deallocate(THSP)
!  allocate(THSP(NBONDS,0:MAXL,-MAXL:MAXL), STAT=ERR)
!  if (ERR .ne. 0) then
!    call show_error ("Impossible to allocate memory"//CHAR(0), &
!                     "Function: sphericals"//CHAR(0), "Table: THSP"//CHAR(0))
!    sphericals=0
!    goto 001
!  endif
  if (allocated(ATHSP)) deallocate(ATHSP)
  allocate(ATHSP(0:MAXL,-MAXL:MAXL), STAT=ERR)
  if (ERR .ne. 0) then
    call show_error ("Impossible to allocate memory"//CHAR(0), &
                     "Function: sphericals"//CHAR(0), "Table: ATHSP"//CHAR(0))
    sphericals=0
    goto 001
  endif
  if (allocated(SPTSHP)) deallocate(SPTSHP)
  allocate(SPTSHP(NSP,0:MAXL,-MAXL:MAXL), STAT=ERR)
  if (ERR .ne. 0) then
    call show_error ("Impossible to allocate memory"//CHAR(0), &
                     "Function: sphericals"//CHAR(0), "Table: SPTSHP"//CHAR(0))
    sphericals=0
    goto 001
  endif

  do l=0, MAXL
    do m=-MAXL, MAXL
!      do n=1, NBONDS
!        THSP(n,l,m)=0.0d0
!      enddo
      ATHSP(l,m)=0.0d0
      do n=1, NSP
        SPTSHP(n,l,m)=0.0d0
      enddo
    enddo
  enddo

  do i=1, NS
    if (NS .gt. 1) then
      pro=(i-1)/(1.0*NS)
      call setprogress (pro)
    endif
    do j=1, NA
      SPHRUN=.true.
      do k=1, NSP
        NEIGH(k)=0
      enddo
      do k=1, CONTJ(j,i)
        NEIGH(LOT(VOISJ(k,j,i)))=NEIGH(LOT(VOISJ(k,j,i)))+1
      enddo
      do k=1, NSP
        if (NEIGH(k) .ne. COOSPH(k)) then
          SPHRUN=.false.
          exit
        endif
      enddo
      do k=1, CONTJ(j,i)
        call CALCRIJ(j, VOISJ(k,j,i), i, i)
        XC=Rij(1)
        YC=Rij(2)
        ZC=Rij(3)
!        NBONDS=NBONDS+1
        call CART2SPHER(XC, YC, ZC, SR, ST, SP)
        do l=0, MAXL
          do m=0, l
            HSP(l,m) = 0.0d0
            HSP(l,m) = PLEGENDRE (l, m, ST) * cos(SP * m)
            if (m .ne. 0) HSP(l,-m) = (-1)**m*HSP(l,m)
          enddo
          do m=-l, l
!            THSP(NBONDS,l,m)=HSP(l,m)
            SPTSHP(LOT(j),l,m)=SPTSHP(LOT(j),l,m)+HSP(l,m)
            if (SPHRUN) then
              ATHSP(l,m)=ATHSP(l,m)+HSP(l,m)
            endif
          enddo
        enddo
      enddo
    enddo
  enddo

  call cadwidget (SPSH, LISTE_GEOMSA(SPSH+1,GEOSH+1))

!  do l=0, MAXL
!    SPHA(l)=0.0d0
!    do m=-l, l
!      MSP=0.0d0
!      do n=1, NBONDS
!        MSP=MSP+THSP(n,l,m)
!      enddo
!      SPHA(l)=SPHA(l)+(MSP/NBONDS)**2
!    enddo
!    SPHA(l)=sqrt(4*PI*SPHA(l)/(2*l+1))
!    SPHTAB(l)=SPHA(l)
!  enddo

!  call prinsphericals (SPHA)
  o=GRNUM+SQNUM+SKNUM+GQNUM+BDNUM+ANNUM+RINUM
!  call save_curve (MAXL+1, SPHTAB, o)

  do k=1, NSP
    do l=0, MAXL
      SPHA(l)=0.0d0
      do m=-l, l
        SPHA(l)=SPHA(l)+(SPTSHP(k,l,m)/NSPSH(k))**2
      enddo
      SPHA(l)=sqrt(4*PI*SPHA(l)/(2*l+1))
      SPHTAB(l)=SPHA(l)
    enddo
    call spprintspherics (k-1, SPHA)
    call save_curve (MAXL+1, SPHTAB, o)
    o=o+1
  enddo

  do l=0, MAXL
    SPHA(l)=0.0d0
    do m=-l, l
      SPHA(l)=SPHA(l)+(ATHSP(l,m)/ANBONDS)**2
    enddo
    SPHA(l)=sqrt(4*PI*SPHA(l)/(2*l+1))
    SPHTAB(l)=SPHA(l)
  enddo
  call save_curve (MAXL+1, SPHTAB, o)
  call spprintsph (SPHA)

  sphericals = 1
  001 continue

  if (NS.gt.1) then
    pro=1.0d0
    call setprogress (pro)
  endif

  if (allocated(COOSPH)) deallocate (COOSPH)
!  if (allocated(THSP)) deallocate(THSP)
  if (allocated(ATHSP)) deallocate(ATHSP)
  if (allocated(SPTSHP)) deallocate(SPTSHP)
  if (allocated(NSPSH)) deallocate(NSPSH)
  if (allocated(NEIGH)) deallocate(NEIGH)

END FUNCTION

SUBROUTINE CART2SPHER(XC, YC, ZC, RS, TS, PS)

DOUBLE PRECISION, INTENT(IN) :: XC, YC, ZC
DOUBLE PRECISION, INTENT(OUT) :: RS, TS, PS

RS=0.0d0
TS=0.0d0
PS=0.0d0

RS= sqrt(XC**2 + YC**2 + ZC**2)
TS= acos(ZC/RS)
PS= atan2(XC, YC)

END SUBROUTINE

! Adapted from Numercial Recipies !

DOUBLE PRECISION FUNCTION PLEGENDRE (l, m, x)

  DOUBLE PRECISION, PARAMETER :: PI=acos(-1.0)
  INTEGER, INTENT(IN) :: l, m
  DOUBLE PRECISION, INTENT(IN) :: x
  DOUBLE PRECISION :: y
  INTEGER :: i, ll
  DOUBLE PRECISION :: fact, oldfact, pll, pmm, pmmp1, omx2

  y=cos(x)
  pmm=1.0
  if (m > 0) then
    omx2=(1.0-y)*(1.0+y)
    fact=1.0
    i = 1
    do while (i.le.m)
      pmm = pmm * (omx2*fact/(fact+1.0))
      fact = fact + 2.0
      i = i+1
    enddo
  endif
  pmm=sqrt((2*m+1)*pmm/(4.0*PI))
  if (mod(m,2) .eq. 1) pmm=-pmm

  if (l .eq. m) then
    PLEGENDRE=pmm
  else
    pmmp1=y*sqrt(2.0*m+3.0)*pmm
    if (l .eq. (m+1)) then
      PLEGENDRE=pmmp1
    else
      oldfact=sqrt(2.0*m+3.0)
      ll=m+2
      do while (ll.le.l)
        fact=sqrt((4.0*ll*ll-1.0)/(ll*ll-m*m))
        pll=(y*pmmp1-pmm/oldfact)*fact
        oldfact=fact
        pmm=pmmp1
        pmmp1=pll
        ll=ll+1
      enddo
      PLEGENDRE=pll
    endif
  endif

END FUNCTION

DOUBLE PRECISION FUNCTION PLGNDR(l, m, x)

  DOUBLE PRECISION, PARAMETER :: PI=acos(-1.0)
  INTEGER :: j
  INTEGER, INTENT(IN) :: l, m
  DOUBLE PRECISION, INTENT(IN) :: x
  DOUBLE PRECISION :: prod=1.0d0

  INTERFACE
    DOUBLE PRECISION FUNCTION PLEGENDRE(l, m, x)
      INTEGER, INTENT(IN) :: l, m
      DOUBLE PRECISION, INTENT(IN) :: x
    END FUNCTION
  END INTERFACE

  j=l-m+1
  do while (j.le.l+m)
    prod = prod*j
    j=j+1
  enddo

  PLGNDR=sqrt(4.0*PI*prod/(2*l+1))*PLEGENDRE(l,m,x)

END FUNCTION
