! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

INTEGER FUNCTION s_of_q (QM, MQ, NQ)

! Total structure factor
! Partials structure factors

! Total structure factor:

!
!                         00        sin (q*r)
!    S(q)= 1 + 4*PI*rho* /   dr r² ------------ (g(r) - 1)
!                       0              (q*r)
!

! or:

!             1          Rmax                 sin (q*r)    sin (PI*r/Rmax)
!    S(q)= 1 + 4*PI*rho* /   dr r² (g(r) -1) ----------- * ---------------
!                       0                       (q*r)         PI*r/Rmax
!

! Partials structure factors:

!
!                                                       Rmax
!    S  (q)=     delta(a,b) + sqrt(x * x ) *4*PI*rho* /   dr r² (g  (r) - 1)
!     ab                            a   b            0            ab
!
!

! or:

!                                                    Rmax                      sin (q*r)    sin (PI*r/Rmax)
!    S  (q)=  delta(a,b) + sqrt(x * x ) *4*PI*rho*  /   dr r² (g  (r) - go  ) ----------- * ---------------
!     ab                         a   b             0             ab       ab     (q*r)         PI*r/Rmax
!

! with x = Nbre(a)/Nbre(tot)
!       a


USE PARAMETERS

IMPLICIT NONE

DOUBLE PRECISION, INTENT(IN) :: QM, MQ
DOUBLE PRECISION :: pro
DOUBLE PRECISION, DIMENSION (:), ALLOCATABLE :: SQTAB
INTEGER, INTENT(IN) :: NQ

QMAX = QM
QMIN = MQ
NUMBER_OF_Q_POINTS = NQ

if (allocated(Q_POINT)) deallocate(Q_POINT)
allocate(Q_POINT(NUMBER_OF_Q_POINTS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: s_of_q"//CHAR(0), "Table: Q_POINT"//CHAR(0))
  s_of_q = 0
  goto 001
endif
if (allocated(S)) deallocate(S)
allocate(S(NUMBER_OF_Q_POINTS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: s_of_q"//CHAR(0), "Table: S"//CHAR(0))
  s_of_q = 0
  goto 001
endif
if (allocated(XS)) deallocate(XS)
allocate(XS(NUMBER_OF_Q_POINTS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: s_of_q"//CHAR(0), "Table: XS"//CHAR(0))
  s_of_q = 0
  goto 001
endif
if (allocated(Sij)) deallocate(Sij)
allocate(Sij(NUMBER_OF_Q_POINTS,NSP,NSP), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: s_of_q"//CHAR(0), "Table: Sij"//CHAR(0))
  s_of_q = 0
  goto 001
endif

do i=1, NUMBER_OF_Q_POINTS
  S(i)=0.0d0
  XS(i)=0.0d0
  Q_POINT(i)=0.0d0
enddo

do i=1, NSP
do j=1, NSP
do k=1, NUMBER_OF_Q_POINTS
  Sij(k,j,i)=0.0d0
enddo
enddo
enddo

Rmax = (NUMBER_OF_DELTA_R)*DELTA

! 'QMIN' is minimum q modulus accessible considering the analysed
! lattice ie. the minimum modulus of the reciprocal cell vectors,
DELTA_Q=((QMAX-QMIN)/dble(NUMBER_OF_Q_POINTS))
do i=1, NUMBER_OF_Q_POINTS

  if ( mod(i,10) .eq. 0) then
    pro=(i-1)/(1.0*NUMBER_OF_Q_POINTS)
    call setprogress (pro)
  endif
  Q_POINT(i)= dble(i-1)*DELTA_Q+QMIN
  do j=1, NUMBER_OF_DELTA_R
    Phi = Q_POINT(i)*j*DELTA
    Fact_Rmax = PI*j*DELTA/Rmax
    Sinus_Fact_Rmax = sin(Fact_Rmax)/Fact_Rmax
    Sinus_phi = sin(Phi)/Phi
    S(i) = S(i) + SHELL_VOL(j)*(GrTOT(j) - 1)*Sinus_phi!*Sinus_Fact_Rmax
    XS(i) = XS(i) + SHELL_VOL(j)*(XGrTOT(j) - 1)*Sinus_phi
    do k=1, NSP
      do l=1, NSP
         Sij(i,k,l) = Sij(i,k,l) + SHELL_VOL(j)*(Gr_ij(j,k,l) - 1)*Sinus_phi!*Sinus_Fact_Rmax
      enddo
    enddo
  enddo
  S(i) = 1.0d0 + S(i)*TOTAL_DENSITY
  XS(i) = 1.0d0 + XS(i)*TOTAL_DENSITY
  do k=1, NSP
    do l=1, NSP
      if (l .eq. k) then
        Sij(i,k,l) = 1.0d0 + Xi(l)*Sij(i,k,l)*TOTAL_DENSITY
      else
        Sij(i,k,l) = sqrt(Xi(k)*Xi(l))*Sij(i,k,l)*TOTAL_DENSITY
      endif
    enddo
  enddo
enddo

call FZBT

if (allocated(SQTAB)) deallocate(SQTAB)
allocate(SQTAB(NUMBER_OF_Q_POINTS), STAT=ERR)
if (ERR .ne. 0) then
  call show_error ("Impossible to allocate memory"//CHAR(0), &
                   "Subroutine: s_of_q"//CHAR(0), "Table: SQTAB"//CHAR(0))
  s_of_q = 0
  goto 001
endif

pro=1.0d0
call setprogress (pro)

l=GRNUM
do k=1, NUMBER_OF_Q_POINTS
  SQTAB(k)= S(k)
enddo
call save_curve (NUMBER_OF_Q_POINTS, SQTAB, l)
l=l+2
do k=1, NUMBER_OF_Q_POINTS
  SQTAB(k)= (S(k)-1.0)*Q_POINT(k)
enddo
call save_curve (NUMBER_OF_Q_POINTS, SQTAB, l)
l=l+2
do k=1, NUMBER_OF_Q_POINTS
  SQTAB(k)= XS(k)
enddo
call save_curve (NUMBER_OF_Q_POINTS, SQTAB, l)
l=l+2
do k=1, NUMBER_OF_Q_POINTS
  SQTAB(k)= (XS(k)-1.0)*Q_POINT(k)
enddo
call save_curve (NUMBER_OF_Q_POINTS, SQTAB, l)
l=l+2
do i=1, NSP
do j=1, NSP
  do k=1, NUMBER_OF_Q_POINTS
    SQTAB(k)= Sij(k,i,j)
  enddo
  call save_curve (NUMBER_OF_Q_POINTS, SQTAB, l)
  l=l+2
enddo
enddo
do i=1, NSP
do j=1, NSP
  do k=1, NUMBER_OF_Q_POINTS
    SQTAB(k)= FZSij(k,i,j)
  enddo
  call save_curve (NUMBER_OF_Q_POINTS, SQTAB, l)
  l=l+2
enddo
enddo
if (NSP .eq. 2) then
  do i=1, 4
    do j=1, NUMBER_OF_Q_POINTS
      SQTAB(j)= BTij(j,i)
    enddo
    call save_curve (NUMBER_OF_Q_POINTS, SQTAB, l)
    l=l+2
  enddo
endif

s_of_q = 1

pro=0.0d0
call setprogress (pro)

001 continue

if (allocated(SQTAB)) deallocate(SQTAB)
if (allocated(Sij)) deallocate(Sij)
if (allocated(S)) deallocate(S)
if (allocated(XS)) deallocate(XS)
if (allocated(FZSij)) deallocate(FZSij)
if (NSP.eq.2 .and. allocated(BTij)) deallocate(BTij)

END FUNCTION
