/* This file is part of I.S.A.A.C.S.

   I.S.A.A.C.S. is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

    I.S.A.A.C.S. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>

#include "global.h"
#include "interface.h"
#include "callbacks.h"
#include "curve.h"
#include "sqcall.h"

int s_of_q_ (double *, double *, int *);
void smooth_and_save_ (double *, double *, int *, int *, int *);

void update_sq ()
{
  int i;

  if (sqrun)
  {
    for (i= numgr; i < numgr+numsq; i++)
    {
      if (curves[i] -> window != NULL) update_curve (GINT_TO_POINTER(i));
    }
  }
  if (skrun)
  {
    for (i= numgr+numsq; i < numgr+numsq+numsk; i++)
    {
      if (curves[i] -> window != NULL) update_curve (GINT_TO_POINTER(i));
    }
  }
}

void hide_sq_curves ()
{
  GtkWidget * info;
  int i, j;

  if (curves[numgr] -> data != NULL)
  {
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqcheck");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
    if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispsq");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
    if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqfact");
    if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqmoother");
    if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, 0);
  }
  j= numgr;
  i= numgr+numsq;
  hide_curves (&j, &i);
  for ( i = j ; i < j+numsq ; i ++ )
  {
    free(curves[i] -> data);
    curves[i] -> data = NULL;
  }
}

void initsq (int * sqval, GtkTable * tablewidget)
{
  int i, j, k;

  k = * sqval;
  curves[k] -> title = g_strdup_printf("%s", "S(q) Neutrons");
  k =k+1;
  curves[k] -> title = g_strdup_printf("%s", "S(q) Neutrons - smoothed");
  k = k+1;
  curves[k] -> title = g_strdup_printf("%s", "Q(q) Neutrons");
  k = k+1;
  curves[k] -> title = g_strdup_printf("%s", "Q(q) Neutrons - smoothed");
  k = k+1;
  curves[k] -> title = g_strdup_printf("%s", "S(q) X-rays");
  k = k+1;
  curves[k] -> title = g_strdup_printf("%s", "S(q) X-rays - smoothed");
  k = k+1;
  curves[k] -> title = g_strdup_printf("%s", "Q(q) X-rays");
  k = k+1;
  curves[k] -> title = g_strdup_printf("%s", "Q(q) X-rays - smoothed");
  k = k+1;
  for ( i = 0 ; i < nspec ; i++ )
  {
    for ( j = 0 ; j < nspec ; j++ )
    {
      curves[k] -> title = g_strdup_printf("%s%s%s%s%s", "AL(q)[", label[i], ",", label[j], "]");
      k=k+1;
      curves[k] -> title = g_strdup_printf("%s%s%s%s%s", "AL(q)[", label[i], ",", label[j], "] - smoothed");
      k=k+1;
    }
  }
  for ( i = 0 ; i < nspec ; i++ )
  {
    for ( j = 0 ; j < nspec ; j++ )
    {
      curves[k] -> title = g_strdup_printf("%s%s%s%s%s", "FZ(q)[", label[i], ",", label[j], "]");
      k=k+1;
      curves[k] -> title = g_strdup_printf("%s%s%s%s%s", "FZ(q)[", label[i], ",", label[j], "] - smoothed");
      k=k+1;
    }
  }
  if ( nspec == 2 )
  {
    curves[k] -> title = g_strdup_printf("%s", "BT(q)[NN]");
    k=k+1;
    curves[k] -> title = g_strdup_printf("%s", "BT(q)[NN] - smoothed");
    k=k+1;
    curves[k] -> title = g_strdup_printf("%s", "BT(q)[NC]");
    k=k+1;
    curves[k] -> title = g_strdup_printf("%s", "BT(q)[NC] - smoothed");
    k=k+1;
    curves[k] -> title = g_strdup_printf("%s", "BT(q)[CC]");
    k=k+1;
    curves[k] -> title = g_strdup_printf("%s", "BT(q)[CC] - smoothed");
    k=k+1;
    curves[k] -> title = g_strdup_printf("%s", "BT(q)[ZZ]");
    k=k+1;
    curves[k] -> title = g_strdup_printf("%s", "BT(q)[ZZ] - smoothed");
  }
  j = * sqval;
  for ( k = j ; k < j+numsq ; k++ )
  {
    addcurwidget (curves[k] -> title, & k, & j, tablewidget);
    curves[k] -> xlabel = g_strdup_printf("%s", "q [Å-1]");
    curves[k] -> cstart = j;
    curves[k] -> cend = j+numsq;
  }
}

G_MODULE_EXPORT void on_deltaq_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *stq;
  int i;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "deltaq");
  stq=gtk_entry_get_text (EntryGtk);
  i = atof(stq);
  update_entry_int (EntryGtk, & i);
  if (i != num_delta_q)
  {
    num_delta_q = i;
    hide_sq_curves();
  }
}

G_MODULE_EXPORT void on_qmax_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *stq;
  double delq;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "qmax");
  stq = gtk_entry_get_text (EntryGtk);
  delq = atof(stq);
  update_entry_double (EntryGtk, & delq);
  if (delq != qmax)
  {
    qmax = delq;
    hide_sq_curves();
  }
}

G_MODULE_EXPORT void on_sqfact_activate ()
{
  GtkEntry * EntryGtk;
  const gchar *stq;

  EntryGtk = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "sqfact");
  stq = gtk_entry_get_text (EntryGtk);
  sqfact = atof(stq);
  update_entry_double (EntryGtk, & sqfact);
}

G_MODULE_EXPORT void on_Calcsq_released()
{
  int i, j;
  GtkWidget * info;
  GtkWidget * pop;
  GtkWidget * win;

  if (qmax <= qmin)
  {
    show_warning ("You must specify a Qmax\nand Qmax must be > Qmin");
  }
  else if (num_delta_q < 2)
  {
     show_warning ("You must specify a number of δq >= 2");
  }
  else
  {
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqcheck");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
    gtk_widget_set_sensitive (info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispsq");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 0);
    gtk_widget_set_sensitive (info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "deltaq");
    gtk_widget_set_sensitive(info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "qmax");
    gtk_widget_set_sensitive(info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqfact");
    gtk_widget_set_sensitive(info, 0);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqmoother");
    gtk_widget_set_sensitive(info, 0);
    if (! sqrun)
    {
      i = numgr;
      initsq (& i, tablegtksq);
      sqrun = TRUE;
    }
    i = 0;
    setcalcs (& i);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Mainstatusbar");
    j=gtk_statusbar_push (GTK_STATUSBAR(info), 1, "Computing S(q) by FFT[g(r)]");
    win = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqfft");
    gtk_window_set_opacity (GTK_WINDOW (win), opac);
    pop=show_popup ("Please wait S(q) calculation in progress", win);
    gtk_widget_show_all(pop);
    i = s_of_q_ (& qmax, & qmin, & num_delta_q);
    gtk_widget_destroy(pop);
    gtk_window_set_opacity (GTK_WINDOW (win), 1.0);
    gtk_statusbar_remove (GTK_STATUSBAR(info), 1, j);
    if ( i != 1)
    {
      show_error ("The S(q) calculation has failed");
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqmoother");
      gtk_widget_set_sensitive(info, 0);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqfact");
      gtk_widget_set_sensitive(info, 0);
    }
    else
    {
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqcheck");
      gtk_widget_set_sensitive (info, 1);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 1);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispsq");
      gtk_widget_set_sensitive (info, 1);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 1);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqmoother");
      gtk_widget_set_sensitive(info, 1);
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqfact");
      gtk_widget_set_sensitive(info, 1);
    }
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "deltaq");
    gtk_widget_set_sensitive(info, 1);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "qmax");
    gtk_widget_set_sensitive(info, 1);
    i = 1;
    update_sq();
    setcalcs (& i);
  }
}

G_MODULE_EXPORT void on_sqmoother_released (GtkButton * button)
{
  int i, j, k, l;

  if (sqfact == 0.0 || sqfact > 1.0)
  {
    show_warning ("You must specify a smoothing factor\n0.0 < sfact < 1.0");
  }
  else
  {
    l = 0;
    i = numgr+1;
    smooth_and_save_ (curves[i-1] -> data, & sqfact, & i, & num_delta_q, & l);
    i = i+2;
    smooth_and_save_ (curves[i-1] -> data, & sqfact, & i, & num_delta_q, & l);
    i = i+2;
    smooth_and_save_ (curves[i-1] -> data, & sqfact, & i, & num_delta_q, & l);
    i = i+2;
    smooth_and_save_ (curves[i-1] -> data, & sqfact, & i, & num_delta_q, & l);
    k = i+2;
    for (i=0 ; i<nspec ; i++)
    {
      for (j=0 ; j<nspec ; j++)
      {
        smooth_and_save_ (curves[k-1] -> data, & sqfact, & k, & num_delta_q, & l);
        k = k +2;
      }
    }
    for (i=0 ; i<nspec ; i++)
    {
      for (j=0 ; j<nspec ; j++)
      {
        smooth_and_save_ (curves[k-1] -> data, & sqfact, & k, & num_delta_q, & l);
        k = k +2;
      }
    }
    if (nspec == 2)
    {
      for (j=0 ; j<4; j++)
      {
        smooth_and_save_ (curves[k-1] -> data, & sqfact, & k, & num_delta_q, & l);
        k = k +2;
      }
    }
    update_sq();
  }
}

G_MODULE_EXPORT void on_sqcheck_toggled (GtkToggleButton * Button)
{
  GtkWidget * info;
  gboolean status;

  status = gtk_toggle_button_get_active (Button);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "SQlist");
  if (status)
  {
    if (! GTK_WIDGET_VISIBLE(info))
    {
      gtk_widget_show_all(info);
    }
  }
  else
  {
    if (GTK_WIDGET_VISIBLE(info))
    {
      gtk_widget_hide(info);
    }
  }
}
