! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

!
! This file containes various tools
!

!********************************************************************
!
! Transformer un entier en chaine de charactères
! Transform an integer into character
!

SUBROUTINE CHARINT(Word, Num)

IMPLICIT NONE

INTEGER :: lc

INTEGER, INTENT(IN) :: Num

CHARACTER (LEN=16) :: Pseudo

CHARACTER (LEN=15), INTENT(INOUT) :: Word


Word=' '
write (Word, *) Num
lc=1
do while (Word(1:lc) .eq. ' ')

  lc= lc+1

enddo

Pseudo=Word
Word=' '

write (Word, *) Pseudo(lc:LEN_TRIM(Pseudo))

END SUBROUTINE

SUBROUTINE CREAT_LIST(ELEM_CR, CONT_CR)

USE PARAMETERS

IMPLICIT NONE

INTEGER, INTENT(IN) :: ELEM_CR, CONT_CR

TYPE (RING), POINTER :: DEB
TYPE (RING), POINTER :: PREC

nullify(DEB)
nullify(PREC)

allocate(FIRST, STAT=ERR)
if (ERR .ne. 0) then
  write (6, 001) t_p, "FIRST"
  STOP
endif

FIRST%INDICE = 0
FIRST%ELEMENT = ELEM_CR
FIRST%NEIGHBOR = CONT_CR
FIRST%PAST => PREC
FIRST%NEXT => DEB

001 FORMAT (/," Allocation Problem -> No space left ?! :: node ",i3," CREAT_LIST from ",a15)

END SUBROUTINE


SUBROUTINE DO_LIST(ELEM_DO, NEIGB_DO)

USE PARAMETERS

IMPLICIT NONE

INTEGER, INTENT(IN) :: ELEM_DO, NEIGB_DO

TYPE (RING), POINTER :: NEW

allocate(NEW, STAT=ERR)
if (ERR .ne. 0) then
  write (6, 001) t_p, "NEW"
  STOP
endif

ANNEAUX%NEXT => NEW
NEW%PAST => ANNEAUX
NEW%INDICE = ANNEAUX%INDICE+1
NEW%ELEMENT = ELEM_DO
NEW%NEIGHBOR = NEIGB_DO
ANNEAUX => NEW
if (allocated(RPAT)) then
  if (RPAT(ANNEAUX%ELEMENT) .eq. 0) then
    RPAT(ANNEAUX%ELEMENT) = 1
  endif
endif
001 FORMAT (/," Allocation Problem -> No space left ?! :: node ",i3," DO_LIST from ",a15)

END SUBROUTINE


SUBROUTINE CLEAN_LIST(EDEL)

USE PARAMETERS

IMPLICIT NONE

INTEGER, INTENT(IN) :: EDEL
INTEGER :: ELT

do ELT=1, EDEL

  ANNEAUX => ANNEAUX%PAST
  deallocate(ANNEAUX%NEXT)

enddo
deallocate(ANNEAUX)

END SUBROUTINE

!********************************************************************
!
! Calculer une distance
! Compute a distance
!

SUBROUTINE CALCRIJ(AT1, AT2, STEP_1, STEP_2)

USE PARAMETERS

IMPLICIT NONE

INTEGER, INTENT(IN) :: AT1, AT2, STEP_1, STEP_2
INTEGER :: R1, R2, R3, R4, R5, R6, R7, R8, R9, R10
DOUBLE PRECISION, DIMENSION(3) :: COORDA, COORDB
DOUBLE PRECISION, DIMENSION(6) :: Dik
DOUBLE PRECISION, DIMENSION(6,3) ::  Rik

if (STEP_1 .eq. -2) then

  do R1=1, 3
    COORDA(R1) = NFULLPOS(AT1,R1,STEP_2)
    COORDB(R1) = NFULLPOS(AT2,R1,STEP_2)
  enddo

elseif (STEP_1 .eq. -1) then

! Prepos for MSD
  do R1=1, 3
    COORDA(R1) = POB(AT2,R1)
    COORDB(R1) = POA(AT1,R1)
  enddo

else
! Standard inter-atomic distance
  do R1=1, 3
    COORDA(R1) = FULLPOS(AT1,R1,STEP_1)
    COORDB(R1) = FULLPOS(AT2,R1,STEP_2)
  enddo

endif

Dij=0.0d0
do R1=1, 3
  Rij(R1) = COORDA(R1) - COORDB(R1)
  Dij = Dij+Rij(R1)**2
enddo

if (.not.PBC) goto 001

! Test if the simulation box is cubic-like symetric
! For that we check the lattice angles
if ((ALPHA .eq. BETA .and. BETA .eq. GAMA) .and. GLASS) then
!write (6,*) ALPHA, BETA, GAMA
  Dij=0.0d0
  do R1=1, 3
    Rij(R1)=Rij(R1) - ANint(Rij(R1)/(modv(R1)))*(modv(R1))
    Dij=Dij+Rij(R1)**2
  enddo
  goto 001

else

! Else we have to check for symteric transformations
  R2=1

  do while (R2 .eq. 1)

    R2=0
    do R1=1, 3

! We need to see if we can reduce the (x,y,z) components of Rij
! by subscrating or adding the lattice vectors, as many times as required
! to put back the atoms in the same vritual MD box
      R9=0
      do R3=1, 6
        Dik(R3)=0.0d0
      enddo

      do R7=1, 3

! if the lattice is not cubic like we have to modify each component (x,y,z)
! of the distance acoording to the (x,y,z) component of the lattice
! construction vectors.

        R9=R9+1
        do R10=1, 3
! Each modification of the distance is stored before
! beeing compared with the original one
          Rik(R9,R10) = Rij(R10) - lvect(R7,R10)
          Dik(R9)=Dik(R9)+Rik(R9,R10)**2
        enddo

        R9=R9+1
        do R10=1, 3
! Each modification of the distance is stored before
! beeing compared with the original one
          Rik(R9,R10) = Rij(R10) + lvect(R7,R10)
          Dik(R9)=Dik(R9)+Rik(R9,R10)**2
        enddo

      enddo

      do R7=1, R9

! Now we look for the lowest distance
        if (Dik(R7) .lt. Dij) then
          Dij=Dik(R7)
          R2=1
          do R10=1,3
            Rij(R10) = Rik(R7,R10)
          enddo
         endif

       enddo
    enddo

  enddo

! At this point we have simply ''put-back'' the points in the same cell

  fact(1)=1
  fact(2)=-1
  fact(3)=0

! Now we contsruct the different vectors that you can add or substract
! to the inter-atomic distance to find the shortest one according to
! the lattice symetry.

  R5=0
  do R1=1,3
  do R2=1,3
  do R3=1,3
    R5=R5+1
    do R4=1,3
      pvect(R5,R4)=fact(R1)*lvect(1,R4)+fact(R2)*lvect(2,R4)+fact(R3)*lvect(3,R4)
    enddo
  enddo
  enddo
  enddo

  do R1=1,27
    pDij(R1)=0.0d0
  enddo

! Compute all the possible inter-atomic distances

  do R1=1,27
    do R2=1, 3
      pRij(R1,R2)=Rij(R2)+pvect(R1,R2)
      pDij(R1)=pDij(R1)+pRij(R1,R2)**2
    enddo
  enddo

! And then find the shortest one

  do R1=1,27

    if(pDij(R1) .lt. Dij) then

      Dij=pDij(R1)
      do R2=1,3
        Rij(R2)=pRij(R1,R2)
      enddo

    endif

  enddo

endif

001 continue

Dij=sqrt(Dij)

END SUBROUTINE

!********************************************************************
!
! Home made arcosine calculation
!

DOUBLE PRECISION FUNCTION SACOS (ANG)

USE PARAMETERS

DOUBLE PRECISION, INTENT(IN) :: ANG
DOUBLE PRECISION :: SCOS

if (ANG .lt. -1.0d0) then
  SCOS = -2.0d0 - ANG
else if (ANG .gt. 1.0d0) then
  SCOS = 2.0d0 - ANG
else
  SCOS = ANG
endif
SACOS = DACOS(SCOS)*180.d0/PI

END FUNCTION

!********************************************************************
!
! Calculer un angle direct
! Compute a direct angle
!

SUBROUTINE ANGIJK(ATG1, ATG2, ATG3, ASTEP)

USE PARAMETERS

IMPLICIT NONE

INTEGER, INTENT(IN) :: ATG1, ATG2, ATG3, ASTEP
INTEGER :: AG1

INTERFACE
  DOUBLE PRECISION FUNCTION SACOS (ANG)
    DOUBLE PRECISION, INTENT(IN) :: ANG
  END FUNCTION
END INTERFACE

TETA=0.0d0

call CALCRIJ(ATG2,ATG1,ASTEP,ASTEP)
Dil=Dij
do AG1=1, 3
  Ril(AG1) = Rij(AG1)
enddo

call CALCRIJ(ATG2,ATG3,ASTEP,ASTEP)
Vij=0.0d0
do AG1=1, 3
  Vij=Vij+Rij(AG1)*Ril(AG1)
enddo

TETA = SACOS(Vij/(Dij*Dil))

END SUBROUTINE

!********************************************************************
!
! Calculer un angle dièdre
! Compute a dihedral angle
!

SUBROUTINE DIEDRE(DG1, DG2, DG3, DG4, DSTEP)

USE PARAMETERS

IMPLICIT NONE

INTEGER, INTENT(IN) :: DG1, DG2, DG3, DG4, DSTEP
INTEGER :: AG
DOUBLE PRECISION :: D1, D2
DOUBLE PRECISION, DIMENSION(3) :: D12, D23, D34
DOUBLE PRECISION, DIMENSION(3) :: V1, V2

INTERFACE
  DOUBLE PRECISION FUNCTION SACOS (ANG)
    DOUBLE PRECISION, INTENT(IN) :: ANG
  END FUNCTION
END INTERFACE

call CALCRIJ(DG1,DG2,DSTEP,DSTEP)
do AG=1, 3
  D12(AG) = Rij(AG)
enddo
call CALCRIJ(DG2,DG3,DSTEP,DSTEP)
do AG=1, 3
  D23(AG) = Rij(AG)
enddo
call CALCRIJ(DG3,DG4,DSTEP,DSTEP)
do AG=1, 3
  D34(AG) = Rij(AG)
enddo

V1(1) = D12(2)*D23(3) - D12(3)*D23(2)
V1(2) = D12(3)*D23(1) - D12(1)*D23(3)
V1(3) = D12(1)*D23(2) - D12(2)*D23(1)

V2(1) = D23(2)*D34(3) - D23(3)*D34(2)
V2(2) = D23(3)*D34(1) - D23(1)*D34(3)
V2(3) = D23(1)*D34(2) - D23(2)*D34(1)

D1=0.0d0
D2=0.0d0
do AG=1, 3
  D1 = D1 + V1(AG)*V1(AG)
  D2 = D2 + V2(AG)*V2(AG)
enddo
D1=sqrt(D1)
D2=sqrt(D2)
Vij=0.0d0
if (D1.eq.0.0d0 .or. D2.eq.0.0d0) then
  TETA = 0.0d0
else
  do AG=1, 3
    Vij=Vij+V1(AG)*V2(AG)
  enddo
  Vij=Vij/(D1*D2)
  TETA = SACOS (Vij)
endif

END SUBROUTINE

!********************************************************************
!
! Trier une liste de valeur
! Sort a list of values - INTEGER
! This is the most simple sort
! For an improved routine (huge data to sort) see SORT3 in this file
!

SUBROUTINE TRI(TAB, DIMTAB)

IMPLICIT NONE

INTEGER, INTENT(IN) :: DIMTAB
INTEGER, DIMENSION(DIMTAB), INTENT(INOUT) :: TAB
INTEGER :: SORTX, SORTY, VALUE

do SORTX=2, DIMTAB

  VALUE=TAB(SORTX)

  do SORTY=SORTX-1, 1, -1

    if (TAB(SORTY) .le. VALUE) exit

    TAB(SORTY+1)=TAB(SORTY)

  enddo

  TAB(SORTY+1)=VALUE

enddo

END SUBROUTINE

!********************************************************************
!
! Trier une liste de valeur
! Sort a list of values - DOUBLE PRECISION
! This is the most simple sort
! For an improved routine (huge data to sort) see SORT3 in this file
!

SUBROUTINE TRI2DB(TAB, ITAB, DIMTAB)

IMPLICIT NONE

INTEGER, INTENT(IN) :: DIMTAB
INTEGER, DIMENSION(DIMTAB), INTENT(INOUT) :: ITAB
INTEGER :: SORTX, SORTY, VUE

DOUBLE PRECISION, DIMENSION(DIMTAB), INTENT(INOUT) :: TAB
DOUBLE PRECISION :: VALUE

do SORTX=2, DIMTAB

  VALUE=TAB(SORTX)
  VUE=ITAB(SORTX)

  do SORTY=SORTX-1, 1, -1

    if (TAB(SORTY) .le. VALUE) exit

    TAB(SORTY+1)=TAB(SORTY)
    ITAB(SORTY+1)=ITAB(SORTY)

  enddo

  TAB(SORTY+1)=VALUE
  ITAB(SORTY+1)=VUE

enddo

END SUBROUTINE

!********************************************************************
!
! Générateur de nombre aléatoire
! Numerical recipes random number generator 3
!

SUBROUTINE RAN3(idum, ran_3)

INTEGER, INTENT(INOUT) :: idum
INTEGER :: MBIG,MSEED,MZ
INTEGER ::  iii,iff,ii,inext,inextp,k
INTEGER :: mj,mk,ma(55)

DOUBLE PRECISION, INTENT(INOUT) :: ran_3
DOUBLE PRECISION :: FAC

PARAMETER (MBIG=1000000000,MSEED=161803398,MZ=0,FAC=1./MBIG)
SAVE  iff,inext,inextp,ma
DATA  iff /0/
if(idum.lt.0.or.iff.eq.0)then

  iff=1
  mj=MSEED-iabs(idum)
  mj=mod(mj,MBIG)
  ma(55)=mj
  mk=1
  do 11 iii=1,54

    ii=mod(21*iii,55)
    ma(ii)=mk
    mk=mj-mk
    if(mk.lt.MZ)mk=mk+MBIG
    mj=ma(ii)

11  continue

  do 13 k=1,4

    do 12 iii=1,55

      ma(iii)=ma(iii)-ma(1+mod(iii+30,55))
      if(ma(iii).lt.MZ)ma(iii)=ma(iii)+MBIG

12    continue

13  continue
  inext=0
  inextp=31
  idum=1

endif

inext=inext+1
if(inext.eq.56)inext=1
inextp=inextp+1
if(inextp.eq.56)inextp=1
mj=ma(inext)-ma(inextp)
if(mj.lt.MZ)mj=mj+MBIG
ma(inext)=mj
ran_3=mj*FAC

END SUBROUTINE

!********************************************************************
!
! Calculer la somme d'une série de valeurs
! Compute sum of a list of values
!

SUBROUTINE SOMME(TABS, LONGTS, SOMTAB)

INTEGER :: posm
INTEGER, INTENT(IN) :: LONGTS

DOUBLE PRECISION, INTENT(INOUT) :: SOMTAB
DOUBLE PRECISION, INTENT(IN), DIMENSION(LONGTS) :: TABS

SOMTAB=0.0d0

do posm=1, LONGTS

SOMTAB=SOMTAB+TABS(posm)

enddo

END SUBROUTINE

!********************************************************************
!
! Calculer la moyenne d'une série de valeurs
! Compute the average of list of values
!

SUBROUTINE MOYENNE(TABLEAU, LONGTM, MOYTAB)

INTEGER :: posm
INTEGER, INTENT(IN) :: LONGTM

DOUBLE PRECISION, INTENT(INOUT) :: MOYTAB
DOUBLE PRECISION, INTENT(IN), DIMENSION(LONGTM) :: TABLEAU

MOYTAB=0.0d0

do posm=1, LONGTM

MOYTAB=MOYTAB+TABLEAU(posm)

enddo

MOYTAB=MOYTAB/dble(LONGTM)

END SUBROUTINE

!********************************************************************
!
! Calculer l'écart type sur une série de valeurs
! Compute the standard error on a list of values
!

SUBROUTINE ECT_TYPE(MOYENNE, TABLEAU, LONGTE, EC_TYPE)

INTEGER :: pose
INTEGER, INTENT(IN) :: LONGTE

DOUBLE PRECISION, INTENT(IN) :: MOYENNE
DOUBLE PRECISION, INTENT(IN), DIMENSION(LONGTE) :: TABLEAU
DOUBLE PRECISION, INTENT(INOUT) :: EC_TYPE

EC_TYPE=0.0d0

do pose=1, LONGTE

EC_TYPE= EC_TYPE + (TABLEAU(pose) - MOYENNE)**2

enddo

EC_TYPE=sqrt(EC_TYPE/dble(LONGTE-1))

END SUBROUTINE

! Cas particulier lors des statistiques d'anneaux

SUBROUTINE ECT_TYPE_RINGS(MOYENNE, TABLEAU, LONGTE, LREPRES, EC_TYPE)

INTEGER :: pose
INTEGER, INTENT(IN) :: LONGTE, LREPRES

DOUBLE PRECISION, INTENT(IN) :: MOYENNE
DOUBLE PRECISION, INTENT(IN), DIMENSION(LONGTE) :: TABLEAU
DOUBLE PRECISION, INTENT(INOUT) :: EC_TYPE

EC_TYPE=0.0d0

do pose=1, LONGTE

  if (TABLEAU(pose) .ne. 0.0) EC_TYPE= EC_TYPE + (TABLEAU(pose) - MOYENNE)**2

enddo

EC_TYPE=sqrt(EC_TYPE/dble(LREPRES-1))

END SUBROUTINE

!********************************************************************
!
! Lisser une courbe / To smooth a curve
!

SUBROUTINE SMOOTH(TABTOLISS, GTOLISS, DIMTOLISS, SIGMALISS)

! Lissage selon une gaussienne
! Gaussian smoothing - zero based curve

IMPLICIT NONE

INTEGER, INTENT(IN) :: DIMTOLISS
INTEGER :: INDA, INDB, ERR

DOUBLE PRECISION, PARAMETER :: PI=3.14159265359d0
DOUBLE PRECISION :: DQ, DQ2
DOUBLE PRECISION :: FACTLISS
DOUBLE PRECISION, INTENT(IN) :: SIGMALISS

DOUBLE PRECISION, INTENT(INOUT), DIMENSION(DIMTOLISS) :: TABTOLISS
DOUBLE PRECISION, INTENT(IN), DIMENSION(DIMTOLISS) :: GTOLISS
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: NEWTAB

allocate(NEWTAB(DIMTOLISS), STAT=ERR)
if (ERR .ne. 0) then
  write (6, 001)  "NEWTAB"
  STOP
endif

do INDA=1, DIMTOLISS
  NEWTAB(INDA)=0.0d0
enddo

FACTLISS=1.0d0/(SIGMALISS*sqrt(2.0d0*PI))

do INDA=1, DIMTOLISS
  do INDB=1, DIMTOLISS

    if(INDB .eq. 1)then
      DQ= GTOLISS(2)-GTOLISS(1)
    elseif (INDB .eq. DIMTOLISS) then
      DQ = GTOLISS(DIMTOLISS)- GTOLISS(DIMTOLISS-1)
    else
      DQ = (GTOLISS(INDB+1)- GTOLISS(INDB-1))*0.5
    endif

    DQ2=(GTOLISS(INDA)-GTOLISS(INDB))*(GTOLISS(INDA)-GTOLISS(INDB))
    NEWTAB(INDA) = NEWTAB(INDA) + exp(-DQ2/(2.0d0*SIGMALISS*SIGMALISS))*TABTOLISS(INDB)*DQ

  enddo
  NEWTAB(INDA)=FACTLISS*NEWTAB(INDA)
enddo

do INDA=1, DIMTOLISS
  TABTOLISS(INDA)=NEWTAB(INDA)
enddo

deallocate(NEWTAB)

001 FORMAT (/," Allocation Problem -> No space left ?! :: SMOOTH allocation of ",a15)

END SUBROUTINE

SUBROUTINE SMOOTHSK(TABTOLISS, GTOLISS, DIMTOLISS, SIGMALISS)

! Lissage selon une gaussienne
! Gaussian smoothing - non-zero based curve

IMPLICIT NONE

INTEGER, INTENT(IN) :: DIMTOLISS
INTEGER :: INDA, INDB, INDC, ERR

DOUBLE PRECISION, PARAMETER :: PI=3.14159265359d0
DOUBLE PRECISION :: DQ, DQ2
DOUBLE PRECISION :: FACTLISS
DOUBLE PRECISION, INTENT(IN) :: SIGMALISS

DOUBLE PRECISION, INTENT(INOUT), DIMENSION(DIMTOLISS) :: TABTOLISS
DOUBLE PRECISION, INTENT(IN), DIMENSION(DIMTOLISS) :: GTOLISS
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: NEWTAB

allocate(NEWTAB(DIMTOLISS), STAT=ERR)
if (ERR .ne. 0) then
  write (6, 001)  "NEWTAB"
  STOP
endif

do INDA=1, DIMTOLISS
  NEWTAB(INDA)=0.0d0
enddo

FACTLISS=1.0d0/(SIGMALISS*sqrt(2.0d0*PI))

do INDA=1, DIMTOLISS
  if (TABTOLISS(INDA) .ne. -999999.0) then
    do INDB=1, DIMTOLISS
      if (TABTOLISS(INDB) .ne. -999999.0) then
        if(INDB .eq. 1)then
          INDC=2
          do while (TABTOLISS(INDC).eq.-999999.0)
            INDC=INDC+1
          enddo
          DQ= GTOLISS(INDC)-GTOLISS(INDC)
        elseif (INDB .eq. DIMTOLISS) then
          INDC=DIMTOLISS-1
          do while (TABTOLISS(INDC).eq.-999999.0)
            INDC=INDC-1
          enddo
          DQ = GTOLISS(DIMTOLISS)- GTOLISS(INDC)
        else
          INDC=INDB+1
          do while (TABTOLISS(INDC).eq.-999999.0)
            if (INDC .eq. DIMTOLISS) then
              INDC=INDB+1
              goto 002
            endif
            INDC=INDC+1
          enddo
          002 continue
          DQ=GTOLISS(INDC)
          INDC=INDB-1
          do while (TABTOLISS(INDC).eq.-999999.0)
            if (INDC .eq. 1) then
              INDC=INDB-1
              goto 003
            endif
            INDC=INDC-1
          enddo
          003 continue
          DQ = (DQ - GTOLISS(INDC))*0.5
        endif
        DQ2=(GTOLISS(INDA)-GTOLISS(INDB))*(GTOLISS(INDA)-GTOLISS(INDB))
        NEWTAB(INDA) = NEWTAB(INDA) + exp(-DQ2/(2.0d0*SIGMALISS*SIGMALISS))*TABTOLISS(INDB)*DQ
      endif
    enddo
  endif
  NEWTAB(INDA)=FACTLISS*NEWTAB(INDA)
enddo

do INDA=1, DIMTOLISS
  if (TABTOLISS(INDA) .ne. -999999.0) TABTOLISS(INDA)=NEWTAB(INDA)
enddo

deallocate(NEWTAB)

001 FORMAT (/," Allocation Problem -> No space left ?! :: SMOOTHSQ allocation of ",a15)

END SUBROUTINE

!********************************************************************
!
! Sort routine adapted from:
! Numerical sort array 3 - Quicksort
!

SUBROUTINE SORT3(arr, ndum)
!SUBROUTINE SORT3(arr, index, qvx, qvy, qvz, ndum)

INTEGER, INTENT(IN) :: ndum
!INTEGER, DIMENSION(ndum), INTENT(INOUT) :: index
INTEGER, DIMENSION(ndum) :: index

INTEGER, DIMENSION(ndum), INTENT(INOUT) :: arr
!DOUBLE PRECISION, DIMENSION(ndum), INTENT(INOUT) :: arr
!DOUBLE PRECISION, DIMENSION(ndum), INTENT(INOUT) :: qvx, qvy, qvz

call indexx_dp(arr, index, ndum)

arr=arr(index)
!qvx=qvx(index)
!qvy=qvy(index)
!qvz=qvz(index)

END SUBROUTINE

  SUBROUTINE indexx_dp(arr, index, n)

  IMPLICIT NONE
  INTEGER, PARAMETER :: NPAR_ARTH=16,NPAR2_ARTH=8
  INTEGER, DIMENSION(n), INTENT(INOUT) :: index
  INTEGER, PARAMETER :: NN=15, NSTACK=50
  INTEGER, INTENT(IN) :: n
  INTEGER :: k,i,j,indext,jstack,l,r
  INTEGER, DIMENSION(NSTACK) :: istack
  INTEGER :: a

  INTEGER, DIMENSION(n), INTENT(IN) :: arr
!  DOUBLE PRECISION, DIMENSION(n), INTENT(IN) :: arr
!  DOUBLE PRECISION :: a

  call arth(1,1,n,index)
  jstack=0
  l=1
  r=n
  do
    if (r-l < NN) then
      do j=l+1,r
        indext=index(j)
        a=arr(indext)
        do i=j-1,l,-1
          if (arr(index(i)) <= a) exit
          index(i+1)=index(i)
        end do
        index(i+1)=indext
      end do
      if (jstack .eq. 0) RETURN
      r=istack(jstack)
      l=istack(jstack-1)
      jstack=jstack-2
    else
      k=(l+r)/2
      call swap(index(k),index(l+1))
      call icomp_xchg(index(l),index(r))
      call icomp_xchg(index(l+1),index(r))
      call icomp_xchg(index(l),index(l+1))
      i=l+1
      j=r
      indext=index(l+1)
      a=arr(indext)
      do
        do
          i=i+1
          if (arr(index(i)) >= a) exit
        end do
        do
          j=j-1
          if (arr(index(j)) <= a) exit
        end do
        if (j < i) exit
        call swap(index(i),index(j))
      end do
      index(l+1)=index(j)
      index(j)=indext
      jstack=jstack+2
      if (jstack > NSTACK) write (6, *) 'Sort3 error: indexx: NSTACK too small'
      if (r-i+1 >= j-l) then
        istack(jstack)=r
        istack(jstack-1)=i
        r=j-1
      else
        istack(jstack)=j-1
        istack(jstack-1)=l
        l=i
      end if
    end if
  end do
  CONTAINS
!BL
  SUBROUTINE icomp_xchg(i, j)
  INTEGER, INTENT(INOUT) :: i,j
  INTEGER :: swp
  if (arr(j) < arr(i)) then
    swp=i
    i=j
    j=swp
  end if
  END SUBROUTINE icomp_xchg

  SUBROUTINE arth(first, increment, m, arth_i)
  INTEGER, INTENT(IN) :: first,increment,m
  INTEGER, DIMENSION(m) :: arth_i
  INTEGER :: k,k2,temp
  if (m > 0) arth_i(1)=first
  if (m <= NPAR_ARTH) then
    do k=2,m
      arth_i(k)=arth_i(k-1)+increment
    end do
  else
    do k=2,NPAR2_ARTH
      arth_i(k)=arth_i(k-1)+increment
    end do
    temp=increment*NPAR2_ARTH
    k=NPAR2_ARTH
    do
      if (k >= m) exit
      k2=k+k
      arth_i(k+1:min(k2,m))=temp+arth_i(1:min(k,m-k))
      temp=temp+temp
      k=k2
    end do
  end if
  END SUBROUTINE

  SUBROUTINE swap(a,b)
  INTEGER, INTENT(INOUT) :: a,b
  INTEGER :: dum
  dum=a
  a=b
  b=dum
  END SUBROUTINE swap

  END SUBROUTINE indexx_dp
!********************************************************************


INTEGER FUNCTION convpow (VALC, VALS, VALI)

INTEGER, INTENT(IN) :: VALC, VALS, VALI
INTEGER VA, VB, VC

VA = VALC
do VB=VALS-1, VALI+1, -1
  VC = VA/(100**VB)
  VC = VC*(100**VB)
  VA = VA - VC
enddo

convpow = VA/(100**VALI)

END FUNCTION
