! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

! Bond valence calculation

INTEGER FUNCTION bvalence (Dzero, b, Nf, SPBV, GEOBV)

  USE PARAMETERS

  INTEGER, INTENT(IN) :: SPBV, GEOBV
  INTEGER :: NBV
  DOUBLE PRECISION, INTENT(IN) :: Dzero, b, Nf
  DOUBLE PRECISION :: ADDV
  DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: BONDVAL, BVAL
  DOUBLE PRECISION :: pro
  LOGICAL :: BVRUN
  INTERFACE
    LOGICAL FUNCTION PREPNEIGH(SPBV, GEOBV)
      INTEGER, INTENT(IN) :: SPBV, GEOBV
    END FUNCTION
  END INTERFACE

  if (.not.PREPNEIGH(SPBV, GEOBV)) then
    bvalence=0
    goto 001
  endif

  if (allocated(BONDVAL)) deallocate(BONDVAL)
  allocate(BONDVAL(NSP+1), STAT=ERR)
  if (ERR .ne. 0) then
    call show_error ("Impossible to allocate memory"//CHAR(0), &
                     "Function: bvalence"//CHAR(0), "Table: BONDVAL"//CHAR(0))
    bvalence=0
    goto 001
  endif
  if (allocated(BVAL)) deallocate(BVAL)
  allocate(BVAL(NSP+1), STAT=ERR)
  if (ERR .ne. 0) then
    call show_error ("Impossible to allocate memory"//CHAR(0), &
                     "Function: bvalence"//CHAR(0), "Table: BVAL"//CHAR(0))
    bvalence=0
    goto 001
  endif

  do i=1, NSP+1
    BONDVAL(i)=0.0d0
    BVAL(i)=0.0d0
  enddo
  NBV=0
  do i=1, NS
    if (NS .gt. 1) then
      pro=(i-1)/(1.0*NS)
      call setprogress (pro)
    endif
    do j=1, NA
      BVRUN=.true.
      do k=1, NSP
        NEIGH(k)=0
      enddo
      do k=1, CONTJ(j,i)
        NEIGH(LOT(VOISJ(k,j,i)))=NEIGH(LOT(VOISJ(k,j,i)))+1
      enddo
      do k=1, NSP
        if (NEIGH(k) .ne. COOSPH(k)) then
          BVRUN=.false.
          exit
        endif
      enddo
      if (BVRUN) NBV=NBV+1
      do k=1, CONTJ(j,i)
        call CALCRIJ(j, VOISJ(k,j,i), i, i)
        ADDV=exp((Dzero-Dij)/b)
        BONDVAL(LOT(j)) = BONDVAL(LOT(j)) + ADDV
        if (BVRUN) BONDVAL(NSP+1) = BONDVAL(NSP+1) + ADDV
        ADDV=(Dij/Dzero)**(1.0/Nf)
        BVAL(LOT(j)) = BVAL(LOT(j)) + ADDV
        if (BVRUN) BVAL(NSP+1) = BVAL(NSP+1) + ADDV
      enddo
    enddo
  enddo

  j=0
  do i=1, NSP
    BONDVAL(i) = BONDVAL(i) / (NBSPBS(i)*NS)
    call  printvalence (i-1, BONDVAL(i), j)
  enddo
  BONDVAL(NSP+1) = BONDVAL(NSP+1) / NBV
  call  printval (LISTE_GEOMSA(SPBV+1,GEOBV+1), BONDVAL(NSP+1))

  j=1
  do i=1, NSP
    BVAL(i) = BVAL(i) / (NBSPBS(i)*NS)
    call  printvalence (i-1, BVAL(i), j)
  enddo
  BVAL(NSP+1) = BVAL(NSP+1) / NBV
  call  printval (LISTE_GEOMSA(SPBV+1,GEOBV+1), BVAL(NSP+1))

  bvalence = 1
  001 continue

  if (NS.gt.1) then
    pro=1.0d0
    call setprogress (pro)
  endif

  if (allocated(COOSPH)) deallocate (COOSPH)
  if (allocated(NSPSH)) deallocate(NSPSH)
  if (allocated(NEIGH)) deallocate(NEIGH)
  if (allocated(BONDVAL)) deallocate(BONDVAL)
  if (allocated(BVAL)) deallocate(BVAL)

END FUNCTION
