/* This file is part of I.S.A.A.C.S.

   I.S.A.A.C.S. is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

    I.S.A.A.C.S. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <libxml/encoding.h>
#include <libxml/xmlwriter.h>
#include <libxml/xmlreader.h>
#include <libxml/parser.h>

#include "global.h"
#include "interface.h"
#include "callbacks.h"

#define MY_ENCODING "UTF-8"

double new_mass_ (int *, double *);
double new_radius_ (int *, double *);
double new_nscatt_ (int *, double *);
double new_xscatt_ (int *, double *);

size_t strfind (char * ida)
{
  size_t a, b, c;

  a=strlen(ida);
  b = 0;
  for ( c=0 ; c < a ; c++)
  {
    if (ida[c] != ' ')
    {
      b++;
    }
  }
  return b;
}

int XmlwriterFilename (const char *uri)
{
  int rc;
  int i, j;
  xmlTextWriterPtr writer;

  char isaacinfo[16] = " I.S.A.A.C.S. v";
  char xmlinfo[11]=" XML file ";
  const xmlChar intro[50]="";
  char * val;
  char * ncut;
  size_t lgt=10;

  /* Create a new XmlWriter for uri, with no compression. */
  writer = xmlNewTextWriterFilename(uri, 0);
  if (writer == NULL) return 0;
  rc = xmlTextWriterSetIndent(writer, 1);
  if (rc < 0) return 0;
  /* Start the document with the xml default for the version,
   * encoding MY_ENCODING and the default for the standalone
   * declaration. */
  rc = xmlTextWriterStartDocument(writer, NULL, MY_ENCODING, NULL);
  if (rc < 0) return 0;


  strcpy((char *)intro, isaacinfo);
  strcat((char *)intro, VERSION);
  strcat((char *)intro, xmlinfo);
  rc = xmlTextWriterWriteComment(writer, intro);
  if (rc < 0) return 0;
  rc = xmlTextWriterStartElement(writer, BAD_CAST "isaacs-xml");
  if (rc < 0) return 0;

  /* Start the "data" element. Since thist is the first
   * element, this will be the root element of the document.
   <data>
	  <type> </type>
	  <file> <file>
   </data>*/

  rc = xmlTextWriterWriteComment(writer, (const xmlChar *)" Data format and file containing the configuration(s) ");
  if (rc < 0) return 0;
  rc = xmlTextWriterStartElement(writer, BAD_CAST (const xmlChar *)"data");
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"type", "%s", typefile);
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"file", "%s", coordfile);
  if (rc < 0) return 0;
  rc = xmlTextWriterEndElement(writer);
  if (rc < 0) return 0;

  /* Start the "chemistry" element.
  <chemistry>
    <atoms> </atoms>
    <species number=" ">
      <label id="0"> </label>
      ...
    </species>
    <element symbol="">
	  <name> </name>
	  <z> </z>
	  <mass> </mass>
	  <rad> </rad>
	  <radius> </radius>
	  <nscatt> </nscatt>
	  <xscatt> </xscatt>
    </element>
  </chemistry> */

  rc = xmlTextWriterWriteComment(writer, (const xmlChar *)" Chemistry information ");
  if (rc < 0) return 0;
  rc = xmlTextWriterStartElement(writer, BAD_CAST (const xmlChar *)"chemistry");
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"atoms", "%d", natomes);
  if (rc < 0) return 0;
  rc = xmlTextWriterStartElement(writer, BAD_CAST (const xmlChar *)"species");
  if (rc < 0) return 0;
  val=g_strdup_printf("%d",nspec);
  rc = xmlTextWriterWriteAttribute(writer, BAD_CAST (const xmlChar *)"number", BAD_CAST val);
  g_free(val);
  if (rc < 0) return 0;
  for ( i=0 ; i<nspec ; i++)
  {
    rc = xmlTextWriterStartElement(writer, BAD_CAST (const xmlChar *)"label");
    if (rc < 0) return 0;
    val=g_strdup_printf("%d",i);
    rc = xmlTextWriterWriteAttribute(writer, BAD_CAST (const xmlChar *)"id", BAD_CAST val);
    g_free(val);
    if (rc < 0) return 0;
    rc = xmlTextWriterWriteFormatString (writer, "%s", label[i]);
    if (rc < 0) return 0;
    rc = xmlTextWriterEndElement(writer);
    if (rc < 0) return 0;
  }
  rc = xmlTextWriterEndElement(writer);
  if (rc < 0) return 0;
  for ( i=0 ; i<nspec ; i++)
  {
    rc = xmlTextWriterStartElement(writer, BAD_CAST (const xmlChar *)"element");
    if (rc < 0) return 0;
    rc = xmlTextWriterWriteAttribute(writer, BAD_CAST (const xmlChar *)"symbol", BAD_CAST label[i]);
    if (rc < 0) return 0;
    rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"name", "%s", elt_name[i]);
    if (rc < 0) return 0;
    rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"z", "%d", atid[i]);
    if (rc < 0) return 0;
    rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"mass", "%lf", amass[i]);
    if (rc < 0) return 0;
    rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"rad", "%d", radid[i]);
    if (rc < 0) return 0;
    rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"radius", "%lf", rad[i]);
    if (rc < 0) return 0;
    rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"nscatt", "%lf", nscatt[i]);
    if (rc < 0) return 0;
    rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"xscatt", "%lf", xscatt[i]);
    if (rc < 0) return 0;
    rc = xmlTextWriterEndElement(writer);
    if (rc < 0) return 0;
  }
  rc = xmlTextWriterEndElement(writer);
  if (rc < 0) return 0;

  /* Start the "box" element.
  <box>
    <edges>
      <a></a>
      <b></b>
      <c></c>
    </edges>
    <angles> </angles>
    <vectors>
      <a.x></a.x>
      <a.y></a.y>
      <a.z></a.z>
      <b.x></b.x>
      <b.y></b.y>
      <b.z></b.z>
      <c.x></c.x>
      <c.y></c.y>
      <c.z></c.z>
    </vectors>
  </box> */

  rc = xmlTextWriterWriteComment(writer, (const xmlChar *)" Box information ");
  if (rc < 0) return 0;
  rc = xmlTextWriterStartElement(writer, BAD_CAST (const xmlChar *)"box");
  if (rc < 0) return 0;
  rc = xmlTextWriterStartElement(writer, BAD_CAST (const xmlChar *)"edges");
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"a", "%lf", vmod[0]);
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"b", "%lf", vmod[1]);
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"c", "%lf", vmod[2]);
  if (rc < 0) return 0;
  rc = xmlTextWriterEndElement(writer);
  if (rc < 0) return 0;
  rc = xmlTextWriterStartElement(writer, BAD_CAST (const xmlChar *)"angles");
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"alpha", "%lf", langles[0]);
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"beta", "%lf", langles[1]);
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"gamma", "%lf", langles[2]);
  if (rc < 0) return 0;
  rc = xmlTextWriterEndElement(writer);
  if (rc < 0) return 0;
  rc = xmlTextWriterStartElement(writer, BAD_CAST (const xmlChar *)"vectors");
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"a.x", "%lf", cvect[0][0]);
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"a.y", "%lf", cvect[0][1]);
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"a.z", "%lf", cvect[0][2]);
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"b.x", "%lf", cvect[1][0]);
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"b.y", "%lf", cvect[1][1]);
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"b.z", "%lf", cvect[1][2]);
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"c.x", "%lf", cvect[2][0]);
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"c.y", "%lf", cvect[2][1]);
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"c.z", "%lf", cvect[2][2]);
  if (rc < 0) return 0;
  rc = xmlTextWriterEndElement(writer);
  if (rc < 0) return 0;
  rc = xmlTextWriterEndElement(writer);
  if (rc < 0) return 0;

 /* Start the "pbc" element.
  <pbc>
    <apply></apply>
    <fractional></fractional>
    <fractype></fractype>
  </pbc>
  */

  rc = xmlTextWriterWriteComment(writer, (const xmlChar *)" PBC information ");
  if (rc < 0) return 0;
  rc = xmlTextWriterStartElement(writer, BAD_CAST (const xmlChar *)"pbc");
  if (rc < 0) return 0;
  if (apbc)
  {
    rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"apply", "%s", "TRUE");
  }
  else
  {
    rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"apply", "%s", "FALSE");
  }
  if (rc < 0) return 0;
  if (fractional || runfrac)
  {
    rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"fractional", "%s", "TRUE");
  }
  else
  {
    rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"fractional", "%s", "FALSE");
  }
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"fractype", "%d", wtfrac);
  if (rc < 0) return 0;
  rc = xmlTextWriterEndElement(writer);
  if (rc < 0) return 0;

 /* Start the "cutoffs" element.
  <cutoffs>
    <total></total>
    <partials>
      <label[spi]-label[spj]></label[spi]-label[spj]>
    </partials>
  </cutoffs>
  */

  rc = xmlTextWriterWriteComment(writer, (const xmlChar *)" Cutoffs information ");
  if (rc < 0) return 0;
  rc = xmlTextWriterStartElement(writer, BAD_CAST (const xmlChar *)"cutoffs");
  if (rc < 0) return 0;
  rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"total", "%lf", grtotcutoff);
  if (rc < 0) return 0;
  rc = xmlTextWriterStartElement(writer, BAD_CAST (const xmlChar *)"partials");
  if (rc < 0) return 0;
  for ( i=0 ; i<nspec ; i++ )
  {
    for ( j=0 ; j<nspec ; j++ )
    {
      lgt=10;
      lgt+=strfind(label[i]) + strlen("-") + strfind(label[j]);
      ncut=calloc(lgt,sizeof*ncut);
      strncpy (ncut, label[i], strfind(label[i]));
      strncat(ncut, "-", 1);
      strncat(ncut, label[j], strfind(label[j]));
      rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST ncut, "%lf", cutoffs[i][j]);
      if (rc < 0) return 0;
      free(ncut);
      ncut=NULL;
    }
  }
  rc = xmlTextWriterEndElement(writer);
  if (rc < 0) return 0;
  rc = xmlTextWriterEndElement(writer);
  if (rc < 0) return 0;

  /* Start the "time series" element.
  <time-series>
    <dt></dt>
    <unit></unit>
    <ndt></ndt>
  </time-series>
  */

  if (steps > 1)
  {
    rc = xmlTextWriterWriteComment(writer, (const xmlChar *)" Time series ");
    if (rc < 0) return 0;
    rc = xmlTextWriterStartElement(writer, BAD_CAST (const xmlChar *)"time-series");
    if (rc < 0) return 0;
    rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"dt", "%lf", delta_t);
    if (rc < 0) return 0;
    if (tunit == 0)
    {
      rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"unit", "%s", untime[5]);
    }
    else
    {
      rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"unit", "%s", untime[tunit-1]);
    }
    if (rc < 0) return 0;
    rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"ndt", "%d", ndtbs);
    if (rc < 0) return 0;
    rc = xmlTextWriterEndElement(writer);
    if (rc < 0) return 0;
  }

  /* Start the "project" element.
  <project></project>
  */

  rc = xmlTextWriterWriteComment(writer, (const xmlChar *)" Apply project ");
  if (rc < 0) return 0;
  if (prun)
  {
    rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"project", "%s", "TRUE");
  }
  else
  {
    rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST (const xmlChar *)"project", "%s", "FALSE");
  }
  if (rc < 0) return 0;
// Closing "</isaacs-xml>"
  rc = xmlTextWriterEndElement(writer);
  if (rc < 0) return 0;

  rc = xmlTextWriterEndDocument(writer);
  if (rc < 0) return 0;

  xmlFreeTextWriter(writer);
  return 1;
}

int write_xml (const char * filetosave)
{
  /* first, the file version */
  int res=XmlwriterFilename(filetosave);
  /*
   * Cleanup function for the XML library.
   */
  xmlCleanupParser();
  /*
   * this is to debug memory for regression tests
   */
  xmlMemoryDump();
  return res;
}

gboolean file_exists(const char * filename)
{
  FILE * file;
  if ((file = fopen(filename, "r")))
  {
    fclose(file);
    return TRUE;
  }
  return FALSE;
}

xmlNodePtr findnode (xmlNodePtr startnode, char * nname)
{
  xmlNodePtr tmp;

  tmp = startnode;
  while (strcmp((char *)(tmp -> name), nname) != 0)
  {
    if (tmp -> next == NULL)
    {
      return NULL;
    }
    tmp = tmp -> next;
  }
  return tmp;
}

int setprop (xmlNodePtr pnode)
{
  int i, j, res;
  xmlAttrPtr pspec;
  xmlNodePtr idn, ptnode, ptn;
  xmlChar * data;
  double val;

  ptnode=pnode;
  for ( i=0 ; i<nspec ; i++)
  {
    ptnode = findnode(ptnode, "element");
    if (ptnode == NULL)
    {
      res=6;
      goto pend;
    }
    pspec = ptnode -> properties;
    idn = pspec -> children;
    data = xmlNodeGetContent(idn);
    if (strcmp((char *)data, label[i]) != 0)
    {
      res=6;
      goto pend;
    }
    ptnode = ptnode -> children;
    ptn = findnode(ptnode, "name");
    if (ptn == NULL)
    {
      res=6;
      goto pend;
    }
    data = xmlNodeGetContent(ptn);
    if (strcmp((char *)data, elt_name[i]) != 0)
    {
      res=6;
      goto pend;
    }
    ptn = findnode(ptnode, "z");
    if (ptn == NULL)
    {
      res=6;
      goto pend;
    }
    data= xmlNodeGetContent(ptn);
    val = atof((char *)data);
    j = val;
    if (atid[i] != j)
    {
      res=6;
      goto pend;
    }
    ptn = findnode(ptnode, "mass");
    if (ptn == NULL)
    {
      res=6;
      goto pend;
    }
    data= xmlNodeGetContent(ptn);
    val = atof((char *)data);
    if (val != amass[i])
    {
     amass[i] = new_mass_ (& i, & val);
    }
    ptn = findnode(ptnode, "rad");
    if (ptn == NULL)
    {
      res=6;
      goto pend;
    }
    data= xmlNodeGetContent(ptn);
    val = atof((char *)data);
    j = val;
    radid[i]=j;
    ptn = findnode(ptnode, "radius");
    if (ptn == NULL)
    {
      res=6;
      goto pend;
    }
    data= xmlNodeGetContent(ptn);
    val = atof((char *)data);
    if (val != rad[i])
    {
     rad[i] = new_radius_ (& i, & val);
    }
    ptn = findnode(ptnode, "nscatt");
    if (ptn == NULL)
    {
      res=6;
      goto pend;
    }
    data= xmlNodeGetContent(ptn);
    val = atof((char *)data);
    if (val != nscatt[i])
    {
     nscatt[i] = new_nscatt_ (& i, & val);
    }
    ptn = findnode(ptnode, "xscatt");
    if (ptn == NULL)
    {
      res=6;
      goto pend;
    }
    data= xmlNodeGetContent(ptn);
    val = atof((char *)data);
    if (val != xscatt[i])
    {
     xscatt[i] = new_xscatt_ (& i, & val);
    }
    ptnode = ptnode -> parent;
    ptnode = ptnode -> next;
  }
  res = 0;
  gtk_combo_box_set_active(GTK_COMBO_BOX(specbox), res);
  show_spec_info (& res);

pend:
  return res;
}

int testopening (char * tdata, char * tfichier)
{
  char * regtypes[7] = {"XYZ file", "PDB file", "Chem3D file", "CPMD trajectory", "VASP trajectory", "multiple XYZ file", "multiple Chem3D file"};
  int i, j;
  char * err;
  GtkWidget * info;

  j=-2;
  for ( i=0 ; i<nformats ; i ++ )
  {
    if (strcmp(regtypes[i], tdata) == 0) j=i;
  }
  if (j == 5) j=0;
  if (j == 6) j=2;
  if (j != -2)
  {
    if (file_exists (tfichier))
    {
      typefile = tdata;
      coordfile = tfichier;
      info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "FileType");
      gtk_combo_box_set_active (GTK_COMBO_BOX(info), j);
      open_file (& j);
      j=0;
      return j;
    }
    else
    {
      j=4;
      err=g_strdup_printf("The %s\n %s\n from the XML file does not exist\n", tdata, tfichier);
      show_error (err);
      g_free(err);
      return j;
    }
  }
  else
  {
    show_error ("Unknown data format in XML file\n");
    j=4;
    return j;
  }
}

int setchemistry (xmlNodePtr xsnode)
{
  xmlNodePtr idn, cs, xnode;
  xmlAttrPtr xspec;
  xmlChar * data;
  int ats, res, i;
  double val;

  xnode = xsnode -> children;
  cs = findnode(xnode, "atoms");
  if (cs == NULL)
  {
    res=5;
    goto xend;
  }
  data = xmlNodeGetContent(cs);
  val = atof((char *)data);
  ats = val;
  if (ats != natomes)
  {
    show_warning ("The number of atoms in the XML file\n"
                  "is not the same that the number of atoms\n"
                  "in the file containing the coordinates.\n"
                  "Other informations from the XML file\n"
                  "will be ignored\n");
    res=5;
    goto xend;
  }
  else
  {
    cs = findnode(xnode, "species");
    if (xnode == NULL)
    {
      res=5;
      goto xend;
    }
    xspec = cs -> properties;
    idn = xspec -> children;
    data = xmlNodeGetContent(idn);
    val = atof((char *)data);
    ats = val;
    if (ats != nspec)
    {
      show_warning ("The number of chemical species in the XML file\n"
                    "is not the same that the number of chemical species\n"
                    "in the file containing the coordinates.\n"
                    "Other informations from the XML file\n"
                    "will be ignored\n");
      res = 5;
      goto xend;
    }
    else
    {
      cs = cs -> children;
      for ( i=0 ; i<nspec ; i++)
      {
        cs = findnode(cs, "label");
        data = xmlNodeGetContent(cs);
        if (cs == NULL)
        {
          res=5;
          goto xend;
        }
        xspec = cs -> properties;
        idn = xspec -> children;
        data = xmlNodeGetContent(idn);
        val = atof((char *)data);
        ats = val;
        if (ats != i)
        {
          res=5;
          goto xend;
        }
        data = xmlNodeGetContent(cs);
        if (strcmp((char *)data, label[i]) != 0)
        {
          res=5;
          goto xend;
        }
        cs = cs -> next;
      }
      xnode = xsnode -> children;
      res = setprop (xnode);
    }
  }

xend:
  return res;
}

int setbox (xmlNodePtr boxnode)
{
  int box;
  xmlNodePtr bnode, ba, bb;
  xmlChar * data;
  GtkEntry * info;
  double val;

  bnode = boxnode -> children;
  ba=findnode(bnode, "edges");
  if (ba == NULL)
  {
    box=7;
    goto bend;
  }
  ba = ba -> children;
  bb=findnode(ba, "a");
  if (bb == NULL)
  {
    box=7;
    goto bend;
  }
  data =  xmlNodeGetContent(bb);
  val = atof((char *)data);
  vmod[0]= val;
  bb=findnode(ba, "b");
  if (bb == NULL)
  {
    box=7;
    goto bend;
  }
  data =  xmlNodeGetContent(bb);
  val = atof((char *)data);
  vmod[1]= val;
  bb=findnode(ba, "c");
  if (bb == NULL)
  {
    box=7;
    goto bend;
  }
  data =  xmlNodeGetContent(bb);
  val = atof((char *)data);
  vmod[2]= val;
  ba=findnode(bnode, "angles");
  if (ba == NULL)
  {
    box=7;
    goto bend;
  }
  ba = ba -> children;
  bb=findnode(ba, "alpha");
  if (bb == NULL)
  {
    box=7;
    goto bend;
  }
  data =  xmlNodeGetContent(bb);
  val = atof((char *)data);
  langles[0] = val;
  bb=findnode(ba, "beta");
  if (bb == NULL)
  {
    box=7;
    goto bend;
  }
  data =  xmlNodeGetContent(bb);
  val = atof((char *)data);
  langles[1]= val;
  bb=findnode(ba, "gamma");
  if (bb == NULL)
  {
    box=7;
    goto bend;
  }
  data =  xmlNodeGetContent(bb);
  val = atof((char *)data);
  langles[2]= val;
  ba=findnode(bnode, "vectors");
  if (ba == NULL)
  {
    box=7;
    goto bend;
  }
  ba = ba -> children;
  bb=findnode(ba, "a.x");
  if (bb == NULL)
  {
    box=7;
    goto bend;
  }
  data =  xmlNodeGetContent(bb);
  val = atof((char *)data);
  cvect[0][0]= val;
  bb=findnode(ba, "a.y");
  if (bb == NULL)
  {
    box=7;
    goto bend;
  }
  data =  xmlNodeGetContent(bb);
  val = atof((char *)data);
  cvect[0][1]= val;
  bb=findnode(ba, "a.z");
  if (bb == NULL)
  {
    box=7;
    goto bend;
  }
  data =  xmlNodeGetContent(bb);
  val = atof((char *)data);
  cvect[0][2]= val;
  bb=findnode(ba, "b.x");
  if (bb == NULL)
  {
    box=7;
    goto bend;
  }
  data =  xmlNodeGetContent(bb);
  val = atof((char *)data);
  cvect[1][0]= val;
  bb=findnode(ba, "b.y");
  if (bb == NULL)
  {
    box=7;
    goto bend;
  }
  data =  xmlNodeGetContent(bb);
  val = atof((char *)data);
  cvect[1][1]= val;
  bb=findnode(ba, "b.z");
  if (bb == NULL)
  {
    box=7;
    goto bend;
  }
  data =  xmlNodeGetContent(bb);
  val = atof((char *)data);
  cvect[1][2]= val;
  bb=findnode(ba, "c.x");
  if (bb == NULL)
  {
    box=7;
    goto bend;
  }
  data =  xmlNodeGetContent(bb);
  val = atof((char *)data);
  cvect[2][0]= val;
  bb=findnode(ba, "c.y");
  if (bb == NULL)
  {
    box=7;
    goto bend;
  }
  data =  xmlNodeGetContent(bb);
  val = atof((char *)data);
  cvect[2][1]= val;
  bb=findnode(ba, "c.z");
  if (bb == NULL)
  {
    box=7;
    goto bend;
  }
  data =  xmlNodeGetContent(bb);
  val = atof((char *)data);
  cvect[2][2]= val;

  box = 0;

  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "moda");
  update_entry_double (info, & vmod[0]);
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "modb");
  update_entry_double (info, & vmod[1]);
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "modc");
  update_entry_double (info, & vmod[2]);
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "alpha");
  update_entry_double (info, & langles[0]);
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "beta");
  update_entry_double (info, & langles[1]);
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "gamma");
  update_entry_double (info, & langles[2]);
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "a.x");
  update_entry_double (info, & cvect[0][0]);
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "a.y");
  update_entry_double (info, & cvect[0][1]);
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "a.z");
  update_entry_double (info, & cvect[0][2]);
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "b.x");
  update_entry_double (info, & cvect[0][3]);
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "b.y");
  update_entry_double (info, & cvect[1][1]);
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "b.z");
  update_entry_double (info, & cvect[1][2]);
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "c.x");
  update_entry_double (info, & cvect[2][0]);
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "c.y");
  update_entry_double (info, & cvect[2][1]);
  info = (GtkEntry *) gtk_builder_get_object (BuilderGtk, "c.z");
  update_entry_double (info, & cvect[2][2]);

bend:
  return box;
}

int setpbc (xmlNodePtr pbcnode)
{
  int pbc, j;
  xmlNodePtr pnode, bnode;
  xmlChar * data;
  GtkWidget * info;
  double val;

  pnode = pbcnode -> children;
  bnode=findnode(pnode, "apply");
  if (bnode == NULL)
  {
    pbc=8;
    goto pend;
  }
  data = xmlNodeGetContent(bnode);
  apbc=FALSE;
  if (strcmp((char *)data, (char *)"TRUE") == 0)
  {
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "CheckPBC");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 1);
    apbc = TRUE;
  }
  bnode=findnode(pnode, "fractional");
  if (bnode == NULL)
  {
    pbc=8;
    goto pend;
  }
  data = xmlNodeGetContent(bnode);
  fractional=FALSE;
  if (strcmp((char *)data, (char *)"TRUE") == 0)
  {
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Frac");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), 1);
    fractional = TRUE;
    bnode=findnode(pnode, "fractype");
    if (bnode == NULL)
    {
      pbc=8;
      goto pend;
    }
    data = xmlNodeGetContent(bnode);
    val=atof((char *)data);
    j = val;
    tfrac = j;
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "combofrac");
    gtk_combo_box_set_active (GTK_COMBO_BOX(info), tfrac);
  }
  pbc=0;

pend:
  return pbc;
}

int setcutoffs (xmlNodePtr cutnode)
{
  int cut;
  int i, j;
  xmlNodePtr cnode, cn;
  xmlChar * data;
  double val;
  char * ncut;
  size_t lgt;

  cnode = cutnode -> children;
  cn = findnode(cnode, "total");
  if (cn == NULL)
  {
    cut=9;
    goto cend;
  }
  data = xmlNodeGetContent(cn);
  val=atof((char *)data);
  grtotcutoff = val;
  cn = findnode(cnode, "partials");
  if (cn == NULL)
  {
    cut=9;
    goto cend;
  }
  cnode = cn -> children;
  for ( i=0 ; i<nspec ; i++ )
  {
    for ( j=0 ; j<nspec ; j++ )
    {
      lgt=10;
      lgt+=strfind(label[i]) + strlen("-") + strfind(label[j]);
      ncut=calloc(lgt,sizeof*ncut);
      strncpy (ncut, label[i], strfind(label[i]));
      strncat(ncut, "-", 1);
      strncat(ncut, label[j], strfind(label[j]));
      cn = findnode(cnode, ncut);
      if (cn == NULL)
      {
        cut=9;
        goto cend;
      }
      data = xmlNodeGetContent(cn);
      val=atof((char *)data);
      cutoffs[i][j] = val;
      free(ncut);
      ncut=NULL;
    }
  }
  cut=0;

cend:
  return cut;
}

int settime(xmlNodePtr timenode)
{
  int tps;
  int i, j;
  xmlNodePtr tnode, tn;
  xmlChar * data;
  GtkWidget * info;
  double val;

  tnode = timenode -> children;

  tn = findnode(tnode, "dt");
  if (tn == NULL)
  {
    tps=10;
    goto tend;
  }
  data = xmlNodeGetContent(tn);
  val = atof((char *)data);
  delta_t = val;
  tn = findnode(tnode, "unit");
  if (tn == NULL)
  {
    tps=10;
    goto tend;
  }
  data = xmlNodeGetContent(tn);
  j=-1;
  for ( i=0 ; i<6 ; i++)
  {
    if (strcmp((char *)data, untime[i]) == 0) j=i;
  }
  if (j == -1)
  {
    tps=10;
    goto tend;
  }
  if (j < 5)
  {
    tunit = j+1;
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "TpsUnit");
    gtk_combo_box_set_active (GTK_COMBO_BOX(info), tunit-1);
  }
  tn = findnode(tnode, "ndt");
  if (tn != NULL)
  {
    data = xmlNodeGetContent(tn);
    val = atof((char *)data);
    ndtbs = val;
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Ndtbs");
    update_entry_int (GTK_ENTRY(info), & ndtbs);
  }
  tps=0;
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "DeltaT");
  update_entry_double (GTK_ENTRY(info), & delta_t);

tend:
  return tps;
}

int check_xml (const char * filetocheck)
{
  int res;
  xmlDoc * doc;
  xmlTextReaderPtr reader;
  const xmlChar xisaacs[11]="isaacs-xml";
  xmlChar * cdata, * cfichier;
  xmlNodePtr racine, node;
  GtkWidget * info;
  /*
   * build an xmlReader for that file
   */
  reader = xmlReaderForFile(filetocheck, NULL, 0);
  if (reader != NULL)
  {
    res = 0;
    doc = xmlParseFile(filetocheck);
    if (doc == NULL)
    {
      res=1;
      goto end;
    }
    else
    {
      racine = xmlDocGetRootElement(doc);
      if (strcmp((char *)(racine->name), (char *)xisaacs) != 0)
      {
        res=2;
        goto end;
      }
      else
      {
        node = racine -> children;
        node = findnode(node, "data");
        if (node == NULL)
        {
          res=3;
          goto end;
        }
        node = node -> children;
        node = findnode(node, "type");
        if (node == NULL)
        {
          res=3;
          goto end;
        }
        cdata = xmlNodeGetContent(node);
        node = findnode(node, "file");
        if (node == NULL)
        {
          res=3;
          goto end;
        }
        cfichier = xmlNodeGetContent(node);
        res= testopening((char *)cdata, (char *)cfichier);
        if (res != 0)
        {
          goto end;
        }
        else
        {
          node = racine -> children;
          node = findnode(node, "chemistry");
          if (node == NULL)
          {
            res=3;
            goto end;
          }
          res=setchemistry(node);
          if (res != 0)
          {
            goto end;
          }
          node = racine -> children;
          node = findnode(node, "pbc");
          if (node == NULL)
          {
            res=3;
            goto end;
          }
          res=setpbc(node);
          if (res != 0)
          {
             goto end;
          }
          node = racine -> children;
          node = findnode(node, "box");
          if (node == NULL)
          {
            res=3;
            goto end;
          }
          res=setbox(node);
          if (res != 0)
          {
             goto end;
          }
          node = racine -> children;
          node = findnode(node, "cutoffs");
          if (node == NULL)
          {
            res=3;
            goto end;
          }
          res=setcutoffs (node);
          if (res != 0)
          {
             goto end;
          }
          if (steps > 1)
          {
            node = racine -> children;
            node = findnode(node, "time-series");
            if (node == NULL)
            {
              res=3;
              goto end;
            }
            res=settime (node);
            if (res != 0)
            {
               goto end;
            }
          }
          node = racine -> children;
          node = findnode(node, "project");
          if (node == NULL)
          {
            res=3;
            goto end;
          }
          cdata=xmlNodeGetContent(node);
          if (strcmp((char *)cdata, (char *)"TRUE") == 0)
          {
            on_Apply_project_released ();
          }
          else
          {
            info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "ProjectSettings");
            gtk_widget_show(info);
          }
        }
      }
    }
  }
  else
  {
    res=1;
  }

end:
  /*
   * Cleanup function for the XML library.
   */
  if (reader != NULL) xmlFreeTextReader(reader);
  /*
   *Free the global variables that may
   *have been allocated by the parser.
   */
  xmlCleanupParser();

  return res;
}

int open_xml (const char * filetoread)
{
  int oxml;

  oxml = check_xml(filetoread);

  switch(oxml)
  {
    case 0:
      show_info ("Isaacs project file (XML) successfully opened\n");
      break;
    case 1:
      show_error ("Impossible to open the Isaacs project file (XML) file\n");
      break;
    case 2:
      show_error ("The file is not a valid Isaacs project file (XML) file\n");
      break;
    case 3:
      show_error ("The Isaacs project file (XML) file is uncomplete\n");
      break;
    case 4:
      show_error ("Problem(s) in the <data> section of the Isaacs project file (XML)\n");
      break;
    case 5:
      show_error ("Problem(s) in the <chemistry> section of the Isaacs project file (XML)\n");
      break;
    case 6:
      show_error ("Problem(s) in the <element> section of the Isaacs project file (XML)\n");
      break;
    case 7:
      show_error ("Problem(s) in the <box> section of the Isaacs project file (XML)\n");
      break;
    case 8:
      show_error ("Problem(s) in the <pbc> section of the Isaacs project file (XML)\n");
      break;
    case 9:
      show_error ("Problem(s) in the <cutoffs> section of the Isaacs project file (XML)\n");
      break;
    case 10:
      show_error ("Problem(s) in the <time-series> section of the Isaacs project file (XML)\n");
      break;
  }
  return(oxml);
}
