! This file is part of I.S.A.A.C.S.
!
! I.S.A.A.C.S. is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! I.S.A.A.C.S. is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>.

INTEGER (KIND=c_int) FUNCTION read_xyz (xyz_f, lxyz) BIND (C,NAME='read_xyz_')

!INTEGER FUNCTION read_xyz (xyz_file, lxyz)

!
! Lecture of atom types and coordinates - XYZ files
!
USE PARAMETERS

IMPLICIT NONE

LOGICAL :: isok
INTEGER (KIND=c_int), INTENT(IN) :: lxyz
CHARACTER (KIND=c_char), DIMENSION(*), INTENT(IN) :: xyz_f
CHARACTER (LEN=lxyz) :: xyz_file
INTERFACE
INTEGER FUNCTION TEST_LENGTH(UNITFILE, NATL)
  INTEGER, INTENT(IN) :: UNITFILE, NATL
END FUNCTION
END INTERFACE

do i=1, lxyz
  xyz_file(i:i) = xyz_f(i)
enddo

inquire (file=xyz_file, exist=isok)
if (isok) then

  open (unit=20, file=xyz_file, action='read', status='old')
  NS=0
  read (20, *) NA
  NS=TEST_LENGTH(20, NA+2)
  if (NS .ge. 1) then

    rewind(20)
    if (allocated(FULLPOS)) deallocate(FULLPOS)
    allocate (FULLPOS(NA,3,NS), STAT=ERR)
    if (ERR .ne. 0) then
      call show_error ("Impossible to allocate memory"//CHAR(0), &
                       "Subroutine: read_xyz"//CHAR(0), "Table: FULLPOS"//CHAR(0))
    endif
    if (allocated(TAB_OF_TYPE)) deallocate(TAB_OF_TYPE)
    allocate (TAB_OF_TYPE(NA), STAT=ERR)
    if (ERR .ne. 0) then
      call show_error ("Impossible to allocate memory"//CHAR(0), &
                       "Subroutine: read_xyz"//CHAR(0), "Table: TAB_OF_TYPE"//CHAR(0))
    endif
    do i=1, NS
      read (20, *)
      read (20, *)
      do j=1, NA
        read (20, *) TAB_OF_TYPE(j), FULLPOS(j,1,i), FULLPOS(j,2,i), FULLPOS(j,3,i)
      enddo
    enddo
    read_xyz=0
  else
    read_xyz=2
  endif

  close(20)

else
  read_xyz=1
endif

END FUNCTION

INTEGER FUNCTION TEST_LENGTH(UNITFILE, NATL)

IMPLICIT NONE
INTEGER, INTENT(IN) :: UNITFILE, NATL
INTEGER :: LENF=0
INTEGER :: EOF=0

rewind(UNITFILE)
EOF=0
LENF=0
do while (EOF .eq. 0)
  read (UNITFILE, *, iostat=EOF)
  LENF=LENF+1
enddo
LENF=LENF-1

TEST_LENGTH=mod(LENF,NATL)
if (TEST_LENGTH .eq. 0) then
 TEST_LENGTH=LENF/NATL
else
 TEST_LENGTH=0
endif

END FUNCTION


INTEGER (KIND=c_int) FUNCTION write_xyz (xyz_f, lxyz, fxyz, txyz) BIND (C,NAME='write_xyz_')

!
! output of atom types and coordinates - Multiple XYZ files
!

USE PARAMETERS

IMPLICIT NONE

INTEGER (KIND=c_int), INTENT(IN) :: lxyz, fxyz, txyz
CHARACTER (KIND=c_char), DIMENSION(*), INTENT(IN) :: xyz_f
CHARACTER (LEN=lxyz) :: xyz_file
INTERFACE
INTEGER FUNCTION TEST_LENGTH(UNITFILE, NATL)
  INTEGER, INTENT(IN) :: UNITFILE, NATL
END FUNCTION
END INTERFACE

do i=1, lxyz
  xyz_file(i:i) = xyz_f(i)
enddo

write_xyz=0
open (unit=20, file=xyz_file, action='write', status='unknown', err=001)

do i=1, NS
  write (20, '(i10)') NA
  write (20, *)
  do j=1, NA
    if (fxyz .eq. 1) then
      x=FULLPOS(j,1,i)*carttofrac(1,1)+FULLPOS(j,2,i)*carttofrac(2,1)+FULLPOS(j,3,i)*carttofrac(3,1)
      y=FULLPOS(j,1,i)*carttofrac(1,2)+FULLPOS(j,2,i)*carttofrac(2,2)+FULLPOS(j,3,i)*carttofrac(3,2)
      z=FULLPOS(j,1,i)*carttofrac(1,3)+FULLPOS(j,2,i)*carttofrac(2,3)+FULLPOS(j,3,i)*carttofrac(3,3)
    else
      x= FULLPOS(j,1,i)
      y= FULLPOS(j,2,i)
      z= FULLPOS(j,3,i)
      if (txyz .eq. 1) then
        x= x/ANGTOBOHR
        y= y/ANGTOBOHR
        z= z/ANGTOBOHR
      endif
    endif
    write (20, '(a2,3(3x,f15.10))', err=002) TAB_OF_TYPE(j), x, y, z
  enddo
enddo
write_xyz=1

002 continue

close (20)

001 continue

END FUNCTION
